/*  RAxML-V, a program for sequential and parallel estimation of phylogenetic trees 
 *  from nucleotide sequences implementing SEVs (Subtree Equality Vectors), the lazy subtree
 *  rearrangement heuristics, a simulated annealing search algorithm, and a parallel OpenMP-version
 *  for SMPs. Copyright October 2005 by Alexandros Stamatakis
 *
 *  Partially derived from
 *  fastDNAml, a program for estimation of phylogenetic trees from sequences by Gary J. Olsen
 *  
 *  and 
 *
 *  Programs of the PHYLIP package by Joe Felsenstein.
 *
 *  This program is free software; you may redistribute it and/or modify its
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 * 
 *
 *  For any other enquiries send an Email to Alexandros Stamatakis
 *  stamatak@ics.forth.gr
 *
 *  When publishing work that is based on the results from RAxML-V please cite:
 *  
 *  Alexandros Stamatakis: "An Efficient Program for phylogenetic Inference Using Simulated Annealing". 
 *  Proceedings of IPDPS2005, Denver, Colorado, April 2005.
 *  
 *  AND
 *
 *  Alexandros Stamatakis, Thomas Ludwig, and Harald Meier: "RAxML-III: A Fast Program for Maximum Likelihood-based Inference of Large Phylogenetic Trees". 
 *  In Bioinformatics 21(4):456-463.
 *
 *
 *
 *
 */



/* TODO HPC nmlngth ändern z0 entfernen!*/


#define EQUALITIES 16
#define maxcategories 2

/*  Program constants and parameters  */

#define maxlogf       1024  /* maximum number of user trees */
#define smoothings      32  /* maximum smoothing passes through tree */
#define iterations      10  /* maximum iterations of iterations per insert */
#define newzpercycle     1  /* iterations of makenewz per tree traversal */
#define nmlngth         100  /* number of characters in species name */
#define deltaz     0.00001  /* test of net branch length change in update */
#define zmin       1.0E-15  /* max branch prop. to -log(zmin) (= 34) */
#define zmax (1.0 - 1.0E-6) /* min branch prop. to 1.0-zmax (= 1.0E-6) */
#define defaultz       0.9  /* value of z assigned as starting point */
#define unlikely  -1.0E300  /* low likelihood for initialization */

/*  These values are used to rescale the lilelihoods at a given site so that
 *  there is no floating point underflow.
 */
#define twotothe256  \
115792089237316195423570985008687907853269984665640564039457584007913129639936.0
                                                     /*  2**256 (exactly)  */
#define minlikelihood  (1.0/twotothe256)





#define badEval        1.0
#define badZ           0.0
#define badRear         -1
#define badSigma      -1.0

#define TRUE             1
#define FALSE            0

#define treeNone         0
#define treeNewick       1
#define treeProlog       2
#define treePHYLIP       3
#define treeMaxType      3
#define treeDefType  treePHYLIP

#define ABS(x)    (((x)<0)   ?  (-(x)) : (x))
#define MIN(x,y)  (((x)<(y)) ?    (x)  : (y))
#define MAX(x,y)  (((x)>(y)) ?    (x)  : (y))
#define LOG(x)    (((x)>0)   ? log(x)  : hang("log domain error"))
#define NINT(x)   ((int) ((x)>0 ? ((x)+0.5) : ((x)-0.5)))

#define programName        "RAxMl-V"
#define programVersion     "2.1"
#define programVersionInt   2105
#define programDate        "MARCH, 2005"
#define programDateInt      200503

#define TREE_INFERENCE 1
#define TREE_EVALUATION 2
#define FAST_MODE 3
#define SIMULATED_ANNEALING 4
#define TREE_RECURSIVE 5
#define HEAVY_MODE 6
#define PRE_SELECT_MODE 7
#define SPR 8
#define BIG_MODE 9
#define BIG_FAST_MODE 10
#define BIG_RAPID_MODE 11
#define PARALLEL_MODE 12


#define M_HKY85 1
#define M_HKY85CAT 2
#define M_GTR 3
#define M_GTRCAT 4
#define M_PROTEINS 5
#define M_PAM 6
#define M_PMB 7
#define M_JTT 8
#define M_PROTEINSCAT 9
#define M_HKY85GAMMA 10
#define M_HKY85GAMMA2 11
#define M_GTRGAMMA 12

#define REARRANGEMENT_SPECIFICATION 0


typedef  char  yType;
typedef  int     boolean;
typedef  double  xtype;

typedef struct bInf {
  double likelihood;
  int node;
} bestInfo;

typedef struct iL {
  bestInfo *list;
  int n;
  int valid;
} infoList;




typedef  struct  likelihood_vector {
    xtype        a, c, g, t;
    long         exp;
    } likelivector;

typedef  struct  gamma_likelihood_vector 
{
  xtype        a0, c0, g0, t0, a1, c1, g1, t1, a2, c2, g2, t2, a3, c3, g3, t3;
  long         exp;
} 
  gammalikelivector;

typedef  struct  protein_vector {
    xtype        v[20];
    long         exp;
    } proteinlikelivector;


typedef struct protein_matrix {
  double m[20][20];
} proteinmatrix;



typedef  struct  xmantyp {
    struct xmantyp  *prev;
    struct xmantyp  *next;
    struct noderec  *owner;
    likelivector    *lv;
#ifndef HPC
  proteinlikelivector *pv;
  gammalikelivector *gv;
#endif
    } xarray;


typedef struct ratec 
{
  double accumulatedSiteLikelihood;
  double rate;
} rateCategorize;




typedef struct fgh{
  double a, c, g, t;
  long exp;
  boolean set;
} homType;




typedef  struct noderec {
  double           z;
#ifndef HPC
  double z0;
#endif
  struct noderec  *next;
  struct noderec  *back;
  int              number;
  xarray          *x;   
  yType           *tip;          

#ifndef HPC
  int             *equalityVector;
  homType mlv[EQUALITIES]; 
#endif  
    } node, *nodeptr;






typedef struct {
  double sumaq; 
  double sumgq; 
  double sumcq; 
  double sumtq;  
  int exp;
} poutsa;

typedef struct {
  double fx1a, fx1g, fx1c, fx1t, sumag, sumct, sumagct;
  int exp;
} memP;

typedef struct {
  double a, b, c, w;
} mnzMem;


typedef struct {
  double a, b, c, w, dlnLidlz, d2lnLdlz2;
} mnzMem2;



typedef struct {
  double sumfx2rfx2y, fx2r, fx2y;
  double a, c, g, t;
  int exp;
} memQ;

typedef  struct {
  int              numsp;       /* number of species (also tr->mxtips) */
  int              sites;       /* number of input sequence positions */
  yType          **y;           /* sequence data array */
  yType            *y0;  
  
  double *eigmatPROT;
  double **probmatPROT;
  double freqaa[20];
  double invfreqaa[20];
  double crossMAT[20][20][20];
  
  double freqaREV[maxcategories+1], 
    freqcREV[maxcategories+1], 
    freqgREV[maxcategories+1], 
    freqtREV[maxcategories+1], 
    ttratioREV[maxcategories+1];
  boolean matread;
  double EIGV[maxcategories+1][4][4];
  double invfreq[maxcategories+1][4];
  double EIGN[maxcategories+1][4];
  double fracchangeav;
  double initialRates[5];
  double           freqa, freqc, freqg, freqt, ttratio;
  double          freqr, freqy, invfreqr, invfreqy,
                  freqar, freqcy, freqgr, freqty;
  double           xi, xv, fracchange; /* transition/transversion */
/* End of DNA specific values */
    int             *wgt;         /* weight per sequence pos */
    int             *wgt2;        /* weight per pos (booted) */
    int              categs;      /* number of rate categories */
    } rawdata;

typedef  struct {
  int             *alias;       /* site representing a pattern */
  int             *aliaswgt;    /* weight by pattern */
  int             *rateCategory;
  int              endsite;     /* # of sequence patterns */
  int              endsiteNormal;
  int              wgtsum;      /* sum of weights of positions */
  double          *patrat;      /* rates per pattern */
  double          *patratStored;
  double          *wr;          /* weighted rate per pattern */
  double          *wr2;         /* weight*rate**2 per pattern */
} cruncheddata;

typedef struct {
  double lh;
  int node;
} preScore;



typedef struct rmsg {
  int removedNumberP;
  int removedNumberQ;
  int removedNumberR;
  int insertNumberQ;
  int insertNumberR;
  double currentZQR;
  double currentLZR;
  double currentLZQ;
  double currentLZS;
} rearrangementDescriptor;

  




typedef  struct {
  double           startLH;
  double endLH;
    double           likelihood;   
    node           **nodep;
    node            *start;
    node            *outgrnode;
    int              mxtips;
    int              ntips;
    int              nextnode;
    int              opt_level;   
    int              outgr;       /* sequence number to use in rooting tree */
  int              NumberOfCategories;
    boolean          prelabeled;  /* the possible tip names are known */
    boolean          smoothed;
    boolean          rooted;
    rawdata         *rdta;        /* raw data structure */
    cruncheddata    *cdta;        /* crunched data structure */
    double t_value;
   double *modelData;
  int modelParameters;
  double alpha;
  int gammaCATS;
  double *gammaRates;
  double *gammaProbs;
  preScore *nodeScoresOne;
  preScore *nodeScoresAll;
  int *nodeCount;
  double maxLH;
  double *dist;
  char **nameList;

  /************BIG*********/


  double bestOfNode;
  int specialOPT;
  nodeptr removeNode;
  nodeptr insertNode;
  double zqr;
  double currentZQR;

  double currentLZR;
  double currentLZQ;
  double currentLZS;
  double lzs;
  double lzq;
  double lzr;
  int nodeInfo;

  /***********BIG************/

  /********PARALLEL***********/
  int removeNumberP;
  int removeNumberQ;
  int removeNumberR;
  int removedNumberP;
  int removedNumberQ;
  int removedNumberR;
  int insertNumberQ;
  int insertNumberR;
  
  rearrangementDescriptor *rDesc;
  /*************************/
    } tree;





typedef struct subnodetyp {
  int count;
  node **nodeArray;
} subNodes;

typedef struct xyz {
  int difference;
  nodeptr bnode;
} BType;


typedef struct conntyp {
    double           z;           /* branch length */
    node            *p, *q;       /* parent and child sectors */
    void            *valptr;      /* pointer to value of subtree */
    int              descend;     /* pointer to first connect of child */
    int              sibling;     /* next connect from same parent */
    } connect, *connptr;

typedef  struct {
    double           likelihood;
  int              number;
    connect         *links;       /* pointer to first connect (start) */
    node            *start;
    int              nextlink;    /* index of next available connect */
                                  /* tr->start = tpl->links->p */
    int              ntips;
    int              nextnode;
    int              opt_level;   /* degree of branch swapping explored */
    int              scrNum;      /* position in sorted list of scores */
    int              tplNum;      /* position in sorted list of trees */
    
    boolean          prelabeled;  /* the possible tip names are known */
    boolean          smoothed;    /* branch optimization converged? */
    } topol;

typedef struct {
    double           best;        /* highest score saved */
    double           worst;       /* lowest score saved */
    topol           *start;       /* starting tree for optimization */
    topol          **byScore;
    topol          **byTopol;
    int              nkeep;       /* maximum topologies to save */
    int              nvalid;      /* number of topologies saved */
    int              ninit;       /* number of topologies initialized */
    int              numtrees;    /* number of alternatives tested */
    boolean          improved;
    } bestlist;

typedef  struct {
  double timeLimit;
  double t0;
  int categories;
  int repeats; 
  int model;
  int protmodel;
  int              max_rearrange;
  int              stepwidth;
  int              initial;
  int              mode;
  long             boot;        /* bootstrap random number seed */    
  boolean          interleaved; /* input data are in interleaved format */    
  boolean          restart;     /* resume addition to partial tree */  
  boolean          useWeightFile;
  int              maxSubProblemSize;
  int checkpoints;
  int startingTreeOnly;
} analdef;




static double jtteigmat[] =
{0.0,        -0.7031123, -0.6484345, -0.6086499, -0.5514432,
-0.772664, -0.8643413, -1.0620756, -0.9965552, -1.1671808,
-1.2222418,-0.4589201, -1.3103714, -1.4048038, -0.3170582,
-0.347935, -1.5311677, -1.6021194, -1.7991454, -1.8911888};

static double jttprobmat[20][20] =
{{0.076999996, 0.051000003, 0.043000004, 0.051999998, 0.019999996, 0.041,
  0.061999994, 0.073999997, 0.022999999, 0.052000004, 0.090999997, 0.058999988,
  0.024000007, 0.04, 0.050999992, 0.069, 0.059000006, 0.014000008, 0.032000004,
  0.066000005},
 {0.015604455, -0.068062363, 0.020106264, 0.070723273, 0.011702977, 0.009674053,
  0.074000798, -0.169750458, 0.005560808, -0.008208636, -0.012305869,
 -0.063730179, -0.005674643, -0.02116828, 0.104586169, 0.016480839, 0.016765139,
  0.005936994, 0.006046367, -0.0082877},
 {-0.049778281, -0.007118197, 0.003801272, 0.070749616, 0.047506147,
   0.006447017, 0.090522425, -0.053620432, -0.008508175, 0.037170603,
   0.051805545, 0.015413608, 0.019939916, -0.008431976, -0.143511376,
  -0.052486072, -0.032116542, -0.000860626, -0.02535993, 0.03843545},
 {-0.028906423, 0.092952047, -0.009615343, -0.067870117, 0.031970392,
   0.048338335, -0.054396304, -0.135916654, 0.017780083, 0.000129242,
   0.031267424, 0.116333586, 0.007499746, -0.032153596, 0.033517051,
  -0.013719269, -0.00347293, -0.003291821, -0.02158326, -0.008862168},
 {0.037181176, -0.023106564, -0.004482225, -0.029899635, 0.118139633,
 -0.032298569, -0.04683198, 0.05566988, -0.012622847, 0.002023096,
 -0.043921088, -0.04792557, -0.003452711, -0.037744513, 0.020822974,
  0.036580187, 0.02331425, -0.004807711, -0.017504496, 0.01086673},
 {0.044754061, -0.002503471, 0.019452517, -0.015611487, -0.02152807,
 -0.013131425, -0.03465365, -0.047928912, 0.020608851, 0.067843095,
 -0.122130014, 0.002521499, 0.013021646, -0.082891087, -0.061590119,
  0.016270856, 0.051468938, 0.002079063, 0.081019713, 0.082927944},
 {0.058917882, 0.007320741, 0.025278141, 0.000357541, -0.002831285,
 -0.032453034, -0.010177288, -0.069447924, -0.034467324, 0.011422358,
 -0.128478324, 0.04309667, -0.015319944, 0.113302422, -0.035052393,
  0.046885372, 0.06185183, 0.00175743, -0.06224497, 0.020282093},
 {-0.014562092, 0.022522921, -0.007094389, 0.03480089, -0.000326144,
 -0.124039037, 0.020577906, -0.005056454, -0.081841576, -0.004381786,
  0.030826152, 0.091261631, 0.008878828, -0.02829487, 0.042718836,
 -0.011180886, -0.012719227, -0.000753926, 0.048062375, -0.009399129},
 {0.033789571, -0.013512235, 0.088010984, 0.017580292, -0.006608005,
 -0.037836971, -0.061344686, -0.034268357, 0.018190209, -0.068484614,
  0.120024744, -0.00319321, -0.001349477, -0.03000546, -0.073063759,
  0.081912399, 0.0635245, 0.000197, -0.002481798, -0.09108114},
 {-0.113947615, 0.019230545, 0.088819683, 0.064832765, 0.001801467,
 -0.063829682, -0.072001633, 0.018429333, 0.057465965, 0.043901014,
 -0.048050874, -0.001705918, 0.022637173, 0.017404665, 0.043877902,
 -0.017089594, -0.058489485, 0.000127498, -0.029357194, 0.025943972},
 {0.01512923, 0.023603725, 0.006681954, 0.012360216, -0.000181447,
 -0.023011838, -0.008960024, -0.008533239, 0.012569835, 0.03216118,
  0.061986403, -0.001919083, -0.1400832, -0.010669741, -0.003919454,
 -0.003707024, -0.026806029, -0.000611603, -0.001402648, 0.065312824},
 {-0.036405351, 0.020816769, 0.011408213, 0.019787053, 0.038897829,
   0.017641789, 0.020858533, -0.006067252, 0.028617353, -0.064259496,
  -0.081676567, 0.024421823, -0.028751676, 0.07095096, -0.024199434,
  -0.007513119, -0.028108766, -0.01198095, 0.111761119, -0.076198809},
 {0.060831772, 0.144097327, -0.069151377, 0.023754576, -0.003322955,
 -0.071618574, 0.03353154, -0.02795295, 0.039519769, -0.023453968,
 -0.000630308, -0.098024591, 0.017672997, 0.003813378, -0.009266499,
 -0.011192111, 0.016013873, -0.002072968, -0.010022044, -0.012526904},
 {-0.050776604, 0.092833081, 0.044069596, 0.050523021, -0.002628417,
   0.076542572, -0.06388631, -0.00854892, -0.084725311, 0.017401063,
  -0.006262541, -0.094457679, -0.002818678, -0.0044122, -0.002883973,
   0.028729685, -0.004961596, -0.001498627, 0.017994575, -0.000232779},
 {-0.01894566, -0.007760205, -0.015160993, -0.027254587, 0.009800903,
  -0.013443561, -0.032896517, -0.022734138, -0.001983861, 0.00256111,
   0.024823166, -0.021256768, 0.001980052, 0.028136263, -0.012364384,
  -0.013782446, -0.013061091, 0.111173981, 0.021702122, 0.00046654},
 {-0.009444193, -0.042106824, -0.02535015, -0.055125574, 0.006369612,
  -0.02945416, -0.069922064, -0.067221068, -0.003004999, 0.053624311,
   0.128862984, -0.057245803, 0.025550508, 0.087741073, -0.001119043,
  -0.012036202, -0.000913488, -0.034864475, 0.050124813, 0.055534723},
 {0.145782464, -0.024348311, -0.031216873, 0.106174443, 0.00202862,
  0.02653866, -0.113657267, -0.00755018, 0.000307232, -0.051241158,
  0.001310685, 0.035275877, 0.013308898, 0.002957626, -0.002925034,
 -0.065362319, -0.071844582, 0.000475894, -0.000112419, 0.034097762},
 {0.079840455, 0.018769331, 0.078685899, -0.084329807, -0.00277264,
 -0.010099754, 0.059700608, -0.019209715, -0.010442992, -0.042100476,
 -0.006020556, -0.023061786, 0.017246106, -0.001572858, -0.006703785,
  0.056301316, -0.156787357, -0.000303638, 0.001498195, 0.051363455},
 {0.049628261, 0.016475144, 0.094141653, -0.04444633, 0.005206131,
 -0.001827555, 0.02195624, 0.013066683, -0.010415582, -0.022338403,
  0.007837197, -0.023397671, -0.002507095, 0.005177694, 0.017109561,
 -0.202340113, 0.069681441, 0.000120736, 0.002201146, 0.004670849},
 {0.089153689, 0.000233354, 0.010826822, -0.004273519, 0.001440618,
  0.000436077, 0.001182351, -0.002255508, -0.000700465, 0.150589876,
 -0.003911914, -0.00050154, -0.004564983, 0.00012701, -0.001486973,
 -0.018902754, -0.054748555, 0.000217377, -0.000319302, -0.162541651}};





/* static double jtteigmat[20] = {-0.00000000000000042050, -0.31372372780472634890, -0.34649848643426939265, -0.45728273706643091101, -0.56341303983801083177, -0.61333402181352925098, -0.65418447505480870063, -0.72146574701492993675, -0.76948188027116115606, -0.86056434891084798711, -1.00323662179670636263, -1.05596623188043636787, -1.17708886681475100033, -1.27670757990941940996, -1.30699165722397903089, -1.40659039992283485532, -1.53306746928330572821, -1.62112099416155386322, -1.81255342068192804028, -1.90078638411637035333};


static double jttprobmat[20][20] = {{0.60291674248740612096,-0.07085489277328257951,0.02873922137492035747,-0.08708414159233328344,0.16080533650936829071,0.08976940460709023628,-0.26433179074036866618,0.02206560331149652610,-0.13521575737004046247,0.24055253024320391586,-0.11450618464964867904,0.01799370114822089473,-0.42406782472309517695,0.16822375561659272392,-0.14581275694115386488,0.10204311279854458294,0.35129793928545272053,-0.21226371702651189444,0.10084986055440448016,-0.29915905686652122242},{0.60291674248740534381,-0.03513745713718952057,0.16843378947335985174,0.06798804364002701550,-0.11445343910271407739,-0.44970177174856929003,-0.01478257623278464439,-0.18279712711285781013,0.02233308014474476300,0.02897507221264240124,0.06601560017045324247,-0.03932398009913554843,0.11387629228946197224,0.30979770105600068497,-0.60484449183358268698,-0.28165577533273183919,-0.12845920377985203809,-0.04751225788557379009,0.05598925443810346542,0.00046962766782149019},{0.60291674248740312336,-0.09611851897791846844,0.12664396756410439826,0.04937618129809871054,-0.03341303987221153071,0.05122270207240750550,0.01043814922736669565,0.06010414348414046259,-0.10048724989784080242,0.18529582065532265722,-0.46452151846521705369,0.03111748226857309965,0.60828207032013326128,-0.09575878735170430256,0.33829627855846644557,-0.17205511982041898911,-0.22217908975441799657,-0.28433905085106225830,0.38351236628987772637,-0.06360640166663091011},{0.60291674248740501074,-0.14497483026752894242,0.22428966903106359321,0.07458547682394907341,-0.22768537044784367684,0.33877706483375907842,0.49444934014039121362,0.18949058527531706386,0.06376049904408041813,0.00461807021170752147,-0.08367931105818575410,-0.05172772098512534722,0.37592914976790503534,0.07525642222301015538,-0.11511734448374691397,-0.17347061864814442012,0.47203931960147493818,0.20661002878764866963,-0.13586614994496290509,0.01921822333297960525},{0.60291674248740456665,0.13544935406547500834,-0.06627219179062079912,0.32467346693086651666,1.98023885999870374874,-0.27631431452794336101,1.05579123947620501234,0.10939416051388732576,0.24707287264445620956,-0.05828907592993012171,0.07977788325949257409,0.00051073995553892280,0.01729270006831894885,-0.02708061182509786252,0.03720222395337140220,0.02321073530027194590,0.02926606547995642960,0.01751749411453080302,0.05046551412028690148,-0.01654870471860716077},{0.60291674248740512176,-0.08858607849491573205,0.14847389251358003581,0.07512709779340107086,-0.24524342321348016616,-0.28593651348362630893,0.07109612892599781464,0.05045030418120054205,0.06536342526908010819,-0.24283102241974918734,0.23535678393376421869,0.28280665953401784574,-0.46932562629622781580,-0.22069889433223821573,0.36404567942045956697,-0.32838494704580772909,0.13172519610998162376,0.02612112135943266134,-0.00870714545558027436,-0.00197870413413063885},{0.60291674248740667608,-0.14645204097262171095,0.23828143206097804341,0.06450116851554428543,-0.28404691091088229982,0.23956582930052883196,0.54754865925128015647,0.16767662848422015820,0.12083834220417900840,-0.04049402432859762480,0.21387521106228471468,-0.03957513091979730085,-0.35228736397354976084,0.00393470616969096836,-0.09790513653810806183,0.17701074449114032050,-0.40384258549724938447,-0.10839956792674919350,0.05489062621250435231,-0.00249936033828204512},{0.60291674248740434461,-0.07986454962509635658,0.18387873468117824083,-0.00993777712549478477,0.23789260771948755879,0.38200830875694952171,-0.30333358193217774268,-0.36499014827827985785,0.17860320179147906594,-0.33396812042309015345,0.12458573561539150465,0.00384619535044854969,0.07179845372693023642,-0.06966597340189305265,0.07934390622159429030,0.02029094630265065521,-0.00708260410898468081,0.04229248875316305584,0.03834087834328561706,0.00944271154792268759},{0.60291674248740578790,-0.03180899016554650932,0.03379694361595960567,0.22470204848804162245,-0.16488082866999770326,-0.19852034749679886572,-0.13357848764976049116,0.05064318770630669292,-0.24110243045839285791,-0.42327162003097296150,-0.11844539768863431017,0.34945993432789013156,0.67647384842557733986,0.18184042742528039316,-0.33231229777966664329,0.65696933494832132627,0.02568892184680700644,0.07059122258811709183,-0.07723078460513853327,0.00851456808722374052},{0.60291674248740179110,0.00071663619971400563,-0.21387482142388802853,-0.22522613067882227278,-0.00591695502392162835,0.01785267767362070865,0.24174149049455817551,-0.02797334007406349771,-0.37035852119893064582,0.12842971920605941727,0.29715085445868971714,0.00695058114843415727,0.25610200156589474352,0.12742879169046619769,0.11608270479628815530,-0.05123738208483145234,-0.18196560984185791709,0.16203867157617324324,-0.06289938190407393104,-0.67719040768734595215},{0.60291674248740589892,0.06044628336836663923,-0.29058832296520376026,-0.16602794490885477985,-0.16533760852937595676,-0.07872227470041257003,0.23443294668478523168,-0.00530079190676854134,0.32161660567110705866,-0.50581926986867820073,-0.28348049881575121400,-0.03238117051156779586,-0.11836232843879873622,0.19959715912225850509,0.03379048379695254178,0.01156443565151164810,0.00319599923899015811,0.01119653277342255726,0.01669397795050793845,0.00689169787198250054},{0.60291674248740623199,-0.09520262681867788790,0.20079825941042289683,0.06872252898865262305,-0.24413292583672824287,-0.47422929539321639902,0.14236248946735749477,-0.14702085909583967327,-0.00536292529528508300,0.23152967757277581606,-0.00992256343788251727,-0.15453716956292382068,0.00145024483976548897,-0.12252023038556367207,0.36504340420166564440,0.25273974542125898779,0.15619502160376858013,0.05011024652507037325,-0.07113403406904360293,-0.00038399023629508882},{0.60291674248740878550,0.01011217871442918713,-0.21213836200160357537,-0.21649933380335981137,-0.06071165931802807725,-0.05339001631324856445,0.27279757712430996452,-0.02965536471654381390,-0.17895229855396205254,-0.14743409432819842841,0.00779427099499436539,-0.02607765691534287902,0.11124503391740044045,-1.73463955450684093051,-0.43335288263459537106,0.00980047245098399078,0.10430660677181415041,-0.14261021181973912442,-0.03164024019791662062,0.03001384252114482412},{0.60291674248740523279,0.17051365622597533522,-0.45149329592830794722,0.32229233150378894246,-0.29821970584540929972,0.14971002654593690284,-0.06390498726170214250,-0.09302269901320811551,0.67287509920139665365,0.83628119860186855838,0.15031354166842406106,0.06410874772165366109,0.08704163606868765835,-0.06868953702296357045,-0.03210056138715523810,0.01768615337752852540,0.01694684735761131308,0.00243005510845233953,0.02255709630223620352,0.00262245006088374566},{0.60291674248740390052,-0.07243580186224075901,0.01392232649372686717,-0.08275252274637395766,0.16928783862284466077,-0.14590877177383995700,-1.00168313371125239364,0.33765762512540431484,0.33168301967009133913,-0.20807771869533223108,0.33760681375325130027,-0.08907489094284364028,0.24132567759498299576,-0.05901344954064961096,0.03131163250566029732,0.01283048870911927992,-0.00336872664157376878,0.01740342434864068991,0.06652421328772145648,0.00748972436887031072},{0.60291674248741666808,-0.05631318349468034806,0.04145189291079691418,-0.01987979942478768081,0.18771187561407876032,0.04641274237507999473,-0.29869410334006929464,0.03056857722936862781,-0.03105797009794671396,0.19153023762511892092,-0.27117267458769039878,0.01756807982899749251,-0.05650512803594512357,-0.00681451119023730734,0.03632755231417725272,-0.06907601565018613166,-0.22417637128607192931,-0.07538940314283325284,-0.54191856865273291266,0.06438913169627799582},{0.60291674248739535180,-0.06430260069936860612,0.00882132710004505938,-0.08767347418793808300,0.13553422952510923993,0.01885141207484775788,-0.23202939944752615231,0.03646540421881564603,-0.20685742650571001255,0.33337765490815474134,-0.26133960702515196539,0.02163856494529488653,-0.29851301074772973454,-0.10141069958083312463,-0.07734915857027752084,0.01894192033507322362,-0.15952308034888104671,0.43325383778893922226,0.23856643899684254473,0.24360714679230588420},{0.60291674248740523279,2.43099735248706183555,0.41727806904018671652,-0.14001067074298251258,-0.12318765896439920482,0.06279302003913689312,-0.03111904023943605144,0.05671191693445922638,-0.03766569995902683404,0.03986572196102394078,-0.00037913021889041072,0.00492323324573330605,0.00163959448293215311,-0.01830958523815630345,0.02998956463216777280,0.01717129421373365591,0.00886190793828698503,0.00287949422968441495,0.00149441760834284035,-0.00532126597176604040},{0.60291674248740534381,0.17130000419686758351,-0.32824943291627345365,0.67047908629828378846,-0.16543509481003620953,0.13010296542865906755,-0.34523385079354435012,0.02473210729825541312,-0.73129375546921771001,-0.51874884938129228740,0.00858485706941676877,-0.13701881998653650530,-0.22225206242115927435,-0.01257801468085045261,0.05888076763211599202,-0.09490538371229639836,-0.00148909939101503721,-0.00792824425624943756,0.01205816388487477263,0.00162708320613202538},{0.60291674248741011777,-0.00836404148489720660,-0.17373106770239954422,-0.21121514965311677381,0.03506180037750939588,0.04631156247717953950,0.18976778595019094809,-0.02420025421247242944,-0.34887143514851026715,0.14438774538519477675,0.29767491528149137014,0.01074653964165700991,0.12441729972212603461,0.26736964438595289817,0.07927070088898242883,0.00203934444181253557,0.08761657335749901965,-0.14304430295621284319,0.00472090230614827244,0.60991392702611313403}};
*/



/* this PMB matrix decomposition due to Elisabeth Tillier */
static double pmbeigmat[20] =
{0.0000001586972220,-1.8416770496147100, -1.6025046986139100,-1.5801012515121300,
-1.4987794099715900,-1.3520794233801900,-1.3003469390479700,-1.2439503327631300,
-1.1962574080244200,-1.1383730501367500,-1.1153278910708000,-0.4934843510654760,
-0.5419014550215590,-0.9657997830826700,-0.6276075673757390,-0.6675927795018510,
-0.6932641383465870,-0.8897872681859630,-0.8382698977371710,-0.8074694642446040};

static double pmbprobmat[20][20] =
{{0.0771762457248147,0.0531913844998640,0.0393445076407294,0.0466756566755510,
0.0286348361997465,0.0312327748383639,0.0505410248721427,0.0767106611472993,
0.0258916271688597,0.0673140562194124,0.0965705469252199,0.0515979465932174,
0.0250628079438675,0.0503492018628350,0.0399908189418273,0.0641898881894471,
0.0517539616710987,0.0143507440546115,0.0357994592438322,0.0736218495862984},
{0.0368263046116572,-0.0006728917107827,0.0008590805287740,-0.0002764255356960,
0.0020152937187455,0.0055743720652960,0.0003213317669367,0.0000449190281568,
-0.0004226254397134,0.1805040629634510,-0.0272246813586204,0.0005904606533477,
-0.0183743200073889,-0.0009194625608688,0.0008173657533167,-0.0262629806302238,
0.0265738757209787,0.0002176606241904,0.0021315644838566,-0.1823229927207580},
{-0.0194800075560895,0.0012068088610652,-0.0008803318319596,-0.0016044273960017,
-0.0002938633803197,-0.0535796754602196,0.0155163896648621,-0.0015006360762140,
0.0021601372013703,0.0268513218744797,-0.1085292493742730,0.0149753083138452,
0.1346457366717310,-0.0009371698759829,0.0013501708044116,0.0346352293103622,
-0.0276963770242276,0.0003643142783940,0.0002074817333067,-0.0174108903914110},
{0.0557839400850153,0.0023271577185437,0.0183481103396687,0.0023339480096311,
0.0002013267015151,-0.0227406863569852,0.0098644845475047,0.0064721276774396,
0.0001389408104210,-0.0473713878768274,-0.0086984445005797,0.0026913674934634,
0.0283724052562196,0.0001063665179457,0.0027442574779383,-0.1875312134708470,
0.1279864877057640,0.0005103347834563,0.0003155113168637,0.0081451082759554},
{0.0037510125027265,0.0107095920636885,0.0147305410328404,-0.0112351252180332,
-0.0001500408626446,-0.1523450933729730,0.0611532413339872,-0.0005496748939503,
0.0048714378736644,-0.0003826320053999,0.0552010244407311,0.0482555671001955,
-0.0461664995115847,-0.0021165008617978,-0.0004574454232187,0.0233755883688949,
-0.0035484915422384,0.0009090698422851,0.0013840637687758,-0.0073895139302231},
{-0.0111512564930024,0.1025460064723080,0.0396772456883791,-0.0298408501361294,
-0.0001656742634733,-0.0079876311843289,0.0712644184507945,-0.0010780604625230,
-0.0035880882043592,0.0021070399334252,0.0016716329894279,-0.1810123023850110,
0.0015141703608724,-0.0032700852781804,0.0035503782441679,0.0118634302028026,
0.0044561606458028,-0.0001576678495964,0.0023470722225751,-0.0027457045397157},
{0.1474525743949170,-0.0054432538500293,0.0853848892349828,-0.0137787746207348,
-0.0008274830358513,0.0042248844582553,0.0019556229305563,-0.0164191435175148,
-0.0024501858854849,0.0120908948084233,-0.0381456105972653,0.0101271614855119,
-0.0061945941321859,0.0178841099895867,-0.0014577779202600,-0.0752120602555032,
-0.1426985695849920,0.0002862275078983,-0.0081191734261838,0.0313401149422531},
{0.0542034611735289,-0.0078763926211829,0.0060433542506096,0.0033396210615510,
0.0013965072374079,0.0067798903832256,-0.0135291136622509,-0.0089982442731848,
-0.0056744537593887,-0.0766524225176246,0.1881210263933930,-0.0065875518675173,
0.0416627569300375,-0.0953804133524747,-0.0012559228448735,0.0101622644292547,
-0.0304742453119050,0.0011702318499737,0.0454733434783982,-0.1119239362388150},
{0.1069409037912470,0.0805064400880297,-0.1127352030714600,0.1001181253523260,
-0.0021480427488769,-0.0332884841459003,-0.0679837575848452,-0.0043812841356657,
0.0153418716846395,-0.0079441315103188,-0.0121766182046363,-0.0381127991037620,
-0.0036338726532673,0.0195324059593791,-0.0020165963699984,-0.0061222685010268,
-0.0253761448771437,-0.0005246410999057,-0.0112205170502433,0.0052248485517237},
{-0.0325247648326262,0.0238753651653669,0.0203684886605797,0.0295666232678825,
-0.0003946714764213,-0.0157242718469554,-0.0511737848084862,0.0084725632040180,
-0.0167068828528921,0.0686962159427527,-0.0659702890616198,-0.0014289912494271,
-0.0167000964093416,-0.1276689083678200,0.0036575057830967,-0.0205958145531018,
0.0000368919612829,0.0014413626622426,0.1064360941926030,0.0863372661517408},
{-0.0463777468104402,0.0394712148670596,0.1118686750747160,0.0440711686389031,
-0.0026076286506751,-0.0268454015202516,-0.1464943067133240,-0.0137514051835380,
-0.0094395514284145,-0.0144124844774228,0.0249103379323744,-0.0071832157138676,
0.0035592787728526,0.0415627419826693,0.0027040097365669,0.0337523666612066,
0.0316121324137152,-0.0011350177559026,-0.0349998884574440,-0.0302651879823361},
{0.0142360925194728,0.0413145623127025,0.0324976427846929,0.0580930922002398,
-0.0586974207121084,0.0202001168873069,0.0492204086749069,0.1126593173463060,
0.0116620013776662,-0.0780333711712066,-0.1109786767320410,0.0407775100936731,
-0.0205013161312652,-0.0653458585025237,0.0347351829703865,0.0304448983224773,
0.0068813748197884,-0.0189002309261882,-0.0334507528405279,-0.0668143558699485},
{-0.0131548829657936,0.0044244322828034,-0.0050639951827271,-0.0038668197633889,
-0.1536822386530220,0.0026336969165336,0.0021585651200470,-0.0459233839062969,
0.0046854727140565,0.0393815434593599,0.0619554007991097,0.0027456299925622,
0.0117574347936383,0.0373018612990383,0.0024818527553328,-0.0133956606027299,
-0.0020457128424105,0.0154178819990401,0.0246524142683911,0.0275363065682921},
{-0.1542307272455030,0.0364861558267547,-0.0090880407008181,0.0531673937889863,
0.0157585615170580,0.0029986538457297,0.0180194047699875,0.0652152443589317,
0.0266842840376180,0.0388457366405908,0.0856237634510719,0.0126955778952183,
0.0099593861698250,-0.0013941794862563,0.0294065511237513,-0.1151906949298290,
-0.0852991447389655,0.0028699120202636,-0.0332087026659522,0.0006811857297899},
{0.0281300736924501,-0.0584072081898638,-0.0178386569847853,-0.0536470338171487,
-0.0186881656029960,-0.0240008730656106,-0.0541064820498883,0.2217137098936020,
-0.0260500001542033,0.0234505236798375,0.0311127151218573,-0.0494139126682672,
0.0057093465049849,0.0124937286655911,-0.0298322975915689,0.0006520211333102,
-0.0061018680727128,-0.0007081999479528,-0.0060523759094034,0.0215845995364623},
{0.0295321046399105,-0.0088296411830544,-0.0065057049917325,-0.0053478115612781,
-0.0100646496794634,-0.0015473619084872,0.0008539960632865,-0.0376381933046211,
-0.0328135588935604,0.0672161874239480,0.0667626853916552,-0.0026511651464901,
0.0140451514222062,-0.0544836996133137,0.0427485157912094,0.0097455780205802,
0.0177309072915667,-0.0828759701187452,-0.0729504795471370,0.0670731961252313},
{0.0082646581043963,-0.0319918630534466,-0.0188454445200422,-0.0374976353856606,
0.0037131290686848,-0.0132507796987883,-0.0306958830735725,-0.0044119395527308,
-0.0140786756619672,-0.0180512599925078,-0.0208243802903953,-0.0232202769398931,
-0.0063135878270273,0.0110442171178168,0.1824538048228460,-0.0006644614422758,
-0.0069909097436659,0.0255407650654681,0.0099119399501151,-0.0140911517070698},
{0.0261344441524861,-0.0714454044548650,0.0159436926233439,0.0028462736216688,
-0.0044572637889080,-0.0089474834434532,-0.0177570282144517,-0.0153693244094452,
0.1160919467206400,0.0304911481385036,0.0047047513411774,-0.0456535116423972,
0.0004491494948617,-0.0767108879444462,-0.0012688533741441,0.0192445965934123,
0.0202321954782039,0.0281039933233607,-0.0590403018490048,0.0364080426546883},
{0.0115826306265004,0.1340228176509380,-0.0236200652949049,-0.1284484655137340,
-0.0004742338006503,0.0127617346949511,-0.0428560878860394,0.0060030732454125,
0.0089182609926781,0.0085353834972860,0.0048464809638033,0.0709740071429510,
0.0029940462557054,-0.0483434904493132,-0.0071713680727884,-0.0036840391887209,
0.0031454003250096,0.0246243550241551,-0.0449551277644180,0.0111449232769393},
{0.0140356721886765,-0.0196518236826680,0.0030517022326582,0.0582672093364850,
-0.0000973895685457,0.0021704767224292,0.0341806268602705,-0.0152035987563018,
-0.0903198657739177,0.0259623214586925,0.0155832497882743,-0.0040543568451651,
0.0036477631918247,-0.0532892744763217,-0.0142569373662724,0.0104500681408622,
0.0103483945857315,0.0679534422398752,-0.0768068882938636,0.0280289727046158}}
;


static double pameigmat[] = {0.0, -0.2350753691875762, -0.2701991863800379,
         -0.2931612442853115, -0.4262492032364507, -0.5395980482561625, 
         -0.7141172690079523, -0.7392844756151318, -0.7781761342200766, 
         -0.810032066366362, -0.875299712761124, -1.048227332164386, 
         -1.109594097332267, -1.298616073142234, -1.342036228188581, 
         -1.552599145527578, -1.658762802054814, -1.74893445623765, 
         -1.933280832903272, -2.206353522613025};

static double pamprobmat[20][20] =
 {{0.087683339901135, 0.04051291829598762, 0.04087846315185977, 
   0.04771603459744777, 0.03247095396561266, 0.03784612688594957, 
   0.0504933695604875, 0.0898249006830755, 0.03285885059543713, 
   0.0357514442352119, 0.0852464099207521, 0.07910313444070642, 
   0.01488243946396588, 0.04100101908956829, 0.05158026947089499, 
   0.06975497205982451, 0.05832757042475474, 0.00931264523877807, 
   0.03171540880870517, 0.06303972920984541}, 
  {0.01943453646811026, -0.004492574160484092, 0.007694891061220776, 
   0.01278399096887701, 0.0106157418450234, 0.007542140341575122, 
   0.01326994069032819, 0.02615565199894889, 0.003123125764490066, 
   0.002204507682495444, -0.004782898215768979, 0.01204241965177619, 
   0.0007847400096924341, -0.03043626073172116, 0.01221202591902536, 
   0.01100527004684405, 0.01116495631339549, -0.0925364931988571, 
   -0.02622065387931562, 0.00843494142432107}, 
  {0.01855357100209072, 0.01493642835763868, 0.0127983090766285, 
   0.0200533250704364, -0.1681898360107787, 0.01551657969909255, 
   0.02128060163107209, 0.03100633591848964, 0.00845480845269879, 
   0.000927149370785571, 0.00937207565817036, 0.03490557769673472, 
   0.00300443019551563, -0.02590837220264415, 0.01329376859943192, 
   0.006854110889741407, 0.01102593860528263, 0.003360844186685888, 
   -0.03459712356647764, 0.003351477369404443}, 
  {0.02690642688200102, 0.02131745801890152, 0.0143626616005213, 
   0.02405101425725929, 0.05041008641436849, 0.01430925051050233, 
   0.02362114036816964, 0.04688381789373886, 0.005250115453626377, 
   -0.02040112168595516, -0.0942720776915669, 0.03773004996758644, 
   -0.00822831940782616, -0.1164872809439224, 0.02286281877257392, 
   0.02849551240669926, 0.01468856796295663, 0.02377110964207936, 
   -0.094380545436577, -0.02089068498518036}, 
  {0.00930172577225213, 0.01493463068441099, 0.020186920775608, 
   0.02892154953912524, -0.01224593358361567, 0.01404228329986624, 
   0.02671186617119041, 0.04537535161795231, 0.02229995804098249, 
   -0.04635704133961575, -0.1966910360247138, 0.02796648065439046, 
   -0.02263484732621436, 0.0440490503242072, 0.01148782948302166, 
   0.01989170531824069, 0.001306805142981245, -0.005676690969116321, 
   0.07680476281625202, -0.07967537039721849}, 
  {0.06602274245435476, -0.0966661981471856, -0.005241648783844579, 
   0.00859135188171146, -0.007762129660943368, -0.02888965572526196, 
   0.003592291525888222, 0.1668410669287673, -0.04082039290551406, 
   0.005233775047553415, -0.01758244726137135, -0.1493955762326898, 
   -0.00855819137835548, 0.004211419253492328, 0.01929306335052688, 
   0.03008056746359405, 0.0190444422412472, 0.005577189741419315, 
   0.0000874156155112068, 0.02634091459108298}, 
  {0.01933897472880726, 0.05874583569377844, -0.02293534606228405, 
   -0.07206314017962175, -0.004580681581546643, -0.0628814337610561, 
   -0.0850783812795136, 0.07988417636610614, -0.0852798990133397, 
   0.01649047166155952, -0.05416647263757423, 0.1089834536254064, 
   0.005093403979413865, 0.02520300254161142, 0.0005951431406455604, 
   0.02441251821224675, 0.02796099482240553, -0.002574933994926502, 
   -0.007172237553012804, 0.03002455129086954}, 
  {0.04041118479094272, -0.002476225672095412, -0.01494505811263243, 
   -0.03759443758599911, -0.00892246902492875, -0.003634714029239211, 
   -0.03085671837973749, -0.126176309029931, 0.005814031139083794, 
   0.01313561962646063, -0.04760487162503322, -0.0490563712725484, 
   -0.005082243450421558, -0.01213634309383557, 0.1806666927079249, 
   0.02111663336185495, 0.02963486860587087, -0.0000175020101657785, 
   0.01197155383597686, 0.0357526792184636}, 
  {-0.01184769557720525, 0.01582776076338872, -0.006570708266564639, 
   -0.01471915653734024, 0.00894343616503608, 0.00562664968033149, 
   -0.01465878888356943, 0.05365282692645818, 0.00893509735776116, 
   -0.05879312944436473, 0.0806048683392995, -0.007722897986905326, 
   -0.001819943882718859, 0.0942535573077267, 0.07483883782251654, 
   0.004354639673913651, -0.02828804845740341, -0.001318222184691827, 
   -0.07613149604246563, -0.1251675867732172}, 
  {0.00834167031558193, -0.01509357596974962, 0.007098172811092488, 
   0.03127677418040319, 0.001992448468465455, 0.00915441566808454, 
   0.03430175973499201, -0.0730648147535803, -0.001402707145575659, 
   0.04780949194330815, -0.1115035603461273, -0.01292297197609604, 
   -0.005056270550868528, 0.1112053349612027, -0.03801929822379964, 
   -0.001191241001736563, 0.01872874622910247, 0.0005314214903865993, 
   -0.0882576318311789, 0.07607183599610171}, 
  {-0.01539460099727769, 0.04988596184297883, -0.01187240760647617, 
   -0.06987843637091853, -0.002490472846497859, 0.01009857892494956, 
   -0.07473588067847209, 0.0906009925879084, 0.1243612446505172, 
   0.02152806401345371, -0.03504879644860233, -0.06680752427613573, 
   -0.005574485153629651, 0.001518282948127752, -0.01999168507510701, 
   -0.01478606199529457, -0.02203749419458996, -0.00132680708294333, 
   -0.01137505997867614, 0.05332658773667142}, 
  {-0.06104378736432388, 0.0869446603393548, -0.03298331234537257, 
   0.03128515657456024, 0.003906358569208259, 0.03578694104193928, 
   0.06241936133189683, 0.06182827284921748, -0.05566564263245907, 
   0.02640868588189002, -0.01349751243059039, -0.05507866642582638, 
   -0.006671347738489326, -0.001470096466016046, 0.05185743641479938, 
   -0.07494697511168257, -0.1175185439057584, -0.001188074094105709, 
   0.00937934805737347, 0.05024773745437657}, 
  {-0.07252555582124737, -0.116554459356382, 0.003605361887406413, 
   -0.00836518656029184, 0.004615715410745561, 0.005105376617651312, 
   -0.00944938657024391, 0.05602449420950007, 0.02722719610561933, 
   0.01959357494748446, -0.0258655103753962, 0.1440733975689835, 
   0.01446782819722976, 0.003718896062070054, 0.05825843045655135, 
   -0.06230154142733073, -0.07833704962300169, 0.003160836143568724, 
   -0.001169873777936648, 0.03471745590503304}, 
  {-0.03204352258752698, 0.01019272923862322, 0.04509668708733181, 
   0.05756522429120813, -0.0004601149081726732, -0.0984718150777423, 
   -0.01107826100664925, -0.005680277810520585, 0.01962359392320817, 
   0.01550006899131986, 0.05143956925922197, 0.02462476682588468, 
   -0.0888843861002653, -0.00171553583659411, 0.01606331750661664, 
   0.001176847743518958, -0.02070972978912828, -0.000341523293579971, 
   -0.002654732745607882, 0.02075709428885848}, 
  {0.03595199666430258, -0.02800219615234468, -0.04341570015493925, 
   -0.0748275906176658, 0.0001051403676377422, 0.1137431321746627, 
   0.005852087565974318, 0.003443037513847801, -0.02481931657706633, 
   -0.003651181839831423, 0.03195794176786321, 0.04135411406392523, 
   -0.07562030263210619, 0.001769332364699, -0.01984381173403915, 
   -0.005029750745010152, 0.02649253902476472, 0.000518085571702734, 
   0.001062936684474851, 0.01295950668914449}, 
  {-0.16164552322896, -0.0006050035060464324, 0.0258380054414968, 
   0.003188424740960557, -0.0002058911341821877, 0.03157555987384681, 
   -0.01678913462596107, 0.03096216145389774, -0.0133791110666919, 
   0.1125249625204277, -0.00769017706442472, -0.02653938062180483, 
   -0.002555329863523985, -0.00861833362947954, 0.01775148884754278, 
   0.02529310679774722, 0.0826243417011238, -0.0001036728183032624, 
   0.001963562313294209, -0.0935900561309786}, 
  {0.1652394174588469, -0.002814245280784351, -0.0328982001821263, 
   -0.02000104712964131, 0.0002208121995725443, -0.02733462178511839, 
   0.02648078162927627, -0.01788316626401427, 0.01630747623755998, 
   0.1053849023838147, -0.005447706553811218, 0.01810876922536839, 
   -0.001808914710282444, -0.007687912115607397, -0.01332593672114388, 
   -0.02110750894891371, -0.07456116592983384, 0.000219072589592394, 
   0.001270886972191055, -0.1083616930749109}, 
  {0.02453279389716254, -0.005820072356487439, 0.100260287284095, 
   0.01277522280305745, -0.003184943445296999, 0.05814689527984152, 
   -0.0934012278200201, -0.03017986487349484, -0.03136625380994165, 
   0.00988668352785117, -0.00358900410973142, -0.02017443675004764, 
   0.000915384582922184, -0.001460963415183106, -0.01370112443251124, 
   0.1130040979284457, -0.1196161771323699, -0.0005800211204222045, 
   -0.0006153403201024954, 0.00416806428223025}, 
  {-0.0778089244252535, -0.007055161182430869, -0.0349307504860869, 
   -0.0811915584276571, -0.004689825871599125, -0.03726108871471753, 
   0.1072225647141469, -0.00917015113070944, 0.01381628985996913, 
   -0.00123227881492089, 0.001815954515275675, 0.005708744099349901, 
   -0.0001448985044877925, -0.001306578795561384, -0.006992743514185243, 
   0.1744720240732789, -0.05353628497814023, -0.0007613684227234787, 
   -0.0003550282315997644, 0.01340106423804634}, 
  {-0.0159527329868513, -0.007622151568160798, -0.1389875105184963, 
   0.1165051999914764, -0.002217810389087748, 0.01550003226513692, 
   -0.07427664222230566, -0.003371438498619264, 0.01385754771325365, 
   0.004759020167383304, 0.001624078805220564, 0.02011638303109029, 
   -0.001717827082842178, -0.0007424036708598594, -0.003978884451898934, 
   0.0866418927301209, -0.01280817739158123, -0.00023039242454603, 
   0.002309205802479111, 0.0005926106991001195}};





int NWC;
int myId, numOfWorkers;
infoList iList;
xarray *usedxtip, *freextip;
FILE   *INFILE;
double globTime = 0.0;
int treeStringLength;
extern char *optarg;                  
extern int  optind,opterr,optopt;      
int Multiple = 1;
int Thorough = 0;
int branchCounter;
char *seq_data_str = NULL, run_id[40] = "", workdir[2048] = "", 
  seq_file[2048] = "", tree_file[2048]="", weightFileName[2048] = "";
int equalities = 0;
    bestlist *bt;
char *likelihood_key   = "likelihood";
char *ntaxa_key        = "ntaxa";
char *opt_level_key    = "opt_level";
char *smoothed_key     = "smoothed";
int map[EQUALITIES];
int* map2;
FILE *permutationFile, *logFile, *infoFile;
char permFileName[1024], resultFileName[1024], 
  logFileName[1024], checkpointFileName[1024], infoFileName[1024]; 
int buildConsensus = 1; 
int consensusTreeCounter = 0;
double masterTime;
int checkPointCounter = 0;
analdef      *adef;
char *treeString (char *treestr, tree *tr, nodeptr p, int form);
void optimizeRates(tree *tr);
void optimizeTT(tree *tr);
void optimizeRateCategories(tree *tr, int categorized, int _maxCategories);
int optimizeModel(tree *tr, analdef *adef, int finalOptimization);

double str_readTreeLikelihood (char *treestr);
void treeEvaluation(tree *tr, analdef *adef);
void checkTime(analdef *adef, tree *tr);
boolean initrav (tree *tr, nodeptr p);
void hookup (nodeptr p, nodeptr q, double z);


boolean (*newview) (tree *, nodeptr);
double (*evaluate)(tree *, nodeptr);   
double (*makenewz) (tree *, nodeptr, nodeptr, double, int);
double (*evaluatePartial)(tree *, nodeptr, int, double);
boolean (*newviewPartial)(tree *, nodeptr, int, double);
boolean (*initravPartial) (tree *, nodeptr, int, double);


boolean newviewGTR (tree *, nodeptr);
double evaluateGTR (tree *, nodeptr);   
double makenewzGTR (tree *, nodeptr, nodeptr, double, int);

boolean newviewGTRCAT (tree *, nodeptr);
double evaluateGTRCAT (tree *, nodeptr);   
double makenewzGTRCAT (tree *, nodeptr, nodeptr, double, int);

boolean newviewHKY85 (tree *, nodeptr);
double evaluateHKY85 (tree *, nodeptr);   
double makenewzHKY85 (tree *, nodeptr, nodeptr, double, int);

boolean newviewHKY85CAT (tree *, nodeptr);
double evaluateHKY85CAT (tree *, nodeptr);   
double makenewzHKY85CAT (tree *, nodeptr, nodeptr, double, int);

boolean newviewHKY85GAMMA (tree *, nodeptr);
double evaluateHKY85GAMMA (tree *, nodeptr);   
double makenewzHKY85GAMMA (tree *, nodeptr, nodeptr, double, int);

boolean newviewHKY85GAMMA_2 (tree *, nodeptr);
double evaluateHKY85GAMMA_2 (tree *, nodeptr);   
double makenewzHKY85GAMMA_2 (tree *, nodeptr, nodeptr, double, int);



boolean newviewPROT (tree *, nodeptr);
double evaluatePROT (tree *, nodeptr);   
double makenewzPROT (tree *, nodeptr, nodeptr, double, int);

boolean newviewPROTCAT (tree *, nodeptr);
double evaluatePROTCAT (tree *, nodeptr);   
double makenewzPROTCAT (tree *, nodeptr, nodeptr, double, int);

void makeGammaCats(tree *tr);

boolean newviewGTRGAMMA (tree *, nodeptr);
double evaluateGTRGAMMA (tree *, nodeptr);   
double makenewzGTRGAMMA (tree *, nodeptr, nodeptr, double, int);

int optimizeModel (tree *, analdef *, int );

void compute(tree *, analdef *);
