/*  RAxML-V, a program for sequential and parallel estimation of phylogenetic trees 
 *  from nucleotide sequences implementing SEVs (Subtree Equality Vectors), the lazy subtree
 *  rearrangement heuristics, a simulated annealing search algorithm, and a parallel OpenMP-version
 *  for SMPs. Copyright October 2005 by Alexandros Stamatakis
 *
 *  Partially derived from
 *  fastDNAml, a program for estimation of phylogenetic trees from sequences by Gary J. Olsen
 *  
 *  and 
 *
 *  Programs of the PHYLIP package by Joe Felsenstein.
 *
 *  This program is free software; you may redistribute it and/or modify its
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 * 
 *
 *  For any other enquiries send an Email to Alexandros Stamatakis
 *  stamatak@ics.forth.gr
 *
 *  When publishing work that is based on the results from RAxML-V please cite:
 *  
 *  Alexandros Stamatakis: "An Efficient Program for phylogenetic Inference Using Simulated Annealing". 
 *  Proceedings of IPDPS2005,  Denver, Colorado, April 2005.
 *  
 *  AND
 *
 *  Alexandros Stamatakis, Thomas Ludwig, and Harald Meier: "RAxML-III: A Fast Program for Maximum Likelihood-based Inference of Large Phylogenetic Trees". 
 *  In Bioinformatics 21(4):456-463.
 *
 *
 */

#ifdef WIN32
   #include <sys/timeb.h>
   #include "getopt.h"
   #include <direct.h>
#else
   #include <sys/times.h>
   #include <sys/types.h>
   #include <sys/time.h>
   #include <unistd.h> 
#endif

#include <math.h>
#include <time.h> 
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>

#ifdef PARALLEL
#include <mpi.h>
#endif

#include "axml.h"

/*=======================================================================*/
/*                              PROGRAM                                  */
/*=======================================================================*/


double gettime()
{
#ifdef WIN32
   struct _timeb timebuffer;
   _ftime(&timebuffer);
   return timebuffer.time + timebuffer.millitm * 0.000001;
#else
   struct timeval ttime;
    gettimeofday(&ttime , NULL);
    return ttime.tv_sec + ttime.tv_usec * 0.000001;
#endif
}

int gettimeSrand()
{
#ifdef WIN32
   struct _timeb timebuffer;
   _ftime(&timebuffer);
   return timebuffer.time + timebuffer.millitm;
#else
   struct timeval ttime;
    gettimeofday(&ttime , NULL);
    return ttime.tv_sec + ttime.tv_usec;
#endif
}


void  *tipValPtr (nodeptr p)
  { return  (void *) & p->number;
    }


int  cmpTipVal (void *v1, void *v2)
  { /* cmpTipVal */
    int  i1, i2;

    i1 = *((int *) v1);
    i2 = *((int *) v2);
    return  (i1 < i2) ? -1 : ((i1 == i2) ? 0 : 1);
  } /* cmpTipVal */


/*  These are the only routines that need to UNDERSTAND topologies */




topol  *setupTopol (int maxtips, int nsites)
  { /* setupTopol */
    topol   *tpl;

    if (! (tpl = (topol *) malloc(sizeof(topol))) || 
        ! (tpl->links = (connptr) malloc((2*maxtips-3) * sizeof(connect))))
      {
	printf("ERROR: Unable to get topology memory");
	tpl = (topol *) NULL;
      }
    else 
      {
	tpl->likelihood  = unlikely;
	tpl->start       = (node *) NULL;
	tpl->nextlink    = 0;
	tpl->ntips       = 0;
	tpl->nextnode    = 0;
	tpl->opt_level   = 0;     /* degree of branch swapping explored */
	tpl->scrNum      = 0;     /* position in sorted list of scores */
	tpl->tplNum      = 0;     /* position in sorted list of trees */
	tpl->prelabeled  = TRUE;
	tpl->smoothed    = FALSE; /* branch optimization converged? */
      }

    return  tpl;
  } /* setupTopol */


void  freeTopol (topol *tpl)
  { /* freeTopol */
    free(tpl->links);
    free(tpl);
  } /* freeTopol */


int  saveSubtree (nodeptr p, topol *tpl)
    /*  Save a subtree in a standard order so that earlier branches
     *  from a node contain lower value tips than do second branches from
     *  the node.  This code works with arbitrary furcations in the tree.
     */
  { /* saveSubtree */
    connptr  r, r0;
    nodeptr  q, s;
    int      t, t0, t1;

    r0 = tpl->links;
    r = r0 + (tpl->nextlink)++;
    r->p = p;
    r->q = q = p->back;
    r->z = p->z;
    r->descend = 0;                     /* No children (yet) */

    if (q->tip) {
      r->valptr = tipValPtr(q);         /* Assign value */
      }

    else {                              /* Internal node, look at children */
      s = q->next;                      /* First child */
      do {
        t = saveSubtree(s, tpl);        /* Generate child's subtree */

        t0 = 0;                         /* Merge child into list */
        t1 = r->descend;
        while (t1 && (cmpTipVal(r0[t1].valptr, r0[t].valptr) < 0)) {
          t0 = t1;
          t1 = r0[t1].sibling;
          }
        if (t0) r0[t0].sibling = t;  else  r->descend = t;
        r0[t].sibling = t1;

        s = s->next;                    /* Next child */
        } while (s != q);

      r->valptr = r0[r->descend].valptr;   /* Inherit first child's value */
      }                                 /* End of internal node processing */

    return  r - r0;
  } /* saveSubtree */


nodeptr  minSubtreeTip (nodeptr  p0)
  { /* minTreeTip */
    nodeptr  minTip, p, testTip;

    if (p0->tip) return p0;

    p = p0->next;
    minTip = minSubtreeTip(p->back);
    while ((p = p->next) != p0) {
      testTip = minSubtreeTip(p->back);
      if (cmpTipVal(tipValPtr(testTip), tipValPtr(minTip)) < 0)
        minTip = testTip;
      }
    return minTip;
  } /* minTreeTip */


nodeptr  minTreeTip (nodeptr  p)
  { /* minTreeTip */
    nodeptr  minp, minpb;

    minp  = minSubtreeTip(p);
    minpb = minSubtreeTip(p->back);
    return cmpTipVal(tipValPtr(minp), tipValPtr(minpb)) < 0 ? minp : minpb;
  } /* minTreeTip */


void saveTree (tree *tr, topol *tpl)
    /*  Save a tree topology in a standard order so that first branches
     *  from a node contain lower value tips than do second branches from
     *  the node.  The root tip should have the lowest value of all.
     */
  { /* saveTree */
    connptr  r;
    int  i;

    tpl->nextlink = 0;                             /* Reset link pointer */
    r = tpl->links + saveSubtree(minTreeTip(tr->start), tpl);  /* Save tree */
    r->sibling = 0;

    tpl->number     = tr->nodeInfo;
    tpl->likelihood = tr->likelihood;
    tpl->start      = tr->start;
    tpl->ntips      = tr->ntips;
    tpl->nextnode   = tr->nextnode;
    tpl->opt_level  = tr->opt_level;
    tpl->prelabeled = tr->prelabeled;
    tpl->smoothed   = tr->smoothed;

   
  } /* saveTree */


void copyTopol (topol *tpl1, topol *tpl2)
  { /* copyTopol */
    connptr  r1, r2, r10, r20;
   
    int  i;

    r10 = tpl1->links;
    r20 = tpl2->links;
    tpl2->nextlink = tpl1->nextlink; 

    r1 = r10;
    r2 = r20;
    i = 2 * tpl1->ntips - 3;
    while (--i >= 0) {
      r2->z = r1->z;
      r2->p = r1->p;
      r2->q = r1->q;
      r2->valptr = r1->valptr;
      r2->descend = r1->descend; 
      r2->sibling = r1->sibling; 
      r1++;
      r2++;
      }

    
    tpl2->likelihood = tpl1->likelihood;
    tpl2->start      = tpl1->start;
    tpl2->ntips      = tpl1->ntips;
    tpl2->nextnode   = tpl1->nextnode;
    tpl2->opt_level  = tpl1->opt_level;
    tpl2->prelabeled = tpl1->prelabeled;
    tpl2->scrNum     = tpl1->scrNum;
    tpl2->tplNum     = tpl1->tplNum;
    tpl2->smoothed   = tpl1->smoothed;
  } /* copyTopol */


boolean restoreTreeRecursive (topol *tpl, tree *tr)
  { /* restoreTree */
    void  hookup();
    boolean  initrav();

    connptr  r;
    nodeptr  p, p0;
    
    int  i;



/*  Copy connections from topology */

    for (r = tpl->links, i = 0; i < tpl->nextlink; r++, i++) {
      hookup(r->p, r->q, r->z);
      }

    tr->likelihood = tpl->likelihood;
    tr->start      = tpl->start;
    tr->ntips      = tpl->ntips;
    
    tr->nextnode   = tpl->nextnode;
    tr->opt_level  = tpl->opt_level;
    tr->prelabeled = tpl->prelabeled;
    tr->smoothed   = tpl->smoothed;

   
    return TRUE;
    
  } /* restoreTree */




boolean restoreTree (topol *tpl, tree *tr)
  { /* restoreTree */
    /*void  hookup();
      boolean  initrav();*/

    connptr  r;
    nodeptr  p, p0;
    
    int  i;

/*  Clear existing connections */

    for (i = 1; i <= 2*(tr->mxtips) - 2; i++) {  /* Uses p = p->next at tip */
      p0 = p = tr->nodep[i];
      do {
        p->back = (nodeptr) NULL;
        p = p->next;
        } while (p != p0);
      }

/*  Copy connections from topology */

    for (r = tpl->links, i = 0; i < tpl->nextlink; r++, i++) {
      hookup(r->p, r->q, r->z);
      }

    tr->likelihood = tpl->likelihood;
    tr->start      = tpl->start;
    tr->ntips      = tpl->ntips;
    
    tr->nextnode   = tpl->nextnode;
    tr->opt_level  = tpl->opt_level;
    tr->prelabeled = tpl->prelabeled;
    tr->smoothed   = tpl->smoothed;

   

    return (initrav(tr, tr->start) && initrav(tr, tr->start->back));
  } /* restoreTree */


int initBestTree (bestlist *bt, int newkeep, int numsp, int sites)
  { /* initBestTree */
    int  i, nlogf;


    bt->nkeep = 0;

    if (bt->ninit <= 0) {
      if (! (bt->start = setupTopol(numsp, sites)))  return  0;
      bt->ninit = -1;
      bt->nvalid = 0;
      bt->numtrees = 0;
      bt->best = unlikely;
      bt->improved = FALSE;
      bt->byScore = (topol **) malloc((newkeep+1) * sizeof(topol *));
      bt->byTopol = (topol **) malloc((newkeep+1) * sizeof(topol *));
      if (! bt->byScore || ! bt->byTopol) {
        printf( "initBestTree: malloc failure\n");
        return 0;
        }
      }
    else if (ABS(newkeep) > bt->ninit) {
      if (newkeep <  0) newkeep = -(bt->ninit);
      else newkeep = bt->ninit;
      }

    if (newkeep < 1) {    /*  Use negative newkeep to clear list  */
      newkeep = -newkeep;
      if (newkeep < 1) newkeep = 1;
      bt->nvalid = 0;
      bt->best = unlikely;
      }

    if (bt->nvalid >= newkeep) {
      bt->nvalid = newkeep;
      bt->worst = bt->byScore[newkeep]->likelihood;
      }
    else {
      bt->worst = unlikely;
      }

    for (i = bt->ninit + 1; i <= newkeep; i++) {
      nlogf = (i <= maxlogf) ? sites : 0;
      if (! (bt->byScore[i] = setupTopol(numsp, nlogf)))  break;
      bt->byTopol[i] = bt->byScore[i];
      bt->ninit = i;
      }

    return  (bt->nkeep = MIN(newkeep, bt->ninit));
  } /* initBestTree */



int resetBestTree (bestlist *bt)
  { /* resetBestTree */
    bt->best     = unlikely;
    bt->worst    = unlikely;
    bt->nvalid   = 0;
    bt->improved = FALSE;
  } /* resetBestTree */


boolean  freeBestTree(bestlist *bt)
  { /* freeBestTree */
    while (bt->ninit >= 0)  freeTopol(bt->byScore[(bt->ninit)--]);
    freeTopol(bt->start);
    return TRUE;
  } /* freeBestTree */


/*  Compare two trees, assuming that each is in standard order.  Return
 *  -1 if first preceeds second, 0 if they are identical, or +1 if first
 *  follows second in standard order.  Lower number tips preceed higher
 *  number tips.  A tip preceeds a corresponding internal node.  Internal
 *  nodes are ranked by their lowest number tip.
 */

int  cmpSubtopol (connptr p10, connptr p1, connptr p20, connptr p2)
  { /* cmpSubtopol */
    connptr  p1d, p2d;
    int  cmp;

    if (! p1->descend && ! p2->descend)          /* Two tips */
      return cmpTipVal(p1->valptr, p2->valptr);

    if (! p1->descend) return -1;                /* p1 = tip, p2 = node */
    if (! p2->descend) return  1;                /* p2 = tip, p1 = node */

    p1d = p10 + p1->descend;
    p2d = p20 + p2->descend;
    while (1) {                                  /* Two nodes */
      if (cmp = cmpSubtopol(p10, p1d, p20, p2d))  return cmp; /* Subtrees */
      if (! p1d->sibling && ! p2d->sibling)  return  0; /* Lists done */
      if (! p1d->sibling) return -1;             /* One done, other not */
      if (! p2d->sibling) return  1;             /* One done, other not */
      p1d = p10 + p1d->sibling;                  /* Neither done */
      p2d = p20 + p2d->sibling;
      }
  } /* cmpSubtopol */



int  cmpTopol (void *tpl1, void *tpl2)
  { /* cmpTopol */
    connptr  r1, r2;
    int      cmp;

    r1 = ((topol *) tpl1)->links;
    r2 = ((topol *) tpl2)->links;
    cmp = cmpTipVal(tipValPtr(r1->p), tipValPtr(r2->p));
    if (cmp) return cmp;
    return  cmpSubtopol(r1, r1, r2, r2);
  } /* cmpTopol */



int  cmpTplScore (void *tpl1, void *tpl2)
  { /* cmpTplScore */
    double  l1, l2;

    l1 = ((topol *) tpl1)->likelihood;
    l2 = ((topol *) tpl2)->likelihood;
    return  (l1 > l2) ? -1 : ((l1 == l2) ? 0 : 1);
  } /* cmpTplScore */



/*  Find an item in a sorted list of n items.  If the item is in the list,
 *  return its index.  If it is not in the list, return the negative of the
 *  position into which it should be inserted.
 */

int  findInList (void *item, void *list[], int n, int (* cmpFunc)())
  { /* findInList */
    int  mid, hi, lo, cmp;

    if (n < 1) return  -1;                    /*  No match; first index  */

    lo = 1;
    mid = 0;
    hi = n;
    while (lo < hi) {
      mid = (lo + hi) >> 1;
      cmp = (* cmpFunc)(item, list[mid-1]);
      if (cmp) {
        if (cmp < 0) hi = mid;
        else lo = mid + 1;
        }
      else  return  mid;                        /*  Exact match  */
      }

    if (lo != mid) {
       cmp = (* cmpFunc)(item, list[lo-1]);
       if (cmp == 0) return lo;
       }
    if (cmp > 0) lo++;                         /*  Result of step = 0 test  */
    return  -lo;
  } /* findInList */



int  findTreeInList (bestlist *bt, tree *tr)
  { /* findTreeInList */
    topol  *tpl;

    tpl = bt->byScore[0];
    saveTree(tr, tpl);
    return  findInList((void *) tpl, (void **) (& (bt->byTopol[1])),
                       bt->nvalid, cmpTopol);
  } /* findTreeInList */


int  saveBestTree (bestlist *bt, tree *tr)
  { /* saveBestTree */
   
    topol  *tpl, *reuse;
    int  tplNum, scrNum, reuseScrNum, reuseTplNum, i, oldValid, newValid;

    tplNum = findTreeInList(bt, tr);
    tpl = bt->byScore[0];
    oldValid = newValid = bt->nvalid;

    if (tplNum > 0) {                      /* Topology is in list  */
      reuse = bt->byTopol[tplNum];         /* Matching topol  */
      reuseScrNum = reuse->scrNum;
      reuseTplNum = reuse->tplNum;
      }
                                           /* Good enough to keep? */
    else if (tr->likelihood < bt->worst)  return 0;

    else {                                 /* Topology is not in list */
      tplNum = -tplNum;                    /* Add to list (not replace) */
      if (newValid < bt->nkeep) bt->nvalid = ++newValid;
      reuseScrNum = newValid;              /* Take worst tree */
      reuse = bt->byScore[reuseScrNum];
      reuseTplNum = (newValid > oldValid) ? newValid : reuse->tplNum;
      if (tr->likelihood > bt->start->likelihood) bt->improved = TRUE;
      }

    scrNum = findInList((void *) tpl, (void **) (& (bt->byScore[1])),
                         oldValid, cmpTplScore);
    scrNum = ABS(scrNum);

    if (scrNum < reuseScrNum)
      for (i = reuseScrNum; i > scrNum; i--)
        (bt->byScore[i] = bt->byScore[i-1])->scrNum = i;

    else if (scrNum > reuseScrNum) {
      scrNum--;
      for (i = reuseScrNum; i < scrNum; i++)
        (bt->byScore[i] = bt->byScore[i+1])->scrNum = i;
      }

    if (tplNum < reuseTplNum)
      for (i = reuseTplNum; i > tplNum; i--)
        (bt->byTopol[i] = bt->byTopol[i-1])->tplNum = i;

    else if (tplNum > reuseTplNum) {
      tplNum--;
      for (i = reuseTplNum; i < tplNum; i++)
        (bt->byTopol[i] = bt->byTopol[i+1])->tplNum = i;
      }

   

    tpl->scrNum = scrNum;
    tpl->tplNum = tplNum;
    bt->byTopol[tplNum] = bt->byScore[scrNum] = tpl;
    bt->byScore[0] = reuse;

    if (scrNum == 1)  bt->best = tr->likelihood;
    if (newValid == bt->nkeep) bt->worst = bt->byScore[newValid]->likelihood;

    return  scrNum;
  } /* saveBestTree */




int  recallBestTreeRecursive (bestlist *bt, int rank, tree *tr)
  { /* recallBestTree */
    if (rank < 1)  rank = 1;
    if (rank > bt->nvalid)  rank = bt->nvalid;
    if (rank > 0)  if (! restoreTreeRecursive(bt->byScore[rank], tr)) return FALSE;
    return  rank;
  } /* recallBestTree */



int  recallBestTree (bestlist *bt, int rank, tree *tr)
  { /* recallBestTree */
    if (rank < 1)  rank = 1;
    if (rank > bt->nvalid)  rank = bt->nvalid;
    if (rank > 0)  if (! restoreTree(bt->byScore[rank], tr)) return FALSE;
    return  rank;
  } /* recallBestTree */

/*=======================================================================*/
/*                       End of best tree routines                       */
/*=======================================================================*/


void getnums (rawdata *rdta)
    /* input number of species, number of sites */
  { /* getnums */
    

    if (fscanf(INFILE, "%d %d", & rdta->numsp, & rdta->sites) != 2) 
      {
	printf("ERROR: Problem reading number of species and sites\n");
	exit(-1);
      }
   

    if (rdta->numsp < 4) 
      {
	printf("TOO FEW SPECIES\n");
	exit(-1);
      }

    if (rdta->sites < 1) 
      {
	printf("TOO FEW SITES\n");
	exit(-1);
      }

    return;
  } /* getnums */


boolean digitchar (int ch) 
{
  return (ch >= '0' && ch <= '9'); 
}


boolean whitechar (int ch)
{ 
  return (ch == ' ' || ch == '\n' || ch == '\t');
}


void uppercase (int *chptr)
    /* convert character to upper case -- either ASCII or EBCDIC */
  { /* uppercase */
    int  ch;

    ch = *chptr;
    if ((ch >= 'a' && ch <= 'i') || (ch >= 'j' && ch <= 'r')
                                 || (ch >= 's' && ch <= 'z'))
      *chptr = ch + 'A' - 'a';
  } /* uppercase */


int base36 (int ch)
  { /* base36 */
    if      (ch >= '0' && ch <= '9') return (ch - '0');
    else if (ch >= 'A' && ch <= 'I') return (ch - 'A' + 10);
    else if (ch >= 'J' && ch <= 'R') return (ch - 'J' + 19);
    else if (ch >= 'S' && ch <= 'Z') return (ch - 'S' + 28);
    else if (ch >= 'a' && ch <= 'i') return (ch - 'a' + 10);
    else if (ch >= 'j' && ch <= 'r') return (ch - 'j' + 19);
    else if (ch >= 's' && ch <= 'z') return (ch - 's' + 28);
    else return -1;
  } /* base36 */


int itobase36 (int i)
  { /* itobase36 */
    if      (i <  0) return '?';
    else if (i < 10) return (i      + '0');
    else if (i < 19) return (i - 10 + 'A');
    else if (i < 28) return (i - 19 + 'J');
    else if (i < 36) return (i - 28 + 'S');
    else return '?';
  } /* itobase36 */


int findch (int c)
  { /* findch */
    int ch;

    while ((ch = getc(INFILE)) != EOF && ch != c) ;
    return  ch;
  } /* findch */

void getyspace (rawdata *rdta)
  { /* getyspace */
    long   size;
    int    i;
    yType *y0;

    if (! (rdta->y = (yType **) malloc((rdta->numsp + 1) * sizeof(yType *)))) 
      {
	printf("ERROR: Unable to obtain space for data array pointers\n");
	exit(-1);
      }

    size = 4 * (rdta->sites / 4 + 1);
    if (! (y0 = (yType *) malloc((rdta->numsp + 1) * size * sizeof(yType)))) 
      {
	printf("ERROR: Unable to obtain space for data array\n");
	exit(-1);
      }

    rdta->y0 = y0;

    for (i = 0; i <= rdta->numsp; i++) 
      {
	rdta->y[i] = y0;
	y0 += size;
      }

    return;
  } /* getyspace */


void freeyspace(rawdata *rdta)
{
  long   size;
  int    i;
  yType *y0;
  for (i = 0; i <= rdta->numsp; i++) 
    {
      free(rdta->y[i]);    
    }
}



boolean setupTree (tree *tr, int nsites, analdef *adef)
  { /* setupTree */
    nodeptr  p0, p, q;
    int      i, j, tips, inter;

    tips  = tr->mxtips;
    inter = tr->mxtips - 1;

    tr->rDesc = (rearrangementDescriptor *)malloc(sizeof(rearrangementDescriptor));

 


    tr->nameList = (char **)malloc(sizeof(char *) * (tips + 1));
    /* NAME-REMOVE*/
    tr->outgr     =           1;
          
    /*tr->rearrOrder = (nodeptr *)malloc(tips * sizeof(nodeptr));*/

    if (!(p0 = (nodeptr) malloc((tips + 3*inter) * sizeof(node)))) {
      printf("ERROR: Unable to obtain sufficient tree memory\n");
      return  FALSE;
      }


    if (!(tr->nodep = (nodeptr *) malloc((2*tr->mxtips) * sizeof(nodeptr)))) {
      printf("ERROR: Unable to obtain sufficient tree memory, too\n");
      return  FALSE;
      }

    
    tr->nodep[0] = (node *) NULL;    /* Use as 1-based array */

    for (i = 1; i <= tips; i++) {    /* Set-up tips */
      p = p0++;
      p->x      = (xarray *) NULL;
      p->tip    = (yType *) NULL;
      
#ifndef HPC
      p->equalityVector = (int *) NULL; /*initialize eq vector with zero */
#endif


      p->number = i;
      p->next   = p;
      p->back   = (node *) NULL;      
      tr->nodep[i] = p;
      }

    for (i = tips + 1; i <= tips + inter; i++) { /* Internal nodes */
      q = (node *) NULL;
      for (j = 1; j <= 3; j++) {
        p = p0++;
        p->x      = (xarray *) NULL;
        p->tip    = (yType *) NULL;
        p->number = i;

#ifndef HPC	
	p->equalityVector = (int *) NULL; /*initialize eq vector with zero */	
#endif
	/* AxML modification end*/

        p->next   = q;
        p->back   = (node *) NULL;
        q = p;
        }
      p->next->next->next = p;
      tr->nodep[i] = p;
      }

    tr->likelihood  = unlikely;
    tr->start       = (node *) NULL;
    tr->outgrnode   = tr->nodep[tr->outgr];
    tr->ntips       = 0;
    tr->nextnode    = 0;
    tr->opt_level   = 0;
    tr->prelabeled  = TRUE;
    tr->smoothed    = FALSE;


    

    return TRUE;
  } /* setupTree */

void freeTreeNode (nodeptr p)   /* Free tree node (sector) associated data */
  { /* freeTreeNode */
    if (p) {
      if (p->x) {
        if (p->x->lv) free(p->x->lv);
        free(p->x);
        }
#ifndef HPC
      free(p->equalityVector);
#endif
      }
  } /* freeTree */



/* AxML modification start*/



#ifndef HPC

boolean setupEqualityVectorGTR(tree *tr) /* new function for initializing and mapping equality vectors */
{
  int siteNum = tr->cdta->endsite;
  nodeptr upper = tr->nodep[tr->mxtips * 2 - 1];
  nodeptr p = tr->nodep[1];
  int *ev, *end;
  char *tip;   
  likelivector *l;
  int j;
  homType *mlvP;
  int refREV[EQUALITIES];
  int code;
  double (*EIGV)[4] = &(tr->rdta->EIGV[0][0]);
  
  for(; p < upper; p++)
    {      
      if( ! (p->equalityVector = 
	     (ev = (int *)malloc(siteNum * sizeof(int)))))
	{
	  printf("ERROR: Failure to get node equality vector  memory\n");
	  return  FALSE; 
	}      
      
      end = & ev[siteNum];
      
      if(p->tip)
	{	 
	  int i;
	  tip = p->tip;
	  mlvP = tr->nodep[p->number]->mlv;
	  
	  for(i = 0; i < equalities; i++)
	    refREV[i] = -1;

	  for(i = 0; i < siteNum; i++)
	    {	    
	      ev[i] = tip[i] - 1; 	     
	    }	  
	}
      else
	{	 	
	  for(; ev < end; ev++)
	    {
	      *ev = 0;	     
	    }
	}
    } 
  
  
  /*freeyspace(tr->rdta);*/

  return TRUE;
}




boolean setupEqualityVectorHKY85(tree *tr) /* new function for initializing and mapping equality vectors */
{
  int siteNum = tr->cdta->endsite;
  nodeptr upper = tr->nodep[tr->mxtips * 2 - 1];
  nodeptr p = tr->nodep[1];
  int *ev, *end;
  char *tip;   
  likelivector *l;
  int i, j;
  homType *mlvP;
  int ref[EQUALITIES];
  int code;
 
 
  


  
  for(; p < upper; p++)
    {      
      if( ! (p->equalityVector = 
	     (ev = (int *)malloc(siteNum * sizeof(int)))))
	{
	  printf("ERROR: Failure to get node equality vector  memory\n");
	  return  FALSE; 
	}      
      
      


      end = & ev[siteNum];
      
      if(p->tip)
	{	 
	  int i;
	  tip = p->tip;
	  mlvP = p->mlv;
	  
	  for(i = 0; i < equalities; i++)
	    ref[i] = -1;

	  for(i = 0; i < siteNum; i++)
	    {	    
	      ev[i] = map[tip[i]];  	  	     	      	     	      
	      
	      if(ref[ev[i]] == -1)
		ref[ev[i]] = 1;
	    }
	  for(i = 0; i < equalities; i++)
	    {	      
	      code = map2[i];
	      if(ref[i] == 1)
		{
		  mlvP->a =  code       & 1;
		  mlvP->c = (code >> 1) & 1; 	
		  mlvP->g = (code >> 2) & 1;
		  mlvP->t = (code >> 3) & 1;
		  mlvP->exp = 0;
		  mlvP->set = TRUE;				  		
		}
	      else
		mlvP->set = FALSE;

	      mlvP++;	 
	    }
	}
      else
	{	 	
	  for(; ev < end; ev++)
	    {
	      *ev = 0;	     
	    }
	}
    } 
  
  
  /*freeyspace(tr->rdta);*/

  return TRUE;
}


#endif





void freeTree (tree *tr)
  { /* freeTree */
    nodeptr  p, q;
    int  i, tips, inter;

    tips  = tr->mxtips;
    inter = tr->mxtips - 1;

    for (i = 1; i <= tips; i++) freeTreeNode(tr->nodep[i]);

    for (i = tips + 1; i <= tips + inter; i++) {
      if (p = tr->nodep[i]) {
        if (q = p->next) {
          freeTreeNode(q->next);
          freeTreeNode(q);
          }
        freeTreeNode(p);
        }
      }
    

    free(tr->nodep[1]);       /* Free the actual nodes */
  } /* freeTree */


boolean getdata (analdef *adef, rawdata *rdta, tree *tr)
    /* read sequences */
  { /* getdata */
    int   i, j, k, l, basesread, basesnew, ch, my_i;
    int   meaning[256];          /*  meaning of input characters */ 
    char *nameptr;
    boolean  allread, firstpass;
    char buffer[300]; 
    int len;

   


    if(adef->model == M_PROTEINS || adef->model == M_PROTEINSCAT)
      {
	for (i = 0; i <= 255; i++) 
	  meaning[i] = -1;
    
	meaning['A'] =  0;  /* alanine */
	meaning['R'] =  1;  /* arginine */
	meaning['N'] =  2;  /*  asparagine*/
	meaning['D'] =  3;  /* aspartic */
	meaning['C'] =  4;  /* cysteine */
	meaning['Q'] =  5;  /* glutamine */
	meaning['E'] =  6;  /* glutamic */
	meaning['G'] =  7;  /* glycine */
	meaning['H'] =  8;  /* histidine */
	meaning['I'] =  9;  /* isoleucine */
	meaning['L'] =  10; /* leucine */
	meaning['K'] =  11; /* lysine */
	meaning['M'] =  12; /* methionine */
	meaning['F'] =  13; /* phenylalanine */
	meaning['P'] =  14; /* proline */
	meaning['S'] =  15; /* serine */
	meaning['T'] =  16; /* threonine */
	meaning['W'] =  17; /* tryptophan */
	meaning['Y'] =  18; /* tyrosine */
	meaning['V'] =  19; /* valine */
	meaning['B'] =  20; /* asparagine, aspartic 2 and 3*/
	meaning['Z'] =  21; /* glutamine glutamic 5 and 6*/
	meaning['X'] =  meaning['?'] = meaning['*'] = meaning['-'] = 22; /* all = 1.0 */
      }
    else
      {
	for (i = 0; i <= 255; i++) meaning[i] = -1;

#ifndef HPC
	meaning['A'] =  1;
	meaning['B'] = 14;
	meaning['C'] =  2;
	meaning['D'] = 13;
	meaning['G'] =  4;
	meaning['H'] = 11;
	meaning['K'] = 12;
	meaning['M'] =  3;
	meaning['N'] = 15;
	meaning['O'] = 16;
	meaning['R'] =  5;
	meaning['S'] =  6;
	meaning['T'] =  8;
	meaning['U'] =  8;
	meaning['V'] =  7;
	meaning['W'] =  9;
	meaning['X'] = 17;
	meaning['Y'] = 10;
	meaning['?'] = 18;
	meaning['-'] = 19;	
#endif
#ifdef HPC 
	meaning['A'] =  1;
	meaning['B'] = 14;
	meaning['C'] =  2;
	meaning['D'] = 13;
	meaning['G'] =  4;
	meaning['H'] = 11;
	meaning['K'] = 12;
	meaning['M'] =  3;
	meaning['N'] = 15;
	meaning['O'] = 15;
	meaning['R'] =  5;
	meaning['S'] =  6;
	meaning['T'] =  8;
	meaning['U'] =  8;
	meaning['V'] =  7;
	meaning['W'] =  9;
	meaning['X'] = 15;
	meaning['Y'] = 10;
	meaning['?'] = 15;
	meaning['-'] = 15;	
#endif 
      }
  


    basesread = basesnew = 0;

    allread = FALSE;
    firstpass = TRUE;
    ch = ' ';

    while (! allread) 
      {
	for (i = 1; i <= tr->mxtips; i++) 
	  {   

	    if (firstpass) 
	      {                      
		/*  Read species names */
		ch = getc(INFILE);
		while(ch == ' ' || ch == '\n' || ch == '\t')
		  {
		    ch = getc(INFILE);
		  }

		my_i = 0;

		do 
		  {
		    buffer[my_i] = ch;
		    ch = getc(INFILE);		   
		    my_i++;
		    if(my_i >= nmlngth)
		      {
			printf("ERROR Names are to long, adapt constant nmlngth in\n");
			printf("axml.h AND dnapars.c, current setting %d\n", nmlngth);
			exit(-1);
		      }
		  }
		while(ch !=  ' ' && ch != '\n' && ch != '\t');
		
		buffer[my_i] = '\0';
		
		/* NAME-REMOVE*/
		len = strlen(buffer) + 1;
		
	       

		tr->nameList[i] = (char *)malloc(sizeof(char) * len);

		strcpy(tr->nameList[i], buffer);
		/*strcpy(tr->nodep[i]->name, buffer);*/
		/* NAME-REMOVE*/
	      }



	    j = basesread;
	    while ((j < rdta->sites)
		   && ((ch = getc(INFILE)) != EOF)
		   && ((! adef->interleaved) || (ch != '\n'))) {
	      uppercase(& ch);
	      if ((meaning[ch] != -1) || ch == '.') 
		{
		  j++;
		  if (ch == '.') {
		    if (i != 1) ch = rdta->y[1][j];
		    else {
		      printf("ERROR: Dot (.) found at site %d of sequence 1\n", j);
		      return  FALSE;
		    }
		  }
		  rdta->y[i][j] = ch;
		}
	      else if (whitechar(ch) || digitchar(ch)) ;
	      else {
		printf("ERROR: Bad base (%c) at site %d of sequence %d\n",
		       ch, j, i);
		return  FALSE;
	      }
	    }

	    if (ch == EOF) {
	      printf("ERROR: End-of-file at site %d of sequence %d\n", j, i);
	      return  FALSE;
	    }

	    if (! firstpass && (j == basesread)) i--;        /* no data on line */
	    else if (i == 1) basesnew = j;
	    else if (j != basesnew) {
	      printf("ERROR: Sequences out of alignment\n");
	      /*printf("%d (instead of %d) residues read in sequence %d %s\n",
		j - basesread, basesnew - basesread, i, tr->nodep[i]->name);*/

	      /* NAME-REMOVE*/
	      printf("%d (instead of %d) residues read in sequence %d %s\n",
		     j - basesread, basesnew - basesread, i, tr->nameList[i]);
	      return  FALSE;
	    }

	    while (ch != '\n' && ch != EOF) ch = getc(INFILE);  /* flush line */
	  }                                                  /* next sequence */
	firstpass = FALSE;
	basesread = basesnew;
	allread = (basesread >= rdta->sites);
      }

    /*  Print listing of sequence alignment */
    if(adef->model == M_PROTEINS || adef->model == M_PROTEINSCAT)
      {
	for (j = 1; j <= tr->mxtips; j++)
	  {
	    for (i = 1; i <= rdta->sites; i++) 
	      {
		rdta->y[j][i] = meaning[rdta->y[j][i]];		
	      }
	  }   
      }
    else
      {
	for (j = 1; j <= tr->mxtips; j++)    /* Convert characters to meanings */
	  for (i = 1; i <= rdta->sites; i++) 
	    {
	      rdta->y[j][i] = meaning[rdta->y[j][i]];
	    }
      }

    return  TRUE;
  } /* getdata */





void inputweights (analdef *adef, rawdata *rdta, cruncheddata *cdta)
    /* input the character weights 0, 1, 2 ... 9, A, B, ... Y, Z */
  { /* inputweights */
    int i, ch, w, fres;
    FILE *weightFile;
    int *wv = (int *)malloc(sizeof(int) *  rdta->sites + 1);
  

    weightFile = fopen(weightFileName, "r");
    if (!weightFile)
      {
	printf( "Could not open weight file: %s\n", weightFileName);
	exit(1);
      }
     
    i = 1;
    
    while((fres = fscanf(weightFile,"%d", &w)) != EOF)
      {
	if(!fres)
	  {
	    printf("error reading weight file probably encountered a non-integer weight value\n");
	    exit(-1);
	  }
	wv[i] = w;
	i++;	
      }
    
   
    if(i != (rdta->sites + 1))
      {
	printf("number %d of weights not equal to number %d of alignment columns\n", i, rdta->sites);
	exit(-1);
      }

    cdta->wgtsum = 0;
    for(i = 1; i <= rdta->sites; i++) 
      {     
	rdta->wgt[i] = wv[i];
	cdta->wgtsum += rdta->wgt[i];
      }

    fclose(weightFile);
    free(wv);
  } /* inputweights */







void getinput (analdef *adef, rawdata *rdta, cruncheddata *cdta, tree *tr)
  { /* getinput */
    int i,j;
#ifndef HPC
    char **names;
#endif

    getnums(rdta);
    tr->outgr     = 1;
    tr->mxtips    = rdta->numsp;
    rdta->wgt      = (int *)    malloc((rdta->sites + 1) * sizeof(int));
    rdta->wgt2     = (int *)    malloc((rdta->sites + 1) * sizeof(int));
    cdta->alias    = (int *)    malloc((rdta->sites + 1) * sizeof(int));
    cdta->aliaswgt = (int *)    malloc((rdta->sites + 1) * sizeof(int));
    cdta->patrat   = (double *) malloc((rdta->sites + 1) * sizeof(double));
    cdta->patratStored   = (double *) malloc((rdta->sites + 1) * sizeof(double));
    cdta->rateCategory = (int *) malloc((rdta->sites + 1) * sizeof(int));
    cdta->wr       = (double *) malloc((rdta->sites + 1) * sizeof(double));
    cdta->wr2      = (double *) malloc((rdta->sites + 1) * sizeof(double));
    
    if(!adef->useWeightFile)
      {
	for (i = 1; i <= rdta->sites; i++) 
	  rdta->wgt[i] = 1;
	cdta->wgtsum = rdta->sites;   
      }
    else
      {	
	inputweights(adef, rdta, cdta);
      }
    


    getyspace(rdta);
    setupTree(tr, rdta->sites, adef);
    getdata(adef, rdta, tr);
#ifndef HPC
    names = (char **)malloc((tr->mxtips + 1) * sizeof(char *));
#endif

    /* NAME-REMOVE*/
    /*for(i = 1; i <= tr->mxtips; i++)
      names[i] = tr->nodep[i]->name;*/
#ifndef HPC
    for(i = 1; i <= tr->mxtips; i++)
      names[i] = tr->nameList[i];
#endif

    /* NAME-REMOVE*/
    
#ifndef HPC
    if((!adef->restart) && (adef->model != M_PROTEINS) && (adef->model != M_PROTEINSCAT))
      RAxML_parsimony_init(tr->mxtips, rdta->sites, rdta->y, names);

    if((adef->model !=  M_PROTEINS) && (adef->model !=  M_PROTEINSCAT))
      {
	for(i = 1; i <= tr->mxtips; i++)
	  {	
	    for(j = 1; j <= rdta->sites; j++)
	      {	    	   	   
		if(rdta->y[i][j] > 15) rdta->y[i][j] = 15; 
	      }
	  }
      }
    free(names);
#endif
    return;
  } /* getinput */




double randum (long  *seed)
    /* random number generator, modified to use 12 bit chunks */
  { /* randum */
    long  sum, mult0, mult1, seed0, seed1, seed2, newseed0, newseed1, newseed2;

    mult0 = 1549;
    seed0 = *seed & 4095;
    sum  = mult0 * seed0;
    newseed0 = sum & 4095;
    sum >>= 12;
    seed1 = (*seed >> 12) & 4095;
    mult1 =  406;
    sum += mult0 * seed1 + mult1 * seed0;
    newseed1 = sum & 4095;
    sum >>= 12;
    seed2 = (*seed >> 24) & 255;
    sum += mult0 * seed2 + mult1 * seed1;
    newseed2 = sum & 255;

    *seed = newseed2 << 24 | newseed1 << 12 | newseed0;
    return  0.00390625 * (newseed2
                          + 0.000244140625 * (newseed1
                                              + 0.000244140625 * newseed0));
  } /* randum */


void makeboot (analdef *adef, rawdata *rdta, cruncheddata *cdta)
  {
    int  i, j, nonzero;
    double  randum();

    nonzero = 0;
    for (i = 1; i <= rdta->sites; i++)  
      if (rdta->wgt[i] > 0) nonzero++;

    for (j = 1; j <= nonzero; j++) 
      cdta->aliaswgt[j] = 0;
    for (j = 1; j <= nonzero; j++)
      cdta->aliaswgt[(int) (nonzero*randum(& adef->boot)) + 1]++;

    j = 0;
    cdta->wgtsum = 0;
    for (i = 1; i <= rdta->sites; i++) 
      {
	if (rdta->wgt[i] > 0)
	  cdta->wgtsum += (rdta->wgt2[i] = rdta->wgt[i] * cdta->aliaswgt[++j]);
	else
	  rdta->wgt2[i] = 0;
      }
  } 


void sitesort (rawdata *rdta, cruncheddata *cdta)
    /* Shell sort keeping sites with identical residues and weights in
     * the original order (i.e., a stable sort).
     * The index created in cdta->alias is 1 based.  The
     * sitecombcrunch routine packs it to a 0 based index.
     */
  { /* sitesort */
    int  gap, i, j, jj, jg, k, n, nsp;
    int  *index, *category;
    boolean  flip, tied;
    yType  **data;

    index    = cdta->alias;
    data     = rdta->y;
    n        = rdta->sites;
    nsp      = rdta->numsp;

    for (gap = n / 2; gap > 0; gap /= 2) {
      for (i = gap + 1; i <= n; i++) {
        j = i - gap;

        do {
          jj = index[j];
          jg = index[j+gap];
          flip = 0;
          tied = 1;
          for (k = 1; (k <= nsp) && tied; k++) {
            flip = (data[k][jj] >  data[k][jg]);
            tied = (data[k][jj] == data[k][jg]);
            }
          if (flip) {
            index[j]     = jg;
            index[j+gap] = jj;
            j -= gap;
            }
          } while (flip && (j > 0));

        }  /* for (i ...   */
      }    /* for (gap ... */
  } /* sitesort */


void sitecombcrunch (rawdata *rdta, cruncheddata *cdta)
    /* combine sites that have identical patterns (and nonzero weight) */
  { /* sitecombcrunch */
    int  i, sitei, j, sitej, k;
    int gaps = 0;
    boolean  tied;

  

    i = 0;
    cdta->alias[0] = cdta->alias[1];
    cdta->aliaswgt[0] = 0;

    for (j = 1; j <= rdta->sites; j++) {
      sitei = cdta->alias[i];
      sitej = cdta->alias[j];
      tied = 1;

      for (k = 1; tied && (k <= rdta->numsp); k++)
        tied = (rdta->y[k][sitei] == rdta->y[k][sitej]);

      if (tied) {
        cdta->aliaswgt[i] += rdta->wgt2[sitej];
        }
      else {
        if (cdta->aliaswgt[i] > 0) i++;
        cdta->aliaswgt[i] = rdta->wgt2[sitej];
        cdta->alias[i] = sitej;
        }
      }

    cdta->endsite = cdta->endsiteNormal = i;
    if (cdta->aliaswgt[i] > 0) cdta->endsite++;
  } /* sitecombcrunch */


boolean makeweights (analdef *adef, rawdata *rdta, cruncheddata *cdta)
    /* make up weights vector to avoid duplicate computations */
  { /* makeweights */
    int  i;

    if (adef->boot)  
      makeboot(adef, rdta, cdta);
    else 
      {
	for (i = 1; i <= rdta->sites; i++)  
	  rdta->wgt2[i] = rdta->wgt[i];
      }

    for (i = 1; i <= rdta->sites; i++)  
      cdta->alias[i] = i;
    sitesort(rdta, cdta);
    sitecombcrunch(rdta, cdta);
    return TRUE;
  } /* makeweights */






boolean makevalues (rawdata *rdta, cruncheddata *cdta)
    /* set up fractional likelihoods at tips */
  { /* makevalues */
    double  temp, wtemp;
    int  i, j;

    for (i = 1; i <= rdta->numsp; i++) {
      for (j = 0; j < cdta->endsite; j++) 
	{
	  rdta->y[i-1][j] = rdta->y[i][cdta->alias[j]];
        }
      }

   
    for (j = 0; j < cdta->endsite; j++) 
      {
	cdta->patrat[j] = temp = 1.0;
	cdta->patratStored[j] = 1.0;
	cdta->rateCategory[j] = 0;
	cdta->wr[j]  = wtemp = temp * cdta->aliaswgt[j];
	cdta->wr2[j] = temp * wtemp;
      }

    return TRUE;
  } /* makevalues */

char *itobase36_str(i)
     int i;
  { 
    static char buf[4];
    int j = 2;
    strcpy(buf,"  0");
    while(i && j >= 0) {
      buf[j--] = itobase36(i % 36);
      i /= 36;
    }
    return buf;
  }

boolean empiricalfreqs (analdef *adef, rawdata *rdta, cruncheddata *cdta)
    /* Get empirical base frequencies from the data */
  { /* empiricalfreqs */
    double  sum, suma, sumc, sumg, sumt, wj, fa, fc, fg, ft, sumb;
    int     i, j, k, code;
    yType  *yptr;
    
    for (j = 0; j < cdta->endsite; j++) 
      {
	for (k = 1; k <= 8; k++) 
	  {
	     rdta->freqa = 0.25;
	     rdta->freqc = 0.25;
	     rdta->freqg = 0.25;
	     rdta->freqt = 0.25;
	    suma = 0.0;
	    sumc = 0.0;
	    sumg = 0.0;
	    sumt = 0.0;
	    for (i = 0; i < rdta->numsp; i++) 
	      {
		yptr = rdta->y[i];
		
		code = yptr[j];
		fa = rdta->freqa * ( code       & 1);
		fc = rdta->freqc * ((code >> 1) & 1);
		fg = rdta->freqg * ((code >> 2) & 1);
		ft = rdta->freqt * ((code >> 3) & 1);
		wj = cdta->aliaswgt[j] / (fa + fc + fg + ft);
		suma += wj * fa;
		sumc += wj * fc;
		sumg += wj * fg;
		sumt += wj * ft;
	      }	  
	  }
	sum = suma + sumc + sumg + sumt;
	rdta->freqa = suma / sum;
	rdta->freqc = sumc / sum;
	rdta->freqg = sumg / sum;
	rdta->freqt = sumt / sum;		
      }

    rdta->freqa = 0.25;
    rdta->freqc = 0.25;
    rdta->freqg = 0.25;
    rdta->freqt = 0.25;

    for (k = 1; k <= 8; k++) {
      suma = 0.0;
      sumc = 0.0;
      sumg = 0.0;
      sumt = 0.0;
      for (i = 0; i < rdta->numsp; i++) {
        yptr = rdta->y[i];
        for (j = 0; j < cdta->endsite; j++) {
          code = *yptr++;
          fa = rdta->freqa * ( code       & 1);
          fc = rdta->freqc * ((code >> 1) & 1);
          fg = rdta->freqg * ((code >> 2) & 1);
          ft = rdta->freqt * ((code >> 3) & 1);
          wj = cdta->aliaswgt[j] / (fa + fc + fg + ft);
          suma += wj * fa;
          sumc += wj * fc;
          sumg += wj * fg;
          sumt += wj * ft;
          }
        }
      sum = suma + sumc + sumg + sumt;
      rdta->freqa = suma / sum;
      rdta->freqc = sumc / sum;
      rdta->freqg = sumg / sum;
      rdta->freqt = sumt / sum;
      }

   rdta->freqr = rdta->freqa + rdta->freqg;
    rdta->invfreqr = 1.0/rdta->freqr;
    rdta->freqar = rdta->freqa * rdta->invfreqr;
    rdta->freqgr = rdta->freqg * rdta->invfreqr;
    rdta->freqy = rdta->freqc + rdta->freqt;
    rdta->invfreqy = 1.0/rdta->freqy;
    rdta->freqcy = rdta->freqc * rdta->invfreqy;
    rdta->freqty = rdta->freqt * rdta->invfreqy;
    
    suma = rdta->ttratio*rdta->freqr*rdta->freqy
         - (rdta->freqa*rdta->freqg + rdta->freqc*rdta->freqt);
    sumb = rdta->freqa*rdta->freqgr + rdta->freqc*rdta->freqty;
    rdta->xi = suma/(suma+sumb);
    rdta->xv = 1.0 - rdta->xi;
    if (rdta->xi <= 0.0) {
      /*printf("WARNING: This transition/transversion ratio\n");
      printf("         is impossible with these base frequencies!\n");
      printf("Transition/transversion parameter reset\n\n");*/
      rdta->xi = 0.000001;
      rdta->xv = 1.0 - rdta->xi;
      rdta->ttratio = (sumb * rdta->xi / rdta->xv 
                       + rdta->freqa * rdta->freqg
                       + rdta->freqc * rdta->freqt)
                    / (rdta->freqr * rdta->freqy);
     
      }
   
    rdta->fracchange = 2.0 * rdta->xi * (rdta->freqa * rdta->freqgr
                                       + rdta->freqc * rdta->freqty)
                     + rdta->xv * (1.0 - rdta->freqa * rdta->freqa
                                       - rdta->freqc * rdta->freqc
                                       - rdta->freqg * rdta->freqg
                                       - rdta->freqt * rdta->freqt);
    
    return TRUE;
  } /* empiricalfreqs */






void alterTT(tree *tr, double t_value)
  { /* reportfreqs */
    double  suma, sumb;       
    
    rawdata *rdta = tr->rdta;
    tr->t_value = t_value;
    suma = t_value*rdta->freqr*rdta->freqy
         - (rdta->freqa*rdta->freqg + rdta->freqc*rdta->freqt);
    sumb = rdta->freqa*rdta->freqgr + rdta->freqc*rdta->freqty;
    rdta->xi = suma/(suma+sumb);
    rdta->xv = 1.0 - rdta->xi;
    if (rdta->xi <= 0.0) {
      /*      printf("WARNING: This transition/transversion ratio\n");
      printf("         is impossible with these base frequencies!\n");
      printf("Transition/transversion parameter reset\n\n");*/
      rdta->xi = 0.000001;
      rdta->xv = 1.0 - rdta->xi;
      rdta->ttratio = (sumb * rdta->xi / rdta->xv 
                       + rdta->freqa * rdta->freqg
                       + rdta->freqc * rdta->freqt)
                    / (rdta->freqr * rdta->freqy);
     
      }
   
    rdta->fracchange = 2.0 * rdta->xi * (rdta->freqa * rdta->freqgr
                                       + rdta->freqc * rdta->freqty)
                     + rdta->xv * (1.0 - rdta->freqa * rdta->freqa
                                       - rdta->freqc * rdta->freqc
                                       - rdta->freqg * rdta->freqg
                                       - rdta->freqt * rdta->freqt);
  } 




boolean linkdata2tree (rawdata *rdta, cruncheddata *cdta, tree *tr, analdef *adef)
    /* Link data array to the tree tips */
  { /* linkdata2tree */
    int  i;
    int  j;
    yType *tip;
    int ref[EQUALITIES];
    tr->rdta       = rdta;
    tr->cdta       = cdta;
    tr->NumberOfCategories = 1;

    if(adef->model == M_PROTEINS || adef->model == M_PROTEINSCAT)
      {
	for (i = 1; i <= tr->mxtips; i++) 
	  {  
	    tr->nodep[i]->tip = &(rdta->y[i-1][0]);
	    tip =  tr->nodep[i]->tip;
	  }
      }
    else
      {     
	for(i = 0; i < EQUALITIES; i++)
	  ref[i] = 0;
	equalities = 0;
	
	
	for (i = 1; i <= tr->mxtips; i++) 
	  {    
	    tr->nodep[i]->tip = &(rdta->y[i-1][0]);
	    tip =  tr->nodep[i]->tip;
	    for(j = 0; j < tr->cdta->endsite; j++)
	      {
		if(ref[tip[j]] == 0)
		  ref[tip[j]] = 1; 
	      }
	  }
	
	for(i = 0; i < EQUALITIES; i++)
	  {
	    if(ref[i] == 1) 
	      {
		map[i] = equalities++;	     	   
	      }
	    else
	      map[i] = -1;
	  }
	
	map2 = (int *)malloc(equalities * sizeof(int));
	
	j = 0;
	
	for(i = 0; i < EQUALITIES; i++)
	  {
	    if(ref[i] == 1)
	      {
		map2[j] = i;
		j++;
	      }
	  }
      }

    return TRUE;
  } /* linkdata2tree */


xarray *setupxarray (int npat, analdef *adef)
  { /* setupxarray */
    xarray        *x;
    likelivector  *data;
    proteinlikelivector *pdata;
    gammalikelivector *gdata;

    x = (xarray *) malloc(sizeof(xarray));

    if(adef->model == M_PROTEINS || adef->model == M_PROTEINSCAT)
      {
#ifndef HPC
	if (x) 
	  {     
	    pdata = (proteinlikelivector *) malloc(npat * sizeof(proteinlikelivector));
	    if (pdata) {
	      x->pv = pdata;
	      x->prev = x->next = x;
	      x->owner = (node *) NULL;	     
	    }
	    else 
	      {
		free(x);
		return (xarray *) NULL;
	      }
	  }
#endif
      }
    else
      {
	if(adef->model == M_HKY85GAMMA || adef->model == M_GTRGAMMA)
	  {
#ifndef HPC
	    if (x) 
	      {     
		gdata = (gammalikelivector *) malloc(npat * sizeof(gammalikelivector));
		if (gdata) 
		  {
		    x->gv = gdata;
		    x->prev = x->next = x;
		    x->owner = (node *) NULL;	      
		  }
		else 
		  {
		    free(x);
		    return (xarray *) NULL;
		  }	  
	      }
#endif
	  }
	else
	  {
	    if (x) 
	      {    	
		data = (likelivector *) malloc(npat * sizeof(likelivector));
		
		if (data) {
		  x->lv = data;
		  x->prev = x->next = x;
		  x->owner = (node *) NULL;
		}
		else 
		  {
		    free(x);
		    return (xarray *) NULL;
		  }
	      }
	  }
      }
    return x;
  } /* setupxarray */


boolean linkxarray (int req, int min, int npat, xarray **freexptr, xarray **usedxptr, analdef *adef)
    /*  Link a set of xarrays */
  { /* linkxarray */
    xarray  *first, *prev, *x;
    int  i;

    first = prev = (xarray *) NULL;
    i = 0;

    do {
      x = setupxarray(npat, adef);
      if (x) {
        if (! first) first = x;
        else {
          prev->next = x;
          x->prev = prev;
          }
        prev = x;
        i++;
        }
      else {
        printf("ERROR: Failure to get requested xarray memory\n");
        if (i < min)  return  FALSE;
        }
      } while ((i < req) && x);

    if (first) {
      first->prev = prev;
      prev->next = first;
      }

    *freexptr = first;
    *usedxptr = (xarray *) NULL;

    return  TRUE;
  } /* linkxarray */


boolean setupnodex (tree *tr, analdef *adef)
  { /* setupnodex */
    nodeptr  p;
    int  i;

    for (i = tr->mxtips + 1; (i <= 2*(tr->mxtips) - 2); i++) {
      p = tr->nodep[i];
      if (! (p->x = setupxarray(tr->cdta->endsite, adef))) {
        printf("ERROR: Failure to get internal node xarray memory\n");
        return  FALSE;
        }
      }

    return  TRUE;
  } /* setupnodex */


xarray *getxtip (nodeptr p)
  { /* getxtip */
    xarray  *new;
    boolean  splice;

    if (! p) return (xarray *) NULL;

    splice = FALSE;

    if (p->x) {                  /* array is there; move to tail of list */
      new = p->x;
      if (new == new->prev) ;             /* linked to self; leave it */
      else if (new == usedxtip) usedxtip = usedxtip->next; /* at head */
      else if (new == usedxtip->prev) ;   /* already at tail */
      else {                              /* move to tail of list */
        new->prev->next = new->next;
        new->next->prev = new->prev;
        splice = TRUE;
        }
      }

    else if (freextip) {                 /* take from unused list */
      p->x = new = freextip;
      new->owner = p;
      if (new->prev != new) {            /* not only member of freelist */
        new->prev->next = new->next;
        new->next->prev = new->prev;
        freextip = new->next;
        }
      else
        freextip = (xarray *) NULL;

      splice = TRUE;
      }

    else if (usedxtip) {                 /* take from head of used list */
      usedxtip->owner->x = (xarray *) NULL;
      p->x = new = usedxtip;
      new->owner = p;
      usedxtip = usedxtip->next;
      }

    else {
      printf("ERROR: Unable to locate memory for tip %d.\n", p->number);
      return  (xarray *) NULL;
      }

    if (splice) {
      if (usedxtip) {                  /* list is not empty */
        usedxtip->prev->next = new;
        new->prev = usedxtip->prev;
        usedxtip->prev = new;
        new->next = usedxtip;
        }
      else
        usedxtip = new->prev = new->next = new;
      }

    return  new;
  } /* getxtip */




xarray *getxnode (nodeptr p)
    /* Ensure that internal node p has memory */
  { /* getxnode */
    nodeptr  s;

    if (! (p->x)) {  /*  Move likelihood array on this node to sector p */
      if ((s = p->next)->x || (s = s->next)->x) {
        p->x = s->x;
        s->x = (xarray *) NULL;
        }
      else {
        printf("ERROR: Unable to locate memory at node %d.\n", p->number);
        exit(1);
        }
      }
    return  p->x;
  } /* getxnode */


 /* AxML modification start*/
 /* optimized version of function newview(), implementing homogeneous and */
 /* partially heterogeneous subtree equality vectors */


#ifdef OPEN_MP


boolean newviewHKY85GAMMA (tree *tr, nodeptr p)        /*  Update likelihoods at node */
  { /* newview */
    double   zq, lzq, xvlzq, zr, lzr, xvlzr;
    nodeptr  q, r;
    gammalikelivector *lp, *lq, *lr;
    int  i, j;

   

    if (p->tip) 
      {      
	gammalikelivector *l;
	int           code;
	yType        *yptr;
      
	if (p->x) return TRUE;  
	  
	if (! getxtip(p)) return FALSE;

	l = p->x->gv;          
	yptr = p->tip;        
#pragma omp parallel for private(code, l)
	for (i = 0; i < tr->cdta->endsite; i++) 
	  {
	    code = yptr[i];

	    l = &p->x->gv[i];  

	    l->a0 =  l->a1 = l->a2 = l->a3 = code       & 1;
	    l->c0 =  l->c1 = l->c2 = l->c3 = (code >> 1) & 1;
	    l->g0 =  l->g1 = l->g2 = l->g3 = (code >> 2) & 1;
	    l->t0 =  l->t1 = l->t2 = l->t3 = (code >> 3) & 1;
	    	   
	    l->exp = 0;	      	     	    
	  }	
    
	return TRUE;
      }

    q = p->next->back;
    r = p->next->next->back;

    while ((! p->x) || (! q->x) || (! r->x)) 
      {
	if (! q->x) if (! newviewHKY85GAMMA(tr, q))  return FALSE;
	if (! r->x) if (! newviewHKY85GAMMA(tr, r))  return FALSE;
	if (! p->x) if (! getxnode(p)) return FALSE;
      }

    lp = p->x->gv;
    lq = q->x->gv;
    lr = r->x->gv;

    zq = q->z;
    lzq = (zq > zmin) ? log(zq) : log(zmin);
    xvlzq = tr->rdta->xv * lzq;
   
    zr = r->z;
    lzr = (zr > zmin) ? log(zr) : log(zmin);
    xvlzr = tr->rdta->xv * lzr;

    {
      double  fxqr, fxqy, fxqn, sumaq, sumgq, sumcq, sumtq,
              fxrr, fxry, fxrn, ki, tempi, tempj;
    
      double zzq[4], zvq[4], zzr[4], zvr[4];
         
      for(j = 0; j < tr->gammaCATS; j++)
	{
	  ki = tr->gammaRates[j];	      
	  zzq[j] = exp(ki *   lzq);
	  zvq[j] = exp(ki * xvlzq);
	  zzr[j] = exp(ki *   lzr);
	  zvr[j] = exp(ki * xvlzr);
	}

#pragma omp parallel for private(fxqr,fxqy,fxqn,tempi,tempj,sumaq,sumgq,sumcq,sumtq,fxrr,fxry,fxrn, lq, lr, lp)           
      for (i = 0; i < tr->cdta->endsite; i++) 
	{     

	  lq = &q->x->gv[i];
	  lp = &p->x->gv[i];
	  lr = &r->x->gv[i];

	  /* CAT 0*/
    	  
	  fxqr = tr->rdta->freqa * lq->a0 + tr->rdta->freqg * lq->g0;
	  fxqy = tr->rdta->freqc * lq->c0 + tr->rdta->freqt * lq->t0;
	  fxqn = fxqr + fxqy;
	  tempi = fxqr * tr->rdta->invfreqr;
	  tempj = zvq[0] * (tempi-fxqn) + fxqn;
	  sumaq = zzq[0] * (lq->a0 - tempi) + tempj;
	  sumgq = zzq[0] * (lq->g0 - tempi) + tempj;
	  tempi = fxqy * tr->rdta->invfreqy;
	  tempj = zvq[0] * (tempi-fxqn) + fxqn;
	  sumcq = zzq[0] * (lq->c0 - tempi) + tempj;
	  sumtq = zzq[0] * (lq->t0 - tempi) + tempj;
	  
	  fxrr = tr->rdta->freqa * lr->a0 + tr->rdta->freqg * lr->g0;
	  fxry = tr->rdta->freqc * lr->c0 + tr->rdta->freqt * lr->t0;
	  fxrn = fxrr + fxry;
	  tempi = fxrr * tr->rdta->invfreqr;
	  tempj = zvr[0] * (tempi-fxrn) + fxrn;
	  lp->a0 = sumaq * (zzr[0] * (lr->a0 - tempi) + tempj);
	  lp->g0 = sumgq * (zzr[0] * (lr->g0 - tempi) + tempj);
	  tempi = fxry * tr->rdta->invfreqy;
	  tempj = zvr[0] * (tempi-fxrn) + fxrn;
	  lp->c0 = sumcq * (zzr[0] * (lr->c0 - tempi) + tempj);
	  lp->t0 = sumtq * (zzr[0] * (lr->t0 - tempi) + tempj);
	
	  /* CAT 1 */
	  
	  fxqr = tr->rdta->freqa * lq->a1 + tr->rdta->freqg * lq->g1;
	  fxqy = tr->rdta->freqc * lq->c1 + tr->rdta->freqt * lq->t1;
	  fxqn = fxqr + fxqy;
	  tempi = fxqr * tr->rdta->invfreqr;
	  tempj = zvq[1] * (tempi-fxqn) + fxqn;
	  sumaq = zzq[1] * (lq->a1 - tempi) + tempj;
	  sumgq = zzq[1] * (lq->g1 - tempi) + tempj;
	  tempi = fxqy * tr->rdta->invfreqy;
	  tempj = zvq[1] * (tempi-fxqn) + fxqn;
	  sumcq = zzq[1] * (lq->c1 - tempi) + tempj;
	  sumtq = zzq[1] * (lq->t1 - tempi) + tempj;
	  
	  fxrr = tr->rdta->freqa * lr->a1 + tr->rdta->freqg * lr->g1;
	  fxry = tr->rdta->freqc * lr->c1 + tr->rdta->freqt * lr->t1;
	  fxrn = fxrr + fxry;
	  tempi = fxrr * tr->rdta->invfreqr;
	  tempj = zvr[1] * (tempi-fxrn) + fxrn;
	  lp->a1 = sumaq * (zzr[1] * (lr->a1 - tempi) + tempj);
	  lp->g1 = sumgq * (zzr[1] * (lr->g1 - tempi) + tempj);
	  tempi = fxry * tr->rdta->invfreqy;
	  tempj = zvr[1] * (tempi-fxrn) + fxrn;
	  lp->c1 = sumcq * (zzr[1] * (lr->c1 - tempi) + tempj);
	  lp->t1 = sumtq * (zzr[1] * (lr->t1 - tempi) + tempj);
	

	  /* CAT 2 */

	  fxqr = tr->rdta->freqa * lq->a2 + tr->rdta->freqg * lq->g2;
	  fxqy = tr->rdta->freqc * lq->c2 + tr->rdta->freqt * lq->t2;
	  fxqn = fxqr + fxqy;
	  tempi = fxqr * tr->rdta->invfreqr;
	  tempj = zvq[2] * (tempi-fxqn) + fxqn;
	  sumaq = zzq[2] * (lq->a2 - tempi) + tempj;
	  sumgq = zzq[2] * (lq->g2 - tempi) + tempj;
	  tempi = fxqy * tr->rdta->invfreqy;
	  tempj = zvq[2] * (tempi-fxqn) + fxqn;
	  sumcq = zzq[2] * (lq->c2 - tempi) + tempj;
	  sumtq = zzq[2] * (lq->t2 - tempi) + tempj;
	  
	  fxrr = tr->rdta->freqa * lr->a2 + tr->rdta->freqg * lr->g2;
	  fxry = tr->rdta->freqc * lr->c2 + tr->rdta->freqt * lr->t2;
	  fxrn = fxrr + fxry;
	  tempi = fxrr * tr->rdta->invfreqr;
	  tempj = zvr[2] * (tempi-fxrn) + fxrn;
	  lp->a2 = sumaq * (zzr[2] * (lr->a2 - tempi) + tempj);
	  lp->g2 = sumgq * (zzr[2] * (lr->g2 - tempi) + tempj);
	  tempi = fxry * tr->rdta->invfreqy;
	  tempj = zvr[2] * (tempi-fxrn) + fxrn;
	  lp->c2 = sumcq * (zzr[2] * (lr->c2 - tempi) + tempj);
	  lp->t2 = sumtq * (zzr[2] * (lr->t2 - tempi) + tempj);
	

	  /* CAT 3 */

	  fxqr = tr->rdta->freqa * lq->a3 + tr->rdta->freqg * lq->g3;
	  fxqy = tr->rdta->freqc * lq->c3 + tr->rdta->freqt * lq->t3;
	  fxqn = fxqr + fxqy;
	  tempi = fxqr * tr->rdta->invfreqr;
	  tempj = zvq[3] * (tempi-fxqn) + fxqn;
	  sumaq = zzq[3] * (lq->a3 - tempi) + tempj;
	  sumgq = zzq[3] * (lq->g3 - tempi) + tempj;
	  tempi = fxqy * tr->rdta->invfreqy;
	  tempj = zvq[3] * (tempi-fxqn) + fxqn;
	  sumcq = zzq[3] * (lq->c3 - tempi) + tempj;
	  sumtq = zzq[3] * (lq->t3 - tempi) + tempj;
	  
	  fxrr = tr->rdta->freqa * lr->a3 + tr->rdta->freqg * lr->g3;
	  fxry = tr->rdta->freqc * lr->c3 + tr->rdta->freqt * lr->t3;
	  fxrn = fxrr + fxry;
	  tempi = fxrr * tr->rdta->invfreqr;
	  tempj = zvr[3] * (tempi-fxrn) + fxrn;
	  lp->a3 = sumaq * (zzr[3] * (lr->a3 - tempi) + tempj);
	  lp->g3 = sumgq * (zzr[3] * (lr->g3 - tempi) + tempj);
	  tempi = fxry * tr->rdta->invfreqy;
	  tempj = zvr[3] * (tempi-fxrn) + fxrn;
	  lp->c3 = sumcq * (zzr[3] * (lr->c3 - tempi) + tempj);
	  lp->t3 = sumtq * (zzr[3] * (lr->t3 - tempi) + tempj);
	

	  /* Scaling */
	  


	  lp->exp = lq->exp + lr->exp;
	  	     
	  if (
	      lp->a0 < minlikelihood && lp->g0 < minlikelihood && lp->c0 < minlikelihood && lp->t0 < minlikelihood &&
	      lp->a1 < minlikelihood && lp->g1 < minlikelihood && lp->c1 < minlikelihood && lp->t1 < minlikelihood &&
	      lp->a2 < minlikelihood && lp->g2 < minlikelihood && lp->c2 < minlikelihood && lp->t2 < minlikelihood &&
	      lp->a3 < minlikelihood && lp->g3 < minlikelihood && lp->c3 < minlikelihood && lp->t3 < minlikelihood
	      ) 
	    {
	      lp->a0   *= twotothe256;
	      lp->g0   *= twotothe256;
	      lp->c0   *= twotothe256;
	      lp->t0   *= twotothe256;
	      
	      lp->a1   *= twotothe256;
	      lp->g1   *= twotothe256;
	      lp->c1   *= twotothe256;
	      lp->t1   *= twotothe256;

	      lp->a2   *= twotothe256;
	      lp->g2   *= twotothe256;
	      lp->c2   *= twotothe256;
	      lp->t2   *= twotothe256;

	      lp->a3   *= twotothe256;
	      lp->g3   *= twotothe256;
	      lp->c3   *= twotothe256;
	      lp->t3   *= twotothe256;

	      lp->exp += 1;
	    }
	    
	  /*lp++;
	    lq++;
	    lr++;*/
	}

      return TRUE;
    }
  } /* newview */



double evaluateHKY85GAMMA (tree *tr, nodeptr p)
  { /* evaluate */
    double   sum, z, lz, xvlz,
             ki, fxpa, fxpc, fxpg, fxpt, fxpr, fxpy, fxqr, fxqy,
             suma, sumb, sumc, term;
    double   zz[4], zv[4];
    gammalikelivector *lp, *lq;
    nodeptr       q;
    int           i, *wptr, j;

   
    
    
    q = p->back;
    while ((! p->x) || (! q->x)) 
      {
	if (! (p->x)) if (! newviewHKY85GAMMA(tr, p)) return badEval;
	if (! (q->x)) if (! newviewHKY85GAMMA(tr, q)) return badEval;
      }
    		
	     
    lp = p->x->gv;
    lq = q->x->gv;   
    
    z = p->z;
    if (z < zmin) z = zmin;
    lz = log(z);
    xvlz = tr->rdta->xv * lz;           	   
       
    sum = 0.0;

    for(j = 0; j < tr->gammaCATS; j++)
      {
	ki = tr->gammaRates[j];
    
	zz[j] = exp(ki *   lz);
	zv[j] = exp(ki * xvlz);      
      }
	
    wptr = &(tr->cdta->aliaswgt[0]);


    
#pragma omp parallel for private(fxpa, fxpg, fxpc, fxpt, suma, fxqr, fxqy, fxpr, fxpy, sumb, sumc, term, lp, lq) reduction(+ : sum)
    for (i = 0; i < tr->cdta->endsite; i++) 
      {                
	lp = &p->x->gv[i];
	lq = &q->x->gv[i];

	fxpa = tr->rdta->freqa * lp->a0;
	fxpg = tr->rdta->freqg * lp->g0;
	fxpc = tr->rdta->freqc * lp->c0;
	fxpt = tr->rdta->freqt * lp->t0;
	suma = fxpa * lq->a0 + fxpc * lq->c0 + fxpg * lq->g0 + fxpt * lq->t0;
	fxqr = tr->rdta->freqa * lq->a0 + tr->rdta->freqg * lq->g0;
	fxqy = tr->rdta->freqc * lq->c0 + tr->rdta->freqt * lq->t0;
	fxpr = fxpa + fxpg;
	fxpy = fxpc + fxpt;
	sumc = (fxpr + fxpy) * (fxqr + fxqy);
	sumb = fxpr * fxqr * tr->rdta->invfreqr + fxpy * fxqy * tr->rdta->invfreqy;
	suma -= sumb;
	sumb -= sumc;

	term = zz[0] * suma + zv[0] * sumb + sumc;

	fxpa = tr->rdta->freqa * lp->a1;
	fxpg = tr->rdta->freqg * lp->g1;
	fxpc = tr->rdta->freqc * lp->c1;
	fxpt = tr->rdta->freqt * lp->t1;
	suma = fxpa * lq->a1 + fxpc * lq->c1 + fxpg * lq->g1 + fxpt * lq->t1;
	fxqr = tr->rdta->freqa * lq->a1 + tr->rdta->freqg * lq->g1;
	fxqy = tr->rdta->freqc * lq->c1 + tr->rdta->freqt * lq->t1;
	fxpr = fxpa + fxpg;
	fxpy = fxpc + fxpt;
	sumc = (fxpr + fxpy) * (fxqr + fxqy);
	sumb = fxpr * fxqr * tr->rdta->invfreqr + fxpy * fxqy * tr->rdta->invfreqy;
	suma -= sumb;
	sumb -= sumc;


	term += zz[1] * suma + zv[1] * sumb + sumc;

	fxpa = tr->rdta->freqa * lp->a2;
	fxpg = tr->rdta->freqg * lp->g2;
	fxpc = tr->rdta->freqc * lp->c2;
	fxpt = tr->rdta->freqt * lp->t2;
	suma = fxpa * lq->a2 + fxpc * lq->c2 + fxpg * lq->g2 + fxpt * lq->t2;
	fxqr = tr->rdta->freqa * lq->a2 + tr->rdta->freqg * lq->g2;
	fxqy = tr->rdta->freqc * lq->c2 + tr->rdta->freqt * lq->t2;
	fxpr = fxpa + fxpg;
	fxpy = fxpc + fxpt;
	sumc = (fxpr + fxpy) * (fxqr + fxqy);
	sumb = fxpr * fxqr * tr->rdta->invfreqr + fxpy * fxqy * tr->rdta->invfreqy;
	suma -= sumb;
	sumb -= sumc;
	

	term += zz[2] * suma + zv[2] * sumb + sumc;

	fxpa = tr->rdta->freqa * lp->a3;
	fxpg = tr->rdta->freqg * lp->g3;
	fxpc = tr->rdta->freqc * lp->c3;
	fxpt = tr->rdta->freqt * lp->t3;
	suma = fxpa * lq->a3 + fxpc * lq->c3 + fxpg * lq->g3 + fxpt * lq->t3;
	fxqr = tr->rdta->freqa * lq->a3 + tr->rdta->freqg * lq->g3;
	fxqy = tr->rdta->freqc * lq->c3 + tr->rdta->freqt * lq->t3;
	fxpr = fxpa + fxpg;
	fxpy = fxpc + fxpt;
	sumc = (fxpr + fxpy) * (fxqr + fxqy);
	sumb = fxpr * fxqr * tr->rdta->invfreqr + fxpy * fxqy * tr->rdta->invfreqy;
	suma -= sumb;
	sumb -= sumc;

	term += zz[3] * suma + zv[3] * sumb + sumc;

	term = log(0.25 * term) + (lp->exp + lq->exp) * log(minlikelihood);

	sum += term * wptr[i]; 	    
      }       

    tr->likelihood = sum;
    return  sum;
  } /* evaluate */



double makenewzHKY85GAMMA (tree *tr, nodeptr p, nodeptr q, double z0, int maxiter)
  { /* makenewz */
    gammalikelivector *lp, *lq;
    double  *abi, *bci, *sumci, *abptr, *bcptr, *sumcptr;
      
    double   dlnLidlz, dlnLdlz, d2lnLdlz2, z, zprev, zstep, lz, xvlz,
      ki, suma, sumb, sumc, ab, bc, inv_Li, t1, t2, t3, Li, t1_0, t1_1, t1_2, t1_3, t2_0, t2_1, t2_2, t2_3,
      fx1a, fx1c, fx1g, fx1t, fx1r, fx1y, fx2r, fx2y, sumc0, ab0, bc0, sumc1, ab1, bc1,
      sumc2, ab2, bc2, sumc3, ab3, bc3, 
      dlnLdlz_0, d2lnLdlz2_0, dlnLdlz_1, d2lnLdlz2_1, dlnLdlz_2, d2lnLdlz2_2, dlnLdlz_3, d2lnLdlz2_3, inv_Li0, inv_Li1, inv_Li2, inv_Li3,
      t1d2, t2d2;
    double   zz[4], zv[4], gammaSquares[4];
    double  *rptr, *wrptr, *wr2ptr;
    int      cat, *cptr, i, curvatOK, j;


    while ((! p->x) || (! q->x)) 
      {
	if (! (p->x)) if (! newviewHKY85GAMMA(tr, p)) return badZ;
	if (! (q->x)) if (! newviewHKY85GAMMA(tr, q)) return badZ;
      }

    lp = p->x->gv;
    lq = q->x->gv;

    { 
      unsigned scratch_size;
      scratch_size = sizeof(double) * tr->cdta->endsite;
      abi   = (double *) malloc(scratch_size * 4);
      bci   = (double *) malloc(scratch_size * 4);
      sumci = (double *) malloc(scratch_size);      
    }
       

    abptr   = abi;
    bcptr   = bci;
    sumcptr = sumci;
    

#pragma omp parallel for private(fx1a, fx1g, fx1c, fx1t, suma, fx2r, fx2y, fx1r, fx1y, sumb, sumc, lp, lq, abptr, bcptr)
    for (i = 0; i < tr->cdta->endsite; i++) 
      {
	lp = &p->x->gv[i];
	lq = &q->x->gv[i]; 
	abptr = &abi[4 * i];
	bcptr = &bci[4 * i];

	fx1a = tr->rdta->freqa * lp->a0;
	fx1g = tr->rdta->freqg * lp->g0;
	fx1c = tr->rdta->freqc * lp->c0;
	fx1t = tr->rdta->freqt * lp->t0;
	suma = fx1a * lq->a0 + fx1c * lq->c0 + fx1g * lq->g0 + fx1t * lq->t0;
	fx2r = tr->rdta->freqa * lq->a0 + tr->rdta->freqg * lq->g0;
	fx2y = tr->rdta->freqc * lq->c0 + tr->rdta->freqt * lq->t0;
	fx1r = fx1a + fx1g;
	fx1y = fx1c + fx1t;
	sumcptr[i] = sumc = (fx1r + fx1y) * (fx2r + fx2y);
	sumb       = fx1r * fx2r * tr->rdta->invfreqr + fx1y * fx2y * tr->rdta->invfreqy;
	*abptr++   = (suma - sumb);
	*bcptr++   = (sumb - sumc);

	fx1a = tr->rdta->freqa * lp->a1;
	fx1g = tr->rdta->freqg * lp->g1;
	fx1c = tr->rdta->freqc * lp->c1;
	fx1t = tr->rdta->freqt * lp->t1;
	suma = fx1a * lq->a1 + fx1c * lq->c1 + fx1g * lq->g1 + fx1t * lq->t1;
	fx2r = tr->rdta->freqa * lq->a1 + tr->rdta->freqg * lq->g1;
	fx2y = tr->rdta->freqc * lq->c1 + tr->rdta->freqt * lq->t1;
	fx1r = fx1a + fx1g;
	fx1y = fx1c + fx1t;
	sumcptr[i] += sumc = (fx1r + fx1y) * (fx2r + fx2y);
	sumb       = fx1r * fx2r * tr->rdta->invfreqr + fx1y * fx2y * tr->rdta->invfreqy;
	*abptr++   = (suma - sumb);
	*bcptr++   = (sumb - sumc);

	fx1a = tr->rdta->freqa * lp->a2;
	fx1g = tr->rdta->freqg * lp->g2;
	fx1c = tr->rdta->freqc * lp->c2;
	fx1t = tr->rdta->freqt * lp->t2;
	suma = fx1a * lq->a2 + fx1c * lq->c2 + fx1g * lq->g2 + fx1t * lq->t2;
	fx2r = tr->rdta->freqa * lq->a2 + tr->rdta->freqg * lq->g2;
	fx2y = tr->rdta->freqc * lq->c2 + tr->rdta->freqt * lq->t2;
	fx1r = fx1a + fx1g;
	fx1y = fx1c + fx1t;
	sumcptr[i] += sumc = (fx1r + fx1y) * (fx2r + fx2y);
	sumb       = fx1r * fx2r * tr->rdta->invfreqr + fx1y * fx2y * tr->rdta->invfreqy;
	*abptr++   = (suma - sumb);
	*bcptr++   = (sumb - sumc);

	fx1a = tr->rdta->freqa * lp->a3;
	fx1g = tr->rdta->freqg * lp->g3;
	fx1c = tr->rdta->freqc * lp->c3;
	fx1t = tr->rdta->freqt * lp->t3;
	suma = fx1a * lq->a3 + fx1c * lq->c3 + fx1g * lq->g3 + fx1t * lq->t3;
	fx2r = tr->rdta->freqa * lq->a3 + tr->rdta->freqg * lq->g3;
	fx2y = tr->rdta->freqc * lq->c3 + tr->rdta->freqt * lq->t3;
	fx1r = fx1a + fx1g;
	fx1y = fx1c + fx1t;
	sumcptr[i] += sumc = (fx1r + fx1y) * (fx2r + fx2y);
	sumb       = fx1r * fx2r * tr->rdta->invfreqr + fx1y * fx2y * tr->rdta->invfreqy;
	*abptr++   = (suma - sumb);
	*bcptr++   = (sumb - sumc);
      }

    gammaSquares[0] = tr->gammaRates[0] * tr->gammaRates[0];
    gammaSquares[1] = tr->gammaRates[1] * tr->gammaRates[1];
    gammaSquares[2] = tr->gammaRates[2] * tr->gammaRates[2];
    gammaSquares[3] = tr->gammaRates[3] * tr->gammaRates[3];


    z = z0;
    do {
      zprev = z;
      zstep = (1.0 - zmax) * z + zmin;
      curvatOK = FALSE;

      do {
        if (z < zmin) z = zmin;
        else if (z > zmax) z = zmax;

        lz    = log(z);
        xvlz  = tr->rdta->xv * lz;
       
	abptr   = abi;
        bcptr   = bci;
        sumcptr = sumci;  


       
	dlnLdlz = 0.0;                 
	d2lnLdlz2 = 0.0;               

	for(j = 0; j < 4; j++)
	  {
	    ki = tr->gammaRates[j];
    
	    zz[j] = exp(ki *   lz);
	    zv[j] = exp(ki * xvlz);      
	  }
	    
	wrptr   = &(tr->cdta->wr[0]);
	wr2ptr  = &(tr->cdta->wr2[0]);	
	    
#pragma omp parallel for private(cat, ab0, bc0, ab1, bc1, ab2, bc2, ab3, bc3, sumc, inv_Li, t1, t2, dlnLidlz, abptr, bcptr, t1d2, t2d2) reduction(+ : dlnLdlz) reduction( + : d2lnLdlz2) 
	for (i = 0; i < tr->cdta->endsite; i++) 
	  {         		

	    abptr = &abi[4 * i];
	    bcptr = &bci[4 * i];

	    ab0 = *abptr++ * zz[0];
	    bc0 = *bcptr++ * zv[0];
	    sumc = ab0 + bc0 + sumci[i];
	    
	    ab1     =  *abptr++ * zz[1];
	    bc1     =  *bcptr++ * zv[1];
	    sumc   +=  ab1 + bc1;
	    
	    ab2    = *abptr++ * zz[2];
	    bc2     = *bcptr++ * zv[2];
	    sumc   +=  ab2 + bc2;	    
	    
	    ab3     = *abptr++ * zz[3];
	    bc3    =  *bcptr++ * zv[3];
	    sumc  +=  ab3 + bc3;

	    inv_Li = 1/sumc;	    

	    t1 = (tr->gammaRates[0] * ab0 + tr->gammaRates[1] * ab1 + tr->gammaRates[2] * ab2 + tr->gammaRates[3] * ab3) * inv_Li;

	    t2 = tr->rdta->xv * (tr->gammaRates[0] * bc0 + tr->gammaRates[1] * bc1 + tr->gammaRates[2] * bc2 + tr->gammaRates[3] * bc3) * inv_Li;

	    dlnLidlz = t1 + t2;
	  	    
	    t1d2 = (gammaSquares[0] * ab0 + gammaSquares[1] * ab1 + 
		    gammaSquares[2] * ab2 + gammaSquares[3] * ab3) * inv_Li;

	    t2d2 = tr->rdta->xv * (gammaSquares[0] * bc0 + gammaSquares[1] * bc1 + 
				   gammaSquares[2] * bc2 + gammaSquares[3] * bc3) * inv_Li;


	    dlnLdlz   += wrptr[i]  * dlnLidlz;
	    d2lnLdlz2 += wr2ptr[i] * (t1d2 + tr->rdta->xv * t2d2 - dlnLidlz * dlnLidlz);					
	  }
      
        if ((d2lnLdlz2 >= 0.0) && (z < zmax))
          zprev = z = 0.37 * z + 0.63;  /*  Bad curvature, shorten branch */
        else
          curvatOK = TRUE;

        } while (! curvatOK);

      if (d2lnLdlz2 < 0.0) {
        z *= exp(-dlnLdlz / d2lnLdlz2);
        if (z < zmin) z = zmin;
        if (z > 0.25 * zprev + 0.75)    /*  Limit steps toward z = 1.0 */
          z = 0.25 * zprev + 0.75;
        }

      if (z > zmax) z = zmax;

      } while ((--maxiter > 0) && (ABS(z - zprev) > zstep));

    free(abi);
    free(bci);
    free(sumci);   

/* printf("makenewz: %le\n", z); */

    return  z;
  } /* makenewz */








boolean newviewGTRGAMMA(tree    *tr, nodeptr  p)
{
  
  if (p->tip) 
    {      
      if (p->x) return TRUE;
      if (! getxtip(p)) return FALSE;  
      {
	int code, i, j, jj;
	double (*EIGV)[4] = &(tr->rdta->EIGV[0][0]);
	gammalikelivector *x1 = p->x->gv;
#pragma omp parallel for private(code, j, x1)      
	for(i = 0; i < tr->cdta->endsite; i++)
	  {	      	  	 
	    x1 = &p->x->gv[i];

	    code = p->tip[i];
	    x1->a0 = 0;
	    x1->c0 = 0;
	    x1->g0 = 0;
	    x1->t0 = 0;
	    for (j = 0; j < 4; j++) 
	      {
		if ((code >> j) & 1) 
		  {
		    int jj = "0213"[j] - '0';		
		    x1->a0 += EIGV[jj][0];
		    x1->c0 += EIGV[jj][1];
		    x1->g0 += EIGV[jj][2];
		    x1->t0 += EIGV[jj][3];
		  }
	      }		
	      
	    x1->a1 = x1->a2 = x1->a3 = x1->a0;
	    x1->c1 = x1->c2 = x1->c3 = x1->c0;
	    x1->g1 = x1->g2 = x1->g3 = x1->g0;
	    x1->t1 = x1->t2 = x1->t3 = x1->t0;

	    x1->exp = 0;	  			  			 
	    /*x1++;	 */
	} 
      return TRUE; 
      }
    }
  
  { 	

    double  *diagptable_x1, *diagptable_x2, *diagptable_x1_start, *diagptable_x2_start;
    double *EIGN    = &(tr->rdta->EIGN[0][0]);
    double *invfreq = &(tr->rdta->invfreq[0][0]);
    double (*EIGV)[4] = &(tr->rdta->EIGV[0][0]);
    xtype tmp_x1_1, tmp_x1_2, tmp_x1_3,
      tmp_x2_1, tmp_x2_2, tmp_x2_3, 
      ump_x1_1, ump_x1_2, ump_x1_3, ump_x1_0, 
      ump_x2_0, ump_x2_1, ump_x2_2, ump_x2_3, x1px2;
    int cat;
    int  *cptr;	  
    int  i, j;
    gammalikelivector   *x1, *x2, *x3;
    double   z1, lz1, z2, lz2, ki;
    nodeptr  q, r;   
    double *eigv, *rptr;

    rptr   = &(tr->cdta->patrat[0]); 
    cptr  = &(tr->cdta->rateCategory[0]);

    q = p->next->back;
    r = p->next->next->back;
      
    while ((! p->x) || (! q->x) || (! r->x)) {
      if (! q->x) if (! newviewGTRGAMMA(tr, q)) return FALSE;
      if (! r->x) if (! newviewGTRGAMMA(tr, r)) return FALSE;
      if (! p->x) if (! getxnode(p)) return FALSE;	
    }

    x1  = q->x->gv;
    z1  = q->z;
    lz1 = (z1 > zmin) ? log(z1) : log(zmin);

    x2  = r->x->gv;
    z2  = r->z;
    lz2 = (z2 > zmin) ? log(z2) : log(zmin);

    x3  = p->x->gv;   
   
    
    diagptable_x1 = diagptable_x1_start = (double *)malloc(sizeof(double) * 4 * 3);
    diagptable_x2 = diagptable_x2_start = (double *)malloc(sizeof(double) * 4 * 3);
    
    
    for(i = 0; i < 4; i++)
      {
	ki = tr->gammaRates[i];
	
	*diagptable_x1++ = exp (EIGN[1] * ki * lz1);
	*diagptable_x1++ = exp (EIGN[2] * ki * lz1);
	*diagptable_x1++ = exp (EIGN[3] * ki * lz1);
	*diagptable_x2++ = exp (EIGN[1] * ki * lz2);
	*diagptable_x2++ = exp (EIGN[2] * ki * lz2);
	*diagptable_x2++ = exp (EIGN[3] * ki * lz2);		
      }

    
#pragma omp parallel for private(tmp_x1_1, tmp_x1_2,tmp_x1_3,ump_x1_0,ump_x1_1,ump_x1_2,ump_x1_3,ump_x2_0,ump_x2_1,ump_x2_2,ump_x2_3,tmp_x2_1,tmp_x2_2,tmp_x2_3,x1px2, diagptable_x1, diagptable_x2, x1, x2, x3)
    for (i = 0; i < tr->cdta->endsite; i++) 
      {	
	x1  = &q->x->gv[i];
	x2  = &r->x->gv[i];
	x3  = &p->x->gv[i];

	/* Rate cat 0 */

	diagptable_x1 = diagptable_x1_start;
	diagptable_x2 = diagptable_x2_start;

	tmp_x1_1 = x1->c0 * *diagptable_x1++;
	tmp_x1_2 = x1->g0 * *diagptable_x1++;
	tmp_x1_3 = x1->t0 * *diagptable_x1++;
	
	ump_x1_0 = tmp_x1_1 * EIGV[0][1];
	ump_x1_0 += tmp_x1_2 * EIGV[0][2];
	ump_x1_0 += tmp_x1_3 * EIGV[0][3];	      	
	ump_x1_0 *= invfreq[0];
	ump_x1_0 += x1->a0;
	
	ump_x1_1 = tmp_x1_1 * EIGV[1][1];
	ump_x1_1 += tmp_x1_2 * EIGV[1][2];
	ump_x1_1 += tmp_x1_3 * EIGV[1][3];	      	
	ump_x1_1 *= invfreq[1];
	ump_x1_1 += x1->a0;
	    
	ump_x1_2 = tmp_x1_1 * EIGV[2][1];
	ump_x1_2 += tmp_x1_2 * EIGV[2][2];
	ump_x1_2 += tmp_x1_3 * EIGV[2][3];	      	
	ump_x1_2 *= invfreq[2];
	ump_x1_2 += x1->a0;

	ump_x1_3 = tmp_x1_1 * EIGV[3][1];
	ump_x1_3 += tmp_x1_2 * EIGV[3][2];
	ump_x1_3 += tmp_x1_3 * EIGV[3][3];	      	
	ump_x1_3 *= invfreq[3];
	ump_x1_3 += x1->a0;
	
	tmp_x2_1 = x2->c0 * *diagptable_x2++;
	tmp_x2_2 = x2->g0 * *diagptable_x2++;
	tmp_x2_3 = x2->t0 * *diagptable_x2++;
	 
	 	    	    	     	     
	ump_x2_0 = tmp_x2_1 * EIGV[0][1];
	ump_x2_0 += tmp_x2_2 * EIGV[0][2];
	ump_x2_0 += tmp_x2_3 * EIGV[0][3];		     
	ump_x2_0 *= invfreq[0];
	ump_x2_0 += x2->a0;
	  
	ump_x2_1 = tmp_x2_1 * EIGV[1][1];
	ump_x2_1 += tmp_x2_2 * EIGV[1][2];
	ump_x2_1 += tmp_x2_3 * EIGV[1][3];		     
	ump_x2_1 *= invfreq[1];
	ump_x2_1 += x2->a0;	 

	ump_x2_2 = tmp_x2_1 * EIGV[2][1];
	ump_x2_2 += tmp_x2_2 * EIGV[2][2];
	ump_x2_2 += tmp_x2_3 * EIGV[2][3];		     
	ump_x2_2 *= invfreq[2];
	ump_x2_2 += x2->a0;	  
		   

	ump_x2_3 = tmp_x2_1 * EIGV[3][1] ;
	ump_x2_3 += tmp_x2_2 * EIGV[3][2];
	ump_x2_3 += tmp_x2_3 * EIGV[3][3];		     
	ump_x2_3 *= invfreq[3];
	ump_x2_3 += x2->a0;	    	  		   	  
     
	
	x1px2 = ump_x1_0 * ump_x2_0;
	x3->a0 = x1px2 *  EIGV[0][0];
	x3->c0 = x1px2 *  EIGV[0][1];
	x3->g0 = x1px2 *  EIGV[0][2];
	x3->t0 = x1px2 *  EIGV[0][3];
	
	x1px2 = ump_x1_1 * ump_x2_1;
	x3->a0 += x1px2  *  EIGV[1][0];
	x3->c0 += x1px2 *   EIGV[1][1];
	x3->g0 += x1px2 *   EIGV[1][2];
	x3->t0 += x1px2 *   EIGV[1][3];
	
	x1px2 = ump_x1_2 * ump_x2_2;
	x3->a0 += x1px2 *  EIGV[2][0];
	x3->c0 += x1px2*   EIGV[2][1];
	x3->g0 += x1px2 *  EIGV[2][2];
	x3->t0 += x1px2 *  EIGV[2][3];
	
	x1px2 = ump_x1_3 * ump_x2_3;
	x3->a0 += x1px2 *   EIGV[3][0];
	x3->c0 += x1px2 *   EIGV[3][1];
	x3->g0 += x1px2 *   EIGV[3][2];
	x3->t0 += x1px2 *   EIGV[3][3];

	/* rate cat 1 */
	
	tmp_x1_1 = x1->c1 * *diagptable_x1++;
	tmp_x1_2 = x1->g1 * *diagptable_x1++;
	tmp_x1_3 = x1->t1 * *diagptable_x1++;
	
	ump_x1_0 = tmp_x1_1 * EIGV[0][1];
	ump_x1_0 += tmp_x1_2 * EIGV[0][2];
	ump_x1_0 += tmp_x1_3 * EIGV[0][3];	      	
	ump_x1_0 *= invfreq[0];
	ump_x1_0 += x1->a1;
	
	ump_x1_1 = tmp_x1_1 * EIGV[1][1];
	ump_x1_1 += tmp_x1_2 * EIGV[1][2];
	ump_x1_1 += tmp_x1_3 * EIGV[1][3];	      	
	ump_x1_1 *= invfreq[1];
	ump_x1_1 += x1->a1;
	    
	ump_x1_2 = tmp_x1_1 * EIGV[2][1];
	ump_x1_2 += tmp_x1_2 * EIGV[2][2];
	ump_x1_2 += tmp_x1_3 * EIGV[2][3];	      	
	ump_x1_2 *= invfreq[2];
	ump_x1_2 += x1->a1;

	ump_x1_3 = tmp_x1_1 * EIGV[3][1];
	ump_x1_3 += tmp_x1_2 * EIGV[3][2];
	ump_x1_3 += tmp_x1_3 * EIGV[3][3];	      	
	ump_x1_3 *= invfreq[3];
	ump_x1_3 += x1->a1;
	
	tmp_x2_1 = x2->c1 * *diagptable_x2++;
	tmp_x2_2 = x2->g1 * *diagptable_x2++;
	tmp_x2_3 = x2->t1 * *diagptable_x2++;
	 
	 	    	    	     	     
	ump_x2_0 = tmp_x2_1 * EIGV[0][1];
	ump_x2_0 += tmp_x2_2 * EIGV[0][2];
	ump_x2_0 += tmp_x2_3 * EIGV[0][3];		     
	ump_x2_0 *= invfreq[0];
	ump_x2_0 += x2->a1;
	  
	ump_x2_1 = tmp_x2_1 * EIGV[1][1];
	ump_x2_1 += tmp_x2_2 * EIGV[1][2];
	ump_x2_1 += tmp_x2_3 * EIGV[1][3];		     
	ump_x2_1 *= invfreq[1];
	ump_x2_1 += x2->a1;	 

	ump_x2_2 = tmp_x2_1 * EIGV[2][1];
	ump_x2_2 += tmp_x2_2 * EIGV[2][2];
	ump_x2_2 += tmp_x2_3 * EIGV[2][3];		     
	ump_x2_2 *= invfreq[2];
	ump_x2_2 += x2->a1;	  
		   

	ump_x2_3 = tmp_x2_1 * EIGV[3][1] ;
	ump_x2_3 += tmp_x2_2 * EIGV[3][2];
	ump_x2_3 += tmp_x2_3 * EIGV[3][3];		     
	ump_x2_3 *= invfreq[3];
	ump_x2_3 += x2->a1;	    	  		   	  
     
	
	x1px2 = ump_x1_0 * ump_x2_0;
	x3->a1 = x1px2 *  EIGV[0][0];
	x3->c1 = x1px2 *  EIGV[0][1];
	x3->g1 = x1px2 *  EIGV[0][2];
	x3->t1 = x1px2 *  EIGV[0][3];
	
	x1px2 = ump_x1_1 * ump_x2_1;
	x3->a1 += x1px2  *  EIGV[1][0];
	x3->c1 += x1px2 *   EIGV[1][1];
	x3->g1 += x1px2 *   EIGV[1][2];
	x3->t1 += x1px2 *   EIGV[1][3];
	
	x1px2 = ump_x1_2 * ump_x2_2;
	x3->a1 += x1px2 *  EIGV[2][0];
	x3->c1 += x1px2*   EIGV[2][1];
	x3->g1 += x1px2 *  EIGV[2][2];
	x3->t1 += x1px2 *  EIGV[2][3];
	
	x1px2 = ump_x1_3 * ump_x2_3;
	x3->a1 += x1px2 *   EIGV[3][0];
	x3->c1 += x1px2 *   EIGV[3][1];
	x3->g1 += x1px2 *   EIGV[3][2];
	x3->t1 += x1px2 *   EIGV[3][3];


	/* rate cat 2 */

	tmp_x1_1 = x1->c2 * *diagptable_x1++;
	tmp_x1_2 = x1->g2 * *diagptable_x1++;
	tmp_x1_3 = x1->t2 * *diagptable_x1++;
	
	ump_x1_0 = tmp_x1_1 * EIGV[0][1];
	ump_x1_0 += tmp_x1_2 * EIGV[0][2];
	ump_x1_0 += tmp_x1_3 * EIGV[0][3];	      	
	ump_x1_0 *= invfreq[0];
	ump_x1_0 += x1->a2;
	
	ump_x1_1 = tmp_x1_1 * EIGV[1][1];
	ump_x1_1 += tmp_x1_2 * EIGV[1][2];
	ump_x1_1 += tmp_x1_3 * EIGV[1][3];	      	
	ump_x1_1 *= invfreq[1];
	ump_x1_1 += x1->a2;
	    
	ump_x1_2 = tmp_x1_1 * EIGV[2][1];
	ump_x1_2 += tmp_x1_2 * EIGV[2][2];
	ump_x1_2 += tmp_x1_3 * EIGV[2][3];	      	
	ump_x1_2 *= invfreq[2];
	ump_x1_2 += x1->a2;

	ump_x1_3 = tmp_x1_1 * EIGV[3][1];
	ump_x1_3 += tmp_x1_2 * EIGV[3][2];
	ump_x1_3 += tmp_x1_3 * EIGV[3][3];	      	
	ump_x1_3 *= invfreq[3];
	ump_x1_3 += x1->a2;
	
	tmp_x2_1 = x2->c2 * *diagptable_x2++;
	tmp_x2_2 = x2->g2 * *diagptable_x2++;
	tmp_x2_3 = x2->t2 * *diagptable_x2++;
	 
	 	    	    	     	     
	ump_x2_0 = tmp_x2_1 * EIGV[0][1];
	ump_x2_0 += tmp_x2_2 * EIGV[0][2];
	ump_x2_0 += tmp_x2_3 * EIGV[0][3];		     
	ump_x2_0 *= invfreq[0];
	ump_x2_0 += x2->a2;
	  
	ump_x2_1 = tmp_x2_1 * EIGV[1][1];
	ump_x2_1 += tmp_x2_2 * EIGV[1][2];
	ump_x2_1 += tmp_x2_3 * EIGV[1][3];		     
	ump_x2_1 *= invfreq[1];
	ump_x2_1 += x2->a2;	 

	ump_x2_2 = tmp_x2_1 * EIGV[2][1];
	ump_x2_2 += tmp_x2_2 * EIGV[2][2];
	ump_x2_2 += tmp_x2_3 * EIGV[2][3];		     
	ump_x2_2 *= invfreq[2];
	ump_x2_2 += x2->a2;	  
		   

	ump_x2_3 = tmp_x2_1 * EIGV[3][1] ;
	ump_x2_3 += tmp_x2_2 * EIGV[3][2];
	ump_x2_3 += tmp_x2_3 * EIGV[3][3];		     
	ump_x2_3 *= invfreq[3];
	ump_x2_3 += x2->a2;	    	  		   	  
     
	
	x1px2 = ump_x1_0 * ump_x2_0;
	x3->a2 = x1px2 *  EIGV[0][0];
	x3->c2 = x1px2 *  EIGV[0][1];
	x3->g2 = x1px2 *  EIGV[0][2];
	x3->t2 = x1px2 *  EIGV[0][3];
	
	x1px2 = ump_x1_1 * ump_x2_1;
	x3->a2 += x1px2  *  EIGV[1][0];
	x3->c2 += x1px2 *   EIGV[1][1];
	x3->g2 += x1px2 *   EIGV[1][2];
	x3->t2 += x1px2 *   EIGV[1][3];
	
	x1px2 = ump_x1_2 * ump_x2_2;
	x3->a2 += x1px2 *  EIGV[2][0];
	x3->c2 += x1px2*   EIGV[2][1];
	x3->g2 += x1px2 *  EIGV[2][2];
	x3->t2 += x1px2 *  EIGV[2][3];
	
	x1px2 = ump_x1_3 * ump_x2_3;
	x3->a2 += x1px2 *   EIGV[3][0];
	x3->c2 += x1px2 *   EIGV[3][1];
	x3->g2 += x1px2 *   EIGV[3][2];
	x3->t2 += x1px2 *   EIGV[3][3];


	/* rate cat 3 */

	tmp_x1_1 = x1->c3 * *diagptable_x1++;
	tmp_x1_2 = x1->g3 * *diagptable_x1++;
	tmp_x1_3 = x1->t3 * *diagptable_x1++;
	
	ump_x1_0 = tmp_x1_1 * EIGV[0][1];
	ump_x1_0 += tmp_x1_2 * EIGV[0][2];
	ump_x1_0 += tmp_x1_3 * EIGV[0][3];	      	
	ump_x1_0 *= invfreq[0];
	ump_x1_0 += x1->a3;
	
	ump_x1_1 = tmp_x1_1 * EIGV[1][1];
	ump_x1_1 += tmp_x1_2 * EIGV[1][2];
	ump_x1_1 += tmp_x1_3 * EIGV[1][3];	      	
	ump_x1_1 *= invfreq[1];
	ump_x1_1 += x1->a3;
	    
	ump_x1_2 = tmp_x1_1 * EIGV[2][1];
	ump_x1_2 += tmp_x1_2 * EIGV[2][2];
	ump_x1_2 += tmp_x1_3 * EIGV[2][3];	      	
	ump_x1_2 *= invfreq[2];
	ump_x1_2 += x1->a3;

	ump_x1_3 = tmp_x1_1 * EIGV[3][1];
	ump_x1_3 += tmp_x1_2 * EIGV[3][2];
	ump_x1_3 += tmp_x1_3 * EIGV[3][3];	      	
	ump_x1_3 *= invfreq[3];
	ump_x1_3 += x1->a3;
	
	tmp_x2_1 = x2->c3 * *diagptable_x2++;
	tmp_x2_2 = x2->g3 * *diagptable_x2++;
	tmp_x2_3 = x2->t3 * *diagptable_x2++;
	 
	 	    	    	     	     
	ump_x2_0 = tmp_x2_1 * EIGV[0][1];
	ump_x2_0 += tmp_x2_2 * EIGV[0][2];
	ump_x2_0 += tmp_x2_3 * EIGV[0][3];		     
	ump_x2_0 *= invfreq[0];
	ump_x2_0 += x2->a3;
	  
	ump_x2_1 = tmp_x2_1 * EIGV[1][1];
	ump_x2_1 += tmp_x2_2 * EIGV[1][2];
	ump_x2_1 += tmp_x2_3 * EIGV[1][3];		     
	ump_x2_1 *= invfreq[1];
	ump_x2_1 += x2->a3;	 

	ump_x2_2 = tmp_x2_1 * EIGV[2][1];
	ump_x2_2 += tmp_x2_2 * EIGV[2][2];
	ump_x2_2 += tmp_x2_3 * EIGV[2][3];		     
	ump_x2_2 *= invfreq[2];
	ump_x2_2 += x2->a3;	  
		   

	ump_x2_3 = tmp_x2_1 * EIGV[3][1] ;
	ump_x2_3 += tmp_x2_2 * EIGV[3][2];
	ump_x2_3 += tmp_x2_3 * EIGV[3][3];		     
	ump_x2_3 *= invfreq[3];
	ump_x2_3 += x2->a3;	    	  		   	  
     
	
	x1px2 = ump_x1_0 * ump_x2_0;
	x3->a3 = x1px2 *  EIGV[0][0];
	x3->c3 = x1px2 *  EIGV[0][1];
	x3->g3 = x1px2 *  EIGV[0][2];
	x3->t3 = x1px2 *  EIGV[0][3];
	
	x1px2 = ump_x1_1 * ump_x2_1;
	x3->a3 += x1px2  *  EIGV[1][0];
	x3->c3 += x1px2 *   EIGV[1][1];
	x3->g3 += x1px2 *   EIGV[1][2];
	x3->t3 += x1px2 *   EIGV[1][3];
	
	x1px2 = ump_x1_2 * ump_x2_2;
	x3->a3 += x1px2 *  EIGV[2][0];
	x3->c3 += x1px2*   EIGV[2][1];
	x3->g3 += x1px2 *  EIGV[2][2];
	x3->t3 += x1px2 *  EIGV[2][3];
	
	x1px2 = ump_x1_3 * ump_x2_3;
	x3->a3 += x1px2 *   EIGV[3][0];
	x3->c3 += x1px2 *   EIGV[3][1];
	x3->g3 += x1px2 *   EIGV[3][2];
	x3->t3 += x1px2 *   EIGV[3][3];


	/* end */
		   
	x3->exp = x1->exp + x2->exp;		  
	
	if (ABS(x3->a0) < minlikelihood && ABS(x3->g0) < minlikelihood && ABS(x3->c0) < minlikelihood && ABS(x3->t0) < minlikelihood &&
	    ABS(x3->a1) < minlikelihood && ABS(x3->g1) < minlikelihood && ABS(x3->c1) < minlikelihood && ABS(x3->t1) < minlikelihood &&
	    ABS(x3->a2) < minlikelihood && ABS(x3->g2) < minlikelihood && ABS(x3->c2) < minlikelihood && ABS(x3->t2) < minlikelihood &&
	    ABS(x3->a3) < minlikelihood && ABS(x3->g3) < minlikelihood && ABS(x3->c3) < minlikelihood && ABS(x3->t3) < minlikelihood) 
	  {	     
	    x3->a0   *= twotothe256;
	    x3->g0   *= twotothe256;
	    x3->c0   *= twotothe256;
	    x3->t0   *= twotothe256;

	    x3->a1   *= twotothe256;
	    x3->g1   *= twotothe256;
	    x3->c1   *= twotothe256;
	    x3->t1   *= twotothe256;

	    x3->a2   *= twotothe256;
	    x3->g2   *= twotothe256;
	    x3->c2   *= twotothe256;
	    x3->t2   *= twotothe256;

	    x3->a3   *= twotothe256;
	    x3->g3   *= twotothe256;
	    x3->c3   *= twotothe256;
	    x3->t3   *= twotothe256;
	   
	    x3->exp += 1;
	  }
	 	
	/*x1++;
	x2++;
	x3++;*/
      
      }
         
    free(diagptable_x1_start); 
    free(diagptable_x2_start);
    return TRUE;
  }
}





double evaluateGTRGAMMA (tr, p)
    tree    *tr;
    nodeptr  p;
  { /* evaluate */
    double   sum, z, lz, term, ki, term2;    
    nodeptr  q;
    int     i, j, cat;
    int     *wptr = tr->cdta->aliaswgt, *cptr;
    
    double  *diagptable, *rptr, *diagptable_start;
    double *EIGN = &(tr->rdta->EIGN[0][0]);
    gammalikelivector   *x1, *x2;
       
    q = p->back;
    while ((! p->x) || (! q->x)) {
      if (! (p->x)) if (! newviewGTRGAMMA(tr, p)) return badEval;
      if (! (q->x)) if (! newviewGTRGAMMA(tr, q)) return badEval;
    }


    rptr  = &(tr->cdta->patrat[0]); 
    cptr  = &(tr->cdta->rateCategory[0]);

    x1  = p->x->gv;
    x2  = q->x->gv;
    z = p->z;
   
    if (z < zmin) z = zmin;
    lz = log(z);

   
    
    diagptable = diagptable_start = (double *)malloc(sizeof(double) * 4 * 3);
    for(i = 0; i < 4; i++)
      {
	ki = tr->gammaRates[i];	 

	*diagptable++ = exp (EIGN[1] * ki * lz);
	*diagptable++ = exp (EIGN[2] * ki * lz);
	*diagptable++ = exp (EIGN[3] * ki * lz);
      }
	
    
    sum = 0.0;



#pragma omp parallel for private(cat, term, x1, x2, diagptable) reduction(+ : sum)  
    for (i = 0; i < tr->cdta->endsite; i++) 
      {	  	 	  
	diagptable = diagptable_start;
	
	x1 = &p->x->gv[i];
	x2 = &q->x->gv[i];

	/* cat 0 */

	term =  x1->a0 * x2->a0;
	term += x1->c0 * x2->c0 * *diagptable++;
	term += x1->g0 * x2->g0 * *diagptable++;
	term += x1->t0 * x2->t0 * *diagptable++;     

	/* cat 1 */

	term += x1->a1 * x2->a1;
	term += x1->c1 * x2->c1 * *diagptable++;
	term += x1->g1 * x2->g1 * *diagptable++;
	term += x1->t1 * x2->t1 * *diagptable++;     

       

	/* cat 2 */

	term += x1->a2 * x2->a2;
	term += x1->c2 * x2->c2 * *diagptable++;
	term += x1->g2 * x2->g2 * *diagptable++;
	term += x1->t2 * x2->t2 * *diagptable++;     

	

	/* cat 3 */
	
	term +=  x1->a3 * x2->a3;
	term += x1->c3 * x2->c3 * *diagptable++;
	term += x1->g3 * x2->g3 * *diagptable++;
	term += x1->t3 * x2->t3 * *diagptable++;     

	term = log(0.25 * term) + (x1->exp + x2->exp)*log(minlikelihood);
	sum += wptr[i] * term;		  
	/*x1++;
	  x2++;*/
      }
    
      
    free(diagptable_start); 
    
    tr->likelihood = sum;
    return  sum;
  } /* evaluate */







double makenewzGTRGAMMA (tr, p, q, z0, maxiter)
    tree    *tr;
    nodeptr  p, q;
    double   z0;
    int  maxiter;
  { /* makenewz */
    double   z, zprev, zstep;
    gammalikelivector  *x1, *x2;
    double  *sumtable, *sum, *diagptable, *diagptable_start;
    int     i;
    double  dlnLdlz = 0.0;                 /*  = d(ln(likelihood))/d(lz) */
    double  d2lnLdlz2 = 0.0;               /*  = d2(ln(likelihood))/d(lz)2 */    
    double sumc, ki;
    
    while ((! p->x) || (! q->x)) 
      {
	if (! (p->x)) if (! newviewGTRGAMMA(tr, p)) return badZ;
	if (! (q->x)) if (! newviewGTRGAMMA(tr, q)) return badZ;
      }

    x1 = p->x->gv;
    x2 = q->x->gv;

    sum = sumtable = (double *)malloc(13 * tr->cdta->endsite * sizeof(double));
    diagptable = diagptable_start = (double *)malloc(sizeof(double) * 4 * 3);
  
   #pragma omp parallel for private(x1, x2, sumc, sum)	
    for (i = 0; i < tr->cdta->endsite; i++) 
      {   
	x1 = &p->x->gv[i];
	x2 = &q->x->gv[i];	

	sum = &sumtable[13 * i];

	sumc = x1->a0 * x2->a0;
	*sum++ = x1->c0 * x2->c0;
	*sum++ = x1->g0 * x2->g0;
	*sum++ = x1->t0 * x2->t0;		    

	sumc += x1->a1 * x2->a1;
	*sum++ = x1->c1 * x2->c1;
	*sum++ = x1->g1 * x2->g1;
	*sum++ = x1->t1 * x2->t1;

	sumc += x1->a2 * x2->a2;
	*sum++ = x1->c2 * x2->c2;
	*sum++ = x1->g2 * x2->g2;
	*sum++ = x1->t2 * x2->t2;

	sumc += x1->a3 * x2->a3;
	*sum++ = x1->c3 * x2->c3;
	*sum++ = x1->g3 * x2->g3;
	*sum++ = x1->t3 * x2->t3;

	*sum++ = sumc;

	/*x1++;
	  x2++;		 */
      }

   
   
    z = z0;
    do {
      int curvatOK = FALSE;
      double tmp_1, tmp_2, tmp_3;
      double t, inv_Li, dlnLidlz, d2lnLidlz2;

      zprev = z;
      
      zstep = (1.0 - zmax) * z + zmin;

      do {	
	double *EIGN    = &(tr->rdta->EIGN[0][0]);
	double lz;
        double *wrptr   = &(tr->cdta->wr[0]);
        double *wr2ptr  = &(tr->cdta->wr2[0]);	

        dlnLdlz = 0.0;                 /*  = d(ln(likelihood))/d(lz) */
        d2lnLdlz2 = 0.0;               /*  = d2(ln(likelihood))/d(lz)2 */


        if (z < zmin) z = zmin;
        else if (z > zmax) z = zmax;
        lz    = log(z);
        
	sum = sumtable;

	diagptable = diagptable_start;
	
	  for(i = 0; i < 4; i++)
	    {
	      ki = tr->gammaRates[i];	 
	      
	      *diagptable++ = exp (EIGN[1] * ki * lz);
	      *diagptable++ = exp (EIGN[2] * ki * lz);
	      *diagptable++ = exp (EIGN[3] * ki * lz);
	    }

	
#pragma omp parallel for private(inv_Li, t, tmp_1, tmp_2, tmp_3, dlnLidlz, d2lnLidlz2, diagptable, sum) reduction(+ :  dlnLdlz) reduction(+ : d2lnLdlz2)
        for (i = 0; i < tr->cdta->endsite; i++) 
	  {	   
	    diagptable = diagptable_start;

	    sum = &sumtable[13 * i];
	    	   
	    inv_Li =  (tmp_1 = *diagptable++ * *sum++);
	    inv_Li += (tmp_2 = *diagptable++ * *sum++);
	    inv_Li += (tmp_3 = *diagptable++ * *sum++);	    	   	    	  	   	   
	    
	    t = tmp_1 * EIGN[1];	   
	    dlnLidlz   = t;
	    d2lnLidlz2 = t * EIGN[1];
	    
	    t = tmp_2 * EIGN[2];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[2];
	    
	    t = tmp_3 * EIGN[3];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[3];
	    
	    dlnLidlz *= tr->gammaRates[0];
	    d2lnLidlz2 *= tr->gammaRates[0] * tr->gammaRates[0];

	    /*t = tmp_1 * EIGN[1] * tr->gammaRates[0];	   

	    dlnLidlz   = t;
	    d2lnLidlz2 = t * EIGN[1] * tr->gammaRates[0];
	    
	    t = tmp_2 * EIGN[2] * tr->gammaRates[0];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[2] * tr->gammaRates[0];
	    
	    t = tmp_3 * EIGN[3] * tr->gammaRates[0];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[3] * tr->gammaRates[0];*/
	   
	    /* rate 1 */
	    
	  
	    inv_Li += (tmp_1 = *sum++ *  *diagptable++);
	    inv_Li += (tmp_2 = *sum++ *  *diagptable++);
	    inv_Li += (tmp_3 = *sum++ *  *diagptable++);	    	   	   	    
 	  
	    t = tmp_1 * EIGN[1] * tr->gammaRates[1];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[1] * tr->gammaRates[1];
	    
	    t = tmp_2 * EIGN[2] * tr->gammaRates[1];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[2] * tr->gammaRates[1];
	    
	    t = tmp_3 * EIGN[3] * tr->gammaRates[1];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[3] * tr->gammaRates[1];
 

	    /* rates 2 */
	 
	    inv_Li += (tmp_1 = *sum++ *  *diagptable++);
	    inv_Li += (tmp_2 = *sum++ *  *diagptable++);
	    inv_Li += (tmp_3 = *sum++ *  *diagptable++);	   	  
 	  	   	    	  
	    t = tmp_1 * EIGN[1] * tr->gammaRates[2];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[1] * tr->gammaRates[2];
	    
	    t = tmp_2 * EIGN[2] * tr->gammaRates[2];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[2] * tr->gammaRates[2];
	    
	    t = tmp_3 * EIGN[3] * tr->gammaRates[2];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[3] * tr->gammaRates[2];
 
	    /* rates 3 */
	  	 
	    inv_Li += (tmp_1 = *sum++ * *diagptable++);
	    inv_Li += (tmp_2 = *sum++ * *diagptable++);
	    inv_Li += (tmp_3 = *sum++ * *diagptable++);	    
	    inv_Li += *sum++;
	   	  	   	    	  
	    t = tmp_1 * EIGN[1] * tr->gammaRates[3];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[1] * tr->gammaRates[3];
	    
	    t = tmp_2 * EIGN[2] * tr->gammaRates[3];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[2] * tr->gammaRates[3];
	    
	    t = tmp_3 * EIGN[3] * tr->gammaRates[3];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[3] * tr->gammaRates[3];
 
	    /* end */

	    inv_Li = 1.0/inv_Li;

	    dlnLidlz   *= inv_Li;
	    d2lnLidlz2 *= inv_Li;

	    dlnLdlz   += wrptr[i]  * dlnLidlz;
	    d2lnLdlz2 += wr2ptr[i] * (d2lnLidlz2 - dlnLidlz * dlnLidlz);
	  }
	
	

        if ((d2lnLdlz2 >= 0.0) && (z < zmax))
          zprev = z = 0.37 * z + 0.63;  /*  Bad curvature, shorten branch */
        else
          curvatOK = TRUE;
	
      } while (! curvatOK);

      if (d2lnLdlz2 < 0.0) {
	double tantmp = -dlnLdlz / d2lnLdlz2;  /* prevent overflow */
	if (tantmp < 100) {
	  z *= exp(tantmp);
	  if (z < zmin) z = zmin;
	  if (z > 0.25 * zprev + 0.75)    /*  Limit steps toward z = 1.0 */
	    z = 0.25 * zprev + 0.75;
	} else {
	  z = 0.25 * zprev + 0.75;
	}
      }
      if (z > zmax) z = zmax;

    } while ((--maxiter > 0) && (ABS(z - zprev) > zstep));

    free(sumtable);
    free(diagptable_start);
  
    return  z;
  } /* makenewz */





boolean newviewGTRCAT(tree    *tr, nodeptr  p)
{
  
  if (p->tip) 
    {      
      if (p->x) return TRUE;
      if (! getxtip(p)) return FALSE;  
      {
	int code, i, j, jj;
	double (*EIGV)[4] = &(tr->rdta->EIGV[0][0]);
	likelivector *x1 = p->x->lv;
#pragma omp parallel for private(code, j)     
	for(i = 0; i < tr->cdta->endsite; i++)
	  {	      	  	 
	    code = p->tip[i];
	    x1[i].a = 0;
	    x1[i].c = 0;
	    x1[i].g = 0;
	    x1[i].t = 0;
	    for (j = 0; j < 4; j++) 
	      {
		if ((code >> j) & 1) 
		  {
		    int jj = "0213"[j] - '0';		
		    x1[i].a += EIGV[jj][0];
		    x1[i].c += EIGV[jj][1];
		    x1[i].g += EIGV[jj][2];
		    x1[i].t += EIGV[jj][3];
		  }
	      }		
	      
	    x1[i].exp = 0;	  			  			 	 	 
	} 
      return TRUE; 
      }
    }
 
  { 	
    double  *diagptable_x1, *diagptable_x2, *diagptable_x1_start, *diagptable_x2_start;
    double *EIGN    = &(tr->rdta->EIGN[0][0]);
    double *invfreq = &(tr->rdta->invfreq[0][0]);
    double (*EIGV)[4] = &(tr->rdta->EIGV[0][0]);
    xtype tmp_x1_1, tmp_x1_2, tmp_x1_3,
      tmp_x2_1, tmp_x2_2, tmp_x2_3, 
      ump_x1_1, ump_x1_2, ump_x1_3, ump_x1_0, 
      ump_x2_0, ump_x2_1, ump_x2_2, ump_x2_3, x1px2;
    int cat;
    int  *cptr;	  
    int  i, j, k;
    likelivector   *x1, *x2, *x3;
    double   z1, lz1, z2, lz2, ki;
    nodeptr  q, r;   
    double *rptr;

    rptr   = &(tr->cdta->patrat[0]); 
    cptr  = &(tr->cdta->rateCategory[0]);

    q = p->next->back;
    r = p->next->next->back;
      
    while ((! p->x) || (! q->x) || (! r->x)) {
      if (! q->x) if (! newviewGTRCAT(tr, q)) return FALSE;
      if (! r->x) if (! newviewGTRCAT(tr, r)) return FALSE;
      if (! p->x) if (! getxnode(p)) return FALSE;	
    }

    x1  = q->x->lv;
    z1  = q->z;
    lz1 = (z1 > zmin) ? log(z1) : log(zmin);

    x2  = r->x->lv;
    z2  = r->z;
    lz2 = (z2 > zmin) ? log(z2) : log(zmin);

    x3  = p->x->lv;   
  

    diagptable_x1 = diagptable_x1_start = (double *)malloc(sizeof(double) * tr->NumberOfCategories * 3);
    diagptable_x2 = diagptable_x2_start = (double *)malloc(sizeof(double) * tr->NumberOfCategories * 3);
#pragma omp parallel for private(ki, j, k)
    for(i = 0; i < tr->NumberOfCategories;i++)
      {
	ki = rptr[i];
	
	j = k = 3 * i;
	
	diagptable_x1[j] = exp(EIGN[1] * lz1 * ki);
	diagptable_x1[++j] = exp(EIGN[2] * lz1 * ki);
	diagptable_x1[++j] = exp(EIGN[3] * lz1 * ki);
	
	diagptable_x2[k] = exp(EIGN[1] * lz2 * ki);	
	diagptable_x2[++k] = exp(EIGN[2] * lz2 * ki);
	diagptable_x2[++k] = exp(EIGN[3] * lz2 * ki);		
      }

    
#pragma omp parallel for private(cat, tmp_x1_1, tmp_x1_2,tmp_x1_3,ump_x1_0,ump_x1_1,ump_x1_2,ump_x1_3,ump_x2_0,ump_x2_1,ump_x2_2,ump_x2_3,tmp_x2_1,tmp_x2_2,tmp_x2_3,x1px2)    
    for (i = 0; i < tr->cdta->endsite; i++) 
      {	
	cat = cptr[i];

	diagptable_x1 = &diagptable_x1_start[cat * 3];
	diagptable_x2 = &diagptable_x2_start[cat * 3];

	tmp_x1_1 = x1[i].c * diagptable_x1[0];
	tmp_x1_2 = x1[i].g * diagptable_x1[1];
	tmp_x1_3 = x1[i].t * diagptable_x1[2];
	
	ump_x1_0 = tmp_x1_1 * EIGV[0][1];
	ump_x1_0 += tmp_x1_2 * EIGV[0][2];
	ump_x1_0 += tmp_x1_3 * EIGV[0][3];	      	
	ump_x1_0 *= invfreq[0];
	ump_x1_0 += x1[i].a;
	
	ump_x1_1 = tmp_x1_1 * EIGV[1][1];
	ump_x1_1 += tmp_x1_2 * EIGV[1][2];
	ump_x1_1 += tmp_x1_3 * EIGV[1][3];	      	
	ump_x1_1 *= invfreq[1];
	ump_x1_1 += x1[i].a;
	    
	ump_x1_2 = tmp_x1_1 * EIGV[2][1];
	ump_x1_2 += tmp_x1_2 * EIGV[2][2];
	ump_x1_2 += tmp_x1_3 * EIGV[2][3];	      	
	ump_x1_2 *= invfreq[2];
	ump_x1_2 += x1[i].a;

	ump_x1_3 = tmp_x1_1 * EIGV[3][1];
	ump_x1_3 += tmp_x1_2 * EIGV[3][2];
	ump_x1_3 += tmp_x1_3 * EIGV[3][3];	      	
	ump_x1_3 *= invfreq[3];
	ump_x1_3 += x1[i].a;
	
	tmp_x2_1 = x2[i].c * diagptable_x2[0];
	tmp_x2_2 = x2[i].g * diagptable_x2[1];
	tmp_x2_3 = x2[i].t * diagptable_x2[2];
	 
	 	    	    	     	     
	ump_x2_0 = tmp_x2_1 * EIGV[0][1];
	ump_x2_0 += tmp_x2_2 * EIGV[0][2];
	ump_x2_0 += tmp_x2_3 * EIGV[0][3];		     
	ump_x2_0 *= invfreq[0];
	ump_x2_0 += x2[i].a;
	  
	ump_x2_1 = tmp_x2_1 * EIGV[1][1];
	ump_x2_1 += tmp_x2_2 * EIGV[1][2];
	ump_x2_1 += tmp_x2_3 * EIGV[1][3];		     
	ump_x2_1 *= invfreq[1];
	ump_x2_1 += x2[i].a;	 

	ump_x2_2 = tmp_x2_1 * EIGV[2][1];
	ump_x2_2 += tmp_x2_2 * EIGV[2][2];
	ump_x2_2 += tmp_x2_3 * EIGV[2][3];		     
	ump_x2_2 *= invfreq[2];
	ump_x2_2 += x2[i].a;	  
		   

	ump_x2_3 = tmp_x2_1 * EIGV[3][1] ;
	ump_x2_3 += tmp_x2_2 * EIGV[3][2];
	ump_x2_3 += tmp_x2_3 * EIGV[3][3];		     
	ump_x2_3 *= invfreq[3];
	ump_x2_3 += x2[i].a;	    	  		   	  	
	
	x1px2 = ump_x1_0 * ump_x2_0;
	x3[i].a = x1px2 *  EIGV[0][0];
	x3[i].c = x1px2 *  EIGV[0][1];
	x3[i].g = x1px2 *  EIGV[0][2];
	x3[i].t = x1px2 *  EIGV[0][3]; 
	
	x1px2 = ump_x1_1 * ump_x2_1;
	x3[i].a += x1px2  *  EIGV[1][0];
	x3[i].c += x1px2 *   EIGV[1][1];
	x3[i].g += x1px2 *   EIGV[1][2];
	x3[i].t += x1px2 *   EIGV[1][3];
	
	x1px2 = ump_x1_2 * ump_x2_2;
	x3[i].a += x1px2 *  EIGV[2][0];
	x3[i].c += x1px2 *  EIGV[2][1];
	x3[i].g += x1px2 *  EIGV[2][2];
	x3[i].t += x1px2 *  EIGV[2][3];
	
	x1px2 = ump_x1_3 * ump_x2_3;
	x3[i].a += x1px2 *   EIGV[3][0];
	x3[i].c += x1px2 *   EIGV[3][1];
	x3[i].g += x1px2 *   EIGV[3][2];
	x3[i].t += x1px2 *   EIGV[3][3];
		   
	x3[i].exp = x1[i].exp + x2[i].exp;		  
	
	if (ABS(x3[i].a) < minlikelihood && ABS(x3[i].g) < minlikelihood
	    && ABS(x3[i].c) < minlikelihood && ABS(x3[i].t) < minlikelihood) 
	  {	     
	    x3[i].a   *= twotothe256;
	    x3[i].g   *= twotothe256;
	    x3[i].c   *= twotothe256;
	    x3[i].t   *= twotothe256;
	    x3[i].exp += 1;
	    }
      }     

   
    free(diagptable_x1_start); 
    free(diagptable_x2_start);

    return TRUE;
  }
}





double evaluateGTRCAT (tr, p)
    tree    *tr;
    nodeptr  p;
  { /* evaluate */
    double   sum, z, lz, term, ki;    
    nodeptr  q;
    int     i, j, cat;
    int     *wptr = tr->cdta->aliaswgt, *cptr;
    
    double  *diagptable, *rptr, *diagptable_start;
    double *EIGN = &(tr->rdta->EIGN[0][0]);
    likelivector   *x1, *x2;
       
    q = p->back;
    while ((! p->x) || (! q->x)) {
      if (! (p->x)) if (! newviewGTRCAT(tr, p)) return badEval;
      if (! (q->x)) if (! newviewGTRCAT(tr, q)) return badEval;
    }


    rptr   = &(tr->cdta->patrat[0]); 
    cptr  = &(tr->cdta->rateCategory[0]);

    x1  = p->x->lv;
    x2  = q->x->lv;
    z = p->z;
   
    if (z < zmin) z = zmin;
    lz = log(z);         

    diagptable = diagptable_start = (double *)malloc(sizeof(double) * tr->NumberOfCategories * 3);
#pragma omp parallel for private(ki, j)
    for(i = 0; i <  tr->NumberOfCategories; i++)
      {
	ki = rptr[i];	 
 
	j = 3 * i;

	diagptable[j] = exp(EIGN[1] * lz * ki);
	diagptable[++j] = exp(EIGN[2] * lz * ki);
	diagptable[++j] = exp(EIGN[3] * lz * ki);
      }
    
    sum = 0.0;
#pragma omp parallel for private(cat, term) reduction(+ : sum)    
    for (i = 0; i < tr->cdta->endsite; i++) 
      {	  	 	  
	cat = cptr[i];
	
	diagptable = &diagptable_start[3 * cat];

	term =  x1[i].a * x2[i].a;
	term += x1[i].c * x2[i].c * diagptable[0];
	term += x1[i].g * x2[i].g * diagptable[1];
	term += x1[i].t * x2[i].t * diagptable[2];     
	term = log(term) + (x1[i].exp + x2[i].exp)*log(minlikelihood);
	sum += wptr[i] * term;	
      }
    
   	
      
    free(diagptable_start); 
    
    tr->likelihood = sum;
    return  sum;
  } /* evaluate */



double makenewzGTRCAT (tr, p, q, z0, maxiter)
    tree    *tr;
    nodeptr  p, q;
    double   z0;
    int  maxiter;
  { /* makenewz */
    double   z, zprev, zstep;
    likelivector  *x1, *x2;
    double  *sum0, *sum1, *sum2, *sum3;
    int     i, *cptr, cat;
    double  dlnLdlz = 0.0;                 /*  = d(ln(likelihood))/d(lz) */
    double  d2lnLdlz2 = 0.0;               /*  = d2(ln(likelihood))/d(lz)2 */    
    double d0, *d1, *d2, *d3, ki, *rptr;
    double e0, e1, e2, e3;
    double *EIGN    = &(tr->rdta->EIGN[0][0]);   
    double *wrptr   = &(tr->cdta->wr[0]);
    double *wr2ptr  = &(tr->cdta->wr2[0]);

    while ((! p->x) || (! q->x)) {
      if (! (p->x)) if (! newviewGTRCAT(tr, p)) return badZ;
      if (! (q->x)) if (! newviewGTRCAT(tr, q)) return badZ;
    }

    x1 = p->x->lv;
    x2 = q->x->lv;

   sum0  = (double *)malloc(tr->cdta->endsite * sizeof(double));
   sum1  = (double *)malloc(tr->cdta->endsite * sizeof(double));  
   sum2  = (double *)malloc(tr->cdta->endsite * sizeof(double)); 
   sum3  = (double *)malloc(tr->cdta->endsite * sizeof(double));
    d1 = (double *)malloc(tr->NumberOfCategories * sizeof(double));
    d2 = (double *)malloc(tr->NumberOfCategories * sizeof(double));
    d3 = (double *)malloc(tr->NumberOfCategories * sizeof(double));

#pragma omp parallel for
   for (i = 0; i < tr->cdta->endsite; i++) 
     {     
       sum0[i] = x1[i].a * x2[i].a;
       sum1[i] = x1[i].c * x2[i].c;
       sum2[i] = x1[i].g * x2[i].g;
       sum3[i] = x1[i].t * x2[i].t;		     
     }

   
   rptr   = &(tr->cdta->patrat[0]); 
   cptr  = &(tr->cdta->rateCategory[0]);

    z = z0;
    do {
      int curvatOK = FALSE;

      zprev = z;
      
      zstep = (1.0 - zmax) * z + zmin;

      do {	
	double tmp_0, tmp_1, tmp_2, tmp_3;
	double t, inv_Li, dlnLidlz, d2lnLidlz2; 
	double lz;
        dlnLdlz = 0.0;                 /*  = d(ln(likelihood))/d(lz) */
        d2lnLdlz2 = 0.0;               /*  = d2(ln(likelihood))/d(lz)2 */

        if (z < zmin) z = zmin;
        else if (z > zmax) z = zmax;
        lz    = log(z);
	

#pragma omp parallel for private(ki)	
	for(i = 0; i < tr->NumberOfCategories; i++)
	  {
	    ki = rptr[i];
	    d1[i] = exp (EIGN[1] * lz * ki); 
	    d2[i] = exp (EIGN[2] * lz * ki); 
	    d3[i] = exp (EIGN[3] * lz * ki); 
	  }




#pragma omp parallel for private(cat, inv_Li, t, tmp_0, tmp_1, tmp_2, tmp_3, dlnLidlz, d2lnLidlz2) reduction(+ :  dlnLdlz) reduction(+ : d2lnLdlz2)
        for (i = 0; i < tr->cdta->endsite; i++) 
	  {
	    cat = cptr[i];
	    	 
	    inv_Li = (tmp_0 = sum0[i]);
	    inv_Li += (tmp_1 = sum1[i] * d1[cat]);
	    inv_Li += (tmp_2 = sum2[i] * d2[cat]);
	    inv_Li += (tmp_3 = sum3[i] * d3[cat]);

	    inv_Li = 1.0/inv_Li;
	    
	  
	    t = tmp_1 * EIGN[1];
	    dlnLidlz   = t;
	    d2lnLidlz2 = t * EIGN[1];
	    
	    t = tmp_2 * EIGN[2];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[2];
	    
	    t = tmp_3 * EIGN[3];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[3];
	    
	    dlnLidlz   *= inv_Li;
	    d2lnLidlz2 *= inv_Li;
	    dlnLdlz   += wrptr[i]  * dlnLidlz;
	    d2lnLdlz2 += wr2ptr[i] * (d2lnLidlz2 - dlnLidlz * dlnLidlz);
	  }
	
        if ((d2lnLdlz2 >= 0.0) && (z < zmax))
          zprev = z = 0.37 * z + 0.63;  /*  Bad curvature, shorten branch */
        else
          curvatOK = TRUE;
	
      } while (! curvatOK);

      if (d2lnLdlz2 < 0.0) {
	double tantmp = -dlnLdlz / d2lnLdlz2;  /* prevent overflow */
	if (tantmp < 100) {
	  z *= exp(tantmp);
	  if (z < zmin) z = zmin;
	  if (z > 0.25 * zprev + 0.75)    /*  Limit steps toward z = 1.0 */
	    z = 0.25 * zprev + 0.75;
	} else {
	  z = 0.25 * zprev + 0.75;
	}
      }
      if (z > zmax) z = zmax;

    } while ((--maxiter > 0) && (ABS(z - zprev) > zstep));

    free (sum0);
    free (sum1);
    free (sum2);
    free (sum3);
    free(d1);
    free(d2);
    free(d3);
    return  z;
  } /* makenewz */


boolean newviewHKY85 (tree *tr, nodeptr p)      
  {
    double   zq, lzq, xvlzq, zr, lzr, xvlzr;
    nodeptr  q, r;
    likelivector *lp, *lq, *lr;
    int  i;
   
    if (p->tip) 
      {         
	likelivector *l;
	int           code;
	yType        *yptr;

	if (p->x) return TRUE;

	if (! getxtip(p)) return FALSE;
	l = p->x->lv;           
	yptr = p->tip;   

#pragma omp parallel for private(code)
	for (i = 0; i < tr->cdta->endsite; i++) 
	  {
	   
	    code = yptr[i];
	    l[i].a =  code       & 1;
	    l[i].c = (code >> 1) & 1;
	    l[i].g = (code >> 2) & 1;
	    l[i].t = (code >> 3) & 1;
	    l[i].exp = 0;	
	  }
	return TRUE;
      }

    q = p->next->back;
    r = p->next->next->back;

    while ((! p->x) || (! q->x) || (! r->x)) {
      if (! q->x) if (! newviewHKY85(tr, q))  return FALSE;
      if (! r->x) if (! newviewHKY85(tr, r))  return FALSE;
      if (! p->x) if (! getxnode(p)) return FALSE;
      }

    lp = p->x->lv;

    lq = q->x->lv;
    zq = q->z;
    lzq = (zq > zmin) ? log(zq) : log(zmin);
    xvlzq = tr->rdta->xv * lzq;

    lr = r->x->lv;
    zr = r->z;
    lzr = (zr > zmin) ? log(zr) : log(zmin);
    xvlzr = tr->rdta->xv * lzr;

    { 
      double  fxqr, fxqy, fxqn, sumaq, sumgq, sumcq, sumtq,
	fxrr, fxry, fxrn, tempi, tempj,
	zzq, zvq, zzr, zvr;
                           
      zzq = exp(lzq);
      zvq = exp(xvlzq);
      zzr = exp(lzr);
      zvr = exp(xvlzr);
#pragma omp parallel for private(fxqr, fxqy, fxqn, tempi, tempj, sumaq, sumgq, sumcq, sumtq, fxrr, fxry, fxrn)
      for (i = 0; i < tr->cdta->endsite; i++) 
	{         	  
          fxqr = tr->rdta->freqa * lq[i].a + tr->rdta->freqg * lq[i].g;
          fxqy = tr->rdta->freqc * lq[i].c + tr->rdta->freqt * lq[i].t;
          fxqn = fxqr + fxqy;
          tempi = fxqr * tr->rdta->invfreqr;
          tempj = zvq * (tempi-fxqn) + fxqn;
          sumaq = zzq * (lq[i].a - tempi) + tempj;
          sumgq = zzq * (lq[i].g - tempi) + tempj;
          tempi = fxqy * tr->rdta->invfreqy;
          tempj = zvq * (tempi-fxqn) + fxqn;
          sumcq = zzq * (lq[i].c - tempi) + tempj;
          sumtq = zzq * (lq[i].t - tempi) + tempj;

         
          fxrr = tr->rdta->freqa * lr[i].a + tr->rdta->freqg * lr[i].g;
          fxry = tr->rdta->freqc * lr[i].c + tr->rdta->freqt * lr[i].t;
          fxrn = fxrr + fxry;
          tempi = fxrr * tr->rdta->invfreqr;
          tempj = zvr * (tempi-fxrn) + fxrn;
          lp[i].a = sumaq * (zzr * (lr[i].a - tempi) + tempj);
          lp[i].g = sumgq * (zzr * (lr[i].g - tempi) + tempj);
          tempi = fxry * tr->rdta->invfreqy;
          tempj = zvr * (tempi-fxrn) + fxrn;
          lp[i].c = sumcq * (zzr * (lr[i].c - tempi) + tempj);
          lp[i].t = sumtq * (zzr * (lr[i].t - tempi) + tempj);         	 	  
	  lp[i].exp = lq[i].exp + lr[i].exp;

	  if (lp[i].a < minlikelihood && lp[i].g < minlikelihood
	      && lp[i].c < minlikelihood && lp[i].t < minlikelihood) 
	    {	      	     
	      lp[i].a   *= twotothe256;
	      lp[i].g   *= twotothe256;
	      lp[i].c   *= twotothe256;
	      lp[i].t   *= twotothe256;
	      lp[i].exp += 1;
	      
	    }
	}

      return TRUE;
    }
  }




double evaluateHKY85 (tree *tr, nodeptr p)
  { /* evaluate */
    double   sum, z, lz, xvlz,
             fxpa, fxpc, fxpg, fxpt, fxpr, fxpy, fxqr, fxqy,
             suma, sumb, sumc, term, zz, zv;   
    likelivector *lp, *lq;
    nodeptr       q;
    int           i, *wptr;

    q = p->back;

    while ((! p->x) || (! q->x)) 
      {
	if (! (p->x)) if (! newviewHKY85(tr, p)) return badEval;
	if (! (q->x)) if (! newviewHKY85(tr, q)) return badEval;
      }

    lp = p->x->lv;
    lq = q->x->lv;

    z = p->z;
    if (z < zmin) z = zmin;
    lz = log(z);
    xvlz = tr->rdta->xv * lz;
   
    zz = exp(lz);
    zv = exp(xvlz);
      
    wptr = &(tr->cdta->aliaswgt[0]);
       
    sum = 0.0;
#pragma omp parallel for private (fxpa, fxpg, fxpc, fxpt, fxqr, fxqy, fxpr, fxpy, sumc, sumb, suma, term)  reduction(+ : sum)
    for (i = 0; i < tr->cdta->endsite; i++) 
      {       
        fxpa = tr->rdta->freqa * lp[i].a;
        fxpg = tr->rdta->freqg * lp[i].g;
        fxpc = tr->rdta->freqc * lp[i].c;
        fxpt = tr->rdta->freqt * lp[i].t;
	fxqr = tr->rdta->freqa * lq[i].a + tr->rdta->freqg * lq[i].g;
        fxqy = tr->rdta->freqc * lq[i].c + tr->rdta->freqt * lq[i].t;
	fxpr = fxpa + fxpg;
        fxpy = fxpc + fxpt;
        suma = fxpa * lq[i].a + fxpc * lq[i].c + 
	  fxpg * lq[i].g + fxpt * lq[i].t;               
        sumc = (fxpr + fxpy) * (fxqr + fxqy);
        sumb = fxpr * fxqr * tr->rdta->invfreqr + fxpy * fxqy * tr->rdta->invfreqy;

        suma -= sumb;
        sumb -= sumc;

	term = log(zz * suma + zv * sumb + sumc) + (lp[i].exp + lq[i].exp)*log(minlikelihood);

        sum += wptr[i] * term;        

      }
    
  
    tr->likelihood = sum;
    
    return  sum;
  }



double makenewzHKY85 (tree *tr, nodeptr p, nodeptr q, double z0, int maxiter)
  { /* makenewz */
    likelivector *lp, *lq;
    double  *abi, *bci, *sumci, dlnLidlz, dlnLdlz, d2lnLdlz2, 
      z, zprev, zstep, lz, xvlz,
      ki, suma, sumb, sumc, ab, bc, inv_Li, t1, t2,
      fx1a, fx1c, fx1g, fx1t, fx1r, fx1y, fx2r, fx2y, 
      zz, zv, *wrptr, *wr2ptr;
    int      i, curvatOK, scratch_size;


    while ((! p->x) || (! q->x)) 
      {
	if (! (p->x)) if (! newviewHKY85(tr, p)) return badZ;
	if (! (q->x)) if (! newviewHKY85(tr, q)) return badZ;
      }

    lp = p->x->lv;
    lq = q->x->lv;
    scratch_size = sizeof(double) * tr->cdta->endsite;
       
    abi   = (double *) malloc(scratch_size);
    bci   = (double *) malloc(scratch_size);
    sumci = (double *) malloc(scratch_size);      
    
#pragma omp parallel for private (fx1a, fx1g, fx1c, fx1t, suma, fx2r, fx2y, fx1r, fx1y, sumc, sumb)
    for (i = 0; i < tr->cdta->endsite; i++) 
      {
	fx1a = tr->rdta->freqa * lp[i].a;
	fx1g = tr->rdta->freqg * lp[i].g;
	fx1c = tr->rdta->freqc * lp[i].c;
	fx1t = tr->rdta->freqt * lp[i].t;
	suma = fx1a * lq[i].a + fx1c * lq[i].c + fx1g * lq[i].g + fx1t * lq[i].t;
	fx2r = tr->rdta->freqa * lq[i].a + tr->rdta->freqg * lq[i].g;
	fx2y = tr->rdta->freqc * lq[i].c + tr->rdta->freqt * lq[i].t;
	fx1r = fx1a + fx1g;
	fx1y = fx1c + fx1t;
	sumci[i] = sumc = (fx1r + fx1y) * (fx2r + fx2y);
	sumb       = fx1r * fx2r * tr->rdta->invfreqr + fx1y * fx2y * tr->rdta->invfreqy;
	abi[i]   = suma - sumb;
	bci[i]   = sumb - sumc;
      }

    wrptr   = &(tr->cdta->wr[0]);
    wr2ptr  = &(tr->cdta->wr2[0]);
    z = z0;

    do 
      {
	zprev = z;
	zstep = (1.0 - zmax) * z + zmin;
	curvatOK = FALSE;

	do 
	  {
	    if (z < zmin) z = zmin;
	    else if (z > zmax) z = zmax;

	    lz    = log(z);
	    xvlz  = tr->rdta->xv * lz;         
        
	    zz = exp(lz);
	    zv = exp(xvlz);
               
	    dlnLdlz = 0.0;                 
	    d2lnLdlz2 = 0.0;              
#pragma omp parallel for private(ab, bc, sumc, inv_Li, t1, t2, 	dlnLidlz) reduction(+ : dlnLdlz) reduction( + : d2lnLdlz2) 
	    for (i = 0; i < tr->cdta->endsite; i++) 
	      {         
		ab     = abi[i] * zz;
		bc     = bci[i] * zv;
		sumc   = sumci[i];
		inv_Li = 1.0/(ab + bc + sumc);
		t1     = ab * inv_Li;
		t2     = tr->rdta->xv * bc * inv_Li;
		dlnLidlz   = t1 + t2;
		dlnLdlz   += wrptr[i]  * dlnLidlz;
		d2lnLdlz2 += wr2ptr[i] * (t1 + tr->rdta->xv * t2 - dlnLidlz * dlnLidlz);
	      }

	    if ((d2lnLdlz2 >= 0.0) && (z < zmax))
	      zprev = z = 0.37 * z + 0.63;
	    else
	      curvatOK = TRUE;

	  } 
	while (! curvatOK);

	if (d2lnLdlz2 < 0.0) 
	  {
	    z *= exp(-dlnLdlz / d2lnLdlz2);
	    if (z < zmin) z = zmin;
	    if (z > 0.25 * zprev + 0.75)
	      z = 0.25 * zprev + 0.75;
        }

	if (z > zmax) z = zmax;

      } 
    while ((--maxiter > 0) && (ABS(z - zprev) > zstep));

    free(abi);
    free(bci);
    free(sumci);
    


    return  z;
  } 


boolean newviewGTR(tree    *tr, nodeptr  p)
{
  
  if (p->tip) 
    {      
      if (p->x) return TRUE;
      if (! getxtip(p)) return FALSE;  
      {
	int code, i, j, jj;
	double (*EIGV)[4] = &(tr->rdta->EIGV[0][0]);
	likelivector *x1 = p->x->lv;
#pragma omp parallel for private(code, j)
	for(i = 0; i < tr->cdta->endsite; i++)
	  {	      	  	 
	    code = p->tip[i];
	    x1[i].a = 0;
	    x1[i].c = 0;
	    x1[i].g = 0;
	    x1[i].t = 0;
	    for (j = 0; j < 4; j++) 
	      {
		if ((code >> j) & 1) 
		  {
		    int jj = "0213"[j] - '0';		
		    x1[i].a += EIGV[jj][0];
		    x1[i].c += EIGV[jj][1];
		    x1[i].g += EIGV[jj][2];
		    x1[i].t += EIGV[jj][3];
		  }
	      }		
	      
	    x1[i].exp = 0;	  			  			 	 
	} 
      return TRUE; 
      }
    }
  else
    { 	
      double  diagptable_x1[4], diagptable_x2[4];
      double *EIGN    = &(tr->rdta->EIGN[0][0]);
      double *invfreq = &(tr->rdta->invfreq[0][0]);
      double (*EIGV)[4] = &(tr->rdta->EIGV[0][0]);
      xtype tmp_x1_1, tmp_x1_2, tmp_x1_3,
      tmp_x2_1, tmp_x2_2, tmp_x2_3, 
      ump_x1_1, ump_x1_2, ump_x1_3, ump_x1_0, 
      ump_x2_0, ump_x2_1, ump_x2_2, ump_x2_3, x1px2;
    int cat;
    int  *cptr;	  
    int  i, j;
    likelivector   *x1, *x2, *x3;
    double   z1, lz1, z2, lz2, ki;
    nodeptr  q, r;   
    double *eigv, *rptr;

    q = p->next->back;
    r = p->next->next->back;
      
    while ((! p->x) || (! q->x) || (! r->x)) {
      if (! q->x) if (! newviewGTR(tr, q)) return FALSE;
      if (! r->x) if (! newviewGTR(tr, r)) return FALSE;
      if (! p->x) if (! getxnode(p)) return FALSE;	
    }

    x1  = q->x->lv;
    z1  = q->z;
    lz1 = (z1 > zmin) ? log(z1) : log(zmin);

    x2  = r->x->lv;
    z2  = r->z;
    lz2 = (z2 > zmin) ? log(z2) : log(zmin);

    x3  = p->x->lv;   
   
    for (i = 1; i < 4; i++) 
      {
	diagptable_x1[i] = exp (EIGN[i] * lz1);
	diagptable_x2[i] = exp (EIGN[i] * lz2);	    
      }	       
      
  
#pragma omp parallel for private(tmp_x1_1, tmp_x1_2,tmp_x1_3,ump_x1_0,ump_x1_1,ump_x1_2,ump_x1_3,ump_x2_0,ump_x2_1,ump_x2_2,ump_x2_3,tmp_x2_1,tmp_x2_2,tmp_x2_3,x1px2)
    for (i = 0; i < tr->cdta->endsite; i++) 
      {	

	tmp_x1_1 = x1[i].c * diagptable_x1[1];
	tmp_x1_2 = x1[i].g * diagptable_x1[2];
	tmp_x1_3 = x1[i].t * diagptable_x1[3];
	
	ump_x1_0 = tmp_x1_1 * EIGV[0][1];
	ump_x1_0 += tmp_x1_2 * EIGV[0][2];
	ump_x1_0 += tmp_x1_3 * EIGV[0][3];	      	
	ump_x1_0 *= invfreq[0];
	ump_x1_0 += x1[i].a;
	
	ump_x1_1 = tmp_x1_1 * EIGV[1][1];
	ump_x1_1 += tmp_x1_2 * EIGV[1][2];
	ump_x1_1 += tmp_x1_3 * EIGV[1][3];	      	
	ump_x1_1 *= invfreq[1];
	ump_x1_1 += x1[i].a;
	    
	ump_x1_2 = tmp_x1_1 * EIGV[2][1];
	ump_x1_2 += tmp_x1_2 * EIGV[2][2];
	ump_x1_2 += tmp_x1_3 * EIGV[2][3];	      	
	ump_x1_2 *= invfreq[2];
	ump_x1_2 += x1[i].a;

	ump_x1_3 = tmp_x1_1 * EIGV[3][1];
	ump_x1_3 += tmp_x1_2 * EIGV[3][2];
	ump_x1_3 += tmp_x1_3 * EIGV[3][3];	      	
	ump_x1_3 *= invfreq[3];
	ump_x1_3 += x1[i].a;
	
	tmp_x2_1 = x2[i].c * diagptable_x2[1];
	tmp_x2_2 = x2[i].g * diagptable_x2[2];
	tmp_x2_3 = x2[i].t * diagptable_x2[3];
	 
	 	    	    	     	     
	ump_x2_0 = tmp_x2_1 * EIGV[0][1];
	ump_x2_0 += tmp_x2_2 * EIGV[0][2];
	ump_x2_0 += tmp_x2_3 * EIGV[0][3];		     
	ump_x2_0 *= invfreq[0];
	ump_x2_0 += x2[i].a;
	  
	ump_x2_1 = tmp_x2_1 * EIGV[1][1];
	ump_x2_1 += tmp_x2_2 * EIGV[1][2];
	ump_x2_1 += tmp_x2_3 * EIGV[1][3];		     
	ump_x2_1 *= invfreq[1];
	ump_x2_1 += x2[i].a;	 

	ump_x2_2 = tmp_x2_1 * EIGV[2][1];
	ump_x2_2 += tmp_x2_2 * EIGV[2][2];
	ump_x2_2 += tmp_x2_3 * EIGV[2][3];		     
	ump_x2_2 *= invfreq[2];
	ump_x2_2 += x2[i].a;	  
		   

	ump_x2_3 = tmp_x2_1 * EIGV[3][1] ;
	ump_x2_3 += tmp_x2_2 * EIGV[3][2];
	ump_x2_3 += tmp_x2_3 * EIGV[3][3];		     
	ump_x2_3 *= invfreq[3];
	ump_x2_3 += x2[i].a;	    	  		   	  	
	
	x1px2 = ump_x1_0 * ump_x2_0;
	x3[i].a = x1px2 *  EIGV[0][0];
	x3[i].c = x1px2 *  EIGV[0][1];
	x3[i].g = x1px2 *  EIGV[0][2];
	x3[i].t = x1px2 *  EIGV[0][3]; 
	
	x1px2 = ump_x1_1 * ump_x2_1;
	x3[i].a += x1px2  *  EIGV[1][0];
	x3[i].c += x1px2 *   EIGV[1][1];
	x3[i].g += x1px2 *   EIGV[1][2];
	x3[i].t += x1px2 *   EIGV[1][3];
	
	x1px2 = ump_x1_2 * ump_x2_2;
	x3[i].a += x1px2 *  EIGV[2][0];
	x3[i].c += x1px2 *  EIGV[2][1];
	x3[i].g += x1px2 *  EIGV[2][2];
	x3[i].t += x1px2 *  EIGV[2][3];
	
	x1px2 = ump_x1_3 * ump_x2_3;
	x3[i].a += x1px2 *   EIGV[3][0];
	x3[i].c += x1px2 *   EIGV[3][1];
	x3[i].g += x1px2 *   EIGV[3][2];
	x3[i].t += x1px2 *   EIGV[3][3];
		   
	x3[i].exp = x1[i].exp + x2[i].exp;		  
	
	if (ABS(x3[i].a) < minlikelihood && ABS(x3[i].g) < minlikelihood
	    && ABS(x3[i].c) < minlikelihood && ABS(x3[i].t) < minlikelihood) 
	  {	     
	    x3[i].a   *= twotothe256;
	    x3[i].g   *= twotothe256;
	    x3[i].c   *= twotothe256;
	    x3[i].t   *= twotothe256;
	    x3[i].exp += 1;
	    }
      }   
    
   return TRUE; 
    }
}





double evaluateGTR (tr, p)
    tree    *tr;
    nodeptr  p;
  { /* evaluate */
    double   sum, z, lz, term;    
    nodeptr  q;
    int     i, j;
    int     *wptr = tr->cdta->aliaswgt;
    
    double  diagptable[4];
    double *EIGN = &(tr->rdta->EIGN[0][0]);
    likelivector   *x1, *x2;
       
    q = p->back;
    while ((! p->x) || (! q->x)) {
      if (! (p->x)) if (! newviewGTR(tr, p)) return badEval;
      if (! (q->x)) if (! newviewGTR(tr, q)) return badEval;
    }


    x1  = p->x->lv;
    x2  = q->x->lv;
    z = p->z;
   
    if (z < zmin) z = zmin;
    lz = log(z);

    diagptable[0] = 1;
    diagptable[1] = exp (EIGN[1] * lz);
    diagptable[2] = exp (EIGN[2] * lz);
    diagptable[3] = exp (EIGN[3] * lz);
    
    
    
    sum = 0.0;
#pragma omp parallel for private(term) reduction(+ : sum)  
    for (i = 0; i < tr->cdta->endsite; i++) 
      {	  	 	  
	term =  x1[i].a * x2[i].a * diagptable[0];
	term += x1[i].c * x2[i].c * diagptable[1];
	term += x1[i].g * x2[i].g * diagptable[2];
	term += x1[i].t * x2[i].t * diagptable[3];     
	term = log(term) + (x1[i].exp + x2[i].exp)*log(minlikelihood);
	sum += wptr[i] * term;		  
      }
    
    tr->likelihood = sum;
    return  sum;
  } /* evaluate */



double makenewzGTR (tr, p, q, z0, maxiter)
    tree    *tr;
    nodeptr  p, q;
    double   z0;
    int  maxiter;
  { /* makenewz */
    double   z, zprev, zstep;
    likelivector  *x1, *x2;
    double  *sum0, *sum1, *sum2, *sum3;
    int     i;
    double  dlnLdlz = 0.0;                 /*  = d(ln(likelihood))/d(lz) */
    double  d2lnLdlz2 = 0.0;               /*  = d2(ln(likelihood))/d(lz)2 */    
    double d0, d1, d2, d3;
    double e0, e1, e2, e3;

    while ((! p->x) || (! q->x)) {
      if (! (p->x)) if (! newviewGTR(tr, p)) return badZ;
      if (! (q->x)) if (! newviewGTR(tr, q)) return badZ;
    }

    x1 = p->x->lv;
    x2 = q->x->lv;

   sum0  = (double *)malloc(tr->cdta->endsite * sizeof(double));
   sum1  = (double *)malloc(tr->cdta->endsite * sizeof(double));  
   sum2  = (double *)malloc(tr->cdta->endsite * sizeof(double)); 
   sum3  = (double *)malloc(tr->cdta->endsite * sizeof(double));

#pragma omp parallel for
   for (i = 0; i < tr->cdta->endsite; i++) 
     {     
       sum0[i] = x1[i].a * x2[i].a;
       sum1[i] = x1[i].c * x2[i].c;
       sum2[i] = x1[i].g * x2[i].g;
       sum3[i] = x1[i].t * x2[i].t;		     
     }

   
   

    z = z0;
    do {
      int curvatOK = FALSE;

      zprev = z;
      
      zstep = (1.0 - zmax) * z + zmin;

      do {
	double  diagptable[4];
	double *EIGN    = &(tr->rdta->EIGN[0][0]);
	double lz;
        double *wrptr   = &(tr->cdta->wr[0]);
        double *wr2ptr  = &(tr->cdta->wr2[0]);
	double tmp_0, tmp_1, tmp_2, tmp_3;
	double t, inv_Li, dlnLidlz, d2lnLidlz2;
        dlnLdlz = 0.0;                 /*  = d(ln(likelihood))/d(lz) */
        d2lnLdlz2 = 0.0;               /*  = d2(ln(likelihood))/d(lz)2 */

        if (z < zmin) z = zmin;
        else if (z > zmax) z = zmax;
        lz    = log(z);
        
	d1 = exp (EIGN[1] * lz); 
	d2 = exp (EIGN[2] * lz); 
	d3 = exp (EIGN[3] * lz); 

#pragma omp parallel for private(inv_Li, t, tmp_0, tmp_1, tmp_2, tmp_3, dlnLidlz, d2lnLidlz2) reduction(+ :  dlnLdlz) reduction(+ : d2lnLdlz2)
        for (i = 0; i < tr->cdta->endsite; i++) 
	  {
	   
	    	 
	    inv_Li = (tmp_0 = sum0[i]);
	    inv_Li += (tmp_1 = sum1[i] * d1);
	    inv_Li += (tmp_2 = sum2[i] * d2);
	    inv_Li += (tmp_3 = sum3[i] * d3);

	    inv_Li = 1.0/inv_Li;
	    
	  
	    t = tmp_1 * EIGN[1];
	    dlnLidlz   = t;
	    d2lnLidlz2 = t * EIGN[1];
	    
	    t = tmp_2 * EIGN[2];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[2];
	    
	    t = tmp_3 * EIGN[3];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[3];
	    
	    dlnLidlz   *= inv_Li;
	    d2lnLidlz2 *= inv_Li;
	    dlnLdlz   += wrptr[i]  * dlnLidlz;
	    d2lnLdlz2 += wr2ptr[i] * (d2lnLidlz2 - dlnLidlz * dlnLidlz);
	  }
	
        if ((d2lnLdlz2 >= 0.0) && (z < zmax))
          zprev = z = 0.37 * z + 0.63;  /*  Bad curvature, shorten branch */
        else
          curvatOK = TRUE;
	
      } while (! curvatOK);

      if (d2lnLdlz2 < 0.0) {
	double tantmp = -dlnLdlz / d2lnLdlz2;  /* prevent overflow */
	if (tantmp < 100) {
	  z *= exp(tantmp);
	  if (z < zmin) z = zmin;
	  if (z > 0.25 * zprev + 0.75)    /*  Limit steps toward z = 1.0 */
	    z = 0.25 * zprev + 0.75;
	} else {
	  z = 0.25 * zprev + 0.75;
	}
      }
      if (z > zmax) z = zmax;

    } while ((--maxiter > 0) && (ABS(z - zprev) > zstep));

    free (sum0);
    free (sum1);
    free (sum2);
    free (sum3);

    return  z;
  } /* makenewz */



boolean newviewHKY85CAT (tree *tr, nodeptr p)        /*  Update likelihoods at node */
  { /* newview */
    double   zq, lzq, xvlzq, zr, lzr, xvlzr;
    nodeptr  q,r;
    likelivector *lp, *lq, *lr;
    int  i;


    if (p->tip) {             /*  Make sure that data are at tip */
      likelivector *l;
      int           code;
      yType        *yptr;

      if (p->x) return TRUE;  /*  They are already there */

      if (! getxtip(p)) return FALSE; /*  They are not, so get memory */
      l = p->x->lv;           /*  Pointer to first likelihood vector value */
      yptr = p->tip;          /*  Pointer to first nucleotide datum */
      

      

      

#pragma omp parallel for private(code)
      for (i = 0; i < tr->cdta->endsite; i++) {
        code = yptr[i];
        l[i].a =  code       & 1;
        l[i].c = (code >> 1) & 1;
        l[i].g = (code >> 2) & 1;
        l[i].t = (code >> 3) & 1;
        l[i].exp = 0;

        }
      
        return TRUE;
    }
   
/*  Internal node needs update */

    q = p->next->back;
    r = p->next->next->back;

    

    while ((! p->x) || (! q->x) || (! r->x)) {
      if (! q->x) if (! newviewHKY85CAT(tr, q))  return FALSE;
      if (! r->x) if (! newviewHKY85CAT(tr, r))  return FALSE;
      if (! p->x) if (! getxnode(p)) return FALSE;
      }

    lp = p->x->lv;

    lq = q->x->lv;
    zq = q->z;
    lzq = (zq > zmin) ? log(zq) : log(zmin);
    xvlzq = tr->rdta->xv * lzq;

    lr = r->x->lv;
    zr = r->z;
    lzr = (zr > zmin) ? log(zr) : log(zmin);
    xvlzr = tr->rdta->xv * lzr;

    

    { double  *zzqtable, *zvqtable,
              *zzrtable, *zvrtable,
             *zzqptr, *zvqptr, *zzrptr, *zvrptr, *rptr;
      double  fxqr, fxqy, fxqn, sumaq, sumgq, sumcq, sumtq,
              fxrr, fxry, fxrn, ki, tempi, tempj;
      int  *cptr;
      double zzq, zvq, zzr, zvr;
      int cat;

      zzqtable = (double *)malloc(tr->NumberOfCategories * sizeof(double));
      zvqtable = (double *)malloc(tr->NumberOfCategories * sizeof(double));
      zzrtable = (double *)malloc(tr->NumberOfCategories * sizeof(double));
      zvrtable = (double *)malloc(tr->NumberOfCategories * sizeof(double));
    
      rptr   = &(tr->cdta->patrat[0]);
      zzqptr = &(zzqtable[0]);
      zvqptr = &(zvqtable[0]);
      zzrptr = &(zzrtable[0]);
      zvrptr = &(zvrtable[0]);
      cptr  = &(tr->cdta->rateCategory[0]);

      

     
 	
#pragma omp parallel for private(ki)
      for (i = 0; i < tr->NumberOfCategories; i++) 
	{   

	  ki = tr->cdta->patrat[i];	
	
	  zzqtable[i] = exp(ki *   lzq);
	  zvqtable[i] = exp(ki * xvlzq);
	  zzrtable[i] = exp(ki *   lzr);
	  zvrtable[i] = exp(ki * xvlzr);
	  
	}
     
#pragma omp parallel for private(cat,zzq,zvq,fxqr,fxqy,fxqn,tempi,tempj,sumaq,sumgq,sumcq,sumtq,zzr,zvr,fxrr,fxry,fxrn)
      for (i = 0; i < tr->cdta->endsite; i++) 
	{
	  cat = tr->cdta->rateCategory[i];
	  
	  zzq = zzqtable[cat];
	  zvq = zvqtable[cat];
	  
	  fxqr = tr->rdta->freqa * lq[i].a + tr->rdta->freqg * lq[i].g;
	  fxqy = tr->rdta->freqc * lq[i].c + tr->rdta->freqt * lq[i].t;
	  fxqn = fxqr + fxqy;
	  tempi = fxqr * tr->rdta->invfreqr;
	  tempj = zvq * (tempi-fxqn) + fxqn;
	  sumaq = zzq * (lq[i].a - tempi) + tempj;
	  sumgq = zzq * (lq[i].g - tempi) + tempj;
	  tempi = fxqy * tr->rdta->invfreqy;
	  tempj = zvq * (tempi-fxqn) + fxqn;
	  sumcq = zzq * (lq[i].c - tempi) + tempj;
	  sumtq = zzq * (lq[i].t - tempi) + tempj;
	  
	  zzr = zzrtable[cat];
	  zvr = zvrtable[cat];
	  fxrr = tr->rdta->freqa * lr[i].a + tr->rdta->freqg * lr[i].g;
	  fxry = tr->rdta->freqc * lr[i].c + tr->rdta->freqt * lr[i].t;
	  fxrn = fxrr + fxry;
	  tempi = fxrr * tr->rdta->invfreqr;
	  tempj = zvr * (tempi-fxrn) + fxrn;
	  lp[i].a = sumaq * (zzr * (lr[i].a - tempi) + tempj);
	  lp[i].g = sumgq * (zzr * (lr[i].g - tempi) + tempj);
	  tempi = fxry * tr->rdta->invfreqy;
	  tempj = zvr * (tempi-fxrn) + fxrn;
	  lp[i].c = sumcq * (zzr * (lr[i].c - tempi) + tempj);
	  lp[i].t = sumtq * (zzr * (lr[i].t - tempi) + tempj);
	  lp[i].exp = lq[i].exp + lr[i].exp;
	 	  
	  if (lp[i].a < minlikelihood && lp[i].g < minlikelihood
	      && lp[i].c < minlikelihood && lp[i].t < minlikelihood) 
	    {
	      lp[i].a   *= twotothe256;
	      lp[i].g   *= twotothe256;
	      lp[i].c   *= twotothe256;
	      lp[i].t   *= twotothe256;
	      lp[i].exp += 1;
            }	  
	}

     

      free(zzqtable);
      free(zvqtable);
      free(zzrtable);
      free(zvrtable);
    }
    return TRUE;
  }




double evaluateHKY85CAT (tree *tr, nodeptr p)
  { /* evaluate */
    double   sum, z, lz, xvlz,
             ki, fxpa, fxpc, fxpg, fxpt, fxpr, fxpy, fxqr, fxqy,
             suma, sumb, sumc, term;


       double   zz, zv;

    double        *zztable, *zvtable,
                 *zzptr, *zvptr;
    double        *rptr;
    likelivector *lp, *lq;
    nodeptr       q;
    int           cat, *cptr, i, *wptr;


    
    q = p->back;
    while ((! p->x) || (! q->x)) {
      if (! (p->x)) if (! newviewHKY85CAT(tr, p)) return badEval;
      if (! (q->x)) if (! newviewHKY85CAT(tr, q)) return badEval;
      }

    lp = p->x->lv;
    lq = q->x->lv;

    z = p->z;
    if (z < zmin) z = zmin;
    lz = log(z);
    xvlz = tr->rdta->xv * lz;

    zztable = (double *)malloc((tr->NumberOfCategories) * sizeof(double));
    zvtable = (double *)malloc((tr->NumberOfCategories) * sizeof(double));

    rptr   = tr->cdta->patrat;
    cptr  = tr->cdta->rateCategory; 
    zzptr = zztable;
    zvptr = zvtable;

    sum = 0.0;
    wptr = &(tr->cdta->aliaswgt[0]);

    

#pragma omp parallel for private(ki)
    for (i = 0; i < tr->NumberOfCategories; i++) 
      {
	ki = tr->cdta->patrat[i];
	zztable[i] = exp(ki *   lz);
	zvtable[i] = exp(ki * xvlz);
      }




    
#pragma omp parallel for private(cat, zz, zv, fxpa, fxpg, fxpc, fxpt, suma, fxqr, fxqy, fxpr, fxpy, sumb, sumc, term) reduction(+ : sum)
    for (i = 0; i < tr->cdta->endsite; i++) 
      {
	cat  = cptr[i];
	zz   = zztable[cat];
	zv   = zvtable[cat];
	fxpa = tr->rdta->freqa * lp[i].a;
        fxpg = tr->rdta->freqg * lp[i].g;
        fxpc = tr->rdta->freqc * lp[i].c;
        fxpt = tr->rdta->freqt * lp[i].t;
        suma = fxpa * lq[i].a + fxpc * lq[i].c + fxpg * lq[i].g + fxpt * lq[i].t;
        fxqr = tr->rdta->freqa * lq[i].a + tr->rdta->freqg * lq[i].g;
        fxqy = tr->rdta->freqc * lq[i].c + tr->rdta->freqt * lq[i].t;
        fxpr = fxpa + fxpg;
        fxpy = fxpc + fxpt;
        sumc = (fxpr + fxpy) * (fxqr + fxqy);
        sumb = fxpr * fxqr * tr->rdta->invfreqr + fxpy * fxqy * tr->rdta->invfreqy;
        suma -= sumb;
        sumb -= sumc;
        term = log(zz * suma + zv * sumb + sumc) + (lp[i].exp + lq[i].exp)*log(minlikelihood);

        sum += tr->cdta->aliaswgt[i] * term;
      }

    
    free(zztable);
    free(zvtable); 
    
    tr->likelihood = sum;
    
    return  sum;
  }


double makenewzHKY85CAT (tree *tr, nodeptr p, nodeptr q, double z0, int maxiter)
  { /* makenewz */
    likelivector *lp, *lq;
    double  *abi, *bci, *sumci, *abptr, *bcptr, *sumcptr;
    double   dlnLidlz, dlnLdlz, d2lnLdlz2, z, zprev, zstep, lz, xvlz,
             ki, suma, sumb, sumc, ab, bc, inv_Li, t1, t2,
             fx1a, fx1c, fx1g, fx1t, fx1r, fx1y, fx2r, fx2y;
    double   *zztable, *zvtable,
            *zzptr, *zvptr;
    double  *rptr, *wrptr, *wr2ptr;
    int      cat, *cptr, i, curvatOK;

    
    while ((! p->x) || (! q->x)) {
      if (! (p->x)) if (! newviewHKY85CAT(tr, p)) return badZ;
      if (! (q->x)) if (! newviewHKY85CAT(tr, q)) return badZ;
      }

    lp = p->x->lv;
    lq = q->x->lv;

    {
      unsigned scratch_size;
      scratch_size = sizeof(double) * tr->cdta->endsite;
      abi   = (double *) malloc(scratch_size);
      bci   = (double *) malloc(scratch_size);
      sumci = (double *) malloc(scratch_size);      
    }
    
    zztable = (double *)malloc(tr->NumberOfCategories * sizeof(double));
    zvtable = (double *)malloc(tr->NumberOfCategories * sizeof(double));

    abptr   = abi;
    bcptr   = bci;
    sumcptr = sumci;
   
#pragma omp parallel for private(fx1a, fx1g, fx1c, fx1t, suma, fx2r, fx2y, fx1r, fx1y, sumb, sumc)
    for (i = 0; i < tr->cdta->endsite; i++) 
      {
	fx1a = tr->rdta->freqa * lp[i].a;
	fx1g = tr->rdta->freqg * lp[i].g;
	fx1c = tr->rdta->freqc * lp[i].c;
	fx1t = tr->rdta->freqt * lp[i].t;
	suma = fx1a * lq[i].a + fx1c * lq[i].c + fx1g * lq[i].g + fx1t * lq[i].t;
	fx2r = tr->rdta->freqa * lq[i].a + tr->rdta->freqg * lq[i].g;
	fx2y = tr->rdta->freqc * lq[i].c + tr->rdta->freqt * lq[i].t;
	fx1r = fx1a + fx1g;
	fx1y = fx1c + fx1t;
	sumcptr[i] = sumc = (fx1r + fx1y) * (fx2r + fx2y);
	sumb       = fx1r * fx2r * tr->rdta->invfreqr + fx1y * fx2y * tr->rdta->invfreqy;
	abptr[i]   = suma - sumb;
	bcptr[i]   = sumb - sumc;
      }

    z = z0;
    do {
      zprev = z;
      zstep = (1.0 - zmax) * z + zmin;
      curvatOK = FALSE;

      do {
        if (z < zmin) z = zmin;
        else if (z > zmax) z = zmax;

        lz    = log(z);
        xvlz  = tr->rdta->xv * lz;
        rptr  = tr->cdta->patrat;
	cptr  = tr->cdta->rateCategory;
        zzptr = zztable;
        zvptr = zvtable;
      	       
        abptr   = abi;
        bcptr   = bci;
        sumcptr = sumci;        
        wrptr   = &(tr->cdta->wr[0]);
        wr2ptr  = &(tr->cdta->wr2[0]);
        dlnLdlz = 0.0;                 /*  = d(ln(likelihood))/d(lz) */
        d2lnLdlz2 = 0.0;               /*  = d2(ln(likelihood))/d(lz)2 */

#pragma omp parallel for private(ki)
        for (i = 0; i < tr->NumberOfCategories; i++) 
	  {
	    ki = tr->cdta->patrat[i];
	    zztable[i] = exp(ki *   lz);
	    zvtable[i] = exp(ki * xvlz);
          }

#pragma omp parallel for private(cat, ab, bc, sumc, inv_Li, t1, t2, dlnLidlz) reduction(+ : dlnLdlz) reduction( + : d2lnLdlz2) 
        for (i = 0; i < tr->cdta->endsite; i++) 
	  {
	    cat    = cptr[i];              /*  ratecategory(i) */
	    ab     = abptr[i] * zztable[cat];
	    bc     = bcptr[i] * zvtable[cat];
	    sumc   = sumcptr[i];
	    inv_Li = 1.0/(ab + bc + sumc);
	    t1     = ab * inv_Li;
	    t2     = tr->rdta->xv * bc * inv_Li;
	    dlnLidlz   = t1 + t2;
	    
	    dlnLdlz   += wrptr[i]  * dlnLidlz;
	    d2lnLdlz2 += wr2ptr[i] * (t1 + tr->rdta->xv * t2 - dlnLidlz * dlnLidlz);
          }

        if ((d2lnLdlz2 >= 0.0) && (z < zmax))
          zprev = z = 0.37 * z + 0.63;  /*  Bad curvature, shorten branch */
        else
          curvatOK = TRUE;

        } while (! curvatOK);

      if (d2lnLdlz2 < 0.0) {
        z *= exp(-dlnLdlz / d2lnLdlz2);
        if (z < zmin) z = zmin;
        if (z > 0.25 * zprev + 0.75)    /*  Limit steps toward z = 1.0 */
          z = 0.25 * zprev + 0.75;
        }

      if (z > zmax) z = zmax;

      } while ((--maxiter > 0) && (ABS(z - zprev) > zstep));
  
  
    free(abi);
    free(bci);
    free(sumci);
    free(zztable);
    free(zvtable); 

/* printf("makenewz: %le\n", z); */

    return  z;
  } /* makenewz */


void make_pmatrix(double matrix[20][20], double lz, double rat, tree *tr)
{
  int l, m;                 
  double p0;
  double *eptr, elambdat[20], *pptr;


  eptr = elambdat;

  for (l = 0; l < 20; l++)     
    *eptr++ = exp(lz * rat * tr->rdta->eigmatPROT[l]);         
 
  for (m = 0; m < 20; m++) 
    {      
      for (l = 0; l <= m; l++) 
	{	  
	  pptr = tr->rdta->crossMAT[m][l];
	  
	  p0 = (pptr[0] * elambdat[0]) + (pptr[1] * elambdat[1]);
	  p0 += (pptr[2] * elambdat[2]) + (pptr[3] * elambdat[3]);
	  p0 += (pptr[4] * elambdat[4]) + (pptr[5] * elambdat[5]);
	  p0 += (pptr[6] * elambdat[6]) + (pptr[7] * elambdat[7]);
	  p0 += (pptr[8] * elambdat[8]) + (pptr[9] * elambdat[9]);
	  p0 += (pptr[10] * elambdat[10]) + (pptr[11] * elambdat[11]);
	  p0 += (pptr[12] * elambdat[12]) + (pptr[13] * elambdat[13]);
	  p0 += (pptr[14] * elambdat[14]) + (pptr[15] * elambdat[15]);
	  p0 += (pptr[16] * elambdat[16]) + (pptr[17] * elambdat[17]);
	  p0 += (pptr[18] * elambdat[18]) + (pptr[19] * elambdat[19]);	  	      	     	 

	  matrix[m][l] =  p0 * tr->rdta->invfreqaa[m];	  
	  matrix[l][m] =  p0 * tr->rdta->invfreqaa[l];
	}           
    }

}  /* make_pmatrix */





boolean newviewPROT (tree *tr, nodeptr p)        /*  Update likelihoods at node */
  { /* newview */
    double   zq, zr;
    nodeptr  q, r;
    proteinlikelivector *lp, *lq, *lr;
    int  i, j, l, m;
    int           code;
    yType        *yptr;
    double i_value, *val;
   
           
    if (p->tip) 
      {              
	if (p->x) return TRUE; 

	if (! getxtip(p)) return FALSE; 
	


	lp = p->x->pv;           
	yptr = p->tip;          
#pragma omp parallel for private(code, val, i_value, j)
	for (i = 0; i < tr->cdta->endsite; i++) 
	  {
	    code = yptr[i];
	    val = & lp[i].v[0];


	    if(code < 22)
	      i_value = 0.0;
	    else
	      i_value = 1.0;	 

	    for(j = 0; j < 20; j++)
	      val[j] = i_value;

	    if(code < 20)
	      val[code] = 1.0;
	    else
	      {
		if(code == 20)	  
		  val[2] = val[3] = 1.0;
		if(code == 21)
		  val[5] = val[6] = 1.0;	  		   
	      }

	    lp[i].exp = 0;
	  }

        return TRUE;
      }
    
    {
       double r_matrix[20][20], q_matrix[20][20], *rmp, *qmp;
       double left_sum, right_sum, *rv, *qv;
       int scale;
       double p0, p1;
       double *eptr, elambdat[20], *eptrQ, elambdatQ[20], *pptr;
   
       q = p->next->back;
       r = p->next->next->back;
       
       while ((! p->x) || (! q->x) || (! r->x)) {
	 if (! q->x) if (! newviewPROT(tr, q))  return FALSE;
	 if (! r->x) if (! newviewPROT(tr, r))  return FALSE;
	 if (! p->x) if (! getxnode(p)) return FALSE;
       }
       
       lp = p->x->pv;
       
       lq = q->x->pv;
       zq = q->z;
       
       lr = r->x->pv;
       zr = r->z;                       	

       /*#pragma omp parallel for*/
       for (l = 0; l < 20; l++)     
	 {
	   elambdat[l] = exp(zr * tr->rdta->eigmatPROT[l]);         
	   elambdatQ[l] = exp(zq * tr->rdta->eigmatPROT[l]);  
	 }

       /*#pragma omp parallel for private(l, pptr, p0, p1) */
       for (m = 0; m < 20; m++) 
	 {      
	   for (l = 0; l <= m; l++) 
	     {	  
	       pptr = tr->rdta->crossMAT[m][l];
	       
	       p0 = (pptr[0] * elambdat[0]) + (pptr[1] * elambdat[1]);		  
	       p1 = (pptr[0] * elambdatQ[0]) + (pptr[1] * elambdatQ[1]);
	       
	       p0 += (pptr[2] * elambdat[2]) + (pptr[3] * elambdat[3]);
	       p1 += (pptr[2] * elambdatQ[2]) + (pptr[3] * elambdatQ[3]);
	       
	       p0 += (pptr[4] * elambdat[4]) + (pptr[5] * elambdat[5]);
	       p1 += (pptr[4] * elambdatQ[4]) + (pptr[5] * elambdatQ[5]);
	       
	       p0 += (pptr[6] * elambdat[6]) + (pptr[7] * elambdat[7]);
	       p1 += (pptr[6] * elambdatQ[6]) + (pptr[7] * elambdatQ[7]);
	       
	       p0 += (pptr[8] * elambdat[8]) + (pptr[9] * elambdat[9]);
	       p1 += (pptr[8] * elambdatQ[8]) + (pptr[9] * elambdatQ[9]);
	       
	       p0 += (pptr[10] * elambdat[10]) + (pptr[11] * elambdat[11]);
	       p1 += (pptr[10] * elambdatQ[10]) + (pptr[11] * elambdatQ[11]);
	       
	       p0 += (pptr[12] * elambdat[12]) + (pptr[13] * elambdat[13]);
	       p1 += (pptr[12] * elambdatQ[12]) + (pptr[13] * elambdatQ[13]);
	       
	       p0 += (pptr[14] * elambdat[14]) + (pptr[15] * elambdat[15]);
	       p1 += (pptr[14] * elambdatQ[14]) + (pptr[15] * elambdatQ[15]);
	       
	       p0 += (pptr[16] * elambdat[16]) + (pptr[17] * elambdat[17]);
	       p1 += (pptr[16] * elambdatQ[16]) + (pptr[17] * elambdatQ[17]);
	       
	       p0 += (pptr[18] * elambdat[18]) + (pptr[19] * elambdat[19]);	  	      	     	 
	       p1 += (pptr[18] * elambdatQ[18]) + (pptr[19] * elambdatQ[19]);		 	  	      	     	 
	       
	       r_matrix[m][l] =  p0 * tr->rdta->invfreqaa[m];	  
	       r_matrix[l][m] =  p0 * tr->rdta->invfreqaa[l];
	       	       
	       q_matrix[m][l] =  p1 * tr->rdta->invfreqaa[m];	  
	       q_matrix[l][m] =  p1 * tr->rdta->invfreqaa[l];
		 
	     }           
	 }         	 	  	  

#pragma omp parallel for private(scale, rv, qv, m, rmp, qmp, left_sum, right_sum)      
       for (i = 0; i < tr->cdta->endsite; i++) 
	 {	 	  	  
	   scale = 1;
	   
	   rv = &(lr[i].v[0]);
	   qv = &(lq[i].v[0]);

	   for (m = 0; m < 20; m++)
	     {
	       rmp = r_matrix[m];
	       qmp = q_matrix[m];
	       
	       left_sum = (rv[0] * rmp[0]) + (rv[1] * rmp[1]);
	       right_sum = (qv[0] * qmp[0]) + (qv[1] * qmp[1]);

	       left_sum += (rv[2] * rmp[2]) + (rv[3] * rmp[3]);
	       right_sum += (qv[2] * qmp[2]) + (qv[3] * qmp[3]);

	       left_sum += (rv[4] * rmp[4]) + (rv[5] * rmp[5]);
	       right_sum += (qv[4] * qmp[4]) + (qv[5] * qmp[5]);

	       left_sum += (rv[6] * rmp[6]) + (rv[7] * rmp[7]);
	       right_sum += (qv[6] * qmp[6]) + (qv[7] * qmp[7]);

	       left_sum += (rv[8] * rmp[8]) + (rv[9] * rmp[9]);
	       right_sum += (qv[8] * qmp[8]) + (qv[9] * qmp[9]);

	       left_sum += (rv[10] * rmp[10]) + (rv[11] * rmp[11]);
	       right_sum += (qv[10] * qmp[10]) + (qv[11] * qmp[11]);

	       left_sum += (rv[12] * rmp[12]) + (rv[13] * rmp[13]);
	       right_sum += (qv[12] * qmp[12]) + (qv[13] * qmp[13]);

	       left_sum += (rv[14] * rmp[14]) + (rv[15] * rmp[15]);
	       right_sum += (qv[14] * qmp[14]) + (qv[15] * qmp[15]);

	       left_sum += (rv[16] * rmp[16]) + (rv[17] * rmp[17]);
	       right_sum += (qv[16] * qmp[16]) + (qv[17] * qmp[17]);

	       left_sum += (rv[18] * rmp[18]) + (rv[19] * rmp[19]);
	       right_sum += (qv[18] * qmp[18]) + (qv[19] * qmp[19]);

	      
	       lp[i].v[m] = left_sum * right_sum;	   

	       if(lp[i].v[m] >= minlikelihood)	      
		 scale = 0;	       	      
	     }

	   lp[i].exp = lq[i].exp + lr[i].exp;

	   if(scale) 
	     {	     
	       for(m = 0; m < 20; m++)
		 lp[i].v[m] *= twotothe256;
	       lp[i].exp += 1;
	     }		
	 }   
    }

    return TRUE;
  }

boolean newviewPartialPROTCAT (tree *tr, nodeptr p, int i, double rate)        /*  Update likelihoods at node */
  { /* newview */
    double   zq, zr;
    nodeptr  q, r;
    proteinlikelivector *lp, *lq, *lr;
    int  j, l, m;
    int           code;
    yType        *yptr;
    double i_value, *val;
   
           
    if (p->tip) 
      {              
	if (p->x) return TRUE; 

	if (! getxtip(p)) return FALSE; 
	


	lp = p->x->pv;           
	yptr = p->tip;          

	code = yptr[i];
	val = & lp[i].v[0];


	if(code < 22)
	  i_value = 0.0;
	else
	  i_value = 1.0;	 

	for(j = 0; j < 20; j++)
	  val[j] = i_value;

	if(code < 20)
	  val[code] = 1.0;
	else
	  {
	    if(code == 20)	  
	      val[2] = val[3] = 1.0;
	    if(code == 21)
	      val[5] = val[6] = 1.0;	  		   
	  }

	lp[i].exp = 0;
	
        return TRUE;
      }
    
    {
       double r_matrix[20][20], q_matrix[20][20], *rmp, *qmp;
       double left_sum, right_sum, *rv, *qv;
       int scale;

   
       q = p->next->back;
       r = p->next->next->back;
       
       while ((! p->x) || (! q->x) || (! r->x)) {
	 if (! q->x) if (! newviewPartialPROTCAT(tr, q, i, rate))  return FALSE;
	 if (! r->x) if (! newviewPartialPROTCAT(tr, r, i, rate))  return FALSE;
	 if (! p->x) if (! getxnode(p)) return FALSE;
       }
       
       lp = p->x->pv;
       
       lq = q->x->pv;
       zq = q->z;
       
       lr = r->x->pv;
       zr = r->z;            
       
       make_pmatrix(r_matrix, zr, rate, tr);
       make_pmatrix(q_matrix, zq, rate, tr);

      
       scale = 1;
	   
       rv = &(lr[i].v[0]);
       qv = &(lq[i].v[0]);

       for (m = 0; m < 20; m++)
	 {
	   rmp = r_matrix[m];
	   qmp = q_matrix[m];
	   
	   left_sum = (rv[0] * rmp[0]) + (rv[1] * rmp[1]);
	   right_sum = (qv[0] * qmp[0]) + (qv[1] * qmp[1]);
	   
	   left_sum += (rv[2] * rmp[2]) + (rv[3] * rmp[3]);
	   right_sum += (qv[2] * qmp[2]) + (qv[3] * qmp[3]);
	   
	   left_sum += (rv[4] * rmp[4]) + (rv[5] * rmp[5]);
	   right_sum += (qv[4] * qmp[4]) + (qv[5] * qmp[5]);
	   
	   left_sum += (rv[6] * rmp[6]) + (rv[7] * rmp[7]);
	   right_sum += (qv[6] * qmp[6]) + (qv[7] * qmp[7]);
	   
	   left_sum += (rv[8] * rmp[8]) + (rv[9] * rmp[9]);
	   right_sum += (qv[8] * qmp[8]) + (qv[9] * qmp[9]);
	   
	   left_sum += (rv[10] * rmp[10]) + (rv[11] * rmp[11]);
	   right_sum += (qv[10] * qmp[10]) + (qv[11] * qmp[11]);
	   
	   left_sum += (rv[12] * rmp[12]) + (rv[13] * rmp[13]);
	   right_sum += (qv[12] * qmp[12]) + (qv[13] * qmp[13]);
	   
	   left_sum += (rv[14] * rmp[14]) + (rv[15] * rmp[15]);
	   right_sum += (qv[14] * qmp[14]) + (qv[15] * qmp[15]);
	   
	   left_sum += (rv[16] * rmp[16]) + (rv[17] * rmp[17]);
	   right_sum += (qv[16] * qmp[16]) + (qv[17] * qmp[17]);
	   
	   left_sum += (rv[18] * rmp[18]) + (rv[19] * rmp[19]);
	   right_sum += (qv[18] * qmp[18]) + (qv[19] * qmp[19]);
	   
	   
	   lp[i].v[m] = left_sum * right_sum;	   
	   
	   if(lp[i].v[m] >= minlikelihood)	      
	     scale = 0;	       	      
	 }

       lp[i].exp = lq[i].exp + lr[i].exp;

       if(scale) 
	 {	     
	   for(m = 0; m < 20; m++)
	     lp[i].v[m] *= twotothe256;
	   lp[i].exp += 1;
	 }		       
    }


    return TRUE;
  }




double evaluatePROT (tree *tr, nodeptr p)
  { /* evaluate */
    double   sum, z, lz, xvlz,
      ki, term, zz, zv, frexm, prodl;
    double        *zztable, *zvtable,
                 *zzptr, *zvptr;
    double        *rptr, *vptr, *mptr;
    proteinlikelivector *lp, *lq;
    nodeptr       q;
    int           cat, *cptr, i, *wptr, m, l;
    double pq_matrix[20][20];
    double p0;
    double *eptr, elambdat[20], *pptr;

    q = p->back;

    while ((! p->x) || (! q->x)) 
      {
	if (! (p->x)) if (! newviewPROT(tr, p)) return badEval;
	if (! (q->x)) if (! newviewPROT(tr, q)) return badEval;
      }

    lp = p->x->pv;
    lq = q->x->pv;

    z = p->z;
    if (z < zmin) z = zmin;
   
    wptr = tr->cdta->aliaswgt;
    sum = 0;
    eptr = elambdat;	      
	
    for (l = 0; l < 20; l++)     
      *eptr++ = exp(z * tr->rdta->eigmatPROT[l]);         

    /*#pragma omp parallel for private(l, pptr, p0)*/
    for (m = 0; m < 20; m++) 
      {      
	for (l = 0; l <= m; l++) 
	  {	  
	    pptr = tr->rdta->crossMAT[m][l];
	    
	    p0 = (pptr[0] * elambdat[0]) + (pptr[1] * elambdat[1]);
	    p0 += (pptr[2] * elambdat[2]) + (pptr[3] * elambdat[3]);
	    p0 += (pptr[4] * elambdat[4]) + (pptr[5] * elambdat[5]);
	    p0 += (pptr[6] * elambdat[6]) + (pptr[7] * elambdat[7]);
	    p0 += (pptr[8] * elambdat[8]) + (pptr[9] * elambdat[9]);
	    p0 += (pptr[10] * elambdat[10]) + (pptr[11] * elambdat[11]);
	    p0 += (pptr[12] * elambdat[12]) + (pptr[13] * elambdat[13]);
	    p0 += (pptr[14] * elambdat[14]) + (pptr[15] * elambdat[15]);
	    p0 += (pptr[16] * elambdat[16]) + (pptr[17] * elambdat[17]);
	    p0 += (pptr[18] * elambdat[18]) + (pptr[19] * elambdat[19]);	  	      	     	 
		
	    pq_matrix[m][l] =  p0 * tr->rdta->invfreqaa[m];	  
	    pq_matrix[l][m] =  p0 * tr->rdta->invfreqaa[l];
	  }           
      }
  
#pragma omp parallel for private(term, m, vptr, mptr, prodl, frexm) reduction(+ : sum)
    for (i = 0; i < tr->cdta->endsite; i++) 
      {
	term = 0;
	for (m = 0; m < 20; m++) 
	  {	   
	    vptr = &(lq[i].v[0]);
	    mptr = pq_matrix[m];

	    prodl = mptr[0] * vptr[0] +  mptr[1] * vptr[1];
	    prodl += mptr[2] * vptr[2] +  mptr[3] * vptr[3];
	    prodl += mptr[4] * vptr[4] +  mptr[5] * vptr[5];
	    prodl += mptr[6] * vptr[6] +  mptr[7] * vptr[7];
	    prodl += mptr[8] * vptr[8] +  mptr[9] * vptr[9];
	    prodl += mptr[10] * vptr[10] +  mptr[11] * vptr[11];
	    prodl += mptr[12] * vptr[12] +  mptr[13] * vptr[13];
	    prodl += mptr[14] * vptr[14] +  mptr[15] * vptr[15];
	    prodl += mptr[16] * vptr[16] +  mptr[17] * vptr[17]; 
	    prodl += mptr[18] * vptr[18] +  mptr[19] * vptr[19];
			   	    
	    frexm = lp[i].v[m] * tr->rdta->freqaa[m];	   
	    term += (prodl * frexm);	    
	  }
		
	term = log(term) + (lq[i].exp + lp[i].exp)*log(minlikelihood);       
	sum += term * wptr[i];
      }
   


    tr->likelihood = sum;
    return  sum;
  }


double evaluatePartialPROTCAT (tree *tr, nodeptr p, int i, double rate)
  { /* evaluate */
    double   sum, z, lz, xvlz,
      ki, term, zz, zv, frexm, prodl;
    double        *zztable, *zvtable,
                 *zzptr, *zvptr;
    double        *rptr;
    proteinlikelivector *lp, *lq;
    nodeptr       q;
    int           cat, *cptr, *wptr, m, l;
    double pq_matrix[20][20];

    q = p->back;

    while ((! p->x) || (! q->x)) 
      {
	if (! (p->x)) if (! newviewPartialPROTCAT(tr, p, i, rate)) return badEval;
	if (! (q->x)) if (! newviewPartialPROTCAT(tr, q, i, rate)) return badEval;
      }

    lp = p->x->pv;
    lq = q->x->pv;

    z = p->z;
    if (z < zmin) z = zmin;
   
    wptr = tr->cdta->aliaswgt;

    
    make_pmatrix(pq_matrix, z, 1.0, tr);       

    sum = 0;



    
    term = 0;
    for (m = 0; m < 20; m++) 
      {
	prodl = 0;	   
	for (l = 0; l < 20; l++)
	  prodl += (pq_matrix[m][l] * lq[i].v[l]);
	frexm = lp[i].v[m] * tr->rdta->freqaa[m];	   
	term += (prodl * frexm);	    
      }
    
	
    term = log(term) + (lq[i].exp + lp[i].exp)*log(minlikelihood);
    sum += term * wptr[i];  
   


    tr->likelihood = sum;
    return  sum;
  }







double makenewzPROT (tree *tr, nodeptr p_node, nodeptr q_node, double z0, int maxiter)
  { /* makenewz */
   
    double   z, zprev, znew;
    int     i, *cptr, cat, l, m, finish, greater, action, extended, curvatOK, decrease;
    double  dlnLdlz, d2lnLdlz2, sum, term;
    double  *rptr, ki;
    proteinlikelivector *x1, *x2;
    double matrix[20][20], matrixD[20][20], matrixDD[20][20];
    double prod4, prod5m, prod6m, prod4m, frexm;  
    double inv_Li, dlnLidlz, d2lnLidlz2 , *v;  

    while ((! p_node->x) || (! q_node->x)) 
      {
	if (! (p_node->x)) if (! newviewPROT(tr, p_node)) return badZ;
	if (! (q_node->x)) if (! newviewPROT(tr, q_node)) return badZ;
      }    
   
    x1 = p_node->x->pv;
    x2 = q_node->x->pv;      

    z = z0;   
    
    do 
      {
	int *wptr =  tr->cdta->aliaswgt;
	double p0, p1, p2;
	double elambdat[20], dlambdat[20], ddlambdat[20];
	double *pptr;

	double *wrptr   = &(tr->cdta->wr[0]);
	double *wr2ptr  = &(tr->cdta->wr2[0]);
	rptr   = &(tr->cdta->patrat[0]); 
	cptr  = &(tr->cdta->rateCategory[0]);	
	
	sum = 0;
	dlnLdlz = 0.0;                 
	d2lnLdlz2 = 0.0; 
	

	zprev = z;

	if (z < zmin) 
	  z = zmin;
	else 
	  if (z > zmax) z = zmax;	
	/*#pragma omp parallel for	*/
	for (l = 0; l < 20; l++) 
	  {
	    elambdat[l] = exp(z * tr->rdta->eigmatPROT[l]); 
	    dlambdat[l] = elambdat[l] * tr->rdta->eigmatPROT[l];
	    ddlambdat[l] = dlambdat[l] * tr->rdta->eigmatPROT[l];
	  }
	/*#pragma omp parallel for private(l, pptr, p0, p1, p2)*/
	for (m = 0; m < 20; m++) 
	  {
	    for (l = 0; l <= m; l++) 
	      {
		pptr = tr->rdta->crossMAT[m][l];
		    
		p0 = (pptr[0] * elambdat[0]) + (pptr[1] * elambdat[1]);
		p1 = (pptr[0] * dlambdat[0]) + (pptr[1] * dlambdat[1]);
		p2 = (pptr[0] * ddlambdat[0]) + (pptr[1] * ddlambdat[1]);
		
		p0 += (pptr[2] * elambdat[2]) + (pptr[3] * elambdat[3]);
		p1 += (pptr[2] * dlambdat[2]) + (pptr[3] * dlambdat[3]);
		p2 += (pptr[2] * ddlambdat[2]) + (pptr[3] * ddlambdat[3]);
		
		p0 += (pptr[4] * elambdat[4]) + (pptr[5] * elambdat[5]);
		p1 += (pptr[4] * dlambdat[4]) + (pptr[5] * dlambdat[5]);
		p2 += (pptr[4] * ddlambdat[4]) + (pptr[5] * ddlambdat[5]);
		
		p0 += (pptr[6] * elambdat[6]) + (pptr[7] * elambdat[7]);
		p1 += (pptr[6] * dlambdat[6]) + (pptr[7] * dlambdat[7]);
		p2 += (pptr[6] * ddlambdat[6]) + (pptr[7] * ddlambdat[7]);
		
		p0 += (pptr[8] * elambdat[8]) + (pptr[9] * elambdat[9]);
		p1 += (pptr[8] * dlambdat[8]) + (pptr[9] * dlambdat[9]);
		p2 += (pptr[8] * ddlambdat[8]) + (pptr[9] * ddlambdat[9]);
		
		p0 += (pptr[10] * elambdat[10]) + (pptr[11] * elambdat[11]);
		p1 += (pptr[10] * dlambdat[10]) + (pptr[11] * dlambdat[11]);
		p2 += (pptr[10] * ddlambdat[10]) + (pptr[11] * ddlambdat[11]);
		
		p0 += (pptr[12] * elambdat[12]) + (pptr[13] * elambdat[13]);
		p1 += (pptr[12] * dlambdat[12]) + (pptr[13] * dlambdat[13]);
		p2 += (pptr[12] * ddlambdat[12]) + (pptr[13] * ddlambdat[13]);
		
		p0 += (pptr[14] * elambdat[14]) + (pptr[15] * elambdat[15]);
		p1 += (pptr[14] * dlambdat[14]) + (pptr[15] * dlambdat[15]);
		p2 += (pptr[14] * ddlambdat[14]) + (pptr[15] * ddlambdat[15]);
		
		p0 += (pptr[16] * elambdat[16]) + (pptr[17] * elambdat[17]);
		p1 += (pptr[16] * dlambdat[16]) + (pptr[17] * dlambdat[17]);
		p2 += (pptr[16] * ddlambdat[16]) + (pptr[17] * ddlambdat[17]);
		
		p0 += (pptr[18] * elambdat[18]) + (pptr[19] * elambdat[19]);
		p1 += (pptr[18] * dlambdat[18]) + (pptr[19] * dlambdat[19]);
		p2 += (pptr[18] * ddlambdat[18]) + (pptr[19] * ddlambdat[19]);
		
		matrix[m][l] = p0 * tr->rdta->invfreqaa[m];
		matrix[l][m] = p0 * tr->rdta->invfreqaa[l];
		matrixD[m][l] = p1 * tr->rdta->invfreqaa[m];
		matrixD[l][m] = p1 * tr->rdta->invfreqaa[l];
		matrixDD[m][l] = p2 * tr->rdta->invfreqaa[m];
		matrixDD[l][m] = p2 * tr->rdta->invfreqaa[l];	      
	      }
	  }            

#pragma omp parallel for private(prod4,  dlnLidlz, d2lnLidlz2, v, m, frexm, prod4m, prod5m, prod6m, inv_Li) reduction(+ :  dlnLdlz) reduction(+ : d2lnLdlz2)
	for (i = 0; i < tr->cdta->endsite; i++) 
	  {
	    prod4 = 0.0;
	    dlnLidlz = 0.0;
	    d2lnLidlz2 = 0.0;
	    
	    v = &(x2[i].v[0]);
	    
	    for (m = 0; m <= 19; m++) 
	      {		
		frexm = x1[i].v[m] * tr->rdta->freqaa[m];				
		
		prod4m = v[0] * matrix[m][0] + v[1] * matrix[m][1];	       
		prod5m = v[0] * matrixD[m][0] + v[1] * matrixD[m][1];		   
		prod6m = v[0] * matrixDD[m][0]+ v[1] * matrixDD[m][1];
		
		prod4m += v[2] * matrix[m][2] + v[3] * matrix[m][3];
		prod5m += v[2] * matrixD[m][2] + v[3] * matrixD[m][3];		   
		prod6m += v[2] * matrixDD[m][2]+ v[3] * matrixDD[m][3];
		
		prod4m += v[4] * matrix[m][4] + v[5] * matrix[m][5];
		prod5m += v[4] * matrixD[m][4] + v[5] * matrixD[m][5];		   
		prod6m += v[4] * matrixDD[m][4]+ v[5] * matrixDD[m][5];
		
		prod4m += v[6] * matrix[m][6] + v[7] * matrix[m][7];
		prod5m += v[6] * matrixD[m][6] + v[7] * matrixD[m][7];		   
		prod6m += v[6] * matrixDD[m][6]+ v[7] * matrixDD[m][7];
		
		prod4m += v[8] * matrix[m][8] + v[9] * matrix[m][9];
		prod5m += v[8] * matrixD[m][8] + v[9] * matrixD[m][9];		   
		prod6m += v[8] * matrixDD[m][8]+ v[9] * matrixDD[m][9];
		
		prod4m += v[10] * matrix[m][10] + v[11] * matrix[m][11];
		prod5m += v[10] * matrixD[m][10] + v[11] * matrixD[m][11];		   
		prod6m += v[10] * matrixDD[m][10]+ v[11] * matrixDD[m][11];
		
		prod4m += v[12] * matrix[m][12] + v[13] * matrix[m][13];
		prod5m += v[12] * matrixD[m][12] + v[13] * matrixD[m][13];		   
		prod6m += v[12] * matrixDD[m][12]+ v[13] * matrixDD[m][13];
		
		prod4m += v[14] * matrix[m][14] + v[15] * matrix[m][15];
		prod5m += v[14] * matrixD[m][14] + v[15] * matrixD[m][15];		   
		prod6m += v[14] * matrixDD[m][14]+ v[15] * matrixDD[m][15];
		
		prod4m += v[16] * matrix[m][16] + v[17] * matrix[m][17];
		prod5m += v[16] * matrixD[m][16] + v[17] * matrixD[m][17];		   
		prod6m += v[16] * matrixDD[m][16]+ v[17] * matrixDD[m][17];
		
		prod4m += v[18] * matrix[m][18] + v[19] * matrix[m][19];
		prod5m += v[18] * matrixD[m][18] + v[19] * matrixD[m][19];		   
		prod6m += v[18] * matrixDD[m][18]+ v[19] * matrixDD[m][19];
		
		
		prod4 += frexm * prod4m;
		dlnLidlz += frexm * prod5m;
		d2lnLidlz2 +=  frexm * prod6m;
	      }
	    
	  
	    inv_Li = 1.0 / prod4;	    	   
	    dlnLidlz   *= inv_Li; 
	    d2lnLidlz2 *= inv_Li; 		
	    dlnLdlz   += wrptr[i]  * dlnLidlz;
	    d2lnLdlz2 += wr2ptr[i] *  (d2lnLidlz2 -  dlnLidlz * dlnLidlz);
	  }
		


	znew = z + dlnLdlz/fabs(d2lnLdlz2);

	if(znew < zmin || znew > zmax)	  	  
	  {	    	    	   
	    znew = z * 0.5;	   
	    action = 1;
	  }
	else	
	  {	    	   	    
	    action = 0;
	  }
       

	if(dlnLdlz < 0)
	  {
	    decrease = 1;
	    if(znew > z)
	      {
		/*printf("ERROR EXPCTED ");	       */
		znew = z * 0.5;
	      }
	  }
	else
	  {
	    decrease = 0;	
	    if(znew < z)
	      {
		/*printf("ERROR EXPCTED ");		*/
		znew = z * 0.37 + 0.63;
	      }
	  }
	z = znew;
	

      }
    while((--maxiter > 0) && fabs(z - zprev) > deltaz);
    /*printf("\n");*/
    return z;      
  }


boolean newviewPROTCAT (tree *tr, nodeptr p)        /*  Update likelihoods at node */
  { /* newview */
    double   zq, zr;
    nodeptr  q, r;
    proteinlikelivector *lp, *lq, *lr;
    int  i, j, l, m;
    int           code;
    yType        *yptr;
    double i_value, *val;
   
           
    if (p->tip) 
      {              
	if (p->x) return TRUE; 

	if (! getxtip(p)) return FALSE; 
	


	lp = p->x->pv;           
	yptr = p->tip;          
#pragma omp parallel for private(code, val, i_value, j)
	for (i = 0; i < tr->cdta->endsite; i++) 
	  {
	    code = yptr[i];
	    val = & lp[i].v[0];


	    if(code < 22)
	      i_value = 0.0;
	    else
	      i_value = 1.0;	 

	    for(j = 0; j < 20; j++)
	      val[j] = i_value;

	    if(code < 20)
	      val[code] = 1.0;
	    else
	      {
		if(code == 20)	  
		  val[2] = val[3] = 1.0;
		if(code == 21)
		  val[5] = val[6] = 1.0;	  		   
	      }

	    lp[i].exp = 0;
	  }

        return TRUE;
      }
    
    {
      proteinmatrix *r_matrix, *q_matrix;
      double *rmp, *qmp;
      double left_sum, right_sum, *rv, *qv;
      int scale, *cptr, cat;
      double *eptr, elambdat[20], *eptrQ, elambdatQ[20];
      double rat, *rptr, *pptr;
      double p0, p1;

      r_matrix = (proteinmatrix *)malloc(sizeof(proteinmatrix) * tr->NumberOfCategories);
      q_matrix = (proteinmatrix *)malloc(sizeof(proteinmatrix) * tr->NumberOfCategories);

   
      q = p->next->back;
      r = p->next->next->back;
       
      while ((! p->x) || (! q->x) || (! r->x)) {
	if (! q->x) if (! newviewPROTCAT(tr, q))  return FALSE;
	if (! r->x) if (! newviewPROTCAT(tr, r))  return FALSE;
	if (! p->x) if (! getxnode(p)) return FALSE;
      }
       
      lp = p->x->pv;
      
      lq = q->x->pv;
      zq = q->z;
      
      lr = r->x->pv;
      zr = r->z;            
                  
      rptr = &(tr->cdta->patrat[0]); 
      cptr  = &(tr->cdta->rateCategory[0]);
#pragma omp parallel for private(m, l, p0, p1, pptr, elambdat, elambdatQ)
      for(i = 0; i < tr->NumberOfCategories; i++)
	{    	     
	  for (l = 0; l < 20; l++)     
	    {
	      elambdat[l] = exp(zr * rptr[i] * tr->rdta->eigmatPROT[l]);         
	      elambdatQ[l] = exp(zq * rptr[i] * tr->rdta->eigmatPROT[l]); 
	    }

	  for (m = 0; m < 20; m++) 
	    {      
	      for (l = 0; l <= m; l++) 
		{	  
		  pptr = tr->rdta->crossMAT[m][l];
	      
		  p0 = (pptr[0] * elambdat[0]) + (pptr[1] * elambdat[1]);		  
		  p1 = (pptr[0] * elambdatQ[0]) + (pptr[1] * elambdatQ[1]);

		  p0 += (pptr[2] * elambdat[2]) + (pptr[3] * elambdat[3]);
		  p1 += (pptr[2] * elambdatQ[2]) + (pptr[3] * elambdatQ[3]);

		  p0 += (pptr[4] * elambdat[4]) + (pptr[5] * elambdat[5]);
		  p1 += (pptr[4] * elambdatQ[4]) + (pptr[5] * elambdatQ[5]);

		  p0 += (pptr[6] * elambdat[6]) + (pptr[7] * elambdat[7]);
		  p1 += (pptr[6] * elambdatQ[6]) + (pptr[7] * elambdatQ[7]);

		  p0 += (pptr[8] * elambdat[8]) + (pptr[9] * elambdat[9]);
		  p1 += (pptr[8] * elambdatQ[8]) + (pptr[9] * elambdatQ[9]);

		  p0 += (pptr[10] * elambdat[10]) + (pptr[11] * elambdat[11]);
		  p1 += (pptr[10] * elambdatQ[10]) + (pptr[11] * elambdatQ[11]);

		  p0 += (pptr[12] * elambdat[12]) + (pptr[13] * elambdat[13]);
		  p1 += (pptr[12] * elambdatQ[12]) + (pptr[13] * elambdatQ[13]);
		  
		  p0 += (pptr[14] * elambdat[14]) + (pptr[15] * elambdat[15]);
		  p1 += (pptr[14] * elambdatQ[14]) + (pptr[15] * elambdatQ[15]);

		  p0 += (pptr[16] * elambdat[16]) + (pptr[17] * elambdat[17]);
		  p1 += (pptr[16] * elambdatQ[16]) + (pptr[17] * elambdatQ[17]);

		  p0 += (pptr[18] * elambdat[18]) + (pptr[19] * elambdat[19]);	  	      	     	 
		  p1 += (pptr[18] * elambdatQ[18]) + (pptr[19] * elambdatQ[19]);	
		  		  		 		 		 		 		 		 		  		  
		  r_matrix[i].m[m][l] =  p0 * tr->rdta->invfreqaa[m];	  
		  r_matrix[i].m[l][m] =  p0 * tr->rdta->invfreqaa[l];
		  
		  q_matrix[i].m[m][l] =  p1 * tr->rdta->invfreqaa[m];	  
		  q_matrix[i].m[l][m] =  p1 * tr->rdta->invfreqaa[l];		   
		}           
	    }
	}    
                    
#pragma omp parallel for private(cat, scale, rv, qv, m, rmp, qmp, left_sum, right_sum)  
       for (i = 0; i < tr->cdta->endsite; i++) 
	 {
	   cat = cptr[i];
	 	  	  
	   scale = 1;
	   
	   rv = &(lr[i].v[0]);
	   qv = &(lq[i].v[0]);

	   for (m = 0; m < 20; m++)
	     {
	       rmp = r_matrix[cat].m[m];
	       qmp = q_matrix[cat].m[m];
	       
	       left_sum = (rv[0] * rmp[0]) + (rv[1] * rmp[1]);
	       right_sum = (qv[0] * qmp[0]) + (qv[1] * qmp[1]);

	       left_sum += (rv[2] * rmp[2]) + (rv[3] * rmp[3]);
	       right_sum += (qv[2] * qmp[2]) + (qv[3] * qmp[3]);

	       left_sum += (rv[4] * rmp[4]) + (rv[5] * rmp[5]);
	       right_sum += (qv[4] * qmp[4]) + (qv[5] * qmp[5]);

	       left_sum += (rv[6] * rmp[6]) + (rv[7] * rmp[7]);
	       right_sum += (qv[6] * qmp[6]) + (qv[7] * qmp[7]);

	       left_sum += (rv[8] * rmp[8]) + (rv[9] * rmp[9]);
	       right_sum += (qv[8] * qmp[8]) + (qv[9] * qmp[9]);

	       left_sum += (rv[10] * rmp[10]) + (rv[11] * rmp[11]);
	       right_sum += (qv[10] * qmp[10]) + (qv[11] * qmp[11]);

	       left_sum += (rv[12] * rmp[12]) + (rv[13] * rmp[13]);
	       right_sum += (qv[12] * qmp[12]) + (qv[13] * qmp[13]);

	       left_sum += (rv[14] * rmp[14]) + (rv[15] * rmp[15]);
	       right_sum += (qv[14] * qmp[14]) + (qv[15] * qmp[15]);

	       left_sum += (rv[16] * rmp[16]) + (rv[17] * rmp[17]);
	       right_sum += (qv[16] * qmp[16]) + (qv[17] * qmp[17]);

	       left_sum += (rv[18] * rmp[18]) + (rv[19] * rmp[19]);
	       right_sum += (qv[18] * qmp[18]) + (qv[19] * qmp[19]);

	      
	       lp[i].v[m] = left_sum * right_sum;	   

	       if(lp[i].v[m] >= minlikelihood)	      
		 scale = 0;	       	      
	     }

	   lp[i].exp = lq[i].exp + lr[i].exp;

	   if(scale) 
	     {	     
	       for(m = 0; m < 20; m++)
		 lp[i].v[m] *= twotothe256;
	       lp[i].exp += 1;
	     }		
	 }
    
       free(r_matrix);
       free(q_matrix);
    }

    return TRUE;
  }




double evaluatePROTCAT (tree *tr, nodeptr p)
  { /* evaluate */
    double   sum, z, lz, xvlz,
      ki, term, zz, zv, frexm, prodl;
    double        *zztable, *zvtable,
                 *zzptr, *zvptr;
    double        *rptr, *pqptr, *vptr;
    proteinlikelivector *lp, *lq;
    nodeptr       q;
    int           cat, *cptr, i, *wptr, m, l;
    proteinmatrix *pq_matrix;
    double p0;
    double *eptr, elambdat[20], *pptr;
    double rat;

    q = p->back;
    pq_matrix = (proteinmatrix *)malloc(sizeof(proteinmatrix) * tr->NumberOfCategories);
     
    while ((! p->x) || (! q->x)) 
      {
	if (! (p->x)) if (! newviewPROTCAT(tr, p)) return badEval;
	if (! (q->x)) if (! newviewPROTCAT(tr, q)) return badEval;
      }

    lp = p->x->pv;
    lq = q->x->pv;

    z = p->z;
    if (z < zmin) z = zmin;
   
    wptr = tr->cdta->aliaswgt;
    cptr  = &(tr->cdta->rateCategory[0]);    
    rptr   = &(tr->cdta->patrat[0]); 

    sum = 0;
#pragma omp parallel for private(l, m, pptr, p0, elambdat)
    for(i = 0; i < tr->NumberOfCategories; i++)
      {    	
	for (l = 0; l < 20; l++)     
	  elambdat[l] = exp(z * rptr[i] * tr->rdta->eigmatPROT[l]);         

	for (m = 0; m < 20; m++) 
	  {      
	    for (l = 0; l <= m; l++) 
	      {	  
		pptr = tr->rdta->crossMAT[m][l];
		
		p0 = (pptr[0] * elambdat[0]) + (pptr[1] * elambdat[1]);
		p0 += (pptr[2] * elambdat[2]) + (pptr[3] * elambdat[3]);
		p0 += (pptr[4] * elambdat[4]) + (pptr[5] * elambdat[5]);
		p0 += (pptr[6] * elambdat[6]) + (pptr[7] * elambdat[7]);
		p0 += (pptr[8] * elambdat[8]) + (pptr[9] * elambdat[9]);
		p0 += (pptr[10] * elambdat[10]) + (pptr[11] * elambdat[11]);
		p0 += (pptr[12] * elambdat[12]) + (pptr[13] * elambdat[13]);
		p0 += (pptr[14] * elambdat[14]) + (pptr[15] * elambdat[15]);
		p0 += (pptr[16] * elambdat[16]) + (pptr[17] * elambdat[17]);
		p0 += (pptr[18] * elambdat[18]) + (pptr[19] * elambdat[19]);	  	      	     	 
		
		pq_matrix[i].m[m][l] =  p0 * tr->rdta->invfreqaa[m];	  
		pq_matrix[i].m[l][m] =  p0 * tr->rdta->invfreqaa[l];
	      }           
	  }
      }    


#pragma omp parallel for private(cat, m, pqptr, vptr, prodl, frexm, term) reduction(+ : sum)
    for (i = 0; i < tr->cdta->endsite; i++) 
      {
	cat = cptr[i];

	term = 0;
	for (m = 0; m < 20; m++) 
	  {
	   
	    pqptr = pq_matrix[cat].m[m];
	    vptr = &(lq[i].v[0]);

	   
	    prodl = pqptr[0] * vptr[0] +  pqptr[1] * vptr[1];
	    prodl += pqptr[2] * vptr[2] +  pqptr[3] * vptr[3];
	    prodl += pqptr[4] * vptr[4] +  pqptr[5] * vptr[5];
	    prodl += pqptr[6] * vptr[6] +  pqptr[7] * vptr[7];
	    prodl += pqptr[8] * vptr[8] +  pqptr[9] * vptr[9];
	    prodl += pqptr[10] * vptr[10] +  pqptr[11] * vptr[11];
	    prodl += pqptr[12] * vptr[12] +  pqptr[13] * vptr[13];
	    prodl += pqptr[14] * vptr[14] +  pqptr[15] * vptr[15];
	    prodl += pqptr[16] * vptr[16] +  pqptr[17] * vptr[17]; 
	    prodl += pqptr[18] * vptr[18] +  pqptr[19] * vptr[19];

	    frexm = lp[i].v[m] * tr->rdta->freqaa[m];	   
	    term += (prodl * frexm);	    
	  }
	
	
	term = log(term) + (lq[i].exp + lp[i].exp)*log(minlikelihood);
	sum += term * wptr[i];
      }
   
    free(pq_matrix);
    
    tr->likelihood = sum;
    return  sum;
  }



double makenewzPROTCAT (tree *tr, nodeptr p_node, nodeptr q_node, double z0, int maxiter)
  { /* makenewz */
   
    double   z, zprev, znew;
    int     i, *cptr, cat, l, m, finish, greater, action, extended, curvatOK, decrease;
    double  dlnLdlz, d2lnLdlz2, sum, term;
    double  *rptr, ki;
    proteinlikelivector *x1, *x2;
    proteinmatrix *matrix, *matrixD, *matrixDD;
    double prod4, prod5m, prod6m, prod4m, frexm;  
    double inv_Li, dlnLidlz, d2lnLidlz2 , *v;  

    matrix = (proteinmatrix *)malloc(sizeof(proteinmatrix) * tr->NumberOfCategories);
    matrixD = (proteinmatrix *)malloc(sizeof(proteinmatrix) * tr->NumberOfCategories);
    matrixDD = (proteinmatrix *)malloc(sizeof(proteinmatrix) * tr->NumberOfCategories);

    while ((! p_node->x) || (! q_node->x)) 
      {
	if (! (p_node->x)) if (! newviewPROTCAT(tr, p_node)) return badZ;
	if (! (q_node->x)) if (! newviewPROTCAT(tr, q_node)) return badZ;
      }    
   
    x1 = p_node->x->pv;
    x2 = q_node->x->pv;      

    z = z0;   
    
    do 
      {
	int *wptr =  tr->cdta->aliaswgt, cat;
	double *wrptr   = &(tr->cdta->wr[0]);
	double *wr2ptr  = &(tr->cdta->wr2[0]);
	double *mptr, *mptrD, *mptrDD;       
	double p0, p1, p2;
	double elambdat[20], dlambdat[20], ddlambdat[20];
	double *pptr;
	double rat, *rptr;
	
	cptr  = &(tr->cdta->rateCategory[0]);	
	rptr   = &(tr->cdta->patrat[0]);

	sum = 0;
	dlnLdlz = 0.0;                 
	d2lnLdlz2 = 0.0; 
	

	zprev = z;

	if (z < zmin) 
	  z = zmin;
	else 
	  if (z > zmax) z = zmax;	

#pragma omp parallel for private(rat, l, m, pptr, p0, p1, p2, elambdat, dlambdat, ddlambdat)		      
	for(i = 0; i < tr->NumberOfCategories; i++)
	  {
	    rat =  rptr[i];
	    
	    for (l = 0; l < 20; l++) 
	      {
		elambdat[l] = exp(z * rat * tr->rdta->eigmatPROT[l]); 
		dlambdat[l] = elambdat[l] * rat * tr->rdta->eigmatPROT[l];
		ddlambdat[l] = dlambdat[l] * rat * tr->rdta->eigmatPROT[l];
	      }

	    for (m = 0; m < 20; m++) 
	      {
		for (l = 0; l <= m; l++) 
		  {
		    pptr = tr->rdta->crossMAT[m][l];
		    
		    p0 = (pptr[0] * elambdat[0]) + (pptr[1] * elambdat[1]);
		    p1 = (pptr[0] * dlambdat[0]) + (pptr[1] * dlambdat[1]);
		    p2 = (pptr[0] * ddlambdat[0]) + (pptr[1] * ddlambdat[1]);

		    p0 += (pptr[2] * elambdat[2]) + (pptr[3] * elambdat[3]);
		    p1 += (pptr[2] * dlambdat[2]) + (pptr[3] * dlambdat[3]);
		    p2 += (pptr[2] * ddlambdat[2]) + (pptr[3] * ddlambdat[3]);

		    p0 += (pptr[4] * elambdat[4]) + (pptr[5] * elambdat[5]);
		    p1 += (pptr[4] * dlambdat[4]) + (pptr[5] * dlambdat[5]);
		    p2 += (pptr[4] * ddlambdat[4]) + (pptr[5] * ddlambdat[5]);

		    p0 += (pptr[6] * elambdat[6]) + (pptr[7] * elambdat[7]);
		    p1 += (pptr[6] * dlambdat[6]) + (pptr[7] * dlambdat[7]);
		    p2 += (pptr[6] * ddlambdat[6]) + (pptr[7] * ddlambdat[7]);

		    p0 += (pptr[8] * elambdat[8]) + (pptr[9] * elambdat[9]);
		    p1 += (pptr[8] * dlambdat[8]) + (pptr[9] * dlambdat[9]);
		    p2 += (pptr[8] * ddlambdat[8]) + (pptr[9] * ddlambdat[9]);

		    p0 += (pptr[10] * elambdat[10]) + (pptr[11] * elambdat[11]);
		    p1 += (pptr[10] * dlambdat[10]) + (pptr[11] * dlambdat[11]);
		    p2 += (pptr[10] * ddlambdat[10]) + (pptr[11] * ddlambdat[11]);

		    p0 += (pptr[12] * elambdat[12]) + (pptr[13] * elambdat[13]);
		    p1 += (pptr[12] * dlambdat[12]) + (pptr[13] * dlambdat[13]);
		    p2 += (pptr[12] * ddlambdat[12]) + (pptr[13] * ddlambdat[13]);

		    p0 += (pptr[14] * elambdat[14]) + (pptr[15] * elambdat[15]);
		    p1 += (pptr[14] * dlambdat[14]) + (pptr[15] * dlambdat[15]);
		    p2 += (pptr[14] * ddlambdat[14]) + (pptr[15] * ddlambdat[15]);

		    p0 += (pptr[16] * elambdat[16]) + (pptr[17] * elambdat[17]);
		    p1 += (pptr[16] * dlambdat[16]) + (pptr[17] * dlambdat[17]);
		    p2 += (pptr[16] * ddlambdat[16]) + (pptr[17] * ddlambdat[17]);

		    p0 += (pptr[18] * elambdat[18]) + (pptr[19] * elambdat[19]);
		    p1 += (pptr[18] * dlambdat[18]) + (pptr[19] * dlambdat[19]);
		    p2 += (pptr[18] * ddlambdat[18]) + (pptr[19] * ddlambdat[19]);
		   		  		    		   		   		   		   		   		   	     
		    matrix[i].m[m][l] = p0 * tr->rdta->invfreqaa[m];
		    matrix[i].m[l][m] = p0 * tr->rdta->invfreqaa[l];
		    matrixD[i].m[m][l] = p1 * tr->rdta->invfreqaa[m];
		    matrixD[i].m[l][m] = p1 * tr->rdta->invfreqaa[l];
		    matrixDD[i].m[m][l] = p2 * tr->rdta->invfreqaa[m];
		    matrixDD[i].m[l][m] = p2 * tr->rdta->invfreqaa[l];	      
		  }
	      }
	  }      
	
#pragma omp parallel for private(mptr, mptrD, mptrDD, cat, prod4,  dlnLidlz, d2lnLidlz2, v, m, frexm, prod4m, prod5m, prod6m, inv_Li) reduction(+ :  dlnLdlz) reduction(+ : d2lnLdlz2)
	for (i = 0; i < tr->cdta->endsite; i++) 
	  {
	    prod4 = 0.0;
	    dlnLidlz = 0.0;
	    d2lnLidlz2 = 0.0;
	    
	    cat = cptr[i];

	    v = &(x2[i].v[0]);
	    
	    for (m = 0; m <= 19; m++) 
	      {		
		mptr = matrix[cat].m[m];
		mptrD = matrixD[cat].m[m]; 
		mptrDD = matrixDD[cat].m[m]; 

		frexm = x1[i].v[m] * tr->rdta->freqaa[m];				
		
		prod4m = v[0] * mptr[0] + v[1] * mptr[1];	       
		prod5m = v[0] * mptrD[0] + v[1] * mptrD[1];		   
		prod6m = v[0] * mptrDD[0]+ v[1] * mptrDD[1];
		
		prod4m += v[2] * mptr[2] + v[3] * mptr[3];
		prod5m += v[2] * mptrD[2] + v[3] * mptrD[3];		   
		prod6m += v[2] * mptrDD[2]+ v[3] * mptrDD[3];
		
		prod4m += v[4] * mptr[4] + v[5] * mptr[5];
		prod5m += v[4] * mptrD[4] + v[5] * mptrD[5];		   
		prod6m += v[4] * mptrDD[4]+ v[5] * mptrDD[5];
		
		prod4m += v[6] * mptr[6] + v[7] * mptr[7];
		prod5m += v[6] * mptrD[6] + v[7] * mptrD[7];		   
		prod6m += v[6] * mptrDD[6]+ v[7] * mptrDD[7];
		
		prod4m += v[8] * mptr[8] + v[9] * mptr[9];
		prod5m += v[8] * mptrD[8] + v[9] * mptrD[9];		   
		prod6m += v[8] * mptrDD[8]+ v[9] * mptrDD[9];
		
		prod4m += v[10] * mptr[10] + v[11] * mptr[11];
		prod5m += v[10] * mptrD[10] + v[11] * mptrD[11];		   
		prod6m += v[10] * mptrDD[10]+ v[11] * mptrDD[11];
		
		prod4m += v[12] * mptr[12] + v[13] * mptr[13];
		prod5m += v[12] * mptrD[12] + v[13] * mptrD[13];		   
		prod6m += v[12] * mptrDD[12]+ v[13] * mptrDD[13];
		
		prod4m += v[14] * mptr[14] + v[15] * mptr[15];
		prod5m += v[14] * mptrD[14] + v[15] * mptrD[15];		   
		prod6m += v[14] * mptrDD[14]+ v[15] * mptrDD[15];
		
		prod4m += v[16] * mptr[16] + v[17] * mptr[17];
		prod5m += v[16] * mptrD[16] + v[17] * mptrD[17];		   
		prod6m += v[16] * mptrDD[16]+ v[17] * mptrDD[17];
		
		prod4m += v[18] * mptr[18] + v[19] * mptr[19];
		prod5m += v[18] * mptrD[18] + v[19] * mptrD[19];		   
		prod6m += v[18] * mptrDD[18]+ v[19] * mptrDD[19];
		
		
		prod4 += frexm * prod4m;
		dlnLidlz += frexm * prod5m;
		d2lnLidlz2 +=  frexm * prod6m;
	      }
	    
	  
	    inv_Li = 1.0 / prod4;	    	   
	    dlnLidlz   *= inv_Li; 
	    d2lnLidlz2 *= inv_Li; 		
	    dlnLdlz   += wrptr[i]  * dlnLidlz;
	    d2lnLdlz2 += wr2ptr[i] *  (d2lnLidlz2 -  dlnLidlz * dlnLidlz);
	  }
		


	znew = z + dlnLdlz/fabs(d2lnLdlz2);

	if(znew < zmin || znew > zmax)	  	  
	  {	    	    	   
	    znew = z * 0.5;	   
	    action = 1;
	  }
	else	
	  {	    	   	    
	    action = 0;
	  }
       

	if(dlnLdlz < 0)
	  {
	    decrease = 1;
	    if(znew > z)
	      {
		/*printf("ERROR EXPCTED ");	       */
		znew = z * 0.5;
	      }
	  }
	else
	  {
	    decrease = 0;	
	    if(znew < z)
	      {
		/*printf("ERROR EXPCTED ");		*/
		znew = z * 0.37 + 0.63;
	      }
	  }
	z = znew;
	

      }
    while((--maxiter > 0) && fabs(z - zprev) > deltaz);
    /*printf("\n");*/

    free(matrix);
    free(matrixD);
    free(matrixDD);

    return z;      
  }



#endif

#ifdef NORMAL
#ifndef HPC
void make_pmatrix(double matrix[20][20], double lz, double rat, tree *tr)
{
  int l, m;                 
  double p0;
  double *eptr, elambdat[20], *pptr;


  eptr = elambdat;

  for (l = 0; l < 20; l++)     
    *eptr++ = exp(lz * rat * tr->rdta->eigmatPROT[l]);         
 
  for (m = 0; m < 20; m++) 
    {      
      for (l = 0; l <= m; l++) 
	{	  
	  pptr = tr->rdta->crossMAT[m][l];
	  
	  p0 = (pptr[0] * elambdat[0]) + (pptr[1] * elambdat[1]);
	  p0 += (pptr[2] * elambdat[2]) + (pptr[3] * elambdat[3]);
	  p0 += (pptr[4] * elambdat[4]) + (pptr[5] * elambdat[5]);
	  p0 += (pptr[6] * elambdat[6]) + (pptr[7] * elambdat[7]);
	  p0 += (pptr[8] * elambdat[8]) + (pptr[9] * elambdat[9]);
	  p0 += (pptr[10] * elambdat[10]) + (pptr[11] * elambdat[11]);
	  p0 += (pptr[12] * elambdat[12]) + (pptr[13] * elambdat[13]);
	  p0 += (pptr[14] * elambdat[14]) + (pptr[15] * elambdat[15]);
	  p0 += (pptr[16] * elambdat[16]) + (pptr[17] * elambdat[17]);
	  p0 += (pptr[18] * elambdat[18]) + (pptr[19] * elambdat[19]);	  	      	     	 

	  matrix[m][l] =  p0 * tr->rdta->invfreqaa[m];	  
	  matrix[l][m] =  p0 * tr->rdta->invfreqaa[l];
	}           
    }

}  /* make_pmatrix */





boolean newviewPROT (tree *tr, nodeptr p)        /*  Update likelihoods at node */
  { /* newview */
    double   zq, zr;
    nodeptr  q, r;
    proteinlikelivector *lp, *lq, *lr;
    int  i, j, l, m;
    int           code;
    yType        *yptr;
    double i_value, *val;
   
           
    if (p->tip) 
      {              
	if (p->x) return TRUE; 

	if (! getxtip(p)) return FALSE; 
	


	lp = p->x->pv;           
	yptr = p->tip;          

	for (i = 0; i < tr->cdta->endsite; i++) 
	  {
	    code = yptr[i];
	    val = & lp[i].v[0];


	    if(code < 22)
	      i_value = 0.0;
	    else
	      i_value = 1.0;	 

	    for(j = 0; j < 20; j++)
	      val[j] = i_value;

	    if(code < 20)
	      val[code] = 1.0;
	    else
	      {
		if(code == 20)	  
		  val[2] = val[3] = 1.0;
		if(code == 21)
		  val[5] = val[6] = 1.0;	  		   
	      }

	    lp[i].exp = 0;
	  }

        return TRUE;
      }
    
    {
       double r_matrix[20][20], q_matrix[20][20], *rmp, *qmp;
       double left_sum, right_sum, *rv, *qv;
       int scale;
       double p0, p1;
       double *eptr, elambdat[20], *eptrQ, elambdatQ[20], *pptr;
   
       q = p->next->back;
       r = p->next->next->back;
       
       while ((! p->x) || (! q->x) || (! r->x)) {
	 if (! q->x) if (! newviewPROT(tr, q))  return FALSE;
	 if (! r->x) if (! newviewPROT(tr, r))  return FALSE;
	 if (! p->x) if (! getxnode(p)) return FALSE;
       }
       
       lp = p->x->pv;
       
       lq = q->x->pv;
       zq = q->z;
       
       lr = r->x->pv;
       zr = r->z;            
           	
       eptr = elambdat;
       eptrQ = elambdatQ;

       for (l = 0; l < 20; l++)     
	 {
	   *eptr++ = exp(zr * tr->rdta->eigmatPROT[l]);         
	   *eptrQ++ = exp(zq * tr->rdta->eigmatPROT[l]);  
	 }
 
       for (m = 0; m < 20; m++) 
	 {      
	   for (l = 0; l <= m; l++) 
	     {	  
	       pptr = tr->rdta->crossMAT[m][l];
	       
	       p0 = (pptr[0] * elambdat[0]) + (pptr[1] * elambdat[1]);		  
	       p1 = (pptr[0] * elambdatQ[0]) + (pptr[1] * elambdatQ[1]);
	       
	       p0 += (pptr[2] * elambdat[2]) + (pptr[3] * elambdat[3]);
	       p1 += (pptr[2] * elambdatQ[2]) + (pptr[3] * elambdatQ[3]);
	       
	       p0 += (pptr[4] * elambdat[4]) + (pptr[5] * elambdat[5]);
	       p1 += (pptr[4] * elambdatQ[4]) + (pptr[5] * elambdatQ[5]);
	       
	       p0 += (pptr[6] * elambdat[6]) + (pptr[7] * elambdat[7]);
	       p1 += (pptr[6] * elambdatQ[6]) + (pptr[7] * elambdatQ[7]);
	       
	       p0 += (pptr[8] * elambdat[8]) + (pptr[9] * elambdat[9]);
	       p1 += (pptr[8] * elambdatQ[8]) + (pptr[9] * elambdatQ[9]);
	       
	       p0 += (pptr[10] * elambdat[10]) + (pptr[11] * elambdat[11]);
	       p1 += (pptr[10] * elambdatQ[10]) + (pptr[11] * elambdatQ[11]);
	       
	       p0 += (pptr[12] * elambdat[12]) + (pptr[13] * elambdat[13]);
	       p1 += (pptr[12] * elambdatQ[12]) + (pptr[13] * elambdatQ[13]);
	       
	       p0 += (pptr[14] * elambdat[14]) + (pptr[15] * elambdat[15]);
	       p1 += (pptr[14] * elambdatQ[14]) + (pptr[15] * elambdatQ[15]);
	       
	       p0 += (pptr[16] * elambdat[16]) + (pptr[17] * elambdat[17]);
	       p1 += (pptr[16] * elambdatQ[16]) + (pptr[17] * elambdatQ[17]);
	       
	       p0 += (pptr[18] * elambdat[18]) + (pptr[19] * elambdat[19]);	  	      	     	 
	       p1 += (pptr[18] * elambdatQ[18]) + (pptr[19] * elambdatQ[19]);		 	  	      	     	 
	       
	       r_matrix[m][l] =  p0 * tr->rdta->invfreqaa[m];	  
	       r_matrix[l][m] =  p0 * tr->rdta->invfreqaa[l];
	       	       
	       q_matrix[m][l] =  p1 * tr->rdta->invfreqaa[m];	  
	       q_matrix[l][m] =  p1 * tr->rdta->invfreqaa[l];
		 
	     }           
	 }         	 	  	  
       
       for (i = 0; i < tr->cdta->endsite; i++) 
	 {	 	  	  
	   scale = 1;
	   
	   rv = &(lr[i].v[0]);
	   qv = &(lq[i].v[0]);

	   for (m = 0; m < 20; m++)
	     {
	       rmp = r_matrix[m];
	       qmp = q_matrix[m];
	       
	       left_sum = (rv[0] * rmp[0]) + (rv[1] * rmp[1]);
	       right_sum = (qv[0] * qmp[0]) + (qv[1] * qmp[1]);

	       left_sum += (rv[2] * rmp[2]) + (rv[3] * rmp[3]);
	       right_sum += (qv[2] * qmp[2]) + (qv[3] * qmp[3]);

	       left_sum += (rv[4] * rmp[4]) + (rv[5] * rmp[5]);
	       right_sum += (qv[4] * qmp[4]) + (qv[5] * qmp[5]);

	       left_sum += (rv[6] * rmp[6]) + (rv[7] * rmp[7]);
	       right_sum += (qv[6] * qmp[6]) + (qv[7] * qmp[7]);

	       left_sum += (rv[8] * rmp[8]) + (rv[9] * rmp[9]);
	       right_sum += (qv[8] * qmp[8]) + (qv[9] * qmp[9]);

	       left_sum += (rv[10] * rmp[10]) + (rv[11] * rmp[11]);
	       right_sum += (qv[10] * qmp[10]) + (qv[11] * qmp[11]);

	       left_sum += (rv[12] * rmp[12]) + (rv[13] * rmp[13]);
	       right_sum += (qv[12] * qmp[12]) + (qv[13] * qmp[13]);

	       left_sum += (rv[14] * rmp[14]) + (rv[15] * rmp[15]);
	       right_sum += (qv[14] * qmp[14]) + (qv[15] * qmp[15]);

	       left_sum += (rv[16] * rmp[16]) + (rv[17] * rmp[17]);
	       right_sum += (qv[16] * qmp[16]) + (qv[17] * qmp[17]);

	       left_sum += (rv[18] * rmp[18]) + (rv[19] * rmp[19]);
	       right_sum += (qv[18] * qmp[18]) + (qv[19] * qmp[19]);

	      
	       lp[i].v[m] = left_sum * right_sum;	   

	       if(lp[i].v[m] >= minlikelihood)	      
		 scale = 0;	       	      
	     }

	   lp[i].exp = lq[i].exp + lr[i].exp;

	   if(scale) 
	     {	     
	       for(m = 0; m < 20; m++)
		 lp[i].v[m] *= twotothe256;
	       lp[i].exp += 1;
	     }		
	 }   
    }

    return TRUE;
  }

boolean newviewPartialPROTCAT (tree *tr, nodeptr p, int i, double rate)        /*  Update likelihoods at node */
  { /* newview */
    double   zq, zr;
    nodeptr  q, r;
    proteinlikelivector *lp, *lq, *lr;
    int  j, l, m;
    int           code;
    yType        *yptr;
    double i_value, *val;
   
           
    if (p->tip) 
      {              
	if (p->x) return TRUE; 

	if (! getxtip(p)) return FALSE; 
	


	lp = p->x->pv;           
	yptr = p->tip;          

	code = yptr[i];
	val = & lp[i].v[0];


	if(code < 22)
	  i_value = 0.0;
	else
	  i_value = 1.0;	 

	for(j = 0; j < 20; j++)
	  val[j] = i_value;

	if(code < 20)
	  val[code] = 1.0;
	else
	  {
	    if(code == 20)	  
	      val[2] = val[3] = 1.0;
	    if(code == 21)
	      val[5] = val[6] = 1.0;	  		   
	  }

	lp[i].exp = 0;
	
        return TRUE;
      }
    
    {
       double r_matrix[20][20], q_matrix[20][20], *rmp, *qmp;
       double left_sum, right_sum, *rv, *qv;
       int scale;

   
       q = p->next->back;
       r = p->next->next->back;
       
       while ((! p->x) || (! q->x) || (! r->x)) {
	 if (! q->x) if (! newviewPartialPROTCAT(tr, q, i, rate))  return FALSE;
	 if (! r->x) if (! newviewPartialPROTCAT(tr, r, i, rate))  return FALSE;
	 if (! p->x) if (! getxnode(p)) return FALSE;
       }
       
       lp = p->x->pv;
       
       lq = q->x->pv;
       zq = q->z;
       
       lr = r->x->pv;
       zr = r->z;            
       
       make_pmatrix(r_matrix, zr, rate, tr);
       make_pmatrix(q_matrix, zq, rate, tr);

      
       scale = 1;
	   
       rv = &(lr[i].v[0]);
       qv = &(lq[i].v[0]);

       for (m = 0; m < 20; m++)
	 {
	   rmp = r_matrix[m];
	   qmp = q_matrix[m];
	   
	   left_sum = (rv[0] * rmp[0]) + (rv[1] * rmp[1]);
	   right_sum = (qv[0] * qmp[0]) + (qv[1] * qmp[1]);
	   
	   left_sum += (rv[2] * rmp[2]) + (rv[3] * rmp[3]);
	   right_sum += (qv[2] * qmp[2]) + (qv[3] * qmp[3]);
	   
	   left_sum += (rv[4] * rmp[4]) + (rv[5] * rmp[5]);
	   right_sum += (qv[4] * qmp[4]) + (qv[5] * qmp[5]);
	   
	   left_sum += (rv[6] * rmp[6]) + (rv[7] * rmp[7]);
	   right_sum += (qv[6] * qmp[6]) + (qv[7] * qmp[7]);
	   
	   left_sum += (rv[8] * rmp[8]) + (rv[9] * rmp[9]);
	   right_sum += (qv[8] * qmp[8]) + (qv[9] * qmp[9]);
	   
	   left_sum += (rv[10] * rmp[10]) + (rv[11] * rmp[11]);
	   right_sum += (qv[10] * qmp[10]) + (qv[11] * qmp[11]);
	   
	   left_sum += (rv[12] * rmp[12]) + (rv[13] * rmp[13]);
	   right_sum += (qv[12] * qmp[12]) + (qv[13] * qmp[13]);
	   
	   left_sum += (rv[14] * rmp[14]) + (rv[15] * rmp[15]);
	   right_sum += (qv[14] * qmp[14]) + (qv[15] * qmp[15]);
	   
	   left_sum += (rv[16] * rmp[16]) + (rv[17] * rmp[17]);
	   right_sum += (qv[16] * qmp[16]) + (qv[17] * qmp[17]);
	   
	   left_sum += (rv[18] * rmp[18]) + (rv[19] * rmp[19]);
	   right_sum += (qv[18] * qmp[18]) + (qv[19] * qmp[19]);
	   
	   
	   lp[i].v[m] = left_sum * right_sum;	   
	   
	   if(lp[i].v[m] >= minlikelihood)	      
	     scale = 0;	       	      
	 }

       lp[i].exp = lq[i].exp + lr[i].exp;

       if(scale) 
	 {	     
	   for(m = 0; m < 20; m++)
	     lp[i].v[m] *= twotothe256;
	   lp[i].exp += 1;
	 }		       
    }


    return TRUE;
  }


double evaluatePROT (tree *tr, nodeptr p)
  { /* evaluate */
    double   sum, z, lz, xvlz,
      ki, term, zz, zv, frexm, prodl;
    double        *zztable, *zvtable,
                 *zzptr, *zvptr;
    double        *rptr, *vptr, *mptr;
    proteinlikelivector *lp, *lq;
    nodeptr       q;
    int           cat, *cptr, i, *wptr, m, l;
    double pq_matrix[20][20];
    double p0;
    double *eptr, elambdat[20], *pptr;

    q = p->back;

    while ((! p->x) || (! q->x)) 
      {
	if (! (p->x)) if (! newviewPROT(tr, p)) return badEval;
	if (! (q->x)) if (! newviewPROT(tr, q)) return badEval;
      }

    lp = p->x->pv;
    lq = q->x->pv;

    z = p->z;
    if (z < zmin) z = zmin;
   
    wptr = tr->cdta->aliaswgt;
    sum = 0;
    eptr = elambdat;	      
	
    for (l = 0; l < 20; l++)     
      *eptr++ = exp(z * tr->rdta->eigmatPROT[l]);         

    for (m = 0; m < 20; m++) 
      {      
	for (l = 0; l <= m; l++) 
	  {	  
	    pptr = tr->rdta->crossMAT[m][l];
	    
	    p0 = (pptr[0] * elambdat[0]) + (pptr[1] * elambdat[1]);
	    p0 += (pptr[2] * elambdat[2]) + (pptr[3] * elambdat[3]);
	    p0 += (pptr[4] * elambdat[4]) + (pptr[5] * elambdat[5]);
	    p0 += (pptr[6] * elambdat[6]) + (pptr[7] * elambdat[7]);
	    p0 += (pptr[8] * elambdat[8]) + (pptr[9] * elambdat[9]);
	    p0 += (pptr[10] * elambdat[10]) + (pptr[11] * elambdat[11]);
	    p0 += (pptr[12] * elambdat[12]) + (pptr[13] * elambdat[13]);
	    p0 += (pptr[14] * elambdat[14]) + (pptr[15] * elambdat[15]);
	    p0 += (pptr[16] * elambdat[16]) + (pptr[17] * elambdat[17]);
	    p0 += (pptr[18] * elambdat[18]) + (pptr[19] * elambdat[19]);	  	      	     	 
		
	    pq_matrix[m][l] =  p0 * tr->rdta->invfreqaa[m];	  
	    pq_matrix[l][m] =  p0 * tr->rdta->invfreqaa[l];
	  }           
      }
  

    for (i = 0; i < tr->cdta->endsite; i++) 
      {
	term = 0;
	for (m = 0; m < 20; m++) 
	  {	   
	    vptr = &(lq[i].v[0]);
	    mptr = pq_matrix[m];

	    prodl = mptr[0] * vptr[0] +  mptr[1] * vptr[1];
	    prodl += mptr[2] * vptr[2] +  mptr[3] * vptr[3];
	    prodl += mptr[4] * vptr[4] +  mptr[5] * vptr[5];
	    prodl += mptr[6] * vptr[6] +  mptr[7] * vptr[7];
	    prodl += mptr[8] * vptr[8] +  mptr[9] * vptr[9];
	    prodl += mptr[10] * vptr[10] +  mptr[11] * vptr[11];
	    prodl += mptr[12] * vptr[12] +  mptr[13] * vptr[13];
	    prodl += mptr[14] * vptr[14] +  mptr[15] * vptr[15];
	    prodl += mptr[16] * vptr[16] +  mptr[17] * vptr[17]; 
	    prodl += mptr[18] * vptr[18] +  mptr[19] * vptr[19];
			   	    
	    frexm = lp[i].v[m] * tr->rdta->freqaa[m];	   
	    term += (prodl * frexm);	    
	  }
		
	term = log(term) + (lq[i].exp + lp[i].exp)*log(minlikelihood);
	sum += term * wptr[i];
      }
   
   
    tr->likelihood = sum;
    return  sum;
  }


double evaluatePartialPROTCAT (tree *tr, nodeptr p, int i, double rate)
  { /* evaluate */
    double   sum, z, lz, xvlz,
      ki, term, zz, zv, frexm, prodl;
    double        *zztable, *zvtable,
                 *zzptr, *zvptr;
    double        *rptr;
    proteinlikelivector *lp, *lq;
    nodeptr       q;
    int           cat, *cptr, *wptr, m, l;
    double pq_matrix[20][20];

    q = p->back;

    while ((! p->x) || (! q->x)) 
      {
	if (! (p->x)) if (! newviewPartialPROTCAT(tr, p, i, rate)) return badEval;
	if (! (q->x)) if (! newviewPartialPROTCAT(tr, q, i, rate)) return badEval;
      }

    lp = p->x->pv;
    lq = q->x->pv;

    z = p->z;
    if (z < zmin) z = zmin;
   
    wptr = tr->cdta->aliaswgt;

    
    make_pmatrix(pq_matrix, z, 1.0, tr);       

    sum = 0;



    
    term = 0;
    for (m = 0; m < 20; m++) 
      {
	prodl = 0;	   
	for (l = 0; l < 20; l++)
	  prodl += (pq_matrix[m][l] * lq[i].v[l]);
	frexm = lp[i].v[m] * tr->rdta->freqaa[m];	   
	term += (prodl * frexm);	    
      }
    
	
    term = log(term) + (lq[i].exp + lp[i].exp)*log(minlikelihood);
    sum += term * wptr[i];  
   


    tr->likelihood = sum;
    return  sum;
  }







double makenewzPROT (tree *tr, nodeptr p_node, nodeptr q_node, double z0, int maxiter)
  { /* makenewz */
   
    double   z, zprev, znew;
    int     i, *cptr, cat, l, m, finish, greater, action, extended, curvatOK, decrease;
    double  dlnLdlz, d2lnLdlz2, sum, term;
    double  *rptr, ki;
    proteinlikelivector *x1, *x2;
    double matrix[20][20], matrixD[20][20], matrixDD[20][20];
    double prod4, prod5m, prod6m, prod4m, frexm;  
    double inv_Li, dlnLidlz, d2lnLidlz2 , *v;  

    while ((! p_node->x) || (! q_node->x)) 
      {
	if (! (p_node->x)) if (! newviewPROT(tr, p_node)) return badZ;
	if (! (q_node->x)) if (! newviewPROT(tr, q_node)) return badZ;
      }    
   
    x1 = p_node->x->pv;
    x2 = q_node->x->pv;      

    z = z0;   
    
    do 
      {
	int *wptr =  tr->cdta->aliaswgt;
	double p0, p1, p2;
	double elambdat[20], dlambdat[20], ddlambdat[20];
	double *pptr;

	double *wrptr   = &(tr->cdta->wr[0]);
	double *wr2ptr  = &(tr->cdta->wr2[0]);
	rptr   = &(tr->cdta->patrat[0]); 
	cptr  = &(tr->cdta->rateCategory[0]);	
	
	sum = 0;
	dlnLdlz = 0.0;                 
	d2lnLdlz2 = 0.0; 
	

	zprev = z;

	if (z < zmin) 
	  z = zmin;
	else 
	  if (z > zmax) z = zmax;	
	
	for (l = 0; l < 20; l++) 
	  {
	    elambdat[l] = exp(z * tr->rdta->eigmatPROT[l]); 
	    dlambdat[l] = elambdat[l] * tr->rdta->eigmatPROT[l];
	    ddlambdat[l] = dlambdat[l] * tr->rdta->eigmatPROT[l];
	  }

	for (m = 0; m < 20; m++) 
	  {
	    for (l = 0; l <= m; l++) 
	      {
		pptr = tr->rdta->crossMAT[m][l];
		    
		p0 = (pptr[0] * elambdat[0]) + (pptr[1] * elambdat[1]);
		p1 = (pptr[0] * dlambdat[0]) + (pptr[1] * dlambdat[1]);
		p2 = (pptr[0] * ddlambdat[0]) + (pptr[1] * ddlambdat[1]);
		
		p0 += (pptr[2] * elambdat[2]) + (pptr[3] * elambdat[3]);
		p1 += (pptr[2] * dlambdat[2]) + (pptr[3] * dlambdat[3]);
		p2 += (pptr[2] * ddlambdat[2]) + (pptr[3] * ddlambdat[3]);
		
		p0 += (pptr[4] * elambdat[4]) + (pptr[5] * elambdat[5]);
		p1 += (pptr[4] * dlambdat[4]) + (pptr[5] * dlambdat[5]);
		p2 += (pptr[4] * ddlambdat[4]) + (pptr[5] * ddlambdat[5]);
		
		p0 += (pptr[6] * elambdat[6]) + (pptr[7] * elambdat[7]);
		p1 += (pptr[6] * dlambdat[6]) + (pptr[7] * dlambdat[7]);
		p2 += (pptr[6] * ddlambdat[6]) + (pptr[7] * ddlambdat[7]);
		
		p0 += (pptr[8] * elambdat[8]) + (pptr[9] * elambdat[9]);
		p1 += (pptr[8] * dlambdat[8]) + (pptr[9] * dlambdat[9]);
		p2 += (pptr[8] * ddlambdat[8]) + (pptr[9] * ddlambdat[9]);
		
		p0 += (pptr[10] * elambdat[10]) + (pptr[11] * elambdat[11]);
		p1 += (pptr[10] * dlambdat[10]) + (pptr[11] * dlambdat[11]);
		p2 += (pptr[10] * ddlambdat[10]) + (pptr[11] * ddlambdat[11]);
		
		p0 += (pptr[12] * elambdat[12]) + (pptr[13] * elambdat[13]);
		p1 += (pptr[12] * dlambdat[12]) + (pptr[13] * dlambdat[13]);
		p2 += (pptr[12] * ddlambdat[12]) + (pptr[13] * ddlambdat[13]);
		
		p0 += (pptr[14] * elambdat[14]) + (pptr[15] * elambdat[15]);
		p1 += (pptr[14] * dlambdat[14]) + (pptr[15] * dlambdat[15]);
		p2 += (pptr[14] * ddlambdat[14]) + (pptr[15] * ddlambdat[15]);
		
		p0 += (pptr[16] * elambdat[16]) + (pptr[17] * elambdat[17]);
		p1 += (pptr[16] * dlambdat[16]) + (pptr[17] * dlambdat[17]);
		p2 += (pptr[16] * ddlambdat[16]) + (pptr[17] * ddlambdat[17]);
		
		p0 += (pptr[18] * elambdat[18]) + (pptr[19] * elambdat[19]);
		p1 += (pptr[18] * dlambdat[18]) + (pptr[19] * dlambdat[19]);
		p2 += (pptr[18] * ddlambdat[18]) + (pptr[19] * ddlambdat[19]);
		
		matrix[m][l] = p0 * tr->rdta->invfreqaa[m];
		matrix[l][m] = p0 * tr->rdta->invfreqaa[l];
		matrixD[m][l] = p1 * tr->rdta->invfreqaa[m];
		matrixD[l][m] = p1 * tr->rdta->invfreqaa[l];
		matrixDD[m][l] = p2 * tr->rdta->invfreqaa[m];
		matrixDD[l][m] = p2 * tr->rdta->invfreqaa[l];	      
	      }
	  }            


	for (i = 0; i < tr->cdta->endsite; i++) 
	  {
	    prod4 = 0.0;
	    dlnLidlz = 0.0;
	    d2lnLidlz2 = 0.0;
	    
	    v = &(x2[i].v[0]);
	    
	    for (m = 0; m <= 19; m++) 
	      {		
		frexm = x1[i].v[m] * tr->rdta->freqaa[m];				
		
		prod4m = v[0] * matrix[m][0] + v[1] * matrix[m][1];	       
		prod5m = v[0] * matrixD[m][0] + v[1] * matrixD[m][1];		   
		prod6m = v[0] * matrixDD[m][0]+ v[1] * matrixDD[m][1];
		
		prod4m += v[2] * matrix[m][2] + v[3] * matrix[m][3];
		prod5m += v[2] * matrixD[m][2] + v[3] * matrixD[m][3];		   
		prod6m += v[2] * matrixDD[m][2]+ v[3] * matrixDD[m][3];
		
		prod4m += v[4] * matrix[m][4] + v[5] * matrix[m][5];
		prod5m += v[4] * matrixD[m][4] + v[5] * matrixD[m][5];		   
		prod6m += v[4] * matrixDD[m][4]+ v[5] * matrixDD[m][5];
		
		prod4m += v[6] * matrix[m][6] + v[7] * matrix[m][7];
		prod5m += v[6] * matrixD[m][6] + v[7] * matrixD[m][7];		   
		prod6m += v[6] * matrixDD[m][6]+ v[7] * matrixDD[m][7];
		
		prod4m += v[8] * matrix[m][8] + v[9] * matrix[m][9];
		prod5m += v[8] * matrixD[m][8] + v[9] * matrixD[m][9];		   
		prod6m += v[8] * matrixDD[m][8]+ v[9] * matrixDD[m][9];
		
		prod4m += v[10] * matrix[m][10] + v[11] * matrix[m][11];
		prod5m += v[10] * matrixD[m][10] + v[11] * matrixD[m][11];		   
		prod6m += v[10] * matrixDD[m][10]+ v[11] * matrixDD[m][11];
		
		prod4m += v[12] * matrix[m][12] + v[13] * matrix[m][13];
		prod5m += v[12] * matrixD[m][12] + v[13] * matrixD[m][13];		   
		prod6m += v[12] * matrixDD[m][12]+ v[13] * matrixDD[m][13];
		
		prod4m += v[14] * matrix[m][14] + v[15] * matrix[m][15];
		prod5m += v[14] * matrixD[m][14] + v[15] * matrixD[m][15];		   
		prod6m += v[14] * matrixDD[m][14]+ v[15] * matrixDD[m][15];
		
		prod4m += v[16] * matrix[m][16] + v[17] * matrix[m][17];
		prod5m += v[16] * matrixD[m][16] + v[17] * matrixD[m][17];		   
		prod6m += v[16] * matrixDD[m][16]+ v[17] * matrixDD[m][17];
		
		prod4m += v[18] * matrix[m][18] + v[19] * matrix[m][19];
		prod5m += v[18] * matrixD[m][18] + v[19] * matrixD[m][19];		   
		prod6m += v[18] * matrixDD[m][18]+ v[19] * matrixDD[m][19];
		
		
		prod4 += frexm * prod4m;
		dlnLidlz += frexm * prod5m;
		d2lnLidlz2 +=  frexm * prod6m;
	      }
	    
	  
	    inv_Li = 1.0 / prod4;	    	   
	    dlnLidlz   *= inv_Li; 
	    d2lnLidlz2 *= inv_Li; 		
	    dlnLdlz   += *wrptr++  * dlnLidlz;
	    d2lnLdlz2 += *wr2ptr++ *  (d2lnLidlz2 -  dlnLidlz * dlnLidlz);
	  }
		


	znew = z + dlnLdlz/fabs(d2lnLdlz2);

	if(znew < zmin || znew > zmax)	  	  
	  {	    	    	   
	    znew = z * 0.5;	   
	    action = 1;
	  }
	else	
	  {	    	   	    
	    action = 0;
	  }
       

	if(dlnLdlz < 0)
	  {
	    decrease = 1;
	    if(znew > z)
	      {
		/*printf("ERROR EXPCTED ");	       */
		znew = z * 0.5;
	      }
	  }
	else
	  {
	    decrease = 0;	
	    if(znew < z)
	      {
		/*printf("ERROR EXPCTED ");		*/
		znew = z * 0.37 + 0.63;
	      }
	  }
	z = znew;
	

      }
    while((--maxiter > 0) && fabs(z - zprev) > deltaz);
    /*printf("\n");*/
    return z;      
  }





void make_pmatrixCAT(proteinmatrix *matrix, double lz, tree *tr)
{
  int l, m, i;                 
  double p0;
  double *eptr, elambdat[20], *pptr, *rptr;
  double rat;

  rptr   = &(tr->cdta->patrat[0]); 
  
  for(i = 0; i < tr->NumberOfCategories; i++)
    {    
      eptr = elambdat;
      
      rat = *rptr++;

      for (l = 0; l < 20; l++)     
	*eptr++ = exp(lz * rat * tr->rdta->eigmatPROT[l]);         

      for (m = 0; m < 20; m++) 
	{      
	  for (l = 0; l <= m; l++) 
	    {	  
	      pptr = tr->rdta->crossMAT[m][l];
	      
	      p0 = (pptr[0] * elambdat[0]) + (pptr[1] * elambdat[1]);
	      p0 += (pptr[2] * elambdat[2]) + (pptr[3] * elambdat[3]);
	      p0 += (pptr[4] * elambdat[4]) + (pptr[5] * elambdat[5]);
	      p0 += (pptr[6] * elambdat[6]) + (pptr[7] * elambdat[7]);
	      p0 += (pptr[8] * elambdat[8]) + (pptr[9] * elambdat[9]);
	      p0 += (pptr[10] * elambdat[10]) + (pptr[11] * elambdat[11]);
	      p0 += (pptr[12] * elambdat[12]) + (pptr[13] * elambdat[13]);
	      p0 += (pptr[14] * elambdat[14]) + (pptr[15] * elambdat[15]);
	      p0 += (pptr[16] * elambdat[16]) + (pptr[17] * elambdat[17]);
	      p0 += (pptr[18] * elambdat[18]) + (pptr[19] * elambdat[19]);	  	      	     	 
	      
	      matrix[i].m[m][l] =  p0 * tr->rdta->invfreqaa[m];	  
	      matrix[i].m[l][m] =  p0 * tr->rdta->invfreqaa[l];
	    }           
	}
    }

}  /* make_pmatrix */





boolean newviewPROTCAT (tree *tr, nodeptr p)        /*  Update likelihoods at node */
  { /* newview */
    double   zq, zr;
    nodeptr  q, r;
    proteinlikelivector *lp, *lq, *lr;
    int  i, j, l, m;
    int           code;
    yType        *yptr;
    double i_value, *val;
   
           
    if (p->tip) 
      {              
	if (p->x) return TRUE; 

	if (! getxtip(p)) return FALSE; 
	


	lp = p->x->pv;           
	yptr = p->tip;          

	for (i = 0; i < tr->cdta->endsite; i++) 
	  {
	    code = yptr[i];
	    val = & lp[i].v[0];


	    if(code < 22)
	      i_value = 0.0;
	    else
	      i_value = 1.0;	 

	    for(j = 0; j < 20; j++)
	      val[j] = i_value;

	    if(code < 20)
	      val[code] = 1.0;
	    else
	      {
		if(code == 20)	  
		  val[2] = val[3] = 1.0;
		if(code == 21)
		  val[5] = val[6] = 1.0;	  		   
	      }

	    lp[i].exp = 0;
	  }

        return TRUE;
      }
    
    {
      proteinmatrix *r_matrix, *q_matrix;
      double *rmp, *qmp;
      double left_sum, right_sum, *rv, *qv;
      int scale, *cptr, cat;
      double *eptr, elambdat[20], *eptrQ, elambdatQ[20];
      double rat, *rptr, *pptr;
      double p0, p1;

      r_matrix = (proteinmatrix *)malloc(sizeof(proteinmatrix) * tr->NumberOfCategories);
      q_matrix = (proteinmatrix *)malloc(sizeof(proteinmatrix) * tr->NumberOfCategories);

   
      q = p->next->back;
      r = p->next->next->back;
       
      while ((! p->x) || (! q->x) || (! r->x)) {
	if (! q->x) if (! newviewPROTCAT(tr, q))  return FALSE;
	if (! r->x) if (! newviewPROTCAT(tr, r))  return FALSE;
	if (! p->x) if (! getxnode(p)) return FALSE;
      }
       
      lp = p->x->pv;
      
      lq = q->x->pv;
      zq = q->z;
      
      lr = r->x->pv;
      zr = r->z;            
                  
      rptr = &(tr->cdta->patrat[0]); 
      cptr  = &(tr->cdta->rateCategory[0]);

      for(i = 0; i < tr->NumberOfCategories; i++)
	{    
	  eptr = elambdat;
	  eptrQ = elambdatQ;
	     
	  rat = *rptr++;

	  for (l = 0; l < 20; l++)     
	    {
	      *eptr++ = exp(zr * rat * tr->rdta->eigmatPROT[l]);         
	      *eptrQ++ = exp(zq * rat * tr->rdta->eigmatPROT[l]); 
	    }

	  for (m = 0; m < 20; m++) 
	    {      
	      for (l = 0; l <= m; l++) 
		{	  
		  pptr = tr->rdta->crossMAT[m][l];
	      
		  p0 = (pptr[0] * elambdat[0]) + (pptr[1] * elambdat[1]);		  
		  p1 = (pptr[0] * elambdatQ[0]) + (pptr[1] * elambdatQ[1]);

		  p0 += (pptr[2] * elambdat[2]) + (pptr[3] * elambdat[3]);
		  p1 += (pptr[2] * elambdatQ[2]) + (pptr[3] * elambdatQ[3]);

		  p0 += (pptr[4] * elambdat[4]) + (pptr[5] * elambdat[5]);
		  p1 += (pptr[4] * elambdatQ[4]) + (pptr[5] * elambdatQ[5]);

		  p0 += (pptr[6] * elambdat[6]) + (pptr[7] * elambdat[7]);
		  p1 += (pptr[6] * elambdatQ[6]) + (pptr[7] * elambdatQ[7]);

		  p0 += (pptr[8] * elambdat[8]) + (pptr[9] * elambdat[9]);
		  p1 += (pptr[8] * elambdatQ[8]) + (pptr[9] * elambdatQ[9]);

		  p0 += (pptr[10] * elambdat[10]) + (pptr[11] * elambdat[11]);
		  p1 += (pptr[10] * elambdatQ[10]) + (pptr[11] * elambdatQ[11]);

		  p0 += (pptr[12] * elambdat[12]) + (pptr[13] * elambdat[13]);
		  p1 += (pptr[12] * elambdatQ[12]) + (pptr[13] * elambdatQ[13]);
		  
		  p0 += (pptr[14] * elambdat[14]) + (pptr[15] * elambdat[15]);
		  p1 += (pptr[14] * elambdatQ[14]) + (pptr[15] * elambdatQ[15]);

		  p0 += (pptr[16] * elambdat[16]) + (pptr[17] * elambdat[17]);
		  p1 += (pptr[16] * elambdatQ[16]) + (pptr[17] * elambdatQ[17]);

		  p0 += (pptr[18] * elambdat[18]) + (pptr[19] * elambdat[19]);	  	      	     	 
		  p1 += (pptr[18] * elambdatQ[18]) + (pptr[19] * elambdatQ[19]);	
		  		  		 		 		 		 		 		 		  		  
		  r_matrix[i].m[m][l] =  p0 * tr->rdta->invfreqaa[m];	  
		  r_matrix[i].m[l][m] =  p0 * tr->rdta->invfreqaa[l];
		  
		  q_matrix[i].m[m][l] =  p1 * tr->rdta->invfreqaa[m];	  
		  q_matrix[i].m[l][m] =  p1 * tr->rdta->invfreqaa[l];		   
		}           
	    }
	}    
                    

       for (i = 0; i < tr->cdta->endsite; i++) 
	 {
	   cat = *cptr++;
	 	  	  
	   scale = 1;
	   
	   rv = &(lr[i].v[0]);
	   qv = &(lq[i].v[0]);

	   for (m = 0; m < 20; m++)
	     {
	       rmp = r_matrix[cat].m[m];
	       qmp = q_matrix[cat].m[m];
	       
	       left_sum = (rv[0] * rmp[0]) + (rv[1] * rmp[1]);
	       right_sum = (qv[0] * qmp[0]) + (qv[1] * qmp[1]);

	       left_sum += (rv[2] * rmp[2]) + (rv[3] * rmp[3]);
	       right_sum += (qv[2] * qmp[2]) + (qv[3] * qmp[3]);

	       left_sum += (rv[4] * rmp[4]) + (rv[5] * rmp[5]);
	       right_sum += (qv[4] * qmp[4]) + (qv[5] * qmp[5]);

	       left_sum += (rv[6] * rmp[6]) + (rv[7] * rmp[7]);
	       right_sum += (qv[6] * qmp[6]) + (qv[7] * qmp[7]);

	       left_sum += (rv[8] * rmp[8]) + (rv[9] * rmp[9]);
	       right_sum += (qv[8] * qmp[8]) + (qv[9] * qmp[9]);

	       left_sum += (rv[10] * rmp[10]) + (rv[11] * rmp[11]);
	       right_sum += (qv[10] * qmp[10]) + (qv[11] * qmp[11]);

	       left_sum += (rv[12] * rmp[12]) + (rv[13] * rmp[13]);
	       right_sum += (qv[12] * qmp[12]) + (qv[13] * qmp[13]);

	       left_sum += (rv[14] * rmp[14]) + (rv[15] * rmp[15]);
	       right_sum += (qv[14] * qmp[14]) + (qv[15] * qmp[15]);

	       left_sum += (rv[16] * rmp[16]) + (rv[17] * rmp[17]);
	       right_sum += (qv[16] * qmp[16]) + (qv[17] * qmp[17]);

	       left_sum += (rv[18] * rmp[18]) + (rv[19] * rmp[19]);
	       right_sum += (qv[18] * qmp[18]) + (qv[19] * qmp[19]);

	      
	       lp[i].v[m] = left_sum * right_sum;	   

	       if(lp[i].v[m] >= minlikelihood)	      
		 scale = 0;	       	      
	     }

	   lp[i].exp = lq[i].exp + lr[i].exp;

	   if(scale) 
	     {	     
	       for(m = 0; m < 20; m++)
		 lp[i].v[m] *= twotothe256;
	       lp[i].exp += 1;
	     }		
	 }
    
       free(r_matrix);
       free(q_matrix);
    }

    return TRUE;
  }




double evaluatePROTCAT (tree *tr, nodeptr p)
  { /* evaluate */
    double   sum, z, lz, xvlz,
      ki, term, zz, zv, frexm, prodl;
    double        *zztable, *zvtable,
                 *zzptr, *zvptr;
    double        *rptr, *pqptr, *vptr;
    proteinlikelivector *lp, *lq;
    nodeptr       q;
    int           cat, *cptr, i, *wptr, m, l;
    proteinmatrix *pq_matrix;
    double p0;
    double *eptr, elambdat[20], *pptr;
    double rat;

    q = p->back;
    pq_matrix = (proteinmatrix *)malloc(sizeof(proteinmatrix) * tr->NumberOfCategories);
     
    while ((! p->x) || (! q->x)) 
      {
	if (! (p->x)) if (! newviewPROTCAT(tr, p)) 
	  {
	    printf("BAD EVAL\n");
	    return badEval;
	  }
	if (! (q->x)) if (! newviewPROTCAT(tr, q)) 
	  {
	    printf("BAD EVAL\n");
	    return badEval;
	  }
      }

    lp = p->x->pv;
    lq = q->x->pv;

    z = p->z;
    if (z < zmin) z = zmin;
   
    wptr = tr->cdta->aliaswgt;
    cptr  = &(tr->cdta->rateCategory[0]);    
    rptr   = &(tr->cdta->patrat[0]); 

    sum = 0;

    for(i = 0; i < tr->NumberOfCategories; i++)
      {    
	eptr = elambdat;
	
	rat = *rptr++;
	
	for (l = 0; l < 20; l++)     
	  *eptr++ = exp(z * rat * tr->rdta->eigmatPROT[l]);         

	for (m = 0; m < 20; m++) 
	  {      
	    for (l = 0; l <= m; l++) 
	      {	  
		pptr = tr->rdta->crossMAT[m][l];
		
		p0 = (pptr[0] * elambdat[0]) + (pptr[1] * elambdat[1]);
		p0 += (pptr[2] * elambdat[2]) + (pptr[3] * elambdat[3]);
		p0 += (pptr[4] * elambdat[4]) + (pptr[5] * elambdat[5]);
		p0 += (pptr[6] * elambdat[6]) + (pptr[7] * elambdat[7]);
		p0 += (pptr[8] * elambdat[8]) + (pptr[9] * elambdat[9]);
		p0 += (pptr[10] * elambdat[10]) + (pptr[11] * elambdat[11]);
		p0 += (pptr[12] * elambdat[12]) + (pptr[13] * elambdat[13]);
		p0 += (pptr[14] * elambdat[14]) + (pptr[15] * elambdat[15]);
		p0 += (pptr[16] * elambdat[16]) + (pptr[17] * elambdat[17]);
		p0 += (pptr[18] * elambdat[18]) + (pptr[19] * elambdat[19]);	  	      	     	 
		
		pq_matrix[i].m[m][l] =  p0 * tr->rdta->invfreqaa[m];	  
		pq_matrix[i].m[l][m] =  p0 * tr->rdta->invfreqaa[l];
	      }           
	  }
      }    



    for (i = 0; i < tr->cdta->endsite; i++) 
      {
	cat = *cptr++;

	term = 0.0;

	for (m = 0; m < 20; m++) 
	  {
	   
	    pqptr = &pq_matrix[cat].m[m][0];
	    vptr = &(lq[i].v[0]);

	   
	    prodl = pqptr[0] * vptr[0] +  pqptr[1] * vptr[1];
	    prodl += (pqptr[2] * vptr[2] +  pqptr[3] * vptr[3]);
	    prodl += (pqptr[4] * vptr[4] +  pqptr[5] * vptr[5]);
	    prodl += (pqptr[6] * vptr[6] +  pqptr[7] * vptr[7]);
	    prodl += (pqptr[8] * vptr[8] +  pqptr[9] * vptr[9]);
	    prodl += (pqptr[10] * vptr[10] +  pqptr[11] * vptr[11]);
	    prodl += (pqptr[12] * vptr[12] +  pqptr[13] * vptr[13]);
	    prodl += (pqptr[14] * vptr[14] +  pqptr[15] * vptr[15]);
	    prodl += (pqptr[16] * vptr[16] +  pqptr[17] * vptr[17]); 
	    prodl += (pqptr[18] * vptr[18] +  pqptr[19] * vptr[19]);

	    /*if(prodl < 0)
	      printf("prodl %f ", prodl);*/

	    frexm = lp[i].v[m] * tr->rdta->freqaa[m];	   
	    /*	    if(frexm < 0)
	      printf("frexm %f ", frexm);*/

	    term += (prodl * frexm);	    
	  }
	
	/*if(term < 0) 
	  printf("%f \n", term);*/

	/*printf("%d %f %d %d \n",term, i, lq[i].exp, lp[i].exp);*/
	
	term = log(term) + (lq[i].exp + lp[i].exp)*log(minlikelihood);
	sum += term * wptr[i];
      }
   
    free(pq_matrix);       

    tr->likelihood = sum;
    return  sum;
  }








double makenewzPROTCAT (tree *tr, nodeptr p_node, nodeptr q_node, double z0, int maxiter)
  { /* makenewz */
   
    double   z, zprev, znew;
    int     i, *cptr, cat, l, m, finish, greater, action, extended, curvatOK, decrease;
    double  dlnLdlz, d2lnLdlz2, sum, term;
    double  *rptr, ki;
    proteinlikelivector *x1, *x2;
    proteinmatrix *matrix, *matrixD, *matrixDD;
    double prod4, prod5m, prod6m, prod4m, frexm;  
    double inv_Li, dlnLidlz, d2lnLidlz2 , *v;  

    matrix = (proteinmatrix *)malloc(sizeof(proteinmatrix) * tr->NumberOfCategories);
    matrixD = (proteinmatrix *)malloc(sizeof(proteinmatrix) * tr->NumberOfCategories);
    matrixDD = (proteinmatrix *)malloc(sizeof(proteinmatrix) * tr->NumberOfCategories);

    while ((! p_node->x) || (! q_node->x)) 
      {
	if (! (p_node->x)) if (! newviewPROTCAT(tr, p_node)) return badZ;
	if (! (q_node->x)) if (! newviewPROTCAT(tr, q_node)) return badZ;
      }    
   
    x1 = p_node->x->pv;
    x2 = q_node->x->pv;      

    z = z0;   
    
    do 
      {
	int *wptr =  tr->cdta->aliaswgt, cat;
	double *wrptr   = &(tr->cdta->wr[0]);
	double *wr2ptr  = &(tr->cdta->wr2[0]);
	double *mptr, *mptrD, *mptrDD;       
	double p0, p1, p2;
	double elambdat[20], dlambdat[20], ddlambdat[20];
	double *pptr;
	double rat, *rptr;
	
	cptr  = &(tr->cdta->rateCategory[0]);	
	rptr   = &(tr->cdta->patrat[0]);

	sum = 0;
	dlnLdlz = 0.0;                 
	d2lnLdlz2 = 0.0; 
	

	zprev = z;

	if (z < zmin) 
	  z = zmin;
	else 
	  if (z > zmax) z = zmax;	
		     	 
	for(i = 0; i < tr->NumberOfCategories; i++)
	  {
	    rat =  *rptr++;
	    
	    for (l = 0; l < 20; l++) 
	      {
		elambdat[l] = exp(z * rat * tr->rdta->eigmatPROT[l]); 
		dlambdat[l] = elambdat[l] * rat * tr->rdta->eigmatPROT[l];
		ddlambdat[l] = dlambdat[l] * rat * tr->rdta->eigmatPROT[l];
	      }

	    for (m = 0; m < 20; m++) 
	      {
		for (l = 0; l <= m; l++) 
		  {
		    pptr = tr->rdta->crossMAT[m][l];
		    
		    p0 = (pptr[0] * elambdat[0]) + (pptr[1] * elambdat[1]);
		    p1 = (pptr[0] * dlambdat[0]) + (pptr[1] * dlambdat[1]);
		    p2 = (pptr[0] * ddlambdat[0]) + (pptr[1] * ddlambdat[1]);

		    p0 += (pptr[2] * elambdat[2]) + (pptr[3] * elambdat[3]);
		    p1 += (pptr[2] * dlambdat[2]) + (pptr[3] * dlambdat[3]);
		    p2 += (pptr[2] * ddlambdat[2]) + (pptr[3] * ddlambdat[3]);

		    p0 += (pptr[4] * elambdat[4]) + (pptr[5] * elambdat[5]);
		    p1 += (pptr[4] * dlambdat[4]) + (pptr[5] * dlambdat[5]);
		    p2 += (pptr[4] * ddlambdat[4]) + (pptr[5] * ddlambdat[5]);

		    p0 += (pptr[6] * elambdat[6]) + (pptr[7] * elambdat[7]);
		    p1 += (pptr[6] * dlambdat[6]) + (pptr[7] * dlambdat[7]);
		    p2 += (pptr[6] * ddlambdat[6]) + (pptr[7] * ddlambdat[7]);

		    p0 += (pptr[8] * elambdat[8]) + (pptr[9] * elambdat[9]);
		    p1 += (pptr[8] * dlambdat[8]) + (pptr[9] * dlambdat[9]);
		    p2 += (pptr[8] * ddlambdat[8]) + (pptr[9] * ddlambdat[9]);

		    p0 += (pptr[10] * elambdat[10]) + (pptr[11] * elambdat[11]);
		    p1 += (pptr[10] * dlambdat[10]) + (pptr[11] * dlambdat[11]);
		    p2 += (pptr[10] * ddlambdat[10]) + (pptr[11] * ddlambdat[11]);

		    p0 += (pptr[12] * elambdat[12]) + (pptr[13] * elambdat[13]);
		    p1 += (pptr[12] * dlambdat[12]) + (pptr[13] * dlambdat[13]);
		    p2 += (pptr[12] * ddlambdat[12]) + (pptr[13] * ddlambdat[13]);

		    p0 += (pptr[14] * elambdat[14]) + (pptr[15] * elambdat[15]);
		    p1 += (pptr[14] * dlambdat[14]) + (pptr[15] * dlambdat[15]);
		    p2 += (pptr[14] * ddlambdat[14]) + (pptr[15] * ddlambdat[15]);

		    p0 += (pptr[16] * elambdat[16]) + (pptr[17] * elambdat[17]);
		    p1 += (pptr[16] * dlambdat[16]) + (pptr[17] * dlambdat[17]);
		    p2 += (pptr[16] * ddlambdat[16]) + (pptr[17] * ddlambdat[17]);

		    p0 += (pptr[18] * elambdat[18]) + (pptr[19] * elambdat[19]);
		    p1 += (pptr[18] * dlambdat[18]) + (pptr[19] * dlambdat[19]);
		    p2 += (pptr[18] * ddlambdat[18]) + (pptr[19] * ddlambdat[19]);
		   		  		    		   		   		   		   		   		   	     
		    matrix[i].m[m][l] = p0 * tr->rdta->invfreqaa[m];
		    matrix[i].m[l][m] = p0 * tr->rdta->invfreqaa[l];
		    matrixD[i].m[m][l] = p1 * tr->rdta->invfreqaa[m];
		    matrixD[i].m[l][m] = p1 * tr->rdta->invfreqaa[l];
		    matrixDD[i].m[m][l] = p2 * tr->rdta->invfreqaa[m];
		    matrixDD[i].m[l][m] = p2 * tr->rdta->invfreqaa[l];	      
		  }
	      }
	  }      
	
	for (i = 0; i < tr->cdta->endsite; i++) 
	  {
	    prod4 = 0.0;
	    dlnLidlz = 0.0;
	    d2lnLidlz2 = 0.0;
	    
	    cat = *cptr++;

	    v = &(x2[i].v[0]);
	    
	    for (m = 0; m <= 19; m++) 
	      {		
		mptr = matrix[cat].m[m];
		mptrD = matrixD[cat].m[m]; 
		mptrDD = matrixDD[cat].m[m]; 

		frexm = x1[i].v[m] * tr->rdta->freqaa[m];				
		
		prod4m = v[0] * mptr[0] + v[1] * mptr[1];	       
		prod5m = v[0] * mptrD[0] + v[1] * mptrD[1];		   
		prod6m = v[0] * mptrDD[0]+ v[1] * mptrDD[1];
		
		prod4m += v[2] * mptr[2] + v[3] * mptr[3];
		prod5m += v[2] * mptrD[2] + v[3] * mptrD[3];		   
		prod6m += v[2] * mptrDD[2]+ v[3] * mptrDD[3];
		
		prod4m += v[4] * mptr[4] + v[5] * mptr[5];
		prod5m += v[4] * mptrD[4] + v[5] * mptrD[5];		   
		prod6m += v[4] * mptrDD[4]+ v[5] * mptrDD[5];
		
		prod4m += v[6] * mptr[6] + v[7] * mptr[7];
		prod5m += v[6] * mptrD[6] + v[7] * mptrD[7];		   
		prod6m += v[6] * mptrDD[6]+ v[7] * mptrDD[7];
		
		prod4m += v[8] * mptr[8] + v[9] * mptr[9];
		prod5m += v[8] * mptrD[8] + v[9] * mptrD[9];		   
		prod6m += v[8] * mptrDD[8]+ v[9] * mptrDD[9];
		
		prod4m += v[10] * mptr[10] + v[11] * mptr[11];
		prod5m += v[10] * mptrD[10] + v[11] * mptrD[11];		   
		prod6m += v[10] * mptrDD[10]+ v[11] * mptrDD[11];
		
		prod4m += v[12] * mptr[12] + v[13] * mptr[13];
		prod5m += v[12] * mptrD[12] + v[13] * mptrD[13];		   
		prod6m += v[12] * mptrDD[12]+ v[13] * mptrDD[13];
		
		prod4m += v[14] * mptr[14] + v[15] * mptr[15];
		prod5m += v[14] * mptrD[14] + v[15] * mptrD[15];		   
		prod6m += v[14] * mptrDD[14]+ v[15] * mptrDD[15];
		
		prod4m += v[16] * mptr[16] + v[17] * mptr[17];
		prod5m += v[16] * mptrD[16] + v[17] * mptrD[17];		   
		prod6m += v[16] * mptrDD[16]+ v[17] * mptrDD[17];
		
		prod4m += v[18] * mptr[18] + v[19] * mptr[19];
		prod5m += v[18] * mptrD[18] + v[19] * mptrD[19];		   
		prod6m += v[18] * mptrDD[18]+ v[19] * mptrDD[19];
		
		
		prod4 += frexm * prod4m;
		dlnLidlz += frexm * prod5m;
		d2lnLidlz2 +=  frexm * prod6m;
	      }
	    
	  
	    inv_Li = 1.0 / prod4;	    	   
	    dlnLidlz   *= inv_Li; 
	    d2lnLidlz2 *= inv_Li; 		
	    dlnLdlz   += *wrptr++  * dlnLidlz;
	    d2lnLdlz2 += *wr2ptr++ *  (d2lnLidlz2 -  dlnLidlz * dlnLidlz);
	  }
		


	znew = z + dlnLdlz/fabs(d2lnLdlz2);

	if(znew < zmin || znew > zmax)	  	  
	  {	    	    	   
	    znew = z * 0.5;	   
	    action = 1;
	  }
	else	
	  {	    	   	    
	    action = 0;
	  }
       

	if(dlnLdlz < 0)
	  {
	    decrease = 1;
	    if(znew > z)
	      {
		/*printf("ERROR EXPCTED ");	       */
		znew = z * 0.5;
	      }
	  }
	else
	  {
	    decrease = 0;	
	    if(znew < z)
	      {
		/*printf("ERROR EXPCTED ");		*/
		znew = z * 0.37 + 0.63;
	      }
	  }
	z = znew;
	

      }
    while((--maxiter > 0) && fabs(z - zprev) > deltaz);
    /*printf("\n");*/

    free(matrix);
    free(matrixD);
    free(matrixDD);

    return z;      
  }

#endif



#ifndef HPC




boolean newviewHKY85(tree *tr, nodeptr p)        /*  Update likelihoods at node */
  { /* newview */
    double   zq, lzq, xvlzq, zr, lzr, xvlzr;
    nodeptr  q, r, realP;
    likelivector *lp, *lq, *lr;
    homType *mlvP;
    int  i;
           
    if(p->tip) 
      {
	if (p->x) return TRUE;

	if (! getxtip(p)) return FALSE;

	return TRUE;
      }
      
    q = p->next->back;
    r = p->next->next->back;

    while ((! p->x) || (! q->x) || (! r->x)) 
      {
	if (! q->x) if (! newviewHKY85(tr, q))  return FALSE;
	if (! r->x) if (! newviewHKY85(tr, r))  return FALSE;
	if (! p->x) if (! getxnode(p)) return FALSE;
      }
       
    NWC++;
    
    lp = p->x->lv;

    lq = q->x->lv;
    zq = q->z;
    lzq = (zq > zmin) ? log(zq) : log(zmin);
    xvlzq = tr->rdta->xv * lzq;

    lr = r->x->lv;
    zr = r->z;
    lzr = (zr > zmin) ? log(zr) : log(zmin);
    xvlzr = tr->rdta->xv * lzr;

    { double  zzq, zvq,
	zzr, zvr, rp;
      double  fxqr, fxqy, fxqn, sumaq, sumgq, sumcq, sumtq,
              fxrr, fxry, fxrn, ki, tempi, tempj;                
      register  int *s1; 
      register  int *s2;
      register int *eqptr;
      
      homType *mlvR, *mlvQ;
      poutsa calc[EQUALITIES],calcR[EQUALITIES], *calcptr, *calcptrR;
      long ref, ref2, mexp;      
      nodeptr realQ, realR;
    
      /* get equality vectors of p, q, and r */
      
      realP =  tr->nodep[p->number];
    
      mlvP = realP->mlv;
      realQ =  tr->nodep[q->number];
      realR =  tr->nodep[r->number];


      eqptr = realP->equalityVector; 
      s1 = realQ->equalityVector;
      s2 = realR->equalityVector; 
                 
      zzq = exp(lzq);
      zvq = exp(xvlzq);
      zzr = exp(lzr);        
      zvr = exp(xvlzr); 
      
      mlvQ = realQ->mlv;
      mlvR = realR->mlv;


     
           	 	  
      for(i = 0; i < equalities; i++)
	{
	  
	  if(mlvQ->set)
	    {
	      calcptr = &calc[i];		 
	      fxqr = tr->rdta->freqa * mlvQ->a + tr->rdta->freqg * mlvQ->g;
	      fxqy = tr->rdta->freqc * mlvQ->c + tr->rdta->freqt * mlvQ->t;
	      fxqn = fxqr + fxqy;
	      tempi = fxqr * tr->rdta->invfreqr;
	      tempj = zvq * (tempi-fxqn) + fxqn;
	      calcptr->sumaq = zzq * (mlvQ->a - tempi) + tempj;
	      calcptr->sumgq = zzq * (mlvQ->g - tempi) + tempj;
	      tempi = fxqy * tr->rdta->invfreqy;
	      tempj = zvq * (tempi-fxqn) + fxqn;
	      calcptr->sumcq = zzq * (mlvQ->c - tempi) + tempj;
	      calcptr->sumtq = zzq * (mlvQ->t - tempi) + tempj;
	      calcptr->exp = mlvQ->exp;
	    }
	  if(mlvR->set)
	    {
	      calcptrR = &calcR[i];		 
	      fxrr = tr->rdta->freqa * mlvR->a + tr->rdta->freqg * mlvR->g;
	      fxry = tr->rdta->freqc * mlvR->c + tr->rdta->freqt * mlvR->t;
	      fxrn = fxrr + fxry;
	      tempi = fxrr * tr->rdta->invfreqr;
	      tempj = zvr * (tempi-fxrn) + fxrn;
	      calcptrR->sumaq = zzr * (mlvR->a - tempi) + tempj;
	      calcptrR->sumgq = zzr * (mlvR->g - tempi) + tempj;
	      tempi = fxry * tr->rdta->invfreqy;
	      tempj = zvr * (tempi-fxrn) + fxrn;
	      calcptrR->sumcq = zzr * (mlvR->c - tempi) + tempj;
	      calcptrR->sumtq = zzr * (mlvR->t - tempi) + tempj;	     				
	      calcptrR->exp = mlvR->exp;	    	     	
	    }

	  if(mlvR->set && mlvQ->set)
	    {
	      mlvP->a = calcptrR->sumaq * calcptr->sumaq;
	      mlvP->g = calcptrR->sumgq * calcptr->sumgq;		    
	      mlvP->c = calcptrR->sumcq * calcptr->sumcq; 
	      mlvP->t = calcptrR->sumtq * calcptr->sumtq;
	      mlvP->exp = calcptrR->exp + calcptr->exp;

	      if (mlvP->a < minlikelihood && mlvP->g < minlikelihood
		  && mlvP->c < minlikelihood && mlvP->t < minlikelihood) {
		mlvP->a   *= twotothe256;
		mlvP->g   *= twotothe256;
		mlvP->c   *= twotothe256;
		mlvP->t   *= twotothe256;
		mlvP->exp += 1;
	      }
	 
	      mlvP->set = TRUE;
	    }
	  else
	    mlvP->set = FALSE;
	  
	  mlvQ++;
	  mlvR++;
	  mlvP++;
	}


      if(r->tip && q->tip)
	{

	 

	   for (i = 0; i < tr->cdta->endsite; i++) 
	    {
	      ref = *s2++;
	      ref2 = *s1++;
	     	      	      		  	
	      if(ref2 != ref)
		{
		  *eqptr++ = -1;
		  calcptrR = &calcR[ref];
		  calcptr = &calc[ref2];
		  lp->a = calcptrR->sumaq * calcptr->sumaq;
		  lp->g = calcptrR->sumgq * calcptr->sumgq;		    
		  lp->c = calcptrR->sumcq * calcptr->sumcq; 
		  lp->t = calcptrR->sumtq * calcptr->sumtq;
		  lp->exp = calcptrR->exp + calcptr->exp;
		  if (lp->a < minlikelihood && lp->g < minlikelihood
		      && lp->c < minlikelihood && lp->t < minlikelihood) {
		    lp->a   *= twotothe256;
		    lp->g   *= twotothe256;
		    lp->c   *= twotothe256;
		    lp->t   *= twotothe256;
		    lp->exp += 1;
		  }
		  lp++;   
		}
	      else
		{
		  *eqptr++ = ref;		    		
		}
	   	      	    
	    }          
	}
      else
	{
	  	  

	 
	  
	  for (i = 0; i < tr->cdta->endsite; i++) 
	    {
	      ref = *s2++;
	      ref2 = *s1++;
	      
	      if(ref2 >= 0)
		{		  	
		  if(ref2 == ref)
		    {
		      *eqptr++ = ref;		     		  
		      goto incTip;		    		  		 		  	    		
		    }
		  calcptr = &calc[ref2];
		  sumaq = calcptr->sumaq;
		  sumgq = calcptr->sumgq;
		  sumcq = calcptr->sumcq;
		  sumtq = calcptr->sumtq;		 		  	
		  mexp = calcptr->exp;	       		  
		}	    	  	  	 	 
	      else
		{		
		  fxqr = tr->rdta->freqa * lq->a + tr->rdta->freqg * lq->g;
		  fxqy = tr->rdta->freqc * lq->c + tr->rdta->freqt * lq->t;
		  fxqn = fxqr + fxqy;
		  tempi = fxqr * tr->rdta->invfreqr;
		  tempj = zvq * (tempi-fxqn) + fxqn;
		  sumaq = zzq * (lq->a - tempi) + tempj;
		  sumgq = zzq * (lq->g - tempi) + tempj;
		  tempi = fxqy * tr->rdta->invfreqy;
		  tempj = zvq * (tempi-fxqn) + fxqn;
		  sumcq = zzq * (lq->c - tempi) + tempj;
		  sumtq = zzq * (lq->t - tempi) + tempj;
		  mexp = lq->exp;
		  lq++;
		}
	      	      
	      if(ref >= 0)
		{
		  calcptr = &calcR[ref];		 	    
		  lp->a = sumaq * calcptr->sumaq;
		  lp->g = sumgq * calcptr->sumgq;		    
		  lp->c = sumcq * calcptr->sumcq; 
		  lp->t = sumtq * calcptr->sumtq;
		  lp->exp = mexp + calcptr->exp;		  	  
		}
	      else
		{
		  fxrr = tr->rdta->freqa * lr->a + tr->rdta->freqg * lr->g;
		  fxry = tr->rdta->freqc * lr->c + tr->rdta->freqt * lr->t;
		  fxrn = fxrr + fxry;
		  tempi = fxrr * tr->rdta->invfreqr;
		  tempj = zvr * (tempi-fxrn) + fxrn;
		  lp->a = sumaq * (zzr * (lr->a - tempi) + tempj);
		  lp->g = sumgq * (zzr * (lr->g - tempi) + tempj);
		  tempi = fxry * tr->rdta->invfreqy;
		  tempj = zvr * (tempi-fxrn) + fxrn;
		  lp->c = sumcq * (zzr * (lr->c - tempi) + tempj);
		  lp->t = sumtq * (zzr * (lr->t - tempi) + tempj);
		  lp->exp = mexp + lr->exp;
		  lr++;
		}
	      
	      if (lp->a < minlikelihood && lp->g < minlikelihood
		  && lp->c < minlikelihood && lp->t < minlikelihood) {
		lp->a   *= twotothe256;
		lp->g   *= twotothe256;
		lp->c   *= twotothe256;
		lp->t   *= twotothe256;
		lp->exp += 1;
	      }
	      
	      *eqptr++ = -1;
	      lp++;
	    incTip:	 ;
	    }      
	
    
	}
    }
    
   
    

    return TRUE;  
  } /* newview */

 /* AxML modification end*/





double evaluateHKY85(tree *tr, nodeptr p)
  { /* evaluate */
    double   sum, z, lz, xvlz,
             ki, fx1a, fx1c, fx1g, fx1t, fx1r, fx1y, fx2r, fx2y,
             suma, sumb, sumc, term, zzp, zvp;           
    likelivector *lp, *lq;
    homType *mlvP, *mlvQ;
    nodeptr       q, realQ, realP;
    int           i, *wptr;
    register  int *sQ; 
    register  int *sP;
    long refQ, refP;
    memP calcP[EQUALITIES], *calcptrP;
    memQ calcQ[EQUALITIES], *calcptrQ;

    q = p->back;

    while ((! p->x) || (! q->x)) {
      if (! (p->x)) if (! newviewHKY85(tr, p)) return badEval;
      if (! (q->x)) if (! newviewHKY85(tr, q)) return badEval;
    }

    lp = p->x->lv;
    lq = q->x->lv;

    z = p->z;
    if (z < zmin) z = zmin;
    lz = log(z);
    xvlz = tr->rdta->xv * lz;

    
    zzp = exp(lz);
    zvp = exp(xvlz);
      
    wptr = &(tr->cdta->aliaswgt[0]);
   
    sum = 0.0;
    
    realQ =  tr->nodep[q->number];
    realP =  tr->nodep[p->number];
    mlvP = realP->mlv;
    mlvQ = realQ->mlv;
    sQ = realQ->equalityVector;
    sP = realP->equalityVector; 

    

    for(i = 0; i < equalities; i++)
	{
	 
	  if(mlvP->set)
	    {
	      calcptrP = &calcP[i];		 
	      calcptrP->fx1a = tr->rdta->freqa * mlvP->a;
	      calcptrP->fx1g = tr->rdta->freqg * mlvP->g;
	      calcptrP->fx1c = tr->rdta->freqc * mlvP->c;
	      calcptrP->fx1t = tr->rdta->freqt * mlvP->t;
	      calcptrP->sumag = calcptrP->fx1a + calcptrP->fx1g;
	      calcptrP->sumct = calcptrP->fx1c + calcptrP->fx1t;
	      calcptrP->sumagct = calcptrP->sumag + calcptrP->sumct;	      
	      calcptrP->exp = mlvP->exp;
	    }
	 
	  if(mlvQ->set)
	    {
	      calcptrQ = &calcQ[i];
	      calcptrQ->fx2r = tr->rdta->freqa * mlvQ->a + tr->rdta->freqg * mlvQ->g;	     
	      calcptrQ->fx2y  = tr->rdta->freqc * mlvQ->c + tr->rdta->freqt * mlvQ->t; 	     
	      calcptrQ->sumfx2rfx2y = calcptrQ->fx2r + calcptrQ->fx2y;	      
	      calcptrQ->exp = mlvQ->exp;
	      calcptrQ->a = mlvQ->a;
	      calcptrQ->c = mlvQ->c;
	      calcptrQ->g = mlvQ->g;
	      calcptrQ->t = mlvQ->t;
	    }
	  
	  mlvQ++;	 
	  mlvP++;
	}




      for (i = 0; i < tr->cdta->endsite; i++) {
	    refQ = *sQ++;
	    refP = *sP++;

	    if(refP >= 0)
	      {
		if(refQ >= 0)
		  {
		    calcptrP = &calcP[refP];
		    calcptrQ = &calcQ[refQ];
		    suma = calcptrP->fx1a * calcptrQ->a + 
		      calcptrP->fx1c * calcptrQ->c + 
		      calcptrP->fx1g * calcptrQ->g + calcptrP->fx1t * calcptrQ->t;		 		 
		    
		    sumc = calcptrP->sumagct * calcptrQ->sumfx2rfx2y;
		    sumb = calcptrP->sumag * calcptrQ->fx2r * tr->rdta->invfreqr + calcptrP->sumct * calcptrQ->fx2y * tr->rdta->invfreqy;
		    suma -= sumb;
		    sumb -= sumc;
		    term = log(zzp * suma + zvp * sumb + sumc) + 
		      (calcptrP->exp + calcptrQ->exp)*log(minlikelihood);
		    sum += *wptr++ * term;
		    
		   
		  }
		else
		  {
		    calcptrP = &calcP[refP];
		    suma = calcptrP->fx1a * lq->a + calcptrP->fx1c * lq->c + 
		      calcptrP->fx1g * lq->g + calcptrP->fx1t * lq->t;
		    
		    fx2r = tr->rdta->freqa * lq->a + tr->rdta->freqg * lq->g;
		    fx2y = tr->rdta->freqc * lq->c + tr->rdta->freqt * lq->t;
		    
		    sumc = calcptrP->sumagct * (fx2r + fx2y);
		    sumb       = calcptrP->sumag * fx2r * tr->rdta->invfreqr + calcptrP->sumct * fx2y * tr->rdta->invfreqy;
		    suma -= sumb;
		    sumb -= sumc;
		    term = log(zzp * suma + zvp * sumb + sumc) + 
		      (calcptrP->exp + lq->exp)*log(minlikelihood);
		    sum += *wptr++ * term;
		    
		   
		    lq++;
		  }
	      }
	    else
	      {
		if(refQ >= 0)
		  {
		    calcptrQ = &calcQ[refQ];
		    fx1a = tr->rdta->freqa * lp->a;
		    fx1g = tr->rdta->freqg * lp->g;
		    fx1c = tr->rdta->freqc * lp->c;
		    fx1t = tr->rdta->freqt * lp->t;
		    suma = fx1a * calcptrQ->a + fx1c * calcptrQ->c + 
		      fx1g * calcptrQ->g + fx1t * calcptrQ->t;	  
		    fx1r = fx1a + fx1g;
		    fx1y = fx1c + fx1t;
		    sumc = (fx1r + fx1y) * calcptrQ->sumfx2rfx2y; 
		    sumb = fx1r * calcptrQ->fx2r * tr->rdta->invfreqr + fx1y * calcptrQ->fx2y * tr->rdta->invfreqy;
		    suma -= sumb;
		    sumb -= sumc;	
		    term = log(zzp * suma + zvp * sumb + sumc) + 
		      (lp->exp + calcptrQ->exp)*log(minlikelihood);		
		    sum += *wptr++ * term;	
		    
		    lp++;
		  }
		else
		  {
		    fx1a = tr->rdta->freqa * lp->a;
		    fx1g = tr->rdta->freqg * lp->g;
		    fx1c = tr->rdta->freqc * lp->c;
		    fx1t = tr->rdta->freqt * lp->t;
		    suma = fx1a * lq->a + fx1c * lq->c + fx1g * lq->g + fx1t * lq->t;
		    fx2r = tr->rdta->freqa * lq->a + tr->rdta->freqg * lq->g;
		    fx2y = tr->rdta->freqc * lq->c + tr->rdta->freqt * lq->t;
		    fx1r = fx1a + fx1g;
		    fx1y = fx1c + fx1t;
		    sumc = (fx1r + fx1y) * (fx2r + fx2y);
		    sumb = fx1r * fx2r * tr->rdta->invfreqr + fx1y * fx2y * tr->rdta->invfreqy;
		    suma -= sumb;
		    sumb -= sumc;
		    term = log(zzp * suma + zvp * sumb + sumc) + 
		      (lp->exp + lq->exp)*log(minlikelihood);
		    sum += *wptr++ * term;		
		   
		    lp++;
		    lq++;
		  }
	      }	  
	   
      }

     
    tr->likelihood = sum; 
    return  sum;
   
  } /* evaluate */


double makenewzHKY85 (tree *tr, nodeptr p, nodeptr q, double z0, int maxiter)
{
  likelivector *lp, *lq;
    double  *abi, *bci, *sumci, *abptr, *bcptr, *sumcptr;
    double   dlnLidlz, dlnLdlz, d2lnLdlz2, z, zprev, zstep, lz, xvlz,
             ki, suma, sumb, sumc, ab, bc, inv_Li, t1, t2,
             fx1a, fx1c, fx1g, fx1t, fx1r, fx1y, fx2r, fx2y;
    double  zz, zv;
    double   *wrptr, *wr2ptr;
    int      i, curvatOK;
    homType *mlvP, *mlvQ;
    nodeptr realQ, realP;
    register  int *sQ; 
    register  int *sP;
    long refQ, refP;
    
    while ((! p->x) || (! q->x)) {
      if (! (p->x)) if (! newviewHKY85(tr, p)) return badZ;
      if (! (q->x)) if (! newviewHKY85(tr, q)) return badZ;
      }
    
   

    lp = p->x->lv;
    lq = q->x->lv;

    { 
      
      unsigned scratch_size;
      scratch_size = sizeof(double) * tr->cdta->endsite;
      abi   = (double *) malloc(scratch_size);
      bci   = (double *) malloc(scratch_size);
      sumci = (double *) malloc(scratch_size);              
    }
    
    
    

    abptr   = abi;
    bcptr   = bci;
    sumcptr = sumci;
    realQ =  tr->nodep[q->number];
    realP =  tr->nodep[p->number];
    mlvP = realP->mlv;
    mlvQ = realQ->mlv;
    sQ = realQ->equalityVector;
    sP = realP->equalityVector;


    for (i = 0; i < tr->cdta->endsite; i++) {
      refQ = *sQ++;
      refP = *sP++;
      if(refP >= 0)
	{
	  if(refQ >= 0)
	    {
	      mlvP = &realP->mlv[refP];
	      mlvQ = &realQ->mlv[refQ];
	      fx1a = tr->rdta->freqa * mlvP->a;
	      fx1g = tr->rdta->freqg * mlvP->g;
	      fx1c = tr->rdta->freqc * mlvP->c;
	      fx1t = tr->rdta->freqt * mlvP->t;
	      suma = fx1a * mlvQ->a + fx1c * mlvQ->c + fx1g * mlvQ->g + fx1t * mlvQ->t;
	      fx2r = tr->rdta->freqa * mlvQ->a + tr->rdta->freqg * mlvQ->g;
	      fx2y = tr->rdta->freqc * mlvQ->c + tr->rdta->freqt * mlvQ->t;
	      fx1r = fx1a + fx1g;
	      fx1y = fx1c + fx1t;
	      *sumcptr++ = sumc = (fx1r + fx1y) * (fx2r + fx2y);
	      sumb       = fx1r * fx2r * tr->rdta->invfreqr + fx1y * fx2y * tr->rdta->invfreqy;
	      *abptr++   = suma - sumb;
	      *bcptr++   = sumb - sumc;

		  
	    }
	  else
	    {
	      mlvP = &realP->mlv[refP];
	      fx1a = tr->rdta->freqa * mlvP->a;
	      fx1g = tr->rdta->freqg * mlvP->g;
	      fx1c = tr->rdta->freqc * mlvP->c;
	      fx1t = tr->rdta->freqt * mlvP->t;
	      suma = fx1a * lq->a + fx1c * lq->c + fx1g * lq->g + fx1t * lq->t;
	      fx2r = tr->rdta->freqa * lq->a + tr->rdta->freqg * lq->g;
	      fx2y = tr->rdta->freqc * lq->c + tr->rdta->freqt * lq->t;
	      fx1r = fx1a + fx1g;
	      fx1y = fx1c + fx1t;
	      *sumcptr++ = sumc = (fx1r + fx1y) * (fx2r + fx2y);
	      sumb       = fx1r * fx2r * tr->rdta->invfreqr + fx1y * fx2y * tr->rdta->invfreqy;
	      *abptr++   = suma - sumb;
	      *bcptr++   = sumb - sumc;
    		    		   
	      lq++;
	    }
	}
      else
	{
	  if(refQ >= 0)
	    {
	      mlvQ = &realQ->mlv[refQ];
	
	      fx1a = tr->rdta->freqa * lp->a;
	      fx1g = tr->rdta->freqg * lp->g;
	      fx1c = tr->rdta->freqc * lp->c;
	      fx1t = tr->rdta->freqt * lp->t;
	      suma = fx1a * mlvQ->a + fx1c * mlvQ->c + fx1g * mlvQ->g + fx1t * mlvQ->t;
	      fx2r = tr->rdta->freqa * mlvQ->a + tr->rdta->freqg * mlvQ->g;
	      fx2y = tr->rdta->freqc * mlvQ->c + tr->rdta->freqt * mlvQ->t;
	      fx1r = fx1a + fx1g;
	      fx1y = fx1c + fx1t;
	      *sumcptr++ = sumc = (fx1r + fx1y) * (fx2r + fx2y);
	      sumb       = fx1r * fx2r * tr->rdta->invfreqr + fx1y * fx2y * tr->rdta->invfreqy;
	      *abptr++   = suma - sumb;
	      *bcptr++   = sumb - sumc;
	      
	      lp++;
	    }
	  else
	    {
	      fx1a = tr->rdta->freqa * lp->a;
	      fx1g = tr->rdta->freqg * lp->g;
	      fx1c = tr->rdta->freqc * lp->c;
	      fx1t = tr->rdta->freqt * lp->t;
	      suma = fx1a * lq->a + fx1c * lq->c + fx1g * lq->g + fx1t * lq->t;
	      fx2r = tr->rdta->freqa * lq->a + tr->rdta->freqg * lq->g;
	      fx2y = tr->rdta->freqc * lq->c + tr->rdta->freqt * lq->t;
	      fx1r = fx1a + fx1g;
	      fx1y = fx1c + fx1t;
	      *sumcptr++ = sumc = (fx1r + fx1y) * (fx2r + fx2y);
	      sumb       = fx1r * fx2r * tr->rdta->invfreqr + 
		fx1y * fx2y * tr->rdta->invfreqy;
	      *abptr++   = suma - sumb;
	      *bcptr++   = sumb - sumc;
	      
	      lq++;
	      lp++;
	    }
	}           
    }

    z = z0;
    do {
      zprev = z;
      zstep = (1.0 - zmax) * z + zmin;
      curvatOK = FALSE;

      do {
        if (z < zmin) z = zmin;
        else if (z > zmax) z = zmax;

        lz    = log(z);
        xvlz  = tr->rdta->xv * lz;         
        
	zz = exp(lz);
	zv = exp(xvlz);
          
	abptr   = abi;
	bcptr   = bci;
	sumcptr = sumci;        
	wrptr   = &(tr->cdta->wr[0]);
	wr2ptr  = &(tr->cdta->wr2[0]);
	dlnLdlz = 0.0;                 /*  = d(ln(likelihood))/d(lz) */
	d2lnLdlz2 = 0.0;               /*  = d2(ln(likelihood))/d(lz)2 */



        for (i = 0; i < tr->cdta->endsite; i++) {
         
          ab     = *abptr++ * zz;
          bc     = *bcptr++ * zv;
          sumc   = *sumcptr++;
          inv_Li = 1.0/(ab + bc + sumc);
          t1     = ab * inv_Li;
          t2     = tr->rdta->xv * bc * inv_Li;
          dlnLidlz   = t1 + t2;
          dlnLdlz   += *wrptr++  * dlnLidlz;
          d2lnLdlz2 += *wr2ptr++ * (t1 + tr->rdta->xv * t2 - dlnLidlz * dlnLidlz);
          }



        if ((d2lnLdlz2 >= 0.0) && (z < zmax))
	  {
	    zprev = z = 0.37 * z + 0.63;  /*  Bad curvature, shorten branch */
	   
	  }
        else
          curvatOK = TRUE;

        } while (! curvatOK);

      if (d2lnLdlz2 < 0.0) {
        z *= exp(-dlnLdlz / d2lnLdlz2);
        if (z < zmin) z = zmin;
        if (z > 0.25 * zprev + 0.75)    /*  Limit steps toward z = 1.0 */
          z = 0.25 * zprev + 0.75;
        }

      if (z > zmax) z = zmax;

      } while ((--maxiter > 0) && (ABS(z - zprev) > zstep));

    free(abi);
    free(bci);
    free(sumci);
    

   
    return  z;
}








boolean newviewGTR(tree    *tr, nodeptr  p)
{
  
  if (p->tip) 
    {
      
      if (p->x) return TRUE;
      if (! getxtip(p)) return FALSE;
      {
	homType *mlvP;
	int code, i, j, jj;
	double (*EIGV)[4] = &(tr->rdta->EIGV[0][0]);
	int refREV[EQUALITIES];
	int *ev = p->equalityVector;
                 
      mlvP = tr->nodep[p->number]->mlv;
     
      for(i = 0; i < EQUALITIES; i++)
	{	      	  	 
	  mlvP->a = 0;
	  mlvP->c = 0;
	  mlvP->g = 0;
	  mlvP->t = 0;
	  code = i + 1;
	  for (j = 0; j < 4; j++) 
	    {
	      if ((code >> j) & 1) 
		{
		  int jj = "0213"[j] - '0';		
		  mlvP->a += EIGV[jj][0];
		  mlvP->c += EIGV[jj][1];
		  mlvP->g += EIGV[jj][2];
		  mlvP->t += EIGV[jj][3];
		}
	    }		
	      
	  mlvP->exp = 0;	  			  			 
	  mlvP++;	 
	} 
      return TRUE; 
      }
    }
  
  { 	
    double  diagptable_x1[4], diagptable_x2[4];
    double *EIGN    = &(tr->rdta->EIGN[0][0]);
    double *invfreq = &(tr->rdta->invfreq[0][0]);
    double (*EIGV)[4] = &(tr->rdta->EIGV[0][0]);
    xtype tmp_x1_1, tmp_x1_2, tmp_x1_3,
      tmp_x2_1, tmp_x2_2, tmp_x2_3, 
      ump_x1_1, ump_x1_2, ump_x1_3, ump_x1_0, 
      ump_x2_0, ump_x2_1, ump_x2_2, ump_x2_3, x1px2;
    homType *mlvR, *mlvQ, *mlvP;
    poutsa calcQ[EQUALITIES],calcR[EQUALITIES], *calcptr, *calcptrR;
    long ref, ref2, mexp;      
    nodeptr realQ, realR, realP;	  
    int  i;
    likelivector   *x1, *x2, *x3;
    double   z1, lz1, z2, lz2;
    nodeptr  q, r;
    register int *eqptr, *s1, *s2;
    double *eigv;
    q = p->next->back;
    r = p->next->next->back;
      
    while ((! p->x) || (! q->x) || (! r->x)) {
      if (! q->x) if (! newviewGTR(tr, q)) return FALSE;
      if (! r->x) if (! newviewGTR(tr, r)) return FALSE;
      if (! p->x) if (! getxnode(p)) return FALSE;	
    }

    x1  = q->x->lv;
    z1  = q->z;
    lz1 = (z1 > zmin) ? log(z1) : log(zmin);

    x2  = r->x->lv;
    z2  = r->z;
    lz2 = (z2 > zmin) ? log(z2) : log(zmin);

    x3  = p->x->lv;
      
    realP =  tr->nodep[p->number];
    realQ =  tr->nodep[q->number];
    realR =  tr->nodep[r->number];

    mlvP = realP->mlv;
    mlvQ = realQ->mlv;
    mlvR = realR->mlv;

    eqptr = realP->equalityVector; 
    s1 = realQ->equalityVector;
    s2 = realR->equalityVector; 
     	  
    for (i = 1; i < 4; i++) 
      {
	diagptable_x1[i] = exp (EIGN[i] * lz1);
	diagptable_x2[i] = exp (EIGN[i] * lz2);	    
      }	       
    
    for(i = 0; i < EQUALITIES; i++)
      {	  
	calcptr = &calcQ[i];		 
	tmp_x1_1 = mlvQ->c * diagptable_x1[1];
	tmp_x1_2 = mlvQ->g * diagptable_x1[2];
	tmp_x1_3 = mlvQ->t * diagptable_x1[3];
	
	calcptr->sumaq = tmp_x1_1 * EIGV[0][1];
	calcptr->sumaq += tmp_x1_2 * EIGV[0][2];
	calcptr->sumaq += tmp_x1_3 * EIGV[0][3];	      	
	calcptr->sumaq *= invfreq[0];
	calcptr->sumaq += mlvQ->a;
	
	calcptr->sumcq = tmp_x1_1 * EIGV[1][1];
	calcptr->sumcq += tmp_x1_2 * EIGV[1][2];
	calcptr->sumcq += tmp_x1_3 * EIGV[1][3];	      	
	calcptr->sumcq *= invfreq[1];
	calcptr->sumcq += mlvQ->a;
	
	calcptr->sumgq = tmp_x1_1 * EIGV[2][1];
	calcptr->sumgq += tmp_x1_2 * EIGV[2][2];
	calcptr->sumgq += tmp_x1_3 * EIGV[2][3];	      	
	calcptr->sumgq *= invfreq[2];
	calcptr->sumgq += mlvQ->a;
	
	calcptr->sumtq = tmp_x1_1 * EIGV[3][1];
	calcptr->sumtq += tmp_x1_2 * EIGV[3][2];
	calcptr->sumtq += tmp_x1_3 * EIGV[3][3];	      	
	calcptr->sumtq *= invfreq[3];
	calcptr->sumtq += mlvQ->a;	  
	
	calcptr->exp = mlvQ->exp;
	
	calcptrR = &calcR[i];
	tmp_x2_1 = mlvR->c * diagptable_x2[1];
	tmp_x2_2 = mlvR->g * diagptable_x2[2];
	tmp_x2_3 = mlvR->t * diagptable_x2[3];
	
	calcptrR->sumaq = tmp_x2_1 * EIGV[0][1];
	calcptrR->sumaq += tmp_x2_2 * EIGV[0][2];
	calcptrR->sumaq += tmp_x2_3 * EIGV[0][3];		     
	calcptrR->sumaq *= invfreq[0];
	calcptrR->sumaq += mlvR->a;
	
	calcptrR->sumcq = tmp_x2_1 * EIGV[1][1];
	calcptrR->sumcq += tmp_x2_2 * EIGV[1][2];
	calcptrR->sumcq += tmp_x2_3 * EIGV[1][3];		     
	calcptrR->sumcq *= invfreq[1];
	calcptrR->sumcq += mlvR->a;	 
	
	calcptrR->sumgq = tmp_x2_1 * EIGV[2][1];
	calcptrR->sumgq += tmp_x2_2 * EIGV[2][2];
	calcptrR->sumgq += tmp_x2_3 * EIGV[2][3];		     
	calcptrR->sumgq *= invfreq[2];
	calcptrR->sumgq += mlvR->a;	  
	
	calcptrR->sumtq  = tmp_x2_1 * EIGV[3][1] ;
	calcptrR->sumtq  += tmp_x2_2 * EIGV[3][2];
	calcptrR->sumtq  += tmp_x2_3 * EIGV[3][3];		     
	calcptrR->sumtq  *= invfreq[3];
	calcptrR->sumtq  += mlvR->a;	    	  	      
	
	calcptrR->exp = mlvR->exp;
	



	mlvP->a = calcptrR->sumaq * calcptr->sumaq * EIGV[0][0];
	mlvP->c = calcptrR->sumaq * calcptr->sumaq * EIGV[0][1];
	mlvP->g = calcptrR->sumaq * calcptr->sumaq * EIGV[0][2];
	mlvP->t = calcptrR->sumaq * calcptr->sumaq * EIGV[0][3]; 
	
	mlvP->a += calcptrR->sumcq * calcptr->sumcq * EIGV[1][0];
	mlvP->c += calcptrR->sumcq * calcptr->sumcq * EIGV[1][1];
	mlvP->g += calcptrR->sumcq * calcptr->sumcq * EIGV[1][2];
	mlvP->t += calcptrR->sumcq * calcptr->sumcq * EIGV[1][3];
	
	mlvP->a += calcptrR->sumgq  * calcptr->sumgq * EIGV[2][0];
	mlvP->c += calcptrR->sumgq  * calcptr->sumgq * EIGV[2][1];
	mlvP->g += calcptrR->sumgq  * calcptr->sumgq * EIGV[2][2];
	mlvP->t += calcptrR->sumgq  * calcptr->sumgq * EIGV[2][3];
	
	mlvP->a += calcptrR->sumtq * calcptr->sumtq *  EIGV[3][0];
	mlvP->c += calcptrR->sumtq * calcptr->sumtq *  EIGV[3][1];
	mlvP->g += calcptrR->sumtq * calcptr->sumtq *  EIGV[3][2];
	mlvP->t += calcptrR->sumtq * calcptr->sumtq *  EIGV[3][3];		     
	
	mlvP->exp = calcptr->exp + calcptrR->exp;	
	if (ABS(mlvP->a) < minlikelihood && ABS(mlvP->g) < minlikelihood
	    && ABS(mlvP->c) < minlikelihood && ABS(mlvP->t) < minlikelihood) 
	  {	     
	    mlvP->a   *= twotothe256;
	    mlvP->g   *= twotothe256;
	    mlvP->c   *= twotothe256;
	    mlvP->t   *= twotothe256;
	    mlvP->exp += 1;
	  }
	mlvQ++;
	mlvR++;
	mlvP++;
      }

    if(r->tip && q->tip)
	{	 
	  for (i = 0; i < tr->cdta->endsite; i++) 
	    {
	      ref = *s2++;
	      ref2 = *s1++;
	     	      	      		  	
	      if(ref2 != ref)
		{
		  *eqptr++ = -1;
		  calcptrR = &calcR[ref];
		  calcptr = &calcQ[ref2];
		  eigv = &EIGV[0][0];
		  x1px2 = calcptrR->sumaq * calcptr->sumaq;
		  x3->a =  x1px2 * *eigv++;/*EIGV[0][0];*/
		  x3->c =  x1px2 * *eigv++;/*EIGV[0][1];*/
		  x3->g =  x1px2 * *eigv++;/*EIGV[0][2];*/
		  x3->t =  x1px2 * *eigv++;/*EIGV[0][3];*/
	    
		  x1px2 = calcptrR->sumcq * calcptr->sumcq;
		  x3->a +=  x1px2 * *eigv++;/*EIGV[1][0];*/
		  x3->c +=  x1px2 * *eigv++;/*EIGV[1][1];*/
		  x3->g +=  x1px2 * *eigv++;/*EIGV[1][2];*/
		  x3->t +=  x1px2 * *eigv++;/*EIGV[1][3];*/
	    
		  x1px2 = calcptrR->sumgq * calcptr->sumgq;
		  x3->a += x1px2 * *eigv++;/*EIGV[2][0];*/
		  x3->c += x1px2 * *eigv++;/*EIGV[2][1];*/
		  x3->g += x1px2 * *eigv++;/*EIGV[2][2];*/
		  x3->t += x1px2 * *eigv++;/*EIGV[2][3];*/
	    
		  x1px2 = calcptrR->sumtq * calcptr->sumtq;
		  x3->a += x1px2  *  *eigv++;/*EIGV[3][0];*/
		  x3->c += x1px2  *  *eigv++;/*EIGV[3][1];*/
		  x3->g += x1px2  *  *eigv++;/*EIGV[3][2];*/
		  x3->t += x1px2  *  *eigv++;/*EIGV[3][3];*/		     
		  
		  x3->exp = calcptrR->exp + calcptr->exp;
		  if (ABS(x3->a) < minlikelihood && ABS(x3->g) < minlikelihood
		      && ABS(x3->c) < minlikelihood && ABS(x3->t) < minlikelihood) 
		    {	     
		      x3->a   *= twotothe256;
		      x3->g   *= twotothe256;
		      x3->c   *= twotothe256;
		      x3->t   *= twotothe256;
		      x3->exp += 1;
		    }
		  x3++;   
		}
	      else
		{
		  *eqptr++ = ref;		    		
		}
	   	      	    
	    }          
	}
      else
	{
	  for (i = 0; i < tr->cdta->endsite; i++) 
	    {
	      ref = *s2++;
	      ref2 = *s1++;
	      
	      if(ref2 >= 0)
		{		  	
		  if(ref2 == ref)
		    {
		      *eqptr++ = ref;		     		  
		      goto incTip;		    		  		 		  	    		
		    }
		  calcptr = &calcQ[ref2];
		  ump_x1_0 = calcptr->sumaq;
		  ump_x1_1 = calcptr->sumcq;
		  ump_x1_2 = calcptr->sumgq;
		  ump_x1_3 = calcptr->sumtq;		 		  		       		  		
		  mexp = calcptr->exp;
		}	    	  	  	 	 
	      else
		{		
		  tmp_x1_1 = x1->c * diagptable_x1[1];
		  tmp_x1_2 = x1->g * diagptable_x1[2];
		  tmp_x1_3 = x1->t * diagptable_x1[3];
		  
		  ump_x1_0 = tmp_x1_1 * EIGV[0][1];
		  ump_x1_0 += tmp_x1_2 * EIGV[0][2];
		  ump_x1_0 += tmp_x1_3 * EIGV[0][3];	      	
		  ump_x1_0 *= invfreq[0];
		  ump_x1_0 += x1->a;
		  
		  ump_x1_1 = tmp_x1_1 * EIGV[1][1];
		  ump_x1_1 += tmp_x1_2 * EIGV[1][2];
		  ump_x1_1 += tmp_x1_3 * EIGV[1][3];	      	
		  ump_x1_1 *= invfreq[1];
		  ump_x1_1 += x1->a;
		  
		  ump_x1_2 = tmp_x1_1 * EIGV[2][1];
		  ump_x1_2 += tmp_x1_2 * EIGV[2][2];
		  ump_x1_2 += tmp_x1_3 * EIGV[2][3];	      	
		  ump_x1_2 *= invfreq[2];
		  ump_x1_2 += x1->a;

		  ump_x1_3 = tmp_x1_1 * EIGV[3][1];
		  ump_x1_3 += tmp_x1_2 * EIGV[3][2];
		  ump_x1_3 += tmp_x1_3 * EIGV[3][3];	      	
		  ump_x1_3 *= invfreq[3];
		  ump_x1_3 += x1->a;
		  mexp = x1->exp;
		  x1++;
		}
	      	      
	      if(ref >= 0)
		{
		  calcptr = &calcR[ref];		
		  eigv = &EIGV[0][0];
		  x3->a = ump_x1_0 * calcptr->sumaq * *eigv++;/*EIGV[0][0];*/
		  x3->c = ump_x1_0 * calcptr->sumaq *  *eigv++;/*EIGV[0][1];*/
		  x3->g = ump_x1_0 * calcptr->sumaq *  *eigv++;/*EIGV[0][2];*/
		  x3->t = ump_x1_0 * calcptr->sumaq *  *eigv++;/*EIGV[0][3]; */
		  
		  x3->a += ump_x1_1 * calcptr->sumcq *  *eigv++;/*EIGV[1][0];*/
		  x3->c += ump_x1_1 * calcptr->sumcq *  *eigv++;/*EIGV[1][1];*/
		  x3->g += ump_x1_1 * calcptr->sumcq *  *eigv++;/*EIGV[1][2];*/
		  x3->t += ump_x1_1 * calcptr->sumcq *  *eigv++;/*EIGV[1][3];*/
		  
		  x3->a += ump_x1_2 * calcptr->sumgq *  *eigv++;/*EIGV[2][0];*/
		  x3->c += ump_x1_2 * calcptr->sumgq *  *eigv++;/*EIGV[2][1];*/
		  x3->g += ump_x1_2 * calcptr->sumgq *  *eigv++;/*EIGV[2][2];*/
		  x3->t += ump_x1_2 * calcptr->sumgq *  *eigv++;/*EIGV[2][3];*/
		  
		  x3->a += ump_x1_3 *  calcptr->sumtq *  *eigv++;/*EIGV[3][0];*/
		  x3->c += ump_x1_3 *  calcptr->sumtq *  *eigv++;/*EIGV[3][1];*/
		  x3->g += ump_x1_3 *  calcptr->sumtq *  *eigv++;/*EIGV[3][2];*/
		  x3->t += ump_x1_3 *  calcptr->sumtq *  *eigv++;/*EIGV[3][3];*/
		 
		  x3->exp = mexp + calcptr->exp;
		}
	      else
		{
		   tmp_x2_1 = x2->c * diagptable_x2[1];
		   tmp_x2_2 = x2->g * diagptable_x2[2];
		   tmp_x2_3 = x2->t * diagptable_x2[3];
	 
	 	    	    	     	     
		   ump_x2_0 = tmp_x2_1 * EIGV[0][1];
		   ump_x2_0 += tmp_x2_2 * EIGV[0][2];
		   ump_x2_0 += tmp_x2_3 * EIGV[0][3];		     
		   ump_x2_0 *= invfreq[0];
		   ump_x2_0 += x2->a;
	  
		   ump_x2_1 = tmp_x2_1 * EIGV[1][1];
		   ump_x2_1 += tmp_x2_2 * EIGV[1][2];
		   ump_x2_1 += tmp_x2_3 * EIGV[1][3];		     
		   ump_x2_1 *= invfreq[1];
		   ump_x2_1 += x2->a;	 

		   ump_x2_2 = tmp_x2_1 * EIGV[2][1];
		   ump_x2_2 += tmp_x2_2 * EIGV[2][2];
		   ump_x2_2 += tmp_x2_3 * EIGV[2][3];		     
		   ump_x2_2 *= invfreq[2];
		   ump_x2_2 += x2->a;	  
		   

		   ump_x2_3 = tmp_x2_1 * EIGV[3][1] ;
		   ump_x2_3 += tmp_x2_2 * EIGV[3][2];
		   ump_x2_3 += tmp_x2_3 * EIGV[3][3];		     
		   ump_x2_3 *= invfreq[3];
		   ump_x2_3 += x2->a;	    	  		   	  
		   eigv = &EIGV[0][0];

		   x1px2 = ump_x1_0 * ump_x2_0;
		   x3->a = x1px2 *  *eigv++;/*EIGV[0][0];*/
		   x3->c = x1px2 *  *eigv++;/*EIGV[0][1];*/
		   x3->g = x1px2 *  *eigv++;/*EIGV[0][2];*/
		   x3->t = x1px2 *  *eigv++;/*EIGV[0][3]; */

		   x1px2 = ump_x1_1 * ump_x2_1;
		   x3->a += x1px2  *  *eigv++;/*EIGV[1][0];*/
		   x3->c += x1px2 *  *eigv++;/*EIGV[1][1];*/
		   x3->g += x1px2 *  *eigv++;/*EIGV[1][2];*/
		   x3->t += x1px2 *  *eigv++;/*EIGV[1][3];*/
	   
		   x1px2 = ump_x1_2 * ump_x2_2;
		   x3->a += x1px2 *  *eigv++;/*EIGV[2][0];*/
		   x3->c += x1px2*  *eigv++;/*EIGV[2][1];*/
		   x3->g += x1px2 *  *eigv++;/*EIGV[2][2];*/
		   x3->t += x1px2 *  *eigv++;/*EIGV[2][3];*/

		   x1px2 = ump_x1_3 * ump_x2_3;
		   x3->a += x1px2 *   *eigv++;/*EIGV[3][0];*/
		   x3->c += x1px2 *   *eigv++;/*EIGV[3][1];*/
		   x3->g += x1px2 *   *eigv++;/*EIGV[3][2];*/
		   x3->t += x1px2 *   *eigv++;/*EIGV[3][3];*/
		   
		   x3->exp = mexp + x2->exp;
		   x2++;
		}
	      	    
	      if (ABS(x3->a) < minlikelihood && ABS(x3->g) < minlikelihood
		  && ABS(x3->c) < minlikelihood && ABS(x3->t) < minlikelihood) 
		{	     
		  x3->a   *= twotothe256;
		  x3->g   *= twotothe256;
		  x3->c   *= twotothe256;
		  x3->t   *= twotothe256;
		  x3->exp += 1;
		}


 	      
	      *eqptr++ = -1;
	      x3++;
	    incTip:	 ;
	    }    	  	 
	}
    return TRUE;
  }
}





double evaluateGTR (tr, p)
    tree    *tr;
    nodeptr  p;
  { /* evaluate */
    double   sum, z, lz, term;    
    nodeptr  q;
    int     i, j;
    int     *wptr = tr->cdta->aliaswgt;
    
    double  diagptable[4];
    double *EIGN = &(tr->rdta->EIGN[0][0]);
    likelivector   *x1, *x2;
    homType *mlvP, *mlvQ;
    nodeptr realQ, realP;
    register  int *sQ; 
    register  int *sP;
    long refQ, refP;    
    q = p->back;
    while ((! p->x) || (! q->x)) {
      if (! (p->x)) if (! newviewGTR(tr, p)) return badEval;
      if (! (q->x)) if (! newviewGTR(tr, q)) return badEval;
    }


    x1  = p->x->lv;
    x2  = q->x->lv;
    z = p->z;
   
    if (z < zmin) z = zmin;
    lz = log(z);

    diagptable[0] = 1;
    diagptable[1] = exp (EIGN[1] * lz);
    diagptable[2] = exp (EIGN[2] * lz);
    diagptable[3] = exp (EIGN[3] * lz);
    
    
    
    sum = 0.0;
    realQ =  tr->nodep[q->number];
    realP =  tr->nodep[p->number];
    mlvP = realP->mlv;
    mlvQ = realQ->mlv;
    sQ = realQ->equalityVector;
    sP = realP->equalityVector;

   

    for (i = 0; i < tr->cdta->endsite; i++) {
	    refQ = *sQ++;
	    refP = *sP++;

	    

	    if(refP >= 0)
	      {
		if(refQ >= 0)
		  {
		    mlvP = &realP->mlv[refP];
		    mlvQ = &realQ->mlv[refQ];
		    
		    term =  mlvP->a * mlvQ->a * diagptable[0];
		    term += mlvP->c * mlvQ->c * diagptable[1];
		    term += mlvP->g * mlvQ->g * diagptable[2];
		    term += mlvP->t * mlvQ->t * diagptable[3];     
		    term = log(term) + (mlvQ->exp + mlvP->exp)*log(minlikelihood);
		    sum += *wptr++ * term;
		  }
		else
		  {
		    mlvP = &realP->mlv[refP];
		    term =  mlvP->a * x2->a * diagptable[0];
		    term += mlvP->c * x2->c * diagptable[1];
		    term += mlvP->g * x2->g * diagptable[2];
		    term += mlvP->t * x2->t * diagptable[3];     
		    term = log(term) + (mlvP->exp + x2->exp)*log(minlikelihood);
		   
		    sum += *wptr++ * term;      		    		   
		    x2++;
		  }
	      }
	    else
	      {
		if(refQ >= 0)
		  {
		    mlvQ = &realQ->mlv[refQ];
		    term =  x1->a * mlvQ->a * diagptable[0];
		    term += x1->c * mlvQ->c * diagptable[1];
		    term += x1->g * mlvQ->g * diagptable[2];
		    term += x1->t * mlvQ->t * diagptable[3];     
		    term = log(term) + (x1->exp + mlvQ->exp)*log(minlikelihood); 
		    sum += *wptr++ * term;		   
		    x1++;
		  }
		else
		  {
		    term =  x1->a * x2->a * diagptable[0];
		    term += x1->c * x2->c * diagptable[1];
		    term += x1->g * x2->g * diagptable[2];
		    term += x1->t * x2->t * diagptable[3];     
		    term = log(term) + (x1->exp + x2->exp)*log(minlikelihood);
		    sum += *wptr++ * term;		  
		    x1++;
		    x2++;
		  }
	      }	    
    }
    
    tr->likelihood = sum;
    return  sum;
  } /* evaluate */


double makenewzGTR (tr, p, q, z0, maxiter)
    tree    *tr;
    nodeptr  p, q;
    double   z0;
    int  maxiter;
  { /* makenewz */
    double   z, zprev, zstep;
    likelivector  *x1, *x2;
    double  *sumtable, *sum;
    int     i;
    double  dlnLdlz = 0.0;                 /*  = d(ln(likelihood))/d(lz) */
    double  d2lnLdlz2 = 0.0;               /*  = d2(ln(likelihood))/d(lz)2 */
    homType *mlvP, *mlvQ;
    nodeptr realQ, realP;
    register  int *sQ; 
    register  int *sP;
    long refQ, refP;
    double d0, d1, d2, d3;
    double e0, e1, e2, e3;

    while ((! p->x) || (! q->x)) {
      if (! (p->x)) if (! newviewGTR(tr, p)) return badZ;
      if (! (q->x)) if (! newviewGTR(tr, q)) return badZ;
    }

    x1 = p->x->lv;
    x2 = q->x->lv;

    sum = sumtable = (double *)malloc(4 * tr->cdta->endsite * 
				      sizeof(double));     
    realQ =  tr->nodep[q->number];
    realP =  tr->nodep[p->number];
    mlvP = realP->mlv;
    mlvQ = realQ->mlv;
    sQ = realQ->equalityVector;
    sP = realP->equalityVector;

    for (i = 0; i < tr->cdta->endsite; i++) 
      {     
	refQ = *sQ++;
	refP = *sP++;


	if(refP >= 0)
	      {
		if(refQ >= 0)
		  {
		    mlvP = &realP->mlv[refP];
		    mlvQ = &realQ->mlv[refQ];
		    *sum++ = mlvP->a * mlvQ->a;
		    *sum++ = mlvP->c * mlvQ->c;
		    *sum++ = mlvP->g * mlvQ->g;
		    *sum++ = mlvP->t * mlvQ->t;
		  }
		else
		  {
		    mlvP = &realP->mlv[refP];
		    *sum++ = mlvP->a * x2->a;
		    *sum++ = mlvP->c * x2->c;
		    *sum++ = mlvP->g * x2->g;
		    *sum++ = mlvP->t * x2->t;
    		    		   
		    x2++;
		  }
	      }
	    else
	      {
		if(refQ >= 0)
		  {
		    mlvQ = &realQ->mlv[refQ];
		    *sum++ = x1->a * mlvQ->a;
		    *sum++ = x1->c * mlvQ->c;
		    *sum++ = x1->g * mlvQ->g;
		    *sum++ = x1->t * mlvQ->t;

		    x1++;
		  }
		else
		  {
		    *sum++ = x1->a * x2->a;
		    *sum++ = x1->c * x2->c;
		    *sum++ = x1->g * x2->g;
		    *sum++ = x1->t * x2->t;
		    
		    x1++;
		    x2++;
		  }
	      }

      }

   
   

    z = z0;
    do {
      int curvatOK = FALSE;

      zprev = z;
      
      zstep = (1.0 - zmax) * z + zmin;

      do {
	double  diagptable[4];
	double *EIGN    = &(tr->rdta->EIGN[0][0]);
	double lz;
        double *wrptr   = &(tr->cdta->wr[0]);
        double *wr2ptr  = &(tr->cdta->wr2[0]);

        dlnLdlz = 0.0;                 /*  = d(ln(likelihood))/d(lz) */
        d2lnLdlz2 = 0.0;               /*  = d2(ln(likelihood))/d(lz)2 */

        if (z < zmin) z = zmin;
        else if (z > zmax) z = zmax;
        lz    = log(z);
        
	d1 = exp (EIGN[1] * lz); 
	d2 = exp (EIGN[2] * lz); 
	d3 = exp (EIGN[3] * lz); 
	
	sum = sumtable;
        for (i = 0; i < tr->cdta->endsite; i++) 
	  {
	    double tmp_0, tmp_1, tmp_2, tmp_3;
	    double t, inv_Li, dlnLidlz = 0, d2lnLidlz2 = 0;
	    	 
	    inv_Li = (tmp_0 = *sum++);
	    inv_Li += (tmp_1 = *sum++ * d1);
	    inv_Li += (tmp_2 = *sum++ * d2);
	    inv_Li += (tmp_3 = *sum++ * d3);

	    inv_Li = 1.0/inv_Li;
	     	  	  
	    t = tmp_1 * EIGN[1];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[1];
	    
	    t = tmp_2 * EIGN[2];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[2];
	    
	    t = tmp_3 * EIGN[3];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[3];
	    
	    dlnLidlz   *= inv_Li;
	    d2lnLidlz2 *= inv_Li;
	    dlnLdlz   += *wrptr++  * dlnLidlz;
	    d2lnLdlz2 += *wr2ptr++ * (d2lnLidlz2 - dlnLidlz * dlnLidlz);
	  }
	
        if ((d2lnLdlz2 >= 0.0) && (z < zmax))
          zprev = z = 0.37 * z + 0.63;  /*  Bad curvature, shorten branch */
        else
          curvatOK = TRUE;
	
      } while (! curvatOK);

      if (d2lnLdlz2 < 0.0) {
	double tantmp = -dlnLdlz / d2lnLdlz2;  /* prevent overflow */
	if (tantmp < 100) {
	  z *= exp(tantmp);
	  if (z < zmin) z = zmin;
	  if (z > 0.25 * zprev + 0.75)    /*  Limit steps toward z = 1.0 */
	    z = 0.25 * zprev + 0.75;
	} else {
	  z = 0.25 * zprev + 0.75;
	}
      }
      if (z > zmax) z = zmax;

    } while ((--maxiter > 0) && (ABS(z - zprev) > zstep));

    free (sumtable);

    return  z;
  } /* makenewz */

#endif


boolean newviewHKY85CAT (tree *tr, nodeptr p)        /*  Update likelihoods at node */
  { /* newview */
    double   zq, lzq, xvlzq, zr, lzr, xvlzr, tip_t;
    nodeptr  q, r;
    likelivector *lp, *lq, *lr;
    int  i;

    

    if (p->tip) {             /*  Make sure that data are at tip */      
      likelivector *l;
      int           code;
      yType        *yptr;
      
      if (p->x) 
	{	 
	  return TRUE;  
	}

      if (! getxtip(p)) return FALSE; /*  They are not, so get memory */
      l = p->x->lv;           /*  Pointer to first likelihood vector value */
      yptr = p->tip;          /*  Pointer to first nucleotide datum */
      for (i = 0; i < tr->cdta->endsite; i++) {
        code = *yptr++;
        l->a =  code       & 1;
        l->c = (code >> 1) & 1;
        l->g = (code >> 2) & 1;
        l->t = (code >> 3) & 1;
        l->exp = 0;

	
        l++;
        }
    
      return TRUE;
      }

/*  Internal node needs update */

    q = p->next->back;
    r = p->next->next->back;

    

    while ((! p->x) || (! q->x) || (! r->x)) {
      if (! q->x) if (! newviewHKY85CAT(tr, q))  return FALSE;
      if (! r->x) if (! newviewHKY85CAT(tr, r))  return FALSE;
      if (! p->x) if (! getxnode(p)) return FALSE;
      }

    lp = p->x->lv;

    lq = q->x->lv;
    zq = q->z;
    lzq = (zq > zmin) ? log(zq) : log(zmin);
    xvlzq = tr->rdta->xv * lzq;

    lr = r->x->lv;
    zr = r->z;
    lzr = (zr > zmin) ? log(zr) : log(zmin);
    xvlzr = tr->rdta->xv * lzr;

    { double  *zzqtable, *zvqtable,
              *zzrtable, *zvrtable,
             *zzqptr, *zvqptr, *zzrptr, *zvrptr, *rptr;
      double  fxqr, fxqy, fxqn, sumaq, sumgq, sumcq, sumtq,
              fxrr, fxry, fxrn, ki, tempi, tempj;
      int  *cptr;
      double zzq, zvq, zzr, zvr;
      int cat;

      zzqtable = (double *)malloc(tr->NumberOfCategories * sizeof(double));
      zvqtable = (double *)malloc(tr->NumberOfCategories * sizeof(double));
      zzrtable = (double *)malloc(tr->NumberOfCategories * sizeof(double));
      zvrtable = (double *)malloc(tr->NumberOfCategories * sizeof(double));
     
    
      rptr   = &(tr->cdta->patrat[0]);
      zzqptr = &(zzqtable[0]);
      zvqptr = &(zvqtable[0]);
      zzrptr = &(zzrtable[0]);
      zvrptr = &(zvrtable[0]);
      cptr  = &(tr->cdta->rateCategory[0]);

     

      for (i = 0; i < tr->NumberOfCategories; i++) {   
        ki = *rptr++;	
	
        *zzqptr++ = exp(ki *   lzq);
        *zvqptr++ = exp(ki * xvlzq);
        *zzrptr++ = exp(ki *   lzr);
        *zvrptr++ = exp(ki * xvlzr);
        }

    
     

        for (i = 0; i < tr->cdta->endsite; i++) {
          cat = *cptr++;
	  
          zzq = zzqtable[cat];
          zvq = zvqtable[cat];
	  
          fxqr = tr->rdta->freqa * lq->a + tr->rdta->freqg * lq->g;
          fxqy = tr->rdta->freqc * lq->c + tr->rdta->freqt * lq->t;
          fxqn = fxqr + fxqy;
          tempi = fxqr * tr->rdta->invfreqr;
          tempj = zvq * (tempi-fxqn) + fxqn;
          sumaq = zzq * (lq->a - tempi) + tempj;
          sumgq = zzq * (lq->g - tempi) + tempj;
          tempi = fxqy * tr->rdta->invfreqy;
          tempj = zvq * (tempi-fxqn) + fxqn;
          sumcq = zzq * (lq->c - tempi) + tempj;
          sumtq = zzq * (lq->t - tempi) + tempj;

          zzr = zzrtable[cat];
          zvr = zvrtable[cat];
          fxrr = tr->rdta->freqa * lr->a + tr->rdta->freqg * lr->g;
          fxry = tr->rdta->freqc * lr->c + tr->rdta->freqt * lr->t;
          fxrn = fxrr + fxry;
          tempi = fxrr * tr->rdta->invfreqr;
          tempj = zvr * (tempi-fxrn) + fxrn;
          lp->a = sumaq * (zzr * (lr->a - tempi) + tempj);
          lp->g = sumgq * (zzr * (lr->g - tempi) + tempj);
          tempi = fxry * tr->rdta->invfreqy;
          tempj = zvr * (tempi-fxrn) + fxrn;
          lp->c = sumcq * (zzr * (lr->c - tempi) + tempj);
          lp->t = sumtq * (zzr * (lr->t - tempi) + tempj);
          lp->exp = lq->exp + lr->exp;
	 
	  

          if (lp->a < minlikelihood && lp->g < minlikelihood
           && lp->c < minlikelihood && lp->t < minlikelihood) {
            lp->a   *= twotothe256;
            lp->g   *= twotothe256;
            lp->c   *= twotothe256;
            lp->t   *= twotothe256;
            lp->exp += 1;
            }
          lp++;
          lq++;
          lr++;
          }

	free(zzqtable);
	free(zvqtable);
	free(zzrtable);
	free(zvrtable);


      return TRUE;
      }
  } /* newview */






double evaluateHKY85CAT (tree *tr, nodeptr p)
  { /* evaluate */
    double   sum, z, lz, xvlz,
             ki, fxpa, fxpc, fxpg, fxpt, fxpr, fxpy, fxqr, fxqy,
             suma, sumb, sumc, term;


       double   zz, zv;

    double        *zztable, *zvtable,
                 *zzptr, *zvptr;
    double        *rptr;
    likelivector *lp, *lq;
    nodeptr       q;
    int           cat, *cptr, i, *wptr;

    q = p->back;
    while ((! p->x) || (! q->x)) {
      if (! (p->x)) if (! newviewHKY85CAT(tr, p)) return badEval;
      if (! (q->x)) if (! newviewHKY85CAT(tr, q)) return badEval;
      }

    lp = p->x->lv;
    lq = q->x->lv;

    z = p->z;
    if (z < zmin) z = zmin;
    lz = log(z);
    xvlz = tr->rdta->xv * lz;

    zztable = (double *)malloc(tr->NumberOfCategories * sizeof(double));
    zvtable = (double *)malloc(tr->NumberOfCategories * sizeof(double));

    rptr   = tr->cdta->patrat;
    cptr  = tr->cdta->rateCategory; 
    zzptr = zztable;
    zvptr = zvtable;

    for (i = 0; i < tr->NumberOfCategories; i++) {
      ki = *rptr++;
      *zzptr++ = exp(ki *   lz);
      *zvptr++ = exp(ki * xvlz);
      }

    wptr = &(tr->cdta->aliaswgt[0]);
    
   
    sum = 0.0;


      for (i = 0; i < tr->cdta->endsite; i++) {
        cat  = *cptr++;
        zz   = zztable[cat];
        zv   = zvtable[cat];
        fxpa = tr->rdta->freqa * lp->a;
        fxpg = tr->rdta->freqg * lp->g;
        fxpc = tr->rdta->freqc * lp->c;
        fxpt = tr->rdta->freqt * lp->t;
        suma = fxpa * lq->a + fxpc * lq->c + fxpg * lq->g + fxpt * lq->t;
        fxqr = tr->rdta->freqa * lq->a + tr->rdta->freqg * lq->g;
        fxqy = tr->rdta->freqc * lq->c + tr->rdta->freqt * lq->t;
        fxpr = fxpa + fxpg;
        fxpy = fxpc + fxpt;
        sumc = (fxpr + fxpy) * (fxqr + fxqy);
        sumb = fxpr * fxqr * tr->rdta->invfreqr + fxpy * fxqy * tr->rdta->invfreqy;
        suma -= sumb;
        sumb -= sumc;
        term = log(zz * suma + zv * sumb + sumc) + (lp->exp + lq->exp)*log(minlikelihood);

        sum += *wptr++ * term;
        
        lp++;
        lq++;
        }

      free(zztable);
      free(zvtable); 


    tr->likelihood = sum;
    return  sum;
  } /* evaluate */



double makenewzHKY85CAT (tree *tr, nodeptr p, nodeptr q, double z0, int maxiter)
  { /* makenewz */
    likelivector *lp, *lq;
    double  *abi, *bci, *sumci, *abptr, *bcptr, *sumcptr;
    double   dlnLidlz, dlnLdlz, d2lnLdlz2, z, zprev, zstep, lz, xvlz,
             ki, suma, sumb, sumc, ab, bc, inv_Li, t1, t2,
             fx1a, fx1c, fx1g, fx1t, fx1r, fx1y, fx2r, fx2y;
    double   *zztable, *zvtable,
            *zzptr, *zvptr;
    double  *rptr, *wrptr, *wr2ptr;
    int      cat, *cptr, i, curvatOK;


    while ((! p->x) || (! q->x)) {
      if (! (p->x)) if (! newviewHKY85CAT(tr, p)) return badZ;
      if (! (q->x)) if (! newviewHKY85CAT(tr, q)) return badZ;
      }

    lp = p->x->lv;
    lq = q->x->lv;

    { 
      unsigned scratch_size;
      scratch_size = sizeof(double) * tr->cdta->endsite;
      abi   = (double *) malloc(scratch_size);
      bci   = (double *) malloc(scratch_size);
      sumci = (double *) malloc(scratch_size);      
    }
    
    zztable = (double *)malloc(tr->NumberOfCategories * sizeof(double));
    zvtable = (double *)malloc(tr->NumberOfCategories * sizeof(double));

    abptr   = abi;
    bcptr   = bci;
    sumcptr = sumci;
    


    for (i = 0; i < tr->cdta->endsite; i++) {
      fx1a = tr->rdta->freqa * lp->a;
      fx1g = tr->rdta->freqg * lp->g;
      fx1c = tr->rdta->freqc * lp->c;
      fx1t = tr->rdta->freqt * lp->t;
      suma = fx1a * lq->a + fx1c * lq->c + fx1g * lq->g + fx1t * lq->t;
      fx2r = tr->rdta->freqa * lq->a + tr->rdta->freqg * lq->g;
      fx2y = tr->rdta->freqc * lq->c + tr->rdta->freqt * lq->t;
      fx1r = fx1a + fx1g;
      fx1y = fx1c + fx1t;
      *sumcptr++ = sumc = (fx1r + fx1y) * (fx2r + fx2y);
      sumb       = fx1r * fx2r * tr->rdta->invfreqr + fx1y * fx2y * tr->rdta->invfreqy;
      *abptr++   = suma - sumb;
      *bcptr++   = sumb - sumc;
      lp++;
      lq++;
      }

    z = z0;
    do {
      zprev = z;
      zstep = (1.0 - zmax) * z + zmin;
      curvatOK = FALSE;

      do {
        if (z < zmin) z = zmin;
        else if (z > zmax) z = zmax;

        lz    = log(z);
        xvlz  = tr->rdta->xv * lz;
        rptr  = tr->cdta->patrat;
	cptr  = tr->cdta->rateCategory;
        zzptr = zztable;
        zvptr = zvtable;



        for (i = 0; i < tr->NumberOfCategories; i++) {
          ki = *rptr++;
          *zzptr++ = exp(ki *   lz);
          *zvptr++ = exp(ki * xvlz);
          }

        abptr   = abi;
        bcptr   = bci;
        sumcptr = sumci;        
        wrptr   = &(tr->cdta->wr[0]);
        wr2ptr  = &(tr->cdta->wr2[0]);
        dlnLdlz = 0.0;                 /*  = d(ln(likelihood))/d(lz) */
        d2lnLdlz2 = 0.0;               /*  = d2(ln(likelihood))/d(lz)2 */



        for (i = 0; i < tr->cdta->endsite; i++) {
          cat    = *cptr++;              /*  ratecategory(i) */
          ab     = *abptr++ * zztable[cat];
          bc     = *bcptr++ * zvtable[cat];
          sumc   = *sumcptr++;
          inv_Li = 1.0/(ab + bc + sumc);
          t1     = ab * inv_Li;
          t2     = tr->rdta->xv * bc * inv_Li;
          dlnLidlz   = t1 + t2;
          dlnLdlz   += *wrptr++  * dlnLidlz;
          d2lnLdlz2 += *wr2ptr++ * (t1 + tr->rdta->xv * t2 - dlnLidlz * dlnLidlz);
          }

        if ((d2lnLdlz2 >= 0.0) && (z < zmax))
          zprev = z = 0.37 * z + 0.63;  /*  Bad curvature, shorten branch */
        else
          curvatOK = TRUE;

        } while (! curvatOK);

      if (d2lnLdlz2 < 0.0) {
        z *= exp(-dlnLdlz / d2lnLdlz2);
        if (z < zmin) z = zmin;
        if (z > 0.25 * zprev + 0.75)    /*  Limit steps toward z = 1.0 */
          z = 0.25 * zprev + 0.75;
        }

      if (z > zmax) z = zmax;

      } while ((--maxiter > 0) && (ABS(z - zprev) > zstep));

    free(abi);
    free(bci);
    free(sumci);
    free(zztable);
    free(zvtable); 

/* printf("makenewz: %le\n", z); */

    return  z;
  } /* makenewz */










#ifndef HPC

boolean newviewHKY85GAMMA (tree *tr, nodeptr p)        /*  Update likelihoods at node */
  { /* newview */
    double   zq, lzq, xvlzq, zr, lzr, xvlzr;
    nodeptr  q, r;
    gammalikelivector *lp, *lq, *lr;
    int  i, j;

   

    if (p->tip) 
      {      
	gammalikelivector *l;
	int           code;
	yType        *yptr;
      
	if (p->x) return TRUE;  
	  
	if (! getxtip(p)) return FALSE;

	l = p->x->gv;          
	yptr = p->tip;        
	
	for (i = 0; i < tr->cdta->endsite; i++) 
	  {
	    code = *yptr++;
	     
	    l->a0 =  l->a1 = l->a2 = l->a3 = code       & 1;
	    l->c0 =  l->c1 = l->c2 = l->c3 = (code >> 1) & 1;
	    l->g0 =  l->g1 = l->g2 = l->g3 = (code >> 2) & 1;
	    l->t0 =  l->t1 = l->t2 = l->t3 = (code >> 3) & 1;
	   
	    l->exp = 0;	      
	    l++;
	  }
	
    
	return TRUE;
      }

    q = p->next->back;
    r = p->next->next->back;

    while ((! p->x) || (! q->x) || (! r->x)) 
      {
	if (! q->x) if (! newviewHKY85GAMMA(tr, q))  return FALSE;
	if (! r->x) if (! newviewHKY85GAMMA(tr, r))  return FALSE;
	if (! p->x) if (! getxnode(p)) return FALSE;
      }

    lp = p->x->gv;
    lq = q->x->gv;
    lr = r->x->gv;

    zq = q->z;
    lzq = (zq > zmin) ? log(zq) : log(zmin);
    xvlzq = tr->rdta->xv * lzq;
   
    zr = r->z;
    lzr = (zr > zmin) ? log(zr) : log(zmin);
    xvlzr = tr->rdta->xv * lzr;

    {
      double  fxqr, fxqy, fxqn, sumaq, sumgq, sumcq, sumtq,
              fxrr, fxry, fxrn, ki, tempi, tempj;
    
      double zzq[4], zvq[4], zzr[4], zvr[4];
         
      for(j = 0; j < tr->gammaCATS; j++)
	{
	  ki = tr->gammaRates[j];	      
	  zzq[j] = exp(ki *   lzq);
	  zvq[j] = exp(ki * xvlzq);
	  zzr[j] = exp(ki *   lzr);
	  zvr[j] = exp(ki * xvlzr);
	}
           
      for (i = 0; i < tr->cdta->endsite; i++) 
	{     

	  /* CAT 0*/
    	  
	  fxqr = tr->rdta->freqa * lq->a0 + tr->rdta->freqg * lq->g0;
	  fxqy = tr->rdta->freqc * lq->c0 + tr->rdta->freqt * lq->t0;
	  fxqn = fxqr + fxqy;
	  tempi = fxqr * tr->rdta->invfreqr;
	  tempj = zvq[0] * (tempi-fxqn) + fxqn;
	  sumaq = zzq[0] * (lq->a0 - tempi) + tempj;
	  sumgq = zzq[0] * (lq->g0 - tempi) + tempj;
	  tempi = fxqy * tr->rdta->invfreqy;
	  tempj = zvq[0] * (tempi-fxqn) + fxqn;
	  sumcq = zzq[0] * (lq->c0 - tempi) + tempj;
	  sumtq = zzq[0] * (lq->t0 - tempi) + tempj;
	  
	  fxrr = tr->rdta->freqa * lr->a0 + tr->rdta->freqg * lr->g0;
	  fxry = tr->rdta->freqc * lr->c0 + tr->rdta->freqt * lr->t0;
	  fxrn = fxrr + fxry;
	  tempi = fxrr * tr->rdta->invfreqr;
	  tempj = zvr[0] * (tempi-fxrn) + fxrn;
	  lp->a0 = sumaq * (zzr[0] * (lr->a0 - tempi) + tempj);
	  lp->g0 = sumgq * (zzr[0] * (lr->g0 - tempi) + tempj);
	  tempi = fxry * tr->rdta->invfreqy;
	  tempj = zvr[0] * (tempi-fxrn) + fxrn;
	  lp->c0 = sumcq * (zzr[0] * (lr->c0 - tempi) + tempj);
	  lp->t0 = sumtq * (zzr[0] * (lr->t0 - tempi) + tempj);
	
	  /* CAT 1 */
	  
	  fxqr = tr->rdta->freqa * lq->a1 + tr->rdta->freqg * lq->g1;
	  fxqy = tr->rdta->freqc * lq->c1 + tr->rdta->freqt * lq->t1;
	  fxqn = fxqr + fxqy;
	  tempi = fxqr * tr->rdta->invfreqr;
	  tempj = zvq[1] * (tempi-fxqn) + fxqn;
	  sumaq = zzq[1] * (lq->a1 - tempi) + tempj;
	  sumgq = zzq[1] * (lq->g1 - tempi) + tempj;
	  tempi = fxqy * tr->rdta->invfreqy;
	  tempj = zvq[1] * (tempi-fxqn) + fxqn;
	  sumcq = zzq[1] * (lq->c1 - tempi) + tempj;
	  sumtq = zzq[1] * (lq->t1 - tempi) + tempj;
	  
	  fxrr = tr->rdta->freqa * lr->a1 + tr->rdta->freqg * lr->g1;
	  fxry = tr->rdta->freqc * lr->c1 + tr->rdta->freqt * lr->t1;
	  fxrn = fxrr + fxry;
	  tempi = fxrr * tr->rdta->invfreqr;
	  tempj = zvr[1] * (tempi-fxrn) + fxrn;
	  lp->a1 = sumaq * (zzr[1] * (lr->a1 - tempi) + tempj);
	  lp->g1 = sumgq * (zzr[1] * (lr->g1 - tempi) + tempj);
	  tempi = fxry * tr->rdta->invfreqy;
	  tempj = zvr[1] * (tempi-fxrn) + fxrn;
	  lp->c1 = sumcq * (zzr[1] * (lr->c1 - tempi) + tempj);
	  lp->t1 = sumtq * (zzr[1] * (lr->t1 - tempi) + tempj);
	

	  /* CAT 2 */

	  fxqr = tr->rdta->freqa * lq->a2 + tr->rdta->freqg * lq->g2;
	  fxqy = tr->rdta->freqc * lq->c2 + tr->rdta->freqt * lq->t2;
	  fxqn = fxqr + fxqy;
	  tempi = fxqr * tr->rdta->invfreqr;
	  tempj = zvq[2] * (tempi-fxqn) + fxqn;
	  sumaq = zzq[2] * (lq->a2 - tempi) + tempj;
	  sumgq = zzq[2] * (lq->g2 - tempi) + tempj;
	  tempi = fxqy * tr->rdta->invfreqy;
	  tempj = zvq[2] * (tempi-fxqn) + fxqn;
	  sumcq = zzq[2] * (lq->c2 - tempi) + tempj;
	  sumtq = zzq[2] * (lq->t2 - tempi) + tempj;
	  
	  fxrr = tr->rdta->freqa * lr->a2 + tr->rdta->freqg * lr->g2;
	  fxry = tr->rdta->freqc * lr->c2 + tr->rdta->freqt * lr->t2;
	  fxrn = fxrr + fxry;
	  tempi = fxrr * tr->rdta->invfreqr;
	  tempj = zvr[2] * (tempi-fxrn) + fxrn;
	  lp->a2 = sumaq * (zzr[2] * (lr->a2 - tempi) + tempj);
	  lp->g2 = sumgq * (zzr[2] * (lr->g2 - tempi) + tempj);
	  tempi = fxry * tr->rdta->invfreqy;
	  tempj = zvr[2] * (tempi-fxrn) + fxrn;
	  lp->c2 = sumcq * (zzr[2] * (lr->c2 - tempi) + tempj);
	  lp->t2 = sumtq * (zzr[2] * (lr->t2 - tempi) + tempj);
	

	  /* CAT 3 */

	  fxqr = tr->rdta->freqa * lq->a3 + tr->rdta->freqg * lq->g3;
	  fxqy = tr->rdta->freqc * lq->c3 + tr->rdta->freqt * lq->t3;
	  fxqn = fxqr + fxqy;
	  tempi = fxqr * tr->rdta->invfreqr;
	  tempj = zvq[3] * (tempi-fxqn) + fxqn;
	  sumaq = zzq[3] * (lq->a3 - tempi) + tempj;
	  sumgq = zzq[3] * (lq->g3 - tempi) + tempj;
	  tempi = fxqy * tr->rdta->invfreqy;
	  tempj = zvq[3] * (tempi-fxqn) + fxqn;
	  sumcq = zzq[3] * (lq->c3 - tempi) + tempj;
	  sumtq = zzq[3] * (lq->t3 - tempi) + tempj;
	  
	  fxrr = tr->rdta->freqa * lr->a3 + tr->rdta->freqg * lr->g3;
	  fxry = tr->rdta->freqc * lr->c3 + tr->rdta->freqt * lr->t3;
	  fxrn = fxrr + fxry;
	  tempi = fxrr * tr->rdta->invfreqr;
	  tempj = zvr[3] * (tempi-fxrn) + fxrn;
	  lp->a3 = sumaq * (zzr[3] * (lr->a3 - tempi) + tempj);
	  lp->g3 = sumgq * (zzr[3] * (lr->g3 - tempi) + tempj);
	  tempi = fxry * tr->rdta->invfreqy;
	  tempj = zvr[3] * (tempi-fxrn) + fxrn;
	  lp->c3 = sumcq * (zzr[3] * (lr->c3 - tempi) + tempj);
	  lp->t3 = sumtq * (zzr[3] * (lr->t3 - tempi) + tempj);
	

	  /* Scaling */
	  


	  lp->exp = lq->exp + lr->exp;
	  	     
	  if (
	      lp->a0 < minlikelihood && lp->g0 < minlikelihood && lp->c0 < minlikelihood && lp->t0 < minlikelihood &&
	      lp->a1 < minlikelihood && lp->g1 < minlikelihood && lp->c1 < minlikelihood && lp->t1 < minlikelihood &&
	      lp->a2 < minlikelihood && lp->g2 < minlikelihood && lp->c2 < minlikelihood && lp->t2 < minlikelihood &&
	      lp->a3 < minlikelihood && lp->g3 < minlikelihood && lp->c3 < minlikelihood && lp->t3 < minlikelihood
	      ) 
	    {
	      lp->a0   *= twotothe256;
	      lp->g0   *= twotothe256;
	      lp->c0   *= twotothe256;
	      lp->t0   *= twotothe256;
	      
	      lp->a1   *= twotothe256;
	      lp->g1   *= twotothe256;
	      lp->c1   *= twotothe256;
	      lp->t1   *= twotothe256;

	      lp->a2   *= twotothe256;
	      lp->g2   *= twotothe256;
	      lp->c2   *= twotothe256;
	      lp->t2   *= twotothe256;

	      lp->a3   *= twotothe256;
	      lp->g3   *= twotothe256;
	      lp->c3   *= twotothe256;
	      lp->t3   *= twotothe256;

	      lp->exp += 1;
	    }
	    
	  lp++;
	  lq++;
	  lr++;
	}

      return TRUE;
    }
  } /* newview */



double evaluateHKY85GAMMA (tree *tr, nodeptr p)
  { /* evaluate */
    double   sum, z, lz, xvlz,
             ki, fxpa, fxpc, fxpg, fxpt, fxpr, fxpy, fxqr, fxqy,
             suma, sumb, sumc, term;
    double   zz[4], zv[4];
    gammalikelivector *lp, *lq;
    nodeptr       q;
    int           i, *wptr, j;

   
    
    
    q = p->back;
    while ((! p->x) || (! q->x)) 
      {
	if (! (p->x)) if (! newviewHKY85GAMMA(tr, p)) return badEval;
	if (! (q->x)) if (! newviewHKY85GAMMA(tr, q)) return badEval;
      }
    		
	     
    lp = p->x->gv;
    lq = q->x->gv;   
    
    z = p->z;
    if (z < zmin) z = zmin;
    lz = log(z);
    xvlz = tr->rdta->xv * lz;           	   
       
    sum = 0.0;

    for(j = 0; j < tr->gammaCATS; j++)
      {
	ki = tr->gammaRates[j];
    
	zz[j] = exp(ki *   lz);
	zv[j] = exp(ki * xvlz);      
      }
	
    wptr = &(tr->cdta->aliaswgt[0]);

    for (i = 0; i < tr->cdta->endsite; i++) 
      {                
	fxpa = tr->rdta->freqa * lp->a0;
	fxpg = tr->rdta->freqg * lp->g0;
	fxpc = tr->rdta->freqc * lp->c0;
	fxpt = tr->rdta->freqt * lp->t0;
	suma = fxpa * lq->a0 + fxpc * lq->c0 + fxpg * lq->g0 + fxpt * lq->t0;
	fxqr = tr->rdta->freqa * lq->a0 + tr->rdta->freqg * lq->g0;
	fxqy = tr->rdta->freqc * lq->c0 + tr->rdta->freqt * lq->t0;
	fxpr = fxpa + fxpg;
	fxpy = fxpc + fxpt;
	sumc = (fxpr + fxpy) * (fxqr + fxqy);
	sumb = fxpr * fxqr * tr->rdta->invfreqr + fxpy * fxqy * tr->rdta->invfreqy;
	suma -= sumb;
	sumb -= sumc;

	term = zz[0] * suma + zv[0] * sumb + sumc;

	fxpa = tr->rdta->freqa * lp->a1;
	fxpg = tr->rdta->freqg * lp->g1;
	fxpc = tr->rdta->freqc * lp->c1;
	fxpt = tr->rdta->freqt * lp->t1;
	suma = fxpa * lq->a1 + fxpc * lq->c1 + fxpg * lq->g1 + fxpt * lq->t1;
	fxqr = tr->rdta->freqa * lq->a1 + tr->rdta->freqg * lq->g1;
	fxqy = tr->rdta->freqc * lq->c1 + tr->rdta->freqt * lq->t1;
	fxpr = fxpa + fxpg;
	fxpy = fxpc + fxpt;
	sumc = (fxpr + fxpy) * (fxqr + fxqy);
	sumb = fxpr * fxqr * tr->rdta->invfreqr + fxpy * fxqy * tr->rdta->invfreqy;
	suma -= sumb;
	sumb -= sumc;


	term += zz[1] * suma + zv[1] * sumb + sumc;

	fxpa = tr->rdta->freqa * lp->a2;
	fxpg = tr->rdta->freqg * lp->g2;
	fxpc = tr->rdta->freqc * lp->c2;
	fxpt = tr->rdta->freqt * lp->t2;
	suma = fxpa * lq->a2 + fxpc * lq->c2 + fxpg * lq->g2 + fxpt * lq->t2;
	fxqr = tr->rdta->freqa * lq->a2 + tr->rdta->freqg * lq->g2;
	fxqy = tr->rdta->freqc * lq->c2 + tr->rdta->freqt * lq->t2;
	fxpr = fxpa + fxpg;
	fxpy = fxpc + fxpt;
	sumc = (fxpr + fxpy) * (fxqr + fxqy);
	sumb = fxpr * fxqr * tr->rdta->invfreqr + fxpy * fxqy * tr->rdta->invfreqy;
	suma -= sumb;
	sumb -= sumc;
	

	term += zz[2] * suma + zv[2] * sumb + sumc;

	fxpa = tr->rdta->freqa * lp->a3;
	fxpg = tr->rdta->freqg * lp->g3;
	fxpc = tr->rdta->freqc * lp->c3;
	fxpt = tr->rdta->freqt * lp->t3;
	suma = fxpa * lq->a3 + fxpc * lq->c3 + fxpg * lq->g3 + fxpt * lq->t3;
	fxqr = tr->rdta->freqa * lq->a3 + tr->rdta->freqg * lq->g3;
	fxqy = tr->rdta->freqc * lq->c3 + tr->rdta->freqt * lq->t3;
	fxpr = fxpa + fxpg;
	fxpy = fxpc + fxpt;
	sumc = (fxpr + fxpy) * (fxqr + fxqy);
	sumb = fxpr * fxqr * tr->rdta->invfreqr + fxpy * fxqy * tr->rdta->invfreqy;
	suma -= sumb;
	sumb -= sumc;

	term += zz[3] * suma + zv[3] * sumb + sumc;

	term = log(0.25 * term) + (lp->exp + lq->exp) * log(minlikelihood);

	sum += term * *wptr++; 
	    
	lp++;
	lq++;       
      }       

    tr->likelihood = sum;
    return  sum;
  } /* evaluate */



double makenewzHKY85GAMMA (tree *tr, nodeptr p, nodeptr q, double z0, int maxiter)
  { /* makenewz */
    gammalikelivector *lp, *lq;
    double  *abi, *bci, *sumci, *abptr, *bcptr, *sumcptr;
      
    double   dlnLidlz, dlnLdlz, d2lnLdlz2, z, zprev, zstep, lz, xvlz,
      ki, suma, sumb, sumc, ab, bc, inv_Li, t1, t2, t3, Li, t1_0, t1_1, t1_2, t1_3, t2_0, t2_1, t2_2, t2_3,
      fx1a, fx1c, fx1g, fx1t, fx1r, fx1y, fx2r, fx2y, sumc0, ab0, bc0, sumc1, ab1, bc1,
      sumc2, ab2, bc2, sumc3, ab3, bc3, 
      dlnLdlz_0, d2lnLdlz2_0, dlnLdlz_1, d2lnLdlz2_1, dlnLdlz_2, d2lnLdlz2_2, dlnLdlz_3, d2lnLdlz2_3, inv_Li0, inv_Li1, inv_Li2, inv_Li3,
      t1d2, t2d2;
    double   zz[4], zv[4], gammaSquares[4];
    double  *rptr, *wrptr, *wr2ptr;
    int      cat, *cptr, i, curvatOK, j;


    while ((! p->x) || (! q->x)) 
      {
	if (! (p->x)) if (! newviewHKY85GAMMA(tr, p)) return badZ;
	if (! (q->x)) if (! newviewHKY85GAMMA(tr, q)) return badZ;
      }

    lp = p->x->gv;
    lq = q->x->gv;

    { 
      unsigned scratch_size;
      scratch_size = sizeof(double) * tr->cdta->endsite;
      abi   = (double *) malloc(scratch_size * 4);
      bci   = (double *) malloc(scratch_size * 4);
      sumci = (double *) malloc(scratch_size);      
    }
       

    abptr   = abi;
    bcptr   = bci;
    sumcptr = sumci;
    
    for (i = 0; i < tr->cdta->endsite; i++) 
      {
	fx1a = tr->rdta->freqa * lp->a0;
	fx1g = tr->rdta->freqg * lp->g0;
	fx1c = tr->rdta->freqc * lp->c0;
	fx1t = tr->rdta->freqt * lp->t0;
	suma = fx1a * lq->a0 + fx1c * lq->c0 + fx1g * lq->g0 + fx1t * lq->t0;
	fx2r = tr->rdta->freqa * lq->a0 + tr->rdta->freqg * lq->g0;
	fx2y = tr->rdta->freqc * lq->c0 + tr->rdta->freqt * lq->t0;
	fx1r = fx1a + fx1g;
	fx1y = fx1c + fx1t;
	*sumcptr = sumc = (fx1r + fx1y) * (fx2r + fx2y);
	sumb       = fx1r * fx2r * tr->rdta->invfreqr + fx1y * fx2y * tr->rdta->invfreqy;
	*abptr++   = (suma - sumb);
	*bcptr++   = (sumb - sumc);

	fx1a = tr->rdta->freqa * lp->a1;
	fx1g = tr->rdta->freqg * lp->g1;
	fx1c = tr->rdta->freqc * lp->c1;
	fx1t = tr->rdta->freqt * lp->t1;
	suma = fx1a * lq->a1 + fx1c * lq->c1 + fx1g * lq->g1 + fx1t * lq->t1;
	fx2r = tr->rdta->freqa * lq->a1 + tr->rdta->freqg * lq->g1;
	fx2y = tr->rdta->freqc * lq->c1 + tr->rdta->freqt * lq->t1;
	fx1r = fx1a + fx1g;
	fx1y = fx1c + fx1t;
	*sumcptr += sumc = (fx1r + fx1y) * (fx2r + fx2y);
	sumb       = fx1r * fx2r * tr->rdta->invfreqr + fx1y * fx2y * tr->rdta->invfreqy;
	*abptr++   = (suma - sumb);
	*bcptr++   = (sumb - sumc);

	fx1a = tr->rdta->freqa * lp->a2;
	fx1g = tr->rdta->freqg * lp->g2;
	fx1c = tr->rdta->freqc * lp->c2;
	fx1t = tr->rdta->freqt * lp->t2;
	suma = fx1a * lq->a2 + fx1c * lq->c2 + fx1g * lq->g2 + fx1t * lq->t2;
	fx2r = tr->rdta->freqa * lq->a2 + tr->rdta->freqg * lq->g2;
	fx2y = tr->rdta->freqc * lq->c2 + tr->rdta->freqt * lq->t2;
	fx1r = fx1a + fx1g;
	fx1y = fx1c + fx1t;
	*sumcptr += sumc = (fx1r + fx1y) * (fx2r + fx2y);
	sumb       = fx1r * fx2r * tr->rdta->invfreqr + fx1y * fx2y * tr->rdta->invfreqy;
	*abptr++   = (suma - sumb);
	*bcptr++   = (sumb - sumc);

	fx1a = tr->rdta->freqa * lp->a3;
	fx1g = tr->rdta->freqg * lp->g3;
	fx1c = tr->rdta->freqc * lp->c3;
	fx1t = tr->rdta->freqt * lp->t3;
	suma = fx1a * lq->a3 + fx1c * lq->c3 + fx1g * lq->g3 + fx1t * lq->t3;
	fx2r = tr->rdta->freqa * lq->a3 + tr->rdta->freqg * lq->g3;
	fx2y = tr->rdta->freqc * lq->c3 + tr->rdta->freqt * lq->t3;
	fx1r = fx1a + fx1g;
	fx1y = fx1c + fx1t;
	*sumcptr++ += sumc = (fx1r + fx1y) * (fx2r + fx2y);
	sumb       = fx1r * fx2r * tr->rdta->invfreqr + fx1y * fx2y * tr->rdta->invfreqy;
	*abptr++   = (suma - sumb);
	*bcptr++   = (sumb - sumc);

	lp++;
	lq++;
      }

    gammaSquares[0] = tr->gammaRates[0] * tr->gammaRates[0];
    gammaSquares[1] = tr->gammaRates[1] * tr->gammaRates[1];
    gammaSquares[2] = tr->gammaRates[2] * tr->gammaRates[2];
    gammaSquares[3] = tr->gammaRates[3] * tr->gammaRates[3];


    z = z0;
    do {
      zprev = z;
      zstep = (1.0 - zmax) * z + zmin;
      curvatOK = FALSE;

      do {
        if (z < zmin) z = zmin;
        else if (z > zmax) z = zmax;

        lz    = log(z);
        xvlz  = tr->rdta->xv * lz;
       
	abptr   = abi;
        bcptr   = bci;
        sumcptr = sumci;  


       
	dlnLdlz = 0.0;                 
	d2lnLdlz2 = 0.0;               

	for(j = 0; j < 4; j++)
	  {
	    ki = tr->gammaRates[j];
    
	    zz[j] = exp(ki *   lz);
	    zv[j] = exp(ki * xvlz);      
	  }
	    
	wrptr   = &(tr->cdta->wr[0]);
	wr2ptr  = &(tr->cdta->wr2[0]);	
	    
	for (i = 0; i < tr->cdta->endsite; i++) 
	  {         		
	    ab0 = *abptr++ * zz[0];
	    bc0 = *bcptr++ * zv[0];
	    sumc = ab0 + bc0 + *sumcptr++;
	    
	    ab1     =  *abptr++ * zz[1];
	    bc1     =  *bcptr++ * zv[1];
	    sumc   +=  ab1 + bc1;
	    
	    ab2    = *abptr++ * zz[2];
	    bc2     = *bcptr++ * zv[2];
	    sumc   +=  ab2 + bc2;	    
	    
	    ab3     = *abptr++ * zz[3];
	    bc3    =  *bcptr++ * zv[3];
	    sumc  +=  ab3 + bc3;

	    inv_Li = 1/sumc;	    

	    t1 = (tr->gammaRates[0] * ab0 + tr->gammaRates[1] * ab1 + tr->gammaRates[2] * ab2 + tr->gammaRates[3] * ab3) * inv_Li;

	    t2 = tr->rdta->xv * (tr->gammaRates[0] * bc0 + tr->gammaRates[1] * bc1 + tr->gammaRates[2] * bc2 + tr->gammaRates[3] * bc3) * inv_Li;

	    dlnLidlz = t1 + t2;
	  	    
	    t1d2 = (gammaSquares[0] * ab0 + gammaSquares[1] * ab1 + 
		    gammaSquares[2] * ab2 + gammaSquares[3] * ab3) * inv_Li;

	    t2d2 = tr->rdta->xv * (gammaSquares[0] * bc0 + gammaSquares[1] * bc1 + 
				   gammaSquares[2] * bc2 + gammaSquares[3] * bc3) * inv_Li;


	    dlnLdlz   += wrptr[i]  * dlnLidlz;
	    d2lnLdlz2 += wr2ptr[i] * (t1d2 + tr->rdta->xv * t2d2 - dlnLidlz * dlnLidlz);					
	  }
      
        if ((d2lnLdlz2 >= 0.0) && (z < zmax))
          zprev = z = 0.37 * z + 0.63;  /*  Bad curvature, shorten branch */
        else
          curvatOK = TRUE;

        } while (! curvatOK);

      if (d2lnLdlz2 < 0.0) {
        z *= exp(-dlnLdlz / d2lnLdlz2);
        if (z < zmin) z = zmin;
        if (z > 0.25 * zprev + 0.75)    /*  Limit steps toward z = 1.0 */
          z = 0.25 * zprev + 0.75;
        }

      if (z > zmax) z = zmax;

      } while ((--maxiter > 0) && (ABS(z - zprev) > zstep));

    free(abi);
    free(bci);
    free(sumci);   

/* printf("makenewz: %le\n", z); */

    return  z;
  } /* makenewz */

#endif





boolean newviewGTRCAT(tree    *tr, nodeptr  p)
{
  
  if (p->tip) 
    {      
      if (p->x) return TRUE;
      if (! getxtip(p)) return FALSE;  
      {
	int code, i, j, jj;
	double (*EIGV)[4] = &(tr->rdta->EIGV[0][0]);
	likelivector *x1 = p->x->lv;
     
	for(i = 0; i < tr->cdta->endsite; i++)
	  {	      	  	 
	    code = p->tip[i];
	    x1->a = 0;
	    x1->c = 0;
	    x1->g = 0;
	    x1->t = 0;
	    for (j = 0; j < 4; j++) 
	      {
		if ((code >> j) & 1) 
		  {
		    int jj = "0213"[j] - '0';		
		    x1->a += EIGV[jj][0];
		    x1->c += EIGV[jj][1];
		    x1->g += EIGV[jj][2];
		    x1->t += EIGV[jj][3];
		  }
	      }		
	      
	    x1->exp = 0;	  			  			 
	    x1++;	 
	} 
      return TRUE; 
      }
    }
  
  { 	
    double  *diagptable_x1, *diagptable_x2, *diagptable_x1_start, *diagptable_x2_start;
    double *EIGN    = &(tr->rdta->EIGN[0][0]);
    double *invfreq = &(tr->rdta->invfreq[0][0]);
    double (*EIGV)[4] = &(tr->rdta->EIGV[0][0]);
    xtype tmp_x1_1, tmp_x1_2, tmp_x1_3,
      tmp_x2_1, tmp_x2_2, tmp_x2_3, 
      ump_x1_1, ump_x1_2, ump_x1_3, ump_x1_0, 
      ump_x2_0, ump_x2_1, ump_x2_2, ump_x2_3, x1px2;
    int cat;
    int  *cptr;	  
    int  i, j;
    likelivector   *x1, *x2, *x3;
    double   z1, lz1, z2, lz2, ki;
    nodeptr  q, r;   
    double *eigv, *rptr;

    rptr   = &(tr->cdta->patrat[0]); 
    cptr  = &(tr->cdta->rateCategory[0]);

    q = p->next->back;
    r = p->next->next->back;
      
    while ((! p->x) || (! q->x) || (! r->x)) {
      if (! q->x) if (! newviewGTRCAT(tr, q)) return FALSE;
      if (! r->x) if (! newviewGTRCAT(tr, r)) return FALSE;
      if (! p->x) if (! getxnode(p)) return FALSE;	
    }

    x1  = q->x->lv;
    z1  = q->z;
    lz1 = (z1 > zmin) ? log(z1) : log(zmin);

    x2  = r->x->lv;
    z2  = r->z;
    lz2 = (z2 > zmin) ? log(z2) : log(zmin);

    x3  = p->x->lv;   
   
    
    diagptable_x1 = diagptable_x1_start = (double *)malloc(sizeof(double) * tr->NumberOfCategories * 3);
    diagptable_x2 = diagptable_x2_start = (double *)malloc(sizeof(double) * tr->NumberOfCategories * 3);
    
    
    for(i = 0; i < tr->NumberOfCategories;i++)
      {
	ki = *rptr++;
	
	*diagptable_x1++ = exp (EIGN[1] * ki * lz1);
	*diagptable_x1++ = exp (EIGN[2] * ki * lz1);
	*diagptable_x1++ = exp (EIGN[3] * ki * lz1);
	*diagptable_x2++ = exp (EIGN[1] * ki * lz2);
	*diagptable_x2++ = exp (EIGN[2] * ki * lz2);
	*diagptable_x2++ = exp (EIGN[3] * ki * lz2);		
      }

    
    for (i = 0; i < tr->cdta->endsite; i++) 
      {	
	cat = *cptr++;

	diagptable_x1 = &diagptable_x1_start[cat * 3];
	diagptable_x2 = &diagptable_x2_start[cat * 3];

	tmp_x1_1 = x1->c * *diagptable_x1++;
	tmp_x1_2 = x1->g * *diagptable_x1++;
	tmp_x1_3 = x1->t * *diagptable_x1++;
	
	ump_x1_0 = tmp_x1_1 * EIGV[0][1];
	ump_x1_0 += tmp_x1_2 * EIGV[0][2];
	ump_x1_0 += tmp_x1_3 * EIGV[0][3];	      	
	ump_x1_0 *= invfreq[0];
	ump_x1_0 += x1->a;
	
	ump_x1_1 = tmp_x1_1 * EIGV[1][1];
	ump_x1_1 += tmp_x1_2 * EIGV[1][2];
	ump_x1_1 += tmp_x1_3 * EIGV[1][3];	      	
	ump_x1_1 *= invfreq[1];
	ump_x1_1 += x1->a;
	    
	ump_x1_2 = tmp_x1_1 * EIGV[2][1];
	ump_x1_2 += tmp_x1_2 * EIGV[2][2];
	ump_x1_2 += tmp_x1_3 * EIGV[2][3];	      	
	ump_x1_2 *= invfreq[2];
	ump_x1_2 += x1->a;

	ump_x1_3 = tmp_x1_1 * EIGV[3][1];
	ump_x1_3 += tmp_x1_2 * EIGV[3][2];
	ump_x1_3 += tmp_x1_3 * EIGV[3][3];	      	
	ump_x1_3 *= invfreq[3];
	ump_x1_3 += x1->a;
	
	tmp_x2_1 = x2->c * *diagptable_x2++;
	tmp_x2_2 = x2->g * *diagptable_x2++;
	tmp_x2_3 = x2->t * *diagptable_x2++;
	 
	 	    	    	     	     
	ump_x2_0 = tmp_x2_1 * EIGV[0][1];
	ump_x2_0 += tmp_x2_2 * EIGV[0][2];
	ump_x2_0 += tmp_x2_3 * EIGV[0][3];		     
	ump_x2_0 *= invfreq[0];
	ump_x2_0 += x2->a;
	  
	ump_x2_1 = tmp_x2_1 * EIGV[1][1];
	ump_x2_1 += tmp_x2_2 * EIGV[1][2];
	ump_x2_1 += tmp_x2_3 * EIGV[1][3];		     
	ump_x2_1 *= invfreq[1];
	ump_x2_1 += x2->a;	 

	ump_x2_2 = tmp_x2_1 * EIGV[2][1];
	ump_x2_2 += tmp_x2_2 * EIGV[2][2];
	ump_x2_2 += tmp_x2_3 * EIGV[2][3];		     
	ump_x2_2 *= invfreq[2];
	ump_x2_2 += x2->a;	  
		   

	ump_x2_3 = tmp_x2_1 * EIGV[3][1] ;
	ump_x2_3 += tmp_x2_2 * EIGV[3][2];
	ump_x2_3 += tmp_x2_3 * EIGV[3][3];		     
	ump_x2_3 *= invfreq[3];
	ump_x2_3 += x2->a;	    	  		   	  
	eigv = &EIGV[0][0];
	
	x1px2 = ump_x1_0 * ump_x2_0;
	x3->a = x1px2 *  *eigv++;/*EIGV[0][0];*/
	x3->c = x1px2 *  *eigv++;/*EIGV[0][1];*/
	x3->g = x1px2 *  *eigv++;/*EIGV[0][2];*/
	x3->t = x1px2 *  *eigv++;/*EIGV[0][3]; */
	
	x1px2 = ump_x1_1 * ump_x2_1;
	x3->a += x1px2  *  *eigv++;/*EIGV[1][0];*/
	x3->c += x1px2 *  *eigv++;/*EIGV[1][1];*/
	x3->g += x1px2 *  *eigv++;/*EIGV[1][2];*/
	x3->t += x1px2 *  *eigv++;/*EIGV[1][3];*/
	
	x1px2 = ump_x1_2 * ump_x2_2;
	x3->a += x1px2 *  *eigv++;/*EIGV[2][0];*/
	x3->c += x1px2*  *eigv++;/*EIGV[2][1];*/
	x3->g += x1px2 *  *eigv++;/*EIGV[2][2];*/
	x3->t += x1px2 *  *eigv++;/*EIGV[2][3];*/
	
	x1px2 = ump_x1_3 * ump_x2_3;
	x3->a += x1px2 *   *eigv++;/*EIGV[3][0];*/
	x3->c += x1px2 *   *eigv++;/*EIGV[3][1];*/
	x3->g += x1px2 *   *eigv++;/*EIGV[3][2];*/
	x3->t += x1px2 *   *eigv++;/*EIGV[3][3];*/
		   
	x3->exp = x1->exp + x2->exp;		  
	
	if (ABS(x3->a) < minlikelihood && ABS(x3->g) < minlikelihood
	    && ABS(x3->c) < minlikelihood && ABS(x3->t) < minlikelihood) 
	  {	     
	    x3->a   *= twotothe256;
	    x3->g   *= twotothe256;
	    x3->c   *= twotothe256;
	    x3->t   *= twotothe256;
	    x3->exp += 1;
	  }
	 	
	x1++;
	x2++;
	x3++;
      
      }
         
    free(diagptable_x1_start); 
    free(diagptable_x2_start);
    return TRUE;
  }
}





double evaluateGTRCAT (tr, p)
    tree    *tr;
    nodeptr  p;
  { /* evaluate */
    double   sum, z, lz, term, ki;    
    nodeptr  q;
    int     i, j, cat;
    int     *wptr = tr->cdta->aliaswgt, *cptr;
    
    double  *diagptable, *rptr, *diagptable_start;
    double *EIGN = &(tr->rdta->EIGN[0][0]);
    likelivector   *x1, *x2;
       
    q = p->back;
    while ((! p->x) || (! q->x)) {
      if (! (p->x)) if (! newviewGTRCAT(tr, p)) return badEval;
      if (! (q->x)) if (! newviewGTRCAT(tr, q)) return badEval;
    }


    rptr   = &(tr->cdta->patrat[0]); 
    cptr  = &(tr->cdta->rateCategory[0]);

    x1  = p->x->lv;
    x2  = q->x->lv;
    z = p->z;
   
    if (z < zmin) z = zmin;
    lz = log(z);

   
    
    diagptable = diagptable_start = (double *)malloc(sizeof(double) * tr->NumberOfCategories * 3);
    for(i = 0; i <  tr->NumberOfCategories;i++)
      {
	ki = *rptr++;	 
	*diagptable++ = exp (EIGN[1] * ki * lz);
	*diagptable++ = exp (EIGN[2] * ki * lz);
	*diagptable++ = exp (EIGN[3] * ki * lz);
      }
	
    
    sum = 0.0;
  
    for (i = 0; i < tr->cdta->endsite; i++) 
      {	  	 	  
	cat = *cptr++;
	diagptable = &diagptable_start[3 * cat];

	term =  x1->a * x2->a;
	term += x1->c * x2->c * *diagptable++;
	term += x1->g * x2->g * *diagptable++;
	term += x1->t * x2->t * *diagptable++;     
	term = log(term) + (x1->exp + x2->exp)*log(minlikelihood);
	sum += *wptr++ * term;		  
	x1++;
	x2++;
      }
    
      
    free(diagptable_start); 
    
    tr->likelihood = sum;
    return  sum;
  } /* evaluate */


double makenewzGTRCAT (tr, p, q, z0, maxiter)
    tree    *tr;
    nodeptr  p, q;
    double   z0;
    int  maxiter;
  { /* makenewz */
    double   z, zprev, zstep;
    likelivector  *x1, *x2;
    double  *sumtable, *sum;
    int     i, *cptr, cat;
    double  dlnLdlz = 0.0;                 /*  = d(ln(likelihood))/d(lz) */
    double  d2lnLdlz2 = 0.0;               /*  = d2(ln(likelihood))/d(lz)2 */    
    double *d1, *d2, *d3, *rptr;
    double ki;
    
    while ((! p->x) || (! q->x)) {
      if (! (p->x)) if (! newviewGTRCAT(tr, p)) return badZ;
      if (! (q->x)) if (! newviewGTRCAT(tr, q)) return badZ;
    }

    x1 = p->x->lv;
    x2 = q->x->lv;

    sum = sumtable = (double *)malloc(4 * tr->cdta->endsite * 
				      sizeof(double));
    d1 = (double *)malloc(tr->NumberOfCategories * sizeof(double));
    d2 = (double *)malloc(tr->NumberOfCategories * sizeof(double));
    d3 = (double *)malloc(tr->NumberOfCategories * sizeof(double));
 

    for (i = 0; i < tr->cdta->endsite; i++) 
      {     
	*sum++ = x1->a * x2->a;
	*sum++ = x1->c * x2->c;
	*sum++ = x1->g * x2->g;
	*sum++ = x1->t * x2->t;		    
	x1++;
	x2++;		 
      }

   
   
    z = z0;
    do {
      int curvatOK = FALSE;

      zprev = z;
      
      zstep = (1.0 - zmax) * z + zmin;

      do {	
	double *EIGN    = &(tr->rdta->EIGN[0][0]);
	double lz;
        double *wrptr   = &(tr->cdta->wr[0]);
        double *wr2ptr  = &(tr->cdta->wr2[0]);

        dlnLdlz = 0.0;                 /*  = d(ln(likelihood))/d(lz) */
        d2lnLdlz2 = 0.0;               /*  = d2(ln(likelihood))/d(lz)2 */

        if (z < zmin) z = zmin;
        else if (z > zmax) z = zmax;
        lz    = log(z);
        
	rptr   = &(tr->cdta->patrat[0]); 
	cptr  = &(tr->cdta->rateCategory[0]);
	sum = sumtable;
	
	for(i = 0; i < tr->NumberOfCategories; i++)
	  {
	    ki = *rptr++;
	    d1[i] = exp (EIGN[1] * lz * ki); 
	    d2[i] = exp (EIGN[2] * lz * ki); 
	    d3[i] = exp (EIGN[3] * lz * ki); 
	  }


        for (i = 0; i < tr->cdta->endsite; i++) 
	  {
	    double tmp_0, tmp_1, tmp_2, tmp_3;
	    double t, inv_Li, dlnLidlz = 0, d2lnLidlz2 = 0;
	    	
	    cat = *cptr++;
	    inv_Li = (tmp_0 = *sum++);
	    inv_Li += (tmp_1 = *sum++ * d1[cat]);
	    inv_Li += (tmp_2 = *sum++ * d2[cat]);
	    inv_Li += (tmp_3 = *sum++ * d3[cat]);

	    inv_Li = 1.0/inv_Li;
	    
	  
	    t = tmp_1 * EIGN[1];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[1];
	    
	    t = tmp_2 * EIGN[2];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[2];
	    
	    t = tmp_3 * EIGN[3];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[3];
	    
	    dlnLidlz   *= inv_Li;
	    d2lnLidlz2 *= inv_Li;
	    dlnLdlz   += *wrptr++  * dlnLidlz;
	    d2lnLdlz2 += *wr2ptr++ * (d2lnLidlz2 - dlnLidlz * dlnLidlz);
	  }
	
        if ((d2lnLdlz2 >= 0.0) && (z < zmax))
          zprev = z = 0.37 * z + 0.63;  /*  Bad curvature, shorten branch */
        else
          curvatOK = TRUE;
	
      } while (! curvatOK);

      if (d2lnLdlz2 < 0.0) {
	double tantmp = -dlnLdlz / d2lnLdlz2;  /* prevent overflow */
	if (tantmp < 100) {
	  z *= exp(tantmp);
	  if (z < zmin) z = zmin;
	  if (z > 0.25 * zprev + 0.75)    /*  Limit steps toward z = 1.0 */
	    z = 0.25 * zprev + 0.75;
	} else {
	  z = 0.25 * zprev + 0.75;
	}
      }
      if (z > zmax) z = zmax;

    } while ((--maxiter > 0) && (ABS(z - zprev) > zstep));

    free (sumtable);
    free(d1);
    free(d2);
    free(d3);
    return  z;
  } /* makenewz */


/* GTRGAMMA likelihood functions *******************************************************/


#ifndef HPC

boolean newviewGTRGAMMA(tree    *tr, nodeptr  p)
{
  
  if (p->tip) 
    {      
      if (p->x) return TRUE;
      if (! getxtip(p)) return FALSE;  
      {
	int code, i, j, jj;
	double (*EIGV)[4] = &(tr->rdta->EIGV[0][0]);
	gammalikelivector *x1 = p->x->gv;
     
	for(i = 0; i < tr->cdta->endsite; i++)
	  {	      	  	 
	    code = p->tip[i];
	    x1->a0 = 0;
	    x1->c0 = 0;
	    x1->g0 = 0;
	    x1->t0 = 0;
	    for (j = 0; j < 4; j++) 
	      {
		if ((code >> j) & 1) 
		  {
		    int jj = "0213"[j] - '0';		
		    x1->a0 += EIGV[jj][0];
		    x1->c0 += EIGV[jj][1];
		    x1->g0 += EIGV[jj][2];
		    x1->t0 += EIGV[jj][3];
		  }
	      }		
	      
	    x1->a1 = x1->a2 = x1->a3 = x1->a0;
	    x1->c1 = x1->c2 = x1->c3 = x1->c0;
	    x1->g1 = x1->g2 = x1->g3 = x1->g0;
	    x1->t1 = x1->t2 = x1->t3 = x1->t0;

	    x1->exp = 0;	  			  			 
	    x1++;	 
	} 
      return TRUE; 
      }
    }
  
  { 	

    double  *diagptable_x1, *diagptable_x2, *diagptable_x1_start, *diagptable_x2_start;
    double *EIGN    = &(tr->rdta->EIGN[0][0]);
    double *invfreq = &(tr->rdta->invfreq[0][0]);
    double (*EIGV)[4] = &(tr->rdta->EIGV[0][0]);
    xtype tmp_x1_1, tmp_x1_2, tmp_x1_3,
      tmp_x2_1, tmp_x2_2, tmp_x2_3, 
      ump_x1_1, ump_x1_2, ump_x1_3, ump_x1_0, 
      ump_x2_0, ump_x2_1, ump_x2_2, ump_x2_3, x1px2;
    int cat;
    int  *cptr;	  
    int  i, j;
    gammalikelivector   *x1, *x2, *x3;
    double   z1, lz1, z2, lz2, ki;
    nodeptr  q, r;   
    double *eigv, *rptr;

    rptr   = &(tr->cdta->patrat[0]); 
    cptr  = &(tr->cdta->rateCategory[0]);

    q = p->next->back;
    r = p->next->next->back;
      
    while ((! p->x) || (! q->x) || (! r->x)) {
      if (! q->x) if (! newviewGTRGAMMA(tr, q)) return FALSE;
      if (! r->x) if (! newviewGTRGAMMA(tr, r)) return FALSE;
      if (! p->x) if (! getxnode(p)) return FALSE;	
    }

    x1  = q->x->gv;
    z1  = q->z;
    lz1 = (z1 > zmin) ? log(z1) : log(zmin);

    x2  = r->x->gv;
    z2  = r->z;
    lz2 = (z2 > zmin) ? log(z2) : log(zmin);

    x3  = p->x->gv;   
   
    
    diagptable_x1 = diagptable_x1_start = (double *)malloc(sizeof(double) * 4 * 3);
    diagptable_x2 = diagptable_x2_start = (double *)malloc(sizeof(double) * 4 * 3);
    
    
    for(i = 0; i < 4; i++)
      {
	ki = tr->gammaRates[i];
	
	*diagptable_x1++ = exp (EIGN[1] * ki * lz1);
	*diagptable_x1++ = exp (EIGN[2] * ki * lz1);
	*diagptable_x1++ = exp (EIGN[3] * ki * lz1);
	*diagptable_x2++ = exp (EIGN[1] * ki * lz2);
	*diagptable_x2++ = exp (EIGN[2] * ki * lz2);
	*diagptable_x2++ = exp (EIGN[3] * ki * lz2);		
      }

    
    for (i = 0; i < tr->cdta->endsite; i++) 
      {	

	/* Rate cat 0 */

	diagptable_x1 = diagptable_x1_start;
	diagptable_x2 = diagptable_x2_start;

	tmp_x1_1 = x1->c0 * *diagptable_x1++;
	tmp_x1_2 = x1->g0 * *diagptable_x1++;
	tmp_x1_3 = x1->t0 * *diagptable_x1++;
	
	ump_x1_0 = tmp_x1_1 * EIGV[0][1];
	ump_x1_0 += tmp_x1_2 * EIGV[0][2];
	ump_x1_0 += tmp_x1_3 * EIGV[0][3];	      	
	ump_x1_0 *= invfreq[0];
	ump_x1_0 += x1->a0;
	
	ump_x1_1 = tmp_x1_1 * EIGV[1][1];
	ump_x1_1 += tmp_x1_2 * EIGV[1][2];
	ump_x1_1 += tmp_x1_3 * EIGV[1][3];	      	
	ump_x1_1 *= invfreq[1];
	ump_x1_1 += x1->a0;
	    
	ump_x1_2 = tmp_x1_1 * EIGV[2][1];
	ump_x1_2 += tmp_x1_2 * EIGV[2][2];
	ump_x1_2 += tmp_x1_3 * EIGV[2][3];	      	
	ump_x1_2 *= invfreq[2];
	ump_x1_2 += x1->a0;

	ump_x1_3 = tmp_x1_1 * EIGV[3][1];
	ump_x1_3 += tmp_x1_2 * EIGV[3][2];
	ump_x1_3 += tmp_x1_3 * EIGV[3][3];	      	
	ump_x1_3 *= invfreq[3];
	ump_x1_3 += x1->a0;
	
	tmp_x2_1 = x2->c0 * *diagptable_x2++;
	tmp_x2_2 = x2->g0 * *diagptable_x2++;
	tmp_x2_3 = x2->t0 * *diagptable_x2++;
	 
	 	    	    	     	     
	ump_x2_0 = tmp_x2_1 * EIGV[0][1];
	ump_x2_0 += tmp_x2_2 * EIGV[0][2];
	ump_x2_0 += tmp_x2_3 * EIGV[0][3];		     
	ump_x2_0 *= invfreq[0];
	ump_x2_0 += x2->a0;
	  
	ump_x2_1 = tmp_x2_1 * EIGV[1][1];
	ump_x2_1 += tmp_x2_2 * EIGV[1][2];
	ump_x2_1 += tmp_x2_3 * EIGV[1][3];		     
	ump_x2_1 *= invfreq[1];
	ump_x2_1 += x2->a0;	 

	ump_x2_2 = tmp_x2_1 * EIGV[2][1];
	ump_x2_2 += tmp_x2_2 * EIGV[2][2];
	ump_x2_2 += tmp_x2_3 * EIGV[2][3];		     
	ump_x2_2 *= invfreq[2];
	ump_x2_2 += x2->a0;	  
		   

	ump_x2_3 = tmp_x2_1 * EIGV[3][1] ;
	ump_x2_3 += tmp_x2_2 * EIGV[3][2];
	ump_x2_3 += tmp_x2_3 * EIGV[3][3];		     
	ump_x2_3 *= invfreq[3];
	ump_x2_3 += x2->a0;	    	  		   	  
     
	
	x1px2 = ump_x1_0 * ump_x2_0;
	x3->a0 = x1px2 *  EIGV[0][0];
	x3->c0 = x1px2 *  EIGV[0][1];
	x3->g0 = x1px2 *  EIGV[0][2];
	x3->t0 = x1px2 *  EIGV[0][3];
	
	x1px2 = ump_x1_1 * ump_x2_1;
	x3->a0 += x1px2  *  EIGV[1][0];
	x3->c0 += x1px2 *   EIGV[1][1];
	x3->g0 += x1px2 *   EIGV[1][2];
	x3->t0 += x1px2 *   EIGV[1][3];
	
	x1px2 = ump_x1_2 * ump_x2_2;
	x3->a0 += x1px2 *  EIGV[2][0];
	x3->c0 += x1px2*   EIGV[2][1];
	x3->g0 += x1px2 *  EIGV[2][2];
	x3->t0 += x1px2 *  EIGV[2][3];
	
	x1px2 = ump_x1_3 * ump_x2_3;
	x3->a0 += x1px2 *   EIGV[3][0];
	x3->c0 += x1px2 *   EIGV[3][1];
	x3->g0 += x1px2 *   EIGV[3][2];
	x3->t0 += x1px2 *   EIGV[3][3];

	/* rate cat 1 */
	
	tmp_x1_1 = x1->c1 * *diagptable_x1++;
	tmp_x1_2 = x1->g1 * *diagptable_x1++;
	tmp_x1_3 = x1->t1 * *diagptable_x1++;
	
	ump_x1_0 = tmp_x1_1 * EIGV[0][1];
	ump_x1_0 += tmp_x1_2 * EIGV[0][2];
	ump_x1_0 += tmp_x1_3 * EIGV[0][3];	      	
	ump_x1_0 *= invfreq[0];
	ump_x1_0 += x1->a1;
	
	ump_x1_1 = tmp_x1_1 * EIGV[1][1];
	ump_x1_1 += tmp_x1_2 * EIGV[1][2];
	ump_x1_1 += tmp_x1_3 * EIGV[1][3];	      	
	ump_x1_1 *= invfreq[1];
	ump_x1_1 += x1->a1;
	    
	ump_x1_2 = tmp_x1_1 * EIGV[2][1];
	ump_x1_2 += tmp_x1_2 * EIGV[2][2];
	ump_x1_2 += tmp_x1_3 * EIGV[2][3];	      	
	ump_x1_2 *= invfreq[2];
	ump_x1_2 += x1->a1;

	ump_x1_3 = tmp_x1_1 * EIGV[3][1];
	ump_x1_3 += tmp_x1_2 * EIGV[3][2];
	ump_x1_3 += tmp_x1_3 * EIGV[3][3];	      	
	ump_x1_3 *= invfreq[3];
	ump_x1_3 += x1->a1;
	
	tmp_x2_1 = x2->c1 * *diagptable_x2++;
	tmp_x2_2 = x2->g1 * *diagptable_x2++;
	tmp_x2_3 = x2->t1 * *diagptable_x2++;
	 
	 	    	    	     	     
	ump_x2_0 = tmp_x2_1 * EIGV[0][1];
	ump_x2_0 += tmp_x2_2 * EIGV[0][2];
	ump_x2_0 += tmp_x2_3 * EIGV[0][3];		     
	ump_x2_0 *= invfreq[0];
	ump_x2_0 += x2->a1;
	  
	ump_x2_1 = tmp_x2_1 * EIGV[1][1];
	ump_x2_1 += tmp_x2_2 * EIGV[1][2];
	ump_x2_1 += tmp_x2_3 * EIGV[1][3];		     
	ump_x2_1 *= invfreq[1];
	ump_x2_1 += x2->a1;	 

	ump_x2_2 = tmp_x2_1 * EIGV[2][1];
	ump_x2_2 += tmp_x2_2 * EIGV[2][2];
	ump_x2_2 += tmp_x2_3 * EIGV[2][3];		     
	ump_x2_2 *= invfreq[2];
	ump_x2_2 += x2->a1;	  
		   

	ump_x2_3 = tmp_x2_1 * EIGV[3][1] ;
	ump_x2_3 += tmp_x2_2 * EIGV[3][2];
	ump_x2_3 += tmp_x2_3 * EIGV[3][3];		     
	ump_x2_3 *= invfreq[3];
	ump_x2_3 += x2->a1;	    	  		   	  
     
	
	x1px2 = ump_x1_0 * ump_x2_0;
	x3->a1 = x1px2 *  EIGV[0][0];
	x3->c1 = x1px2 *  EIGV[0][1];
	x3->g1 = x1px2 *  EIGV[0][2];
	x3->t1 = x1px2 *  EIGV[0][3];
	
	x1px2 = ump_x1_1 * ump_x2_1;
	x3->a1 += x1px2  *  EIGV[1][0];
	x3->c1 += x1px2 *   EIGV[1][1];
	x3->g1 += x1px2 *   EIGV[1][2];
	x3->t1 += x1px2 *   EIGV[1][3];
	
	x1px2 = ump_x1_2 * ump_x2_2;
	x3->a1 += x1px2 *  EIGV[2][0];
	x3->c1 += x1px2*   EIGV[2][1];
	x3->g1 += x1px2 *  EIGV[2][2];
	x3->t1 += x1px2 *  EIGV[2][3];
	
	x1px2 = ump_x1_3 * ump_x2_3;
	x3->a1 += x1px2 *   EIGV[3][0];
	x3->c1 += x1px2 *   EIGV[3][1];
	x3->g1 += x1px2 *   EIGV[3][2];
	x3->t1 += x1px2 *   EIGV[3][3];


	/* rate cat 2 */

	tmp_x1_1 = x1->c2 * *diagptable_x1++;
	tmp_x1_2 = x1->g2 * *diagptable_x1++;
	tmp_x1_3 = x1->t2 * *diagptable_x1++;
	
	ump_x1_0 = tmp_x1_1 * EIGV[0][1];
	ump_x1_0 += tmp_x1_2 * EIGV[0][2];
	ump_x1_0 += tmp_x1_3 * EIGV[0][3];	      	
	ump_x1_0 *= invfreq[0];
	ump_x1_0 += x1->a2;
	
	ump_x1_1 = tmp_x1_1 * EIGV[1][1];
	ump_x1_1 += tmp_x1_2 * EIGV[1][2];
	ump_x1_1 += tmp_x1_3 * EIGV[1][3];	      	
	ump_x1_1 *= invfreq[1];
	ump_x1_1 += x1->a2;
	    
	ump_x1_2 = tmp_x1_1 * EIGV[2][1];
	ump_x1_2 += tmp_x1_2 * EIGV[2][2];
	ump_x1_2 += tmp_x1_3 * EIGV[2][3];	      	
	ump_x1_2 *= invfreq[2];
	ump_x1_2 += x1->a2;

	ump_x1_3 = tmp_x1_1 * EIGV[3][1];
	ump_x1_3 += tmp_x1_2 * EIGV[3][2];
	ump_x1_3 += tmp_x1_3 * EIGV[3][3];	      	
	ump_x1_3 *= invfreq[3];
	ump_x1_3 += x1->a2;
	
	tmp_x2_1 = x2->c2 * *diagptable_x2++;
	tmp_x2_2 = x2->g2 * *diagptable_x2++;
	tmp_x2_3 = x2->t2 * *diagptable_x2++;
	 
	 	    	    	     	     
	ump_x2_0 = tmp_x2_1 * EIGV[0][1];
	ump_x2_0 += tmp_x2_2 * EIGV[0][2];
	ump_x2_0 += tmp_x2_3 * EIGV[0][3];		     
	ump_x2_0 *= invfreq[0];
	ump_x2_0 += x2->a2;
	  
	ump_x2_1 = tmp_x2_1 * EIGV[1][1];
	ump_x2_1 += tmp_x2_2 * EIGV[1][2];
	ump_x2_1 += tmp_x2_3 * EIGV[1][3];		     
	ump_x2_1 *= invfreq[1];
	ump_x2_1 += x2->a2;	 

	ump_x2_2 = tmp_x2_1 * EIGV[2][1];
	ump_x2_2 += tmp_x2_2 * EIGV[2][2];
	ump_x2_2 += tmp_x2_3 * EIGV[2][3];		     
	ump_x2_2 *= invfreq[2];
	ump_x2_2 += x2->a2;	  
		   

	ump_x2_3 = tmp_x2_1 * EIGV[3][1] ;
	ump_x2_3 += tmp_x2_2 * EIGV[3][2];
	ump_x2_3 += tmp_x2_3 * EIGV[3][3];		     
	ump_x2_3 *= invfreq[3];
	ump_x2_3 += x2->a2;	    	  		   	  
     
	
	x1px2 = ump_x1_0 * ump_x2_0;
	x3->a2 = x1px2 *  EIGV[0][0];
	x3->c2 = x1px2 *  EIGV[0][1];
	x3->g2 = x1px2 *  EIGV[0][2];
	x3->t2 = x1px2 *  EIGV[0][3];
	
	x1px2 = ump_x1_1 * ump_x2_1;
	x3->a2 += x1px2  *  EIGV[1][0];
	x3->c2 += x1px2 *   EIGV[1][1];
	x3->g2 += x1px2 *   EIGV[1][2];
	x3->t2 += x1px2 *   EIGV[1][3];
	
	x1px2 = ump_x1_2 * ump_x2_2;
	x3->a2 += x1px2 *  EIGV[2][0];
	x3->c2 += x1px2*   EIGV[2][1];
	x3->g2 += x1px2 *  EIGV[2][2];
	x3->t2 += x1px2 *  EIGV[2][3];
	
	x1px2 = ump_x1_3 * ump_x2_3;
	x3->a2 += x1px2 *   EIGV[3][0];
	x3->c2 += x1px2 *   EIGV[3][1];
	x3->g2 += x1px2 *   EIGV[3][2];
	x3->t2 += x1px2 *   EIGV[3][3];


	/* rate cat 3 */

	tmp_x1_1 = x1->c3 * *diagptable_x1++;
	tmp_x1_2 = x1->g3 * *diagptable_x1++;
	tmp_x1_3 = x1->t3 * *diagptable_x1++;
	
	ump_x1_0 = tmp_x1_1 * EIGV[0][1];
	ump_x1_0 += tmp_x1_2 * EIGV[0][2];
	ump_x1_0 += tmp_x1_3 * EIGV[0][3];	      	
	ump_x1_0 *= invfreq[0];
	ump_x1_0 += x1->a3;
	
	ump_x1_1 = tmp_x1_1 * EIGV[1][1];
	ump_x1_1 += tmp_x1_2 * EIGV[1][2];
	ump_x1_1 += tmp_x1_3 * EIGV[1][3];	      	
	ump_x1_1 *= invfreq[1];
	ump_x1_1 += x1->a3;
	    
	ump_x1_2 = tmp_x1_1 * EIGV[2][1];
	ump_x1_2 += tmp_x1_2 * EIGV[2][2];
	ump_x1_2 += tmp_x1_3 * EIGV[2][3];	      	
	ump_x1_2 *= invfreq[2];
	ump_x1_2 += x1->a3;

	ump_x1_3 = tmp_x1_1 * EIGV[3][1];
	ump_x1_3 += tmp_x1_2 * EIGV[3][2];
	ump_x1_3 += tmp_x1_3 * EIGV[3][3];	      	
	ump_x1_3 *= invfreq[3];
	ump_x1_3 += x1->a3;
	
	tmp_x2_1 = x2->c3 * *diagptable_x2++;
	tmp_x2_2 = x2->g3 * *diagptable_x2++;
	tmp_x2_3 = x2->t3 * *diagptable_x2++;
	 
	 	    	    	     	     
	ump_x2_0 = tmp_x2_1 * EIGV[0][1];
	ump_x2_0 += tmp_x2_2 * EIGV[0][2];
	ump_x2_0 += tmp_x2_3 * EIGV[0][3];		     
	ump_x2_0 *= invfreq[0];
	ump_x2_0 += x2->a3;
	  
	ump_x2_1 = tmp_x2_1 * EIGV[1][1];
	ump_x2_1 += tmp_x2_2 * EIGV[1][2];
	ump_x2_1 += tmp_x2_3 * EIGV[1][3];		     
	ump_x2_1 *= invfreq[1];
	ump_x2_1 += x2->a3;	 

	ump_x2_2 = tmp_x2_1 * EIGV[2][1];
	ump_x2_2 += tmp_x2_2 * EIGV[2][2];
	ump_x2_2 += tmp_x2_3 * EIGV[2][3];		     
	ump_x2_2 *= invfreq[2];
	ump_x2_2 += x2->a3;	  
		   

	ump_x2_3 = tmp_x2_1 * EIGV[3][1] ;
	ump_x2_3 += tmp_x2_2 * EIGV[3][2];
	ump_x2_3 += tmp_x2_3 * EIGV[3][3];		     
	ump_x2_3 *= invfreq[3];
	ump_x2_3 += x2->a3;	    	  		   	  
     
	
	x1px2 = ump_x1_0 * ump_x2_0;
	x3->a3 = x1px2 *  EIGV[0][0];
	x3->c3 = x1px2 *  EIGV[0][1];
	x3->g3 = x1px2 *  EIGV[0][2];
	x3->t3 = x1px2 *  EIGV[0][3];
	
	x1px2 = ump_x1_1 * ump_x2_1;
	x3->a3 += x1px2  *  EIGV[1][0];
	x3->c3 += x1px2 *   EIGV[1][1];
	x3->g3 += x1px2 *   EIGV[1][2];
	x3->t3 += x1px2 *   EIGV[1][3];
	
	x1px2 = ump_x1_2 * ump_x2_2;
	x3->a3 += x1px2 *  EIGV[2][0];
	x3->c3 += x1px2*   EIGV[2][1];
	x3->g3 += x1px2 *  EIGV[2][2];
	x3->t3 += x1px2 *  EIGV[2][3];
	
	x1px2 = ump_x1_3 * ump_x2_3;
	x3->a3 += x1px2 *   EIGV[3][0];
	x3->c3 += x1px2 *   EIGV[3][1];
	x3->g3 += x1px2 *   EIGV[3][2];
	x3->t3 += x1px2 *   EIGV[3][3];


	/* end */
		   
	x3->exp = x1->exp + x2->exp;		  
	
	if (ABS(x3->a0) < minlikelihood && ABS(x3->g0) < minlikelihood && ABS(x3->c0) < minlikelihood && ABS(x3->t0) < minlikelihood &&
	    ABS(x3->a1) < minlikelihood && ABS(x3->g1) < minlikelihood && ABS(x3->c1) < minlikelihood && ABS(x3->t1) < minlikelihood &&
	    ABS(x3->a2) < minlikelihood && ABS(x3->g2) < minlikelihood && ABS(x3->c2) < minlikelihood && ABS(x3->t2) < minlikelihood &&
	    ABS(x3->a3) < minlikelihood && ABS(x3->g3) < minlikelihood && ABS(x3->c3) < minlikelihood && ABS(x3->t3) < minlikelihood) 
	  {	     
	    x3->a0   *= twotothe256;
	    x3->g0   *= twotothe256;
	    x3->c0   *= twotothe256;
	    x3->t0   *= twotothe256;

	    x3->a1   *= twotothe256;
	    x3->g1   *= twotothe256;
	    x3->c1   *= twotothe256;
	    x3->t1   *= twotothe256;

	    x3->a2   *= twotothe256;
	    x3->g2   *= twotothe256;
	    x3->c2   *= twotothe256;
	    x3->t2   *= twotothe256;

	    x3->a3   *= twotothe256;
	    x3->g3   *= twotothe256;
	    x3->c3   *= twotothe256;
	    x3->t3   *= twotothe256;
	   
	    x3->exp += 1;
	  }
	 	
	x1++;
	x2++;
	x3++;
      
      }
         
    free(diagptable_x1_start); 
    free(diagptable_x2_start);
    return TRUE;
  }
}





double evaluateGTRGAMMA (tr, p)
    tree    *tr;
    nodeptr  p;
  { /* evaluate */
    double   sum, z, lz, term, ki, term2;    
    nodeptr  q;
    int     i, j, cat;
    int     *wptr = tr->cdta->aliaswgt, *cptr;
    
    double  *diagptable, *rptr, *diagptable_start;
    double *EIGN = &(tr->rdta->EIGN[0][0]);
    gammalikelivector   *x1, *x2;
       
    q = p->back;
    while ((! p->x) || (! q->x)) {
      if (! (p->x)) if (! newviewGTRGAMMA(tr, p)) return badEval;
      if (! (q->x)) if (! newviewGTRGAMMA(tr, q)) return badEval;
    }


    rptr  = &(tr->cdta->patrat[0]); 
    cptr  = &(tr->cdta->rateCategory[0]);

    x1  = p->x->gv;
    x2  = q->x->gv;
    z = p->z;
   
    if (z < zmin) z = zmin;
    lz = log(z);

   
    
    diagptable = diagptable_start = (double *)malloc(sizeof(double) * 4 * 3);
    for(i = 0; i < 4; i++)
      {
	ki = tr->gammaRates[i];	 

	*diagptable++ = exp (EIGN[1] * ki * lz);
	*diagptable++ = exp (EIGN[2] * ki * lz);
	*diagptable++ = exp (EIGN[3] * ki * lz);
      }
	
    
    sum = 0.0;
  
    for (i = 0; i < tr->cdta->endsite; i++) 
      {	  	 	  
	diagptable = diagptable_start;

	/* cat 0 */

	term =  x1->a0 * x2->a0;
	term += x1->c0 * x2->c0 * *diagptable++;
	term += x1->g0 * x2->g0 * *diagptable++;
	term += x1->t0 * x2->t0 * *diagptable++;     

	/* cat 1 */

	term += x1->a1 * x2->a1;
	term += x1->c1 * x2->c1 * *diagptable++;
	term += x1->g1 * x2->g1 * *diagptable++;
	term += x1->t1 * x2->t1 * *diagptable++;     

       

	/* cat 2 */

	term += x1->a2 * x2->a2;
	term += x1->c2 * x2->c2 * *diagptable++;
	term += x1->g2 * x2->g2 * *diagptable++;
	term += x1->t2 * x2->t2 * *diagptable++;     

	

	/* cat 3 */
	
	term +=  x1->a3 * x2->a3;
	term += x1->c3 * x2->c3 * *diagptable++;
	term += x1->g3 * x2->g3 * *diagptable++;
	term += x1->t3 * x2->t3 * *diagptable++;     

	term = log(0.25 * term) + (x1->exp + x2->exp)*log(minlikelihood);
	sum += *wptr++ * term;		  
	x1++;
	x2++;
      }
    
      
    free(diagptable_start); 
    
    tr->likelihood = sum;
    return  sum;
  } /* evaluate */







double makenewzGTRGAMMA (tr, p, q, z0, maxiter)
    tree    *tr;
    nodeptr  p, q;
    double   z0;
    int  maxiter;
  { /* makenewz */
    double   z, zprev, zstep;
    gammalikelivector  *x1, *x2;
    double  *sumtable, *sum, *diagptable, *diagptable_start;
    int     i;
    double  dlnLdlz = 0.0;                 /*  = d(ln(likelihood))/d(lz) */
    double  d2lnLdlz2 = 0.0;               /*  = d2(ln(likelihood))/d(lz)2 */    
    double sumc, ki;
    
    while ((! p->x) || (! q->x)) 
      {
	if (! (p->x)) if (! newviewGTRGAMMA(tr, p)) return badZ;
	if (! (q->x)) if (! newviewGTRGAMMA(tr, q)) return badZ;
      }

    x1 = p->x->gv;
    x2 = q->x->gv;

    sum = sumtable = (double *)malloc(13 * tr->cdta->endsite * sizeof(double));
    diagptable = diagptable_start = (double *)malloc(sizeof(double) * 4 * 3);
  
	
    for (i = 0; i < tr->cdta->endsite; i++) 
      {     
	sumc = x1->a0 * x2->a0;
	*sum++ = x1->c0 * x2->c0;
	*sum++ = x1->g0 * x2->g0;
	*sum++ = x1->t0 * x2->t0;		    

	sumc += x1->a1 * x2->a1;
	*sum++ = x1->c1 * x2->c1;
	*sum++ = x1->g1 * x2->g1;
	*sum++ = x1->t1 * x2->t1;

	sumc += x1->a2 * x2->a2;
	*sum++ = x1->c2 * x2->c2;
	*sum++ = x1->g2 * x2->g2;
	*sum++ = x1->t2 * x2->t2;

	sumc += x1->a3 * x2->a3;
	*sum++ = x1->c3 * x2->c3;
	*sum++ = x1->g3 * x2->g3;
	*sum++ = x1->t3 * x2->t3;

	*sum++ = sumc;

	x1++;
	x2++;		 
      }

   
   
    z = z0;
    do {
      int curvatOK = FALSE;

      zprev = z;
      
      zstep = (1.0 - zmax) * z + zmin;

      do {	
	double *EIGN    = &(tr->rdta->EIGN[0][0]);
	double lz;
        double *wrptr   = &(tr->cdta->wr[0]);
        double *wr2ptr  = &(tr->cdta->wr2[0]);	

        dlnLdlz = 0.0;                 /*  = d(ln(likelihood))/d(lz) */
        d2lnLdlz2 = 0.0;               /*  = d2(ln(likelihood))/d(lz)2 */


        if (z < zmin) z = zmin;
        else if (z > zmax) z = zmax;
        lz    = log(z);
        
	sum = sumtable;

	diagptable = diagptable_start;
	
	  for(i = 0; i < 4; i++)
	    {
	      ki = tr->gammaRates[i];	 
	      
	      *diagptable++ = exp (EIGN[1] * ki * lz);
	      *diagptable++ = exp (EIGN[2] * ki * lz);
	      *diagptable++ = exp (EIGN[3] * ki * lz);
	    }

	


        for (i = 0; i < tr->cdta->endsite; i++) 
	  {
	    double tmp_1, tmp_2, tmp_3;
	    double t, inv_Li, dlnLidlz, d2lnLidlz2;
	    diagptable = diagptable_start;
	    	   
	    inv_Li =  (tmp_1 = *diagptable++ * *sum++);
	    inv_Li += (tmp_2 = *diagptable++ * *sum++);
	    inv_Li += (tmp_3 = *diagptable++ * *sum++);	    	   	    	  	   	   
	    
	    t = tmp_1 * EIGN[1];	   
	    dlnLidlz   = t;
	    d2lnLidlz2 = t * EIGN[1];
	    
	    t = tmp_2 * EIGN[2];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[2];
	    
	    t = tmp_3 * EIGN[3];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[3];
	    
	    dlnLidlz *= tr->gammaRates[0];
	    d2lnLidlz2 *= tr->gammaRates[0] * tr->gammaRates[0];

	    /*t = tmp_1 * EIGN[1] * tr->gammaRates[0];	   

	    dlnLidlz   = t;
	    d2lnLidlz2 = t * EIGN[1] * tr->gammaRates[0];
	    
	    t = tmp_2 * EIGN[2] * tr->gammaRates[0];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[2] * tr->gammaRates[0];
	    
	    t = tmp_3 * EIGN[3] * tr->gammaRates[0];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[3] * tr->gammaRates[0];*/
	   
	    /* rate 1 */
	    
	  
	    inv_Li += (tmp_1 = *sum++ *  *diagptable++);
	    inv_Li += (tmp_2 = *sum++ *  *diagptable++);
	    inv_Li += (tmp_3 = *sum++ *  *diagptable++);	    	   	   	    
 	  
	    t = tmp_1 * EIGN[1] * tr->gammaRates[1];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[1] * tr->gammaRates[1];
	    
	    t = tmp_2 * EIGN[2] * tr->gammaRates[1];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[2] * tr->gammaRates[1];
	    
	    t = tmp_3 * EIGN[3] * tr->gammaRates[1];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[3] * tr->gammaRates[1];
 

	    /* rates 2 */
	 
	    inv_Li += (tmp_1 = *sum++ *  *diagptable++);
	    inv_Li += (tmp_2 = *sum++ *  *diagptable++);
	    inv_Li += (tmp_3 = *sum++ *  *diagptable++);	   	  
 	  	   	    	  
	    t = tmp_1 * EIGN[1] * tr->gammaRates[2];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[1] * tr->gammaRates[2];
	    
	    t = tmp_2 * EIGN[2] * tr->gammaRates[2];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[2] * tr->gammaRates[2];
	    
	    t = tmp_3 * EIGN[3] * tr->gammaRates[2];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[3] * tr->gammaRates[2];
 
	    /* rates 3 */
	  	 
	    inv_Li += (tmp_1 = *sum++ * *diagptable++);
	    inv_Li += (tmp_2 = *sum++ * *diagptable++);
	    inv_Li += (tmp_3 = *sum++ * *diagptable++);	    
	    inv_Li += *sum++;
	   	  	   	    	  
	    t = tmp_1 * EIGN[1] * tr->gammaRates[3];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[1] * tr->gammaRates[3];
	    
	    t = tmp_2 * EIGN[2] * tr->gammaRates[3];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[2] * tr->gammaRates[3];
	    
	    t = tmp_3 * EIGN[3] * tr->gammaRates[3];
	    dlnLidlz   += t;
	    d2lnLidlz2 += t * EIGN[3] * tr->gammaRates[3];
 
	    /* end */

	    inv_Li = 1.0/inv_Li;

	    dlnLidlz   *= inv_Li;
	    d2lnLidlz2 *= inv_Li;

	    dlnLdlz   += *wrptr++  * dlnLidlz;
	    d2lnLdlz2 += *wr2ptr++ * (d2lnLidlz2 - dlnLidlz * dlnLidlz);
	  }
	
	

        if ((d2lnLdlz2 >= 0.0) && (z < zmax))
          zprev = z = 0.37 * z + 0.63;  /*  Bad curvature, shorten branch */
        else
          curvatOK = TRUE;
	
      } while (! curvatOK);

      if (d2lnLdlz2 < 0.0) {
	double tantmp = -dlnLdlz / d2lnLdlz2;  /* prevent overflow */
	if (tantmp < 100) {
	  z *= exp(tantmp);
	  if (z < zmin) z = zmin;
	  if (z > 0.25 * zprev + 0.75)    /*  Limit steps toward z = 1.0 */
	    z = 0.25 * zprev + 0.75;
	} else {
	  z = 0.25 * zprev + 0.75;
	}
      }
      if (z > zmax) z = zmax;

    } while ((--maxiter > 0) && (ABS(z - zprev) > zstep));

    free(sumtable);
    free(diagptable_start);
  
    return  z;
  } /* makenewz */


#endif



#endif















boolean newviewPartialHKY85CAT(tree *tr, nodeptr p, int i, double ki) 
{ 
  double   zq, lzq, xvlzq, zr, lzr, xvlzr;
  nodeptr  q, r;
  likelivector *lp, *lq, *lr;
  
  if (p->tip) {             /*  Make sure that data are at tip */
    likelivector *l;
    int           code;
    yType        *yptr;
    
    if (p->x) return TRUE;  /*  They are already there */
    
    if (! getxtip(p)) return FALSE; /*  They are not, so get memory */
    
    l = &p->x->lv[i];           /*  Pointer to first likelihood vector value */
      yptr = p->tip;          /*  Pointer to first nucleotide datum */
      code = yptr[i];
      l->a =  code       & 1;
      l->c = (code >> 1) & 1;
      l->g = (code >> 2) & 1;
      l->t = (code >> 3) & 1;
      l->exp = 0;
      return TRUE;
  }
  
  /*  Internal node needs update */
  
  q = p->next->back;
  r = p->next->next->back;
  
  
  
  while ((! p->x) || (! q->x) || (! r->x)) {
    if (! q->x) if (! newviewPartialHKY85CAT(tr, q, i, ki))  return FALSE;
    if (! r->x) if (! newviewPartialHKY85CAT(tr, r, i, ki))  return FALSE;
    /*if (! p->x) if (! getxnode(p)) return FALSE;*/
  }
  
  lp = &p->x->lv[i];
  
  lq = &q->x->lv[i];
  zq = q->z;
  lzq = (zq > zmin) ? log(zq) : log(zmin);
  xvlzq = tr->rdta->xv * lzq;
    
  lr = &r->x->lv[i];
  zr = r->z;
  lzr = (zr > zmin) ? log(zr) : log(zmin);
  xvlzr = tr->rdta->xv * lzr;
  
  { 
    double  *rptr;
    double  fxqr, fxqy, fxqn, sumaq, sumgq, sumcq, sumtq,
      fxrr, fxry, fxrn, tempi, tempj;
    double zzq, zvq, zzr, zvr;

    zzq = exp(ki *   lzq);
    zvq = exp(ki * xvlzq);
    zzr = exp(ki *   lzr);
    zvr = exp(ki * xvlzr);
        
    fxqr = tr->rdta->freqa * lq->a + tr->rdta->freqg * lq->g;
    fxqy = tr->rdta->freqc * lq->c + tr->rdta->freqt * lq->t;
    fxqn = fxqr + fxqy;
    tempi = fxqr * tr->rdta->invfreqr;
    tempj = zvq * (tempi-fxqn) + fxqn;
    sumaq = zzq * (lq->a - tempi) + tempj;
    sumgq = zzq * (lq->g - tempi) + tempj;
    tempi = fxqy * tr->rdta->invfreqy;
    tempj = zvq * (tempi-fxqn) + fxqn;
    sumcq = zzq * (lq->c - tempi) + tempj;
    sumtq = zzq * (lq->t - tempi) + tempj;

          
    fxrr = tr->rdta->freqa * lr->a + tr->rdta->freqg * lr->g;
    fxry = tr->rdta->freqc * lr->c + tr->rdta->freqt * lr->t;
    fxrn = fxrr + fxry;
    tempi = fxrr * tr->rdta->invfreqr;
    tempj = zvr * (tempi-fxrn) + fxrn;
    lp->a = sumaq * (zzr * (lr->a - tempi) + tempj);
    lp->g = sumgq * (zzr * (lr->g - tempi) + tempj);
    tempi = fxry * tr->rdta->invfreqy;
    tempj = zvr * (tempi-fxrn) + fxrn;
    lp->c = sumcq * (zzr * (lr->c - tempi) + tempj);
    lp->t = sumtq * (zzr * (lr->t - tempi) + tempj);
    lp->exp = lq->exp + lr->exp;
	
	
	
    if (lp->a < minlikelihood && lp->g < minlikelihood
	&& lp->c < minlikelihood && lp->t < minlikelihood) {
      lp->a   *= twotothe256;
      lp->g   *= twotothe256;
      lp->c   *= twotothe256;
      lp->t   *= twotothe256;
      lp->exp += 1;
    }
    
    return TRUE;
      }
  } /* newview */



double evaluatePartialHKY85CAT (tree *tr, nodeptr p, int i, double ki)
  { /* evaluate */
    double   sum, z, lz, xvlz,
             fxpa, fxpc, fxpg, fxpt, fxpr, fxpy, fxqr, fxqy,
             suma, sumb, sumc, term;

    double partial;
    double   zz, zv;

    double        *rptr;
    likelivector *lp, *lq;
    nodeptr       q;
    int           *wptr;

    q = p->back;
    while ((! p->x) || (! q->x)) {
      if (! (p->x)) if (! newviewPartialHKY85CAT(tr, p, i, ki)) return badEval;
       if (! (q->x)) if (! newviewPartialHKY85CAT(tr, q, i, ki)) return badEval;
      }

    lp = &p->x->lv[i];
    lq = &q->x->lv[i];

    z = p->z;
    if (z < zmin) z = zmin;
    lz = log(z);
    xvlz = tr->rdta->xv * lz;

    rptr  = &(tr->cdta->patrat[0]);    
    wptr = &(tr->cdta->aliaswgt[0]);

    zz   = exp(ki *   lz);
    zv   = exp(ki * xvlz);

    fxpa = tr->rdta->freqa * lp->a;
    fxpg = tr->rdta->freqg * lp->g;
    fxpc = tr->rdta->freqc * lp->c;
    fxpt = tr->rdta->freqt * lp->t;
    suma = fxpa * lq->a + fxpc * lq->c + fxpg * lq->g + fxpt * lq->t;
    fxqr = tr->rdta->freqa * lq->a + tr->rdta->freqg * lq->g;
    fxqy = tr->rdta->freqc * lq->c + tr->rdta->freqt * lq->t;
    fxpr = fxpa + fxpg;
    fxpy = fxpc + fxpt;
    sumc = (fxpr + fxpy) * (fxqr + fxqy);
    sumb = fxpr * fxqr * tr->rdta->invfreqr + fxpy * fxqy * tr->rdta->invfreqy;
    suma -= sumb;
    sumb -= sumc;
    term = log(zz * suma + zv * sumb + sumc) + (lp->exp + lq->exp)*log(minlikelihood);

    partial = wptr[i] * term;
    return partial;

  } /* evaluate */



boolean newviewPartialGTRCAT(tree *tr, nodeptr p, int i, double ki) 
{
 if (p->tip) 
    {      
      if (p->x) return TRUE;

      if (! getxtip(p)) return FALSE; 
      {
	int code, j, jj;
	double (*EIGV)[4] = &(tr->rdta->EIGV[0][0]);
	likelivector *x1 =&(p->x->lv[i]);
     
	code = p->tip[i];
	x1->a = 0;
	x1->c = 0;
	x1->g = 0;
	x1->t = 0;
	for (j = 0; j < 4; j++) 
	  {
	    if ((code >> j) & 1) 
	      {
		int jj = "0213"[j] - '0';		
		x1->a += EIGV[jj][0];
		x1->c += EIGV[jj][1];
		x1->g += EIGV[jj][2];
		x1->t += EIGV[jj][3];
	      }
	  }		
	      
	x1->exp = 0;	  			  			 
		 
	return TRUE; 
	}
      return TRUE; 
    }
  
  { 	
    double  *diagptable_x1, *diagptable_x2;
    double *EIGN    = &(tr->rdta->EIGN[0][0]);
    double *invfreq = &(tr->rdta->invfreq[0][0]);
    double (*EIGV)[4] = &(tr->rdta->EIGV[0][0]);
    xtype tmp_x1_1, tmp_x1_2, tmp_x1_3,
      tmp_x2_1, tmp_x2_2, tmp_x2_3, 
      ump_x1_1, ump_x1_2, ump_x1_3, ump_x1_0, 
      ump_x2_0, ump_x2_1, ump_x2_2, ump_x2_3, x1px2;
    int cat;
    int  *cptr;	  
    int  j;
    likelivector   *x1, *x2, *x3;
    double   z1, lz1, z2, lz2;
    nodeptr  q, r;   
    double *eigv, *rptr;

    q = p->next->back;
    r = p->next->next->back;
      
    while ((! p->x) || (! q->x) || (! r->x)) {
      if (! q->x) if (! newviewPartialGTRCAT(tr, q, i, ki)) return FALSE;
      if (! r->x) if (! newviewPartialGTRCAT(tr, r, i, ki)) return FALSE;
      if (! p->x) if (! getxnode(p)) return FALSE;
    }

    x1  = &(q->x->lv[i]);
    z1  = q->z;
    lz1 = (z1 > zmin) ? log(z1) : log(zmin);

    x2  = &(r->x->lv[i]);
    z2  = r->z;
    lz2 = (z2 > zmin) ? log(z2) : log(zmin);

    x3  = &(p->x->lv[i]);   
    	
    diagptable_x1 = (double *)malloc(sizeof(double) * 4);
    diagptable_x2 = (double *)malloc(sizeof(double) * 4);

    for (j = 1; j < 4; j++) 
      {	    
	diagptable_x1[j] = exp (EIGN[j] * lz1 * ki);
	diagptable_x2[j] = exp (EIGN[j] * lz2 * ki);	    
      }	       
      
    
    

    tmp_x1_1 = x1->c * diagptable_x1[1];
    tmp_x1_2 = x1->g * diagptable_x1[2];
    tmp_x1_3 = x1->t * diagptable_x1[3];
	
    ump_x1_0 = tmp_x1_1 * EIGV[0][1];
    ump_x1_0 += tmp_x1_2 * EIGV[0][2];
    ump_x1_0 += tmp_x1_3 * EIGV[0][3];	      	
    ump_x1_0 *= invfreq[0];
    ump_x1_0 += x1->a;
	
    ump_x1_1 = tmp_x1_1 * EIGV[1][1];
    ump_x1_1 += tmp_x1_2 * EIGV[1][2];
    ump_x1_1 += tmp_x1_3 * EIGV[1][3];	      	
    ump_x1_1 *= invfreq[1];
    ump_x1_1 += x1->a;
	    
    ump_x1_2 = tmp_x1_1 * EIGV[2][1];
    ump_x1_2 += tmp_x1_2 * EIGV[2][2];
    ump_x1_2 += tmp_x1_3 * EIGV[2][3];	      	
    ump_x1_2 *= invfreq[2];
    ump_x1_2 += x1->a;
    
    ump_x1_3 = tmp_x1_1 * EIGV[3][1];
    ump_x1_3 += tmp_x1_2 * EIGV[3][2];
    ump_x1_3 += tmp_x1_3 * EIGV[3][3];	      	
    ump_x1_3 *= invfreq[3];
    ump_x1_3 += x1->a;
	
    tmp_x2_1 = x2->c * diagptable_x2[1];
    tmp_x2_2 = x2->g * diagptable_x2[2];
    tmp_x2_3 = x2->t * diagptable_x2[3];
    
	 	    	    	     	     
    ump_x2_0 = tmp_x2_1 * EIGV[0][1];
    ump_x2_0 += tmp_x2_2 * EIGV[0][2];
    ump_x2_0 += tmp_x2_3 * EIGV[0][3];		     
    ump_x2_0 *= invfreq[0];
    ump_x2_0 += x2->a;
    
    ump_x2_1 = tmp_x2_1 * EIGV[1][1];
    ump_x2_1 += tmp_x2_2 * EIGV[1][2];
    ump_x2_1 += tmp_x2_3 * EIGV[1][3];		     
    ump_x2_1 *= invfreq[1];
    ump_x2_1 += x2->a;	 

    ump_x2_2 = tmp_x2_1 * EIGV[2][1];
    ump_x2_2 += tmp_x2_2 * EIGV[2][2];
    ump_x2_2 += tmp_x2_3 * EIGV[2][3];		     
    ump_x2_2 *= invfreq[2];
    ump_x2_2 += x2->a;	  
		   

    ump_x2_3 = tmp_x2_1 * EIGV[3][1] ;
    ump_x2_3 += tmp_x2_2 * EIGV[3][2];
    ump_x2_3 += tmp_x2_3 * EIGV[3][3];		     
    ump_x2_3 *= invfreq[3];
    ump_x2_3 += x2->a;	    	  		   	  
    eigv = &EIGV[0][0];
	
    x1px2 = ump_x1_0 * ump_x2_0;
    x3->a = x1px2 *  *eigv++;/*EIGV[0][0];*/
    x3->c = x1px2 *  *eigv++;/*EIGV[0][1];*/
    x3->g = x1px2 *  *eigv++;/*EIGV[0][2];*/
    x3->t = x1px2 *  *eigv++;/*EIGV[0][3]; */
    
    x1px2 = ump_x1_1 * ump_x2_1;
    x3->a += x1px2  *  *eigv++;/*EIGV[1][0];*/
    x3->c += x1px2 *  *eigv++;/*EIGV[1][1];*/
    x3->g += x1px2 *  *eigv++;/*EIGV[1][2];*/
    x3->t += x1px2 *  *eigv++;/*EIGV[1][3];*/
	
    x1px2 = ump_x1_2 * ump_x2_2;
    x3->a += x1px2 *  *eigv++;/*EIGV[2][0];*/
    x3->c += x1px2*  *eigv++;/*EIGV[2][1];*/
    x3->g += x1px2 *  *eigv++;/*EIGV[2][2];*/
    x3->t += x1px2 *  *eigv++;/*EIGV[2][3];*/
	
    x1px2 = ump_x1_3 * ump_x2_3;
    x3->a += x1px2 *   *eigv++;/*EIGV[3][0];*/
    x3->c += x1px2 *   *eigv++;/*EIGV[3][1];*/
    x3->g += x1px2 *   *eigv++;/*EIGV[3][2];*/
    x3->t += x1px2 *   *eigv++;/*EIGV[3][3];*/
		   
    x3->exp = x1->exp + x2->exp;		  
    
    if (ABS(x3->a) < minlikelihood && ABS(x3->g) < minlikelihood
	&& ABS(x3->c) < minlikelihood && ABS(x3->t) < minlikelihood) 
      {	     
	x3->a   *= twotothe256;
	x3->g   *= twotothe256;
	x3->c   *= twotothe256;
	x3->t   *= twotothe256;
	x3->exp += 1;
      }
	           
    free(diagptable_x1); 
    free(diagptable_x2);

    return TRUE;

  } 
}


double evaluatePartialGTRCAT (tree *tr, nodeptr p, int i, double ki)
{
  double   sum, z, lz, term, k;    
    nodeptr  q;
    int     j, cat;
    int     *wptr = tr->cdta->aliaswgt, *cptr;
    
    double  diagptable[4], *rptr;
    double *EIGN = &(tr->rdta->EIGN[0][0]);
    likelivector   *x1, *x2;
       
    q = p->back;
    while ((! p->x) || (! q->x)) 
      {
	if (! (p->x)) if (! newviewPartialGTRCAT(tr, p, i, ki)) return badEval;
	if (! (q->x)) if (! newviewPartialGTRCAT(tr, q, i, ki)) return badEval;
      }

    rptr   = &(tr->cdta->patrat[0]); 
    cptr  = &(tr->cdta->rateCategory[0]);

    x1  = &(p->x->lv[i]);
    x2  = &(q->x->lv[i]);
    z = p->z;
   
    if (z < zmin) z = zmin;
    lz = log(z);

    diagptable[0] = 1;
    diagptable[1] = exp (EIGN[1] * lz * ki);
    diagptable[2] = exp (EIGN[2] * lz * ki);
    diagptable[3] = exp (EIGN[3] * lz * ki);
    
   
	
    
    sum = 0.0;
  
    term =  x1->a * x2->a * diagptable[0];
    term += x1->c * x2->c * diagptable[1];
    term += x1->g * x2->g * diagptable[2];
    term += x1->t * x2->t * diagptable[3];     
    term = log(term) + (x1->exp + x2->exp)*log(minlikelihood);
    sum = wptr[i] * term;		  
       
    tr->likelihood = sum;
    return  sum;
}



boolean update (tree *tr, nodeptr p)
  { /* update */
    
	nodeptr  q;
	double   z0, z;
	


	q = p->back;
	z0 = q->z;
	

	if ((z = makenewz(tr, p, q, z0, newzpercycle)) == badZ) 
	  {
	    printf("BADZ\n");
	    return FALSE;
	  }

	p->z = q->z = z;
	if (ABS(z - z0) > deltaz)  tr->smoothed = FALSE;
	

    return TRUE;
  } /* update */



boolean smooth (tree *tr, nodeptr p)
  { /* smooth */
    nodeptr  q;
  
    if (! update(tr, p))               return FALSE; /*  Adjust branch */
    if (! p->tip) 
      {                                  /*  Adjust descendants */
        q = p->next;
        while (q != p) 
	  {
	    if (! smooth(tr, q->back))   return FALSE;
	    q = q->next;
          }	
	if (! newview(tr, p)) return FALSE;	
      }

    return TRUE;
  } /* smooth */


boolean smoothTree (tree *tr, int maxtimes)
  { /* smoothTree */
    nodeptr  p, q;

    p = tr->start;

    while (--maxtimes >= 0) 
      {
	tr->smoothed = TRUE;
	if (! smooth(tr, p->back))       return FALSE;
	if (! p->tip) 
	  {
	    q = p->next;
	    while (q != p) 
	      {
		if (! smooth(tr, q->back))   return FALSE;
		q = q->next;
	      }
	  }
	if (tr->smoothed)  break;
      }

    return TRUE;
  } /* smoothTree */


boolean localSmooth (tree *tr, nodeptr p, int maxtimes)
  { /* localSmooth -- Smooth branches around p */
    nodeptr  q;

    if (p->tip) return FALSE;            /* Should be an error */

    while (--maxtimes >= 0) {
      tr->smoothed = TRUE;
      q = p;
      do {
        if (! update(tr, q)) return FALSE;
        q = q->next;
        } while (q != p);
      if (tr->smoothed)  break;
      }

    tr->smoothed = FALSE;             /* Only smooth locally */
    return TRUE;
  } /* localSmooth */


void hookup (nodeptr p, nodeptr q, double z)
  { /* hookup */
    p->back = q;
    q->back = p;
    p->z = q->z = z;
  } /* hookup */


/* Insert node p into branch q <-> q->back */


boolean insert (tree *tr, nodeptr p, nodeptr q, boolean glob)
/* glob -- Smooth tree globally? */

/*                q
                 /.
             add/ .
               /  .
             pn   .
    s ---- p      .remove
             pnn  .
               \  .
             add\ .
                 \.      pn  = p->next;
                  r      pnn = p->next->next;
 */

  { /* insert */
    nodeptr  r, s;

    r = q->back;
    s = p->back;

    if(Thorough)
    { 
      double  zqr, zqs, zrs, lzqr, lzqs, lzrs, lzsum, lzq, lzr, lzs, lzmax;      
     
	  
      if ((zqr = makenewz(tr, q, r, q->z, iterations)) == badZ) return FALSE; 
      
      if ((zqs = makenewz(tr, q, s, defaultz, iterations)) == badZ) return FALSE;      
      
      if ((zrs = makenewz(tr, r, s, defaultz, iterations)) == badZ) return FALSE;
      
      lzqr = (zqr > zmin) ? log(zqr) : log(zmin); 
      lzqs = (zqs > zmin) ? log(zqs) : log(zmin);
      lzrs = (zrs > zmin) ? log(zrs) : log(zmin);
      lzsum = 0.5 * (lzqr + lzqs + lzrs);
      
      lzq = lzsum - lzrs;
      lzr = lzsum - lzqs;
      lzs = lzsum - lzqr;
      lzmax = log(zmax);
      
      if      (lzq > lzmax) {lzq = lzmax; lzr = lzqr; lzs = lzqs;} 
      else if (lzr > lzmax) {lzr = lzmax; lzq = lzqr; lzs = lzrs;}
      else if (lzs > lzmax) {lzs = lzmax; lzq = lzqs; lzr = lzrs;}
      
      
      hookup(p->next,       q, exp(lzq));
      hookup(p->next->next, r, exp(lzr));
      hookup(p,             s, exp(lzs));      		  
    }
    else
      { 
	
	double  z;

	if(adef->model == M_PROTEINS || adef->model == M_PROTEINSCAT)
	  {
	    printf("NO fast opt mode for proteins should be used, exiting....\n");
	    exit(-1);
	    /*z = q->z / 2;
	    if(z < zmin) 
	      z = q->z;
	    else
	      if(z > zmax)
	      z = zmax;*/
	  }
	else
	  z = sqrt(q->z);
	
	hookup(p->next,       q, z);
	hookup(p->next->next, r, z);
      }


   
    
    if (! newview(tr, p)) return FALSE;
    
   
    tr->opt_level = 0;
   
    if(Thorough)
    {
      if (! localSmooth(tr, p, smoothings)) return FALSE;	
    }           
     
    return  TRUE;
  } /* insert */


nodeptr  removeNode (tree *tr, nodeptr p)

/*                q
                 .|
          remove. |
               .  |
             pn   |
    s ---- p      |add
             pnn  |
               .  |
          remove. |
                 .|      pn  = p->next;
                  r      pnn = p->next->next;
 */

    /* remove p and return where it was */
  { /* removeNode */
    double   zqr;
    nodeptr  q, r;
    
    q = p->next->back;
    r = p->next->next->back;
    zqr = q->z * r->z;
   

    
    if ((zqr = makenewz(tr, q, r, zqr, iterations)) == badZ) 
      return (node *) NULL;      

    

    hookup(q, r, zqr);

    p->next->next->back = p->next->back = (node *) NULL;
     
    return  q;
  } /* removeNode */


boolean initrav (tree *tr, nodeptr p)
  { /* initrav */
    nodeptr  q;

    if (! p->tip) {
      
      q = p->next;
      
      do 
	{
	  if (! initrav(tr, q->back))  return FALSE;	
	  q = q->next;	
        } while (q != p);
      
      
      if (! newview(tr, p)) return FALSE;
	      
      }

    return TRUE;
  } /* initrav */





boolean initravDIST (tree *tr, nodeptr p, int distance)
  {
    nodeptr  q;

    if (! p->tip && distance > 0) 
      {
      
	q = p->next;
      
	do 
	  {
	    if (! initravDIST(tr, q->back, --distance))  return FALSE;	
	    q = q->next;	
	  } 
	while (q != p);
      
      
	if (! newview(tr, p)) return FALSE;	      
      }

    return TRUE;
  } /* initrav */




boolean initravPartialHKY85CAT (tree *tr, nodeptr p, int i, double ki)
  { /* initrav */
    nodeptr  q;

    if (! p->tip) {
      q = p->next;
      do {
        if (! initravPartialHKY85CAT(tr, q->back, i, ki))  return FALSE;
        q = q->next;
        } while (q != p);

      if (! newviewPartialHKY85CAT(tr, p, i, ki)) return FALSE;

      }

    return TRUE;
  } /* initrav */

boolean initravPartialGTRCAT (tree *tr, nodeptr p, int i, double ki)
  { /* initrav */
    nodeptr  q;

    if (! p->tip) {
      q = p->next;
      do {
        if (! initravPartialGTRCAT(tr, q->back, i, ki))  return FALSE;
        q = q->next;
        } while (q != p);

      if (! newviewPartialGTRCAT(tr, p, i, ki)) return FALSE;

      }

    return TRUE;
  } /* initrav */

#ifndef HPC

boolean initravPartialPROTCAT (tree *tr, nodeptr p, int i, double ki)
  { /* initrav */
    nodeptr  q;

    if (! p->tip) {
      q = p->next;
      do {
        if (! initravPartialPROTCAT(tr, q->back, i, ki))  return FALSE;
        q = q->next;
        } while (q != p);

      if (! newviewPartialPROTCAT(tr, p, i, ki)) return FALSE;

      }

    return TRUE;
  } /* initrav */

#endif



boolean readKeyValue (char *string, char *key, char *format, void *value)
  { /* readKeyValue */

    if (!(string = strstr(string, key)))  return FALSE;
    string += strlen(key);
    string = strchr(string, '=');
    if (! (string = strchr(string, '=')))  return FALSE;
    string++;
    return  sscanf(string, format, value);  /* 1 if read, otherwise 0 */
  } /* readKeyValue */



void cacheZ (tree *tr)
  { /* cacheZ */
#ifndef HPC
    nodeptr  p;
    int  nodes;

    nodes = tr->mxtips  +  3 * (tr->mxtips - 2);
    p = tr->nodep[1];
    while (nodes-- > 0) {p->z0 = p->z; p++;}
#endif
  } /* cacheZ */

void restoreZ (tree *tr)
  { /* restoreZ */
#ifndef HPC
    nodeptr  p;
    int  nodes;

    nodes = tr->mxtips  +  3 * (tr->mxtips - 2);
    p = tr->nodep[1];
    while (nodes-- > 0) {p->z = p->z0; p++;}
#endif
  } /* restoreZ */


node * findAnyTip(nodeptr p)
  { /* findAnyTip */
    return  p->tip ? p : findAnyTip(p->next->back);
  } /* findAnyTip */


boolean testInsertParsimony (tree *tr, nodeptr p, nodeptr q)
  { /* testInsert */
    double  qz;
    nodeptr  r;
       
   

    r = q->back;             /* Save original connection */
    qz = q->z;
   
    if (! insert(tr, p, q, FALSE))       return FALSE;         
    
    if (evaluate(tr, p->next->next) == badEval) return FALSE;   
                   
    if(tr->likelihood > tr->endLH)
      {	
	/*printf("%f\n", tr->likelihood);*/
	tr->endLH = tr->likelihood;
      }
    
    saveBestTree(bt, tr);
   
    hookup(q, r, qz);
   
    p->next->next->back = p->next->back = (nodeptr) NULL;
     		            
    restoreZ(tr);           
  
    return TRUE;
  } /* testInsert */

boolean testInsertPre (tree *tr, nodeptr p, nodeptr q)
  { /* testInsert */
    double  qz;
    nodeptr  r;          

    r = q->back;             /* Save original connection */
    qz = q->z;
   
    if (! insert(tr, p, q, FALSE))       return FALSE;         
    
    if (evaluate(tr, p->next->next) == badEval) return FALSE;   
          

    if(tr->likelihood > tr->maxLH)
      tr->maxLH = tr->likelihood;        

    if(tr->likelihood > tr->endLH)
      {	
	printf("%f\n", tr->likelihood);
	tr->endLH = tr->likelihood;
      }
    
    saveBestTree(bt, tr);

   
    hookup(q, r, qz);
   
    p->next->next->back = p->next->back = (nodeptr) NULL;
     		            
    restoreZ(tr);           
  
    return TRUE;
  } /* testInsert */



void addTraverseRec(tree *tr, nodeptr p, nodeptr q,
                 int mintrav, int maxtrav)
{ /* addTraverse */

  
 
  if (--mintrav <= 0) 
    {              
      if (! testInsertParsimony(tr, p, q))  return;        
    }
  
  if ((! q->tip) && (--maxtrav > 0)) 
    {    
      addTraverseRec(tr, p, q->next->back,
		     mintrav, maxtrav);
      addTraverseRec(tr, p, q->next->next->back,
		     mintrav, maxtrav);    
  }

} 


void addTraversePre(tree *tr, nodeptr p, nodeptr q,
                 int mintrav, int maxtrav)
{ /* addTraverse */

  
 
  if (--mintrav <= 0) 
    {              
      if (! testInsertPre(tr, p, q))  return;        
    }
  
  if ((! q->tip) && (--maxtrav > 0)) 
    {    
      addTraversePre(tr, p, q->next->back,
		     mintrav, maxtrav);
      addTraversePre(tr, p, q->next->next->back,
		     mintrav, maxtrav);    
  }

} 







int  rearrangeParsimony(tree *tr, nodeptr p, int mintrav, int maxtrav)
    /* rearranges the tree, globally or locally */
  { /* rearrange */
    double   p1z, p2z, q1z, q2z;
    nodeptr  p1, p2, q, q1, q2;
    int      mintrav2;
    int result = 0;
   
    if (maxtrav < 1 || mintrav > maxtrav)  return 0;
    

    if (! p->tip) {
     
      p1 = p->next->back;
      p2 = p->next->next->back;
      if (! p1->tip || ! p2->tip) 
	{
	  p1z = p1->z;
	  p2z = p2->z;
	  if (! removeNode(tr, p)) return badRear;
	  cacheZ(tr);
	  if (! p1->tip) 
	    {
	      addTraverseRec(tr, p, p1->next->back,
			     mintrav, maxtrav);         
	      addTraverseRec(tr, p, p1->next->next->back,
			     mintrav, maxtrav);          
	    }

	  if (! p2->tip) 
	    {
	      addTraverseRec(tr, p, p2->next->back,
			     mintrav, maxtrav);
	      addTraverseRec(tr, p, p2->next->next->back,
			     mintrav, maxtrav);          
	    }

	  hookup(p->next,       p1, p1z);  /*  Restore original tree */
	  hookup(p->next->next, p2, p2z);
	  if (! (initrav(tr, tr->start)
		 && initrav(tr, tr->start->back))) return badRear;	  
        }
    }   /* if (! p->tip) */

/* Moving subtree backward in tree.  Minimum move is 2 to avoid duplicates */

    q = p->back;

    

    if (! q->tip && maxtrav > 1) {
      q1 = q->next->back;
      q2 = q->next->next->back;
      if (! q1->tip && (!q1->next->back->tip || !q1->next->next->back->tip) ||
          ! q2->tip && (!q2->next->back->tip || !q2->next->next->back->tip)) {
        q1z = q1->z;
        q2z = q2->z;
        if (! removeNode(tr, q)) return badRear;
        cacheZ(tr);
        mintrav2 = mintrav > 2 ? mintrav : 2;

        if (! q1->tip) {
          addTraverseRec(tr, q, q1->next->back,
                                  mintrav2 , maxtrav);
          addTraverseRec(tr, q, q1->next->next->back,
                                  mintrav2 , maxtrav);
         
          }

        if (! q2->tip) {
         addTraverseRec(tr, q, q2->next->back,
                                  mintrav2 , maxtrav);
          addTraverseRec(tr, q, q2->next->next->back,
                                  mintrav2 , maxtrav);
          
          }

        hookup(q->next,       q1, q1z);  /*  Restore original tree */
        hookup(q->next->next, q2, q2z);
	if (! (initrav(tr, tr->start)
	       && initrav(tr, tr->start->back))) return badRear;	  
        }
      }   /* if (! q->tip && maxtrav > 1) */

    return  1;
  } 


int  rearrangePreSelect (tree *tr, nodeptr p, int mintrav, int maxtrav)
    /* rearranges the tree, globally or locally */
  { /* rearrange */
    double   p1z, p2z, q1z, q2z;
    nodeptr  p1, p2, q, q1, q2;
    int      mintrav2;
    int result = 0;
   
    if (maxtrav < 1 || mintrav > maxtrav)  return 0;

    if (! p->tip) {
     
      p1 = p->next->back;
      p2 = p->next->next->back;
      if (! p1->tip || ! p2->tip) 
	{
	  p1z = p1->z;
	  p2z = p2->z;
	  if (! removeNode(tr, p)) return badRear;
	  cacheZ(tr);
	  if (! p1->tip) 
	    {
	      addTraversePre(tr, p, p1->next->back,
			     mintrav, maxtrav);         
	      addTraversePre(tr, p, p1->next->next->back,
			     mintrav, maxtrav);          
	    }

	  if (! p2->tip) 
	    {
	      addTraversePre(tr, p, p2->next->back,
			     mintrav, maxtrav);
	      addTraversePre(tr, p, p2->next->next->back,
			     mintrav, maxtrav);          
	    }

	  hookup(p->next,       p1, p1z);  /*  Restore original tree */
	  hookup(p->next->next, p2, p2z);
	  if (! (initrav(tr, tr->start)
		 && initrav(tr, tr->start->back))) return badRear;	  
        }
    }   /* if (! p->tip) */

/* Moving subtree backward in tree.  Minimum move is 2 to avoid duplicates */

    q = p->back;

    

    if (! q->tip && maxtrav > 1) {
      q1 = q->next->back;
      q2 = q->next->next->back;
      if (! q1->tip && (!q1->next->back->tip || !q1->next->next->back->tip) ||
          ! q2->tip && (!q2->next->back->tip || !q2->next->next->back->tip)) {
        q1z = q1->z;
        q2z = q2->z;
        if (! removeNode(tr, q)) return badRear;
        cacheZ(tr);
        mintrav2 = mintrav > 2 ? mintrav : 2;

        if (! q1->tip) {
          addTraversePre(tr, q, q1->next->back,
                                  mintrav2 , maxtrav);
          addTraversePre(tr, q, q1->next->next->back,
                                  mintrav2 , maxtrav);
         
          }

        if (! q2->tip) {
         addTraversePre(tr, q, q2->next->back,
                                  mintrav2 , maxtrav);
          addTraversePre(tr, q, q2->next->next->back,
                                  mintrav2 , maxtrav);
          
          }

        hookup(q->next,       q1, q1z);  /*  Restore original tree */
        hookup(q->next->next, q2, q2z);
	if (! (initrav(tr, tr->start)
	       && initrav(tr, tr->start->back))) return badRear;	  
        }
      }   /* if (! q->tip && maxtrav > 1) */

    return  1;
  } 




int optimizeTTinvocations = 1;
int optimizeRatesInvocations = 1;
int optimizeRateCategoryInvocations = 1;
int optimizeAlphaInvocations = 1;

void optimizeTT(tree *tr);

















void printTree(FILE *outf, tree *tr, int quoted)
{
  char *str = (char *)malloc(treeStringLength);
  char *outString;
  nodeptr startp;
  startp = tr->start;

  tr->start = tr->nodep[1];
  
  treeString(str, tr, tr->start->back, 1);
  
  tr->start = startp;

  outString = str;
  while(*outString++ != '(');
  fprintf(outf, "("); 
  if(!quoted)
  {
    while(*outString != '\0')
      {
	if(*outString != '\'')
	  fprintf(outf, "%c", *outString); 
	outString++;
      }
  }
      
  free(str);
  fflush(outf);
}

void printResult(tree *tr, int quoted)
{
  char *str = (char *)malloc(treeStringLength);
  char *outString;
  FILE *outf = fopen(resultFileName, "w");
  nodeptr startp;
  startp = tr->start;

  tr->start = tr->nodep[1];

  

  treeString(str, tr, tr->start->back, 1);  
  tr->start = startp; 
  outString = str;
  while(*outString++ != '(');
  fprintf(outf, "("); 
  if(!quoted)
  {
    while(*outString != '\0')
      {
	if(*outString != '\'')
	  fprintf(outf, "%c", *outString); 
	outString++;
      }
  }
      
  free(str);
  fclose(outf);  
}






void printLog(tree *tr, analdef *adef)
{
  
  #ifdef PARALLEL
  if(myId == 0)
    {
  #endif
      if(adef->checkpoints)
	{
	  char thisCheckPointName[2048], str[16];
	  FILE *cpf;
	  
	  printf( "%f %f %d\n", gettime() - masterTime, tr->likelihood, checkPointCounter);
	  fprintf(logFile,"%f %f %d\n", gettime() - masterTime, tr->likelihood, checkPointCounter); 
	  
	  strcpy(thisCheckPointName, checkpointFileName);
	  sprintf(str, "%d", checkPointCounter);
	  strcat(thisCheckPointName, str);
	  checkPointCounter++;
	  cpf = fopen(thisCheckPointName, "w");
	  printTree(cpf, tr, 0);
	  fflush(cpf);
	  fclose(cpf);    
	}
      else
	{
	  printf( "%f %f\n", gettime() - masterTime, tr->likelihood);
	  fprintf(logFile,"%f %f\n", gettime() - masterTime, tr->likelihood); 
	}
      
 
      fflush(logFile);
#ifdef PARALLEL
    }
#endif
    
  

}


void treeOptimize(tree *tr, int _mintrav, int _maxtrav, analdef *adef)
{
  int i;
  int mintrav = _mintrav;
  int maxtrav = _maxtrav;
  int subsequent = 0;
  int initial = adef->initial;
  double optTime;

  /*optTime = gettime();*/

  if (maxtrav > tr->ntips - 3)  
    maxtrav = tr->ntips - 3;
  
  tr->opt_level = 0;
  
  resetBestTree(bt);
  saveBestTree(bt, tr);

  checkTime(adef, tr);
  
  if(Multiple && !Thorough)
    {
      mintrav = 1; 
      
      if(maxtrav < initial)
	maxtrav = 10;
	
    }
  
  /*  Thorough = 1;*/

  /*printf("%d %d %d %d iterations %d\n", mintrav, maxtrav, Multiple, Thorough, iterations);
    printf("starting treeOptimize at %f\n", gettime() - masterTime);*/
  /*fprintf(logFile, "starting treeOptimize at %f\n", gettime() - masterTime);
    fflush(logFile);*/

  tr->startLH = tr->endLH = tr->likelihood;

  for(i = 1; i <= tr->mxtips + tr->mxtips - 2; i++)
    { 
       tr->nodeInfo = i;
      
            
       rearrangeParsimony(tr, tr->nodep[i], mintrav, maxtrav);
       if(Multiple && (tr->endLH > tr->startLH))                 	
	 {	 	  
	   recallBestTree(bt, 1, tr);	   
	   tr->startLH = tr->endLH = tr->likelihood;
	   /*printLog(tr, adef);*/
	   /*printf("%f %f\n", gettime() - optTime, tr->likelihood);*/
	   /*fprintf(logFile, "%f %f\n", gettime() - optTime, tr->likelihood);
	     fflush(logFile);*/
	   subsequent = 1;	     
	 }	
	 
    }
  /*printf("TreeOptTime: %f\n", gettime() - optTime);*/
  /*fprintf(logFile, "TreeOptTime: %f\n", gettime() - optTime);
    fflush(logFile);*/
 
  Multiple = subsequent;
  return;     
}


static int scoreCompare(const void *p1, const void *p2)
{
 preScore *rc1 = (preScore *)p1;
 preScore *rc2 = (preScore *)p2;

 double i = rc1->lh;
 double j = rc2->lh;
  
  if (i > j)
    return (-1);
  if (i < j)
    return (1);
  return (0);
}



void treeOptimizePreSelect(tree *tr, int _mintrav, int _maxtrav, analdef *adef)
{
  int i;
  int mintrav = _mintrav;
  int maxtrav = _maxtrav;
  int subsequent = 0;
  int initial = adef->initial;

  if (maxtrav > tr->ntips - 3)  
    maxtrav = tr->ntips - 3;
  
  tr->opt_level = 0;
  
  resetBestTree(bt);
  saveBestTree(bt, tr);

  checkTime(adef, tr);
  
  
  
  if(Multiple && !Thorough)
    {
      mintrav = 1; 
      
      if(maxtrav < initial)
	maxtrav = 10;	
    }

  tr->startLH = tr->endLH = tr->likelihood;


  if(Multiple)
    {
      for(i = 1; i <= tr->mxtips + tr->mxtips - 2; i++)
	{ 
	  
	  
	  rearrangeParsimony(tr, tr->nodep[i], mintrav, maxtrav);
	  if(Multiple && (tr->endLH > tr->startLH))                 	
	    {	  
	      recallBestTree(bt, 1, tr);
	      /*printf("Tree improved %f\n", tr->likelihood);*/
	      tr->startLH = tr->endLH = tr->likelihood;
	      tr->nodeCount[i] += 1;
	      subsequent = 1;
	    }	
	}
      
  
      Multiple = subsequent;
      return;      
    }
  else
    {      
      printf("NON-MULTIPLE-MODE\n");

      for(i = 1; i <= tr->mxtips + tr->mxtips - 2; i++)    
	{
	  tr->maxLH = unlikely;
	  rearrangePreSelect(tr, tr->nodep[i], 1, 2);  	 
	  tr->nodeScoresOne[i].lh = tr->maxLH;
	  tr->nodeScoresOne[i].node = i;
	}
       
      qsort(&tr->nodeScoresOne[1], 2 * tr->mxtips - 2, sizeof(preScore), scoreCompare);
            
      {
	int span;
	double max;
	int range;
	span = 2 * tr->mxtips - 2;
	
	span = (int)(span/4);
	i = 0;
	
	range = span;
	
	for(i = 1; i <= range; i++)
	  {
	    rearrangeParsimony(tr, tr->nodep[tr->nodeScoresOne[i].node], mintrav, maxtrav);	    	    
	  }
      }

      return;

    }

 
  return;     
}




void treeOptimizePreSelect2(tree *tr, int _mintrav, int _maxtrav, analdef *adef)
{
  int i;
  int mintrav = _mintrav;
  int maxtrav = _maxtrav;
  int subsequent = 0;
  int initial = adef->initial;

  if (maxtrav > tr->ntips - 3)  
    maxtrav = tr->ntips - 3;
  
  tr->opt_level = 0;
  
  resetBestTree(bt);
  saveBestTree(bt, tr);

  checkTime(adef, tr);
  
  
  
  if(Multiple && !Thorough)
    {
      mintrav = 1; 
      
      if(maxtrav < initial)
	maxtrav = 10;	
    }

  tr->startLH = tr->endLH = tr->likelihood;

  for(i = 1; i <= tr->mxtips + tr->mxtips - 2; i++)    
    {
      tr->maxLH = unlikely;
      rearrangePreSelect(tr, tr->nodep[i], 1, 2);  
      if(Multiple && (tr->endLH > tr->startLH))                 	
	{	  
	  recallBestTree(bt, 1, tr);
	  tr->startLH = tr->endLH = tr->likelihood;
	  subsequent = 1;
	  printf("%f\n", tr->likelihood);
	}	
      tr->nodeScoresOne[i].lh = tr->maxLH;
      tr->nodeScoresOne[i].node = i;
    }
  
  recallBestTree(bt, 1, tr);
 
  tr->startLH = tr->endLH = tr->likelihood;
  qsort(&tr->nodeScoresOne[1], 2 * tr->mxtips - 2, sizeof(preScore), scoreCompare);

  /*for(i = 1; i < 2 * tr->mxtips - 1; i++) 
    printf("%f %d\n", tr->nodeScoresOne[i].lh, tr->nodeScoresOne[i].node);*/
  
  {
    int span;
    double max;
    int range;
    span = 2 * tr->mxtips - 2;

    span = (int)(span/4);
    i = 0;
       
    range = span;

    for(i = 1; i <= range; i++)
      {
	rearrangeParsimony(tr, tr->nodep[tr->nodeScoresOne[i].node], mintrav, maxtrav);
	
	if(Multiple && (tr->endLH > tr->startLH))                 	
	  {	  
	    recallBestTree(bt, 1, tr);
	    printf("Tree improved %f\n", tr->likelihood);
	    tr->startLH = tr->endLH = tr->likelihood;
	    subsequent = 1;
	  }	
      }
  }



 
  
  Multiple = subsequent;
  return;     
}












/*===========  This is a problem if tr->start->back is a tip!  ===========*/
/*  All routines should be contrived so that tr->start->back is not a tip */

void initTreeString(char *treestr, tree *tr, int form)
{
   (void) sprintf(treestr, "[&&%s: version = '%s'",
                                 programName, programVersion);
   while (*treestr) treestr++;

   (void) sprintf(treestr, ", %s = %15.13g",
		  likelihood_key, unlikely);
   while (*treestr) treestr++;
   
   (void) sprintf(treestr, ", %s = %d", ntaxa_key, tr->mxtips);
   while (*treestr) treestr++;
   
   (void) sprintf(treestr,", %s = %d", opt_level_key, 0);
   while (*treestr) treestr++;
   
   (void) sprintf(treestr, ", %s = %d", smoothed_key, 0);
   while (*treestr) treestr++;
   
   (void) sprintf(treestr, "]%s", form == treeProlog ? ", " : " ");
   while (*treestr) treestr++;
}


char *treeString (char *treestr, tree *tr, nodeptr p, int form)
    /* write string with representation of tree */
    /*   form == 1 -> Newick tree */
    /*   form == 2 -> Prolog fact */
    /*   form == 3 -> PHYLIP tree */
  { /* treeString */
    double  x, z;
    char  *nameptr;
    int    c;

    if (p == tr->start->back) {
      if (form != treePHYLIP) {
        if (form == treeProlog) {
          (void) sprintf(treestr, "phylip_tree(");
          while (*treestr) treestr++;            /* move pointer to null */
          }

        (void) sprintf(treestr, "[&&%s: version = '%s'",
                                 programName, programVersion);
        while (*treestr) treestr++;

        (void) sprintf(treestr, ", %s = %15.13g",
                                 likelihood_key, tr->likelihood);
        while (*treestr) treestr++;

        (void) sprintf(treestr, ", %s = %d", ntaxa_key, tr->ntips);
        while (*treestr) treestr++;

        (void) sprintf(treestr,", %s = %d", opt_level_key, tr->opt_level);
        while (*treestr) treestr++;

        (void) sprintf(treestr, ", %s = %d", smoothed_key, tr->smoothed);
        while (*treestr) treestr++;

        (void) sprintf(treestr, "]%s", form == treeProlog ? ", " : " ");
        while (*treestr) treestr++;
        }
    }

    if (p->tip) {
      if (form != treePHYLIP) *treestr++ = '\'';

      /* NAME-REMOVE*/
      /*      nameptr = p->name;*/
      nameptr = tr->nameList[p->number];
      /* NAME-REMOVE*/


      while (c = *nameptr++) {
        if (form != treePHYLIP) {if (c == '\'') *treestr++ = '\'';}
        else if (c == ' ') {c = '_';}
        *treestr++ = c;
        }
      if (form != treePHYLIP) *treestr++ = '\'';
    }

    else {
      *treestr++ = '(';
      treestr = treeString(treestr, tr, p->next->back, form);
      *treestr++ = ',';
      treestr = treeString(treestr, tr, p->next->next->back, form);
      if (p == tr->start->back) {
        *treestr++ = ',';
        treestr = treeString(treestr, tr, p->back, form);
        }
      *treestr++ = ')';
    }

    if (p == tr->start->back) 
      {
	(void) sprintf(treestr, ":0.0%s\n", (form != treeProlog) ? ";" : ").");
      }
    else {
      z = p->z;
      if (z < zmin) z = zmin;
      if(adef->model == M_PROTEINS || adef->model == M_PROTEINSCAT)
	x = z;
      else
	{
	  if(adef->model == M_GTR || adef->model == M_GTRCAT || adef->model == M_GTRGAMMA)
	    x = -log(z) * tr->rdta->fracchangeav; 
	  else
	    x = -log(z) * tr->rdta->fracchange;
	}
      (void) sprintf(treestr, ":%8.6f", x);  /* prolog needs the space */
      }

    while (*treestr) treestr++;     /* move pointer up to null termination */
    return  treestr;
  } /* treeString */







void treeOut (FILE *treefile, tree *tr, int form)
    /* write out file with representation of final tree */
  { /* treeOut */
    int    c;
    char  *cptr, *treestr;

    treestr = (char *) malloc((tr->ntips * (nmlngth+32)) + 256);
    if (! treestr) {
      printf( "treeOut: malloc failure\n");
      exit(1);
      }

    (void) treeString(treestr, tr, tr->start->back, form);
    cptr = treestr;
    while (c = *cptr++) putc(c, treefile);

    printf( "%s\n", treestr);
    printf("%s\n", treestr);

    free(treestr);
  } /* treeOut */


/*=======================================================================*/
/*                         Read a tree from a file                       */
/*=======================================================================*/


/*  1.0.A  Processing of quotation marks in comment removed
 */

int treeFinishCom (FILE *fp, char **strp)
  { /* treeFinishCom */
    int  ch;

    while ((ch = getc(fp)) != EOF && ch != ']') {
      if (strp != NULL) *(*strp)++ = ch;    /* save character  */
      if (ch == '[') {                      /* nested comment; find its end */
        if ((ch = treeFinishCom(fp, strp)) == EOF)  break;
        if (strp != NULL) *(*strp)++ = ch;  /* save closing ]  */
        }
      }

    if (strp != NULL) **strp = '\0';        /* terminate string  */
    return  ch;
  } /* treeFinishCom */


int treeGetCh (FILE *fp)         /* get next nonblank, noncomment character */
  { /* treeGetCh */
    int  ch;

    while ((ch = getc(fp)) != EOF) {
      if (whitechar(ch)) ;
      else if (ch == '[') {                   /* comment; find its end */
        if ((ch = treeFinishCom(fp, (char **) NULL)) == EOF)  break;
        }
      else  break;
      }

    return  ch;
  } /* treeGetCh */


boolean  treeLabelEnd (int ch)
  { /* treeLabelEnd */
    switch (ch) {
        case EOF:  case '\0':  case '\t':  case '\n':  case ' ':
        case ':':  case ',':   case '(':   case ')':   case '[':
        case ';':
          return TRUE;
        default:
          break;
        }
    return FALSE;
  } /* treeLabelEnd */


boolean  treeGetLabel (FILE *fp, char *lblPtr, int maxlen)
  { /* treeGetLabel */
    int      ch;
    boolean  done, quoted, lblfound;

    if (--maxlen < 0) lblPtr = (char *) NULL;  /* reserves space for '\0' */
    else if (lblPtr == NULL) maxlen = 0;

    ch = getc(fp);
    done = treeLabelEnd(ch);

    lblfound = ! done;
    quoted = (ch == '\'');
    if (quoted && ! done) {ch = getc(fp); done = (ch == EOF);}

    while (! done) {
      if (quoted) {
        if (ch == '\'') {ch = getc(fp); if (ch != '\'') break;}
        }

      else if (treeLabelEnd(ch)) break;
      /*
	else if (ch == '_') ch = ' ';*/  /* unquoted _ goes to space */

      if (--maxlen >= 0) *lblPtr++ = ch;
      ch = getc(fp);
      if (ch == EOF) break;
      }

    if (ch != EOF)  (void) ungetc(ch, fp);

    if (lblPtr != NULL) *lblPtr = '\0';

    return lblfound;
  } /* treeGetLabel */


boolean  treeFlushLabel (FILE *fp)
  { /* treeFlushLabel */
    return  treeGetLabel(fp, (char *) NULL, (int) 0);
  } /* treeFlushLabel */


int  treeFindTipByLabel (char  *str, tree *tr)
                     /*  str -- label string pointer */
  { /* treeFindTipByLabel */
    nodeptr  q;
    char    *nameptr;
    int      ch, i, n;
    boolean  found;

    for (n = 1; n <= tr->mxtips; n++) {
      q = tr->nodep[n];
      if (! (q->back)) {          /*  Only consider unused tips */
        i = 0;
	 /* NAME-REMOVE*/
        /*nameptr = q->name;*/
	nameptr = tr->nameList[q->number];
	/* NAME-REMOVE*/
        while ((found = (str[i++] == (ch = *nameptr++))) && ch) ;
        if (found) return n;
        }
      }

    printf("ERROR: Cannot find tree species: %s\n", str);

    return  0;
  } /* treeFindTipByLabel */


int  treeFindTipName (FILE *fp, tree *tr)
  { /* treeFindTipName */
    char    *nameptr, str[nmlngth+2];
    int      n;

    if (tr->prelabeled) {
      if (treeGetLabel(fp, str, nmlngth+2))
        n = treeFindTipByLabel(str, tr);
      else
        n = 0;
      }

    else if (tr->ntips < tr->mxtips) {
      n = tr->ntips + 1;
       /* NAME-REMOVE*/
      /*nameptr = tr->nodep[n]->name;*/
      nameptr = tr->nameList[n];
       /* NAME-REMOVE*/
      if (! treeGetLabel(fp, nameptr, nmlngth+1)) n = 0;
      }

    else {
      n = 0;
      }

    return  n;
  } /* treeFindTipName */


void  treeEchoContext (FILE *fp1, FILE *fp2, int n)
 { /* treeEchoContext */
   int      ch;
   boolean  waswhite;

   waswhite = TRUE;

   while (n > 0 && ((ch = getc(fp1)) != EOF)) {
     if (whitechar(ch)) {
       ch = waswhite ? '\0' : ' ';
       waswhite = TRUE;
       }
     else {
       waswhite = FALSE;
       }

     if (ch > '\0') {putc(ch, fp2); n--;}
     }
 } /* treeEchoContext */


boolean treeProcessLength (FILE *fp, double *dptr)
  { /* treeProcessLength */
    int  ch;

    if ((ch = treeGetCh(fp)) == EOF)  return FALSE;    /*  Skip comments */
    (void) ungetc(ch, fp);

    if (fscanf(fp, "%lf", dptr) != 1) {
      printf("ERROR: treeProcessLength: Problem reading branch length\n");
      treeEchoContext(fp, stdout, 40);
      printf("\n");
      return  FALSE;
      }

    return  TRUE;
  } /* treeProcessLength */


int treeFlushLen (FILE  *fp)
  { /* treeFlushLen */
    double  dummy;
    boolean res;
    int     ch;

    ch = treeGetCh(fp);
    
    if (ch == ':') 
      {
	ch = treeGetCh(fp);
	
	    ungetc(ch, fp);
	    if(!treeProcessLength(fp, & dummy)) return 0;
	    return 1;	  
      }
   
    

    if (ch != EOF) (void) ungetc(ch, fp);
    return 1;
  } /* treeFlushLen */





boolean  treeNeedCh (FILE *fp, int c1, char *where)
  { /* treeNeedCh */
    int  c2;

    if ((c2 = treeGetCh(fp)) == c1)  return TRUE;

    printf("ERROR: Expecting '%c' %s tree; found:", c1, where);
    if (c2 == EOF) {
      printf("End-of-File");
      }
    else {
      ungetc(c2, fp);
      treeEchoContext(fp, stdout, 40);
      }
    putchar('\n');
    return FALSE;
  } /* treeNeedCh */





boolean  addElementLen (FILE *fp, tree *tr, nodeptr p)
  { /* addElementLen */
    double   z, branch;
    nodeptr  q;
    int      n, ch, fres;
    
    if ((ch = treeGetCh(fp)) == '(') {     /*  A new internal node */
      n = (tr->nextnode)++;
      if (n > 2*(tr->mxtips) - 2) {
        if (tr->rooted || n > 2*(tr->mxtips) - 1) {
          printf("ERROR: Too many internal nodes.  Is tree rooted?\n");
          printf("       Deepest splitting should be a trifurcation.\n");
          return FALSE;
          }
        else {
          tr->rooted = TRUE;
          }
        }
      q = tr->nodep[n];
      if (! addElementLen(fp, tr, q->next))        return FALSE;
      if (! treeNeedCh(fp, ',', "in"))             return FALSE;
      if (! addElementLen(fp, tr, q->next->next))  return FALSE;
      if (! treeNeedCh(fp, ')', "in"))             return FALSE;
      (void) treeFlushLabel(fp);
      }

    else {                               /*  A new tip */
      ungetc(ch, fp);
      if ((n = treeFindTipName(fp, tr)) <= 0)          return FALSE;
      q = tr->nodep[n];
      if (tr->start->number > n)  tr->start = q;
      (tr->ntips)++;
      }                                  /* End of tip processing */

  

 
    
    fres = treeFlushLen(fp);
    if(!fres) return FALSE;
    
    hookup(p, q, defaultz);
    return TRUE;
      
    
  } /* addElementLen */






int saveTreeCom (char  **comstrp)
  { /* saveTreeCom */
    int  ch;
    boolean  inquote;

    inquote = FALSE;
    while ((ch = getc(INFILE)) != EOF && (inquote || ch != ']')) {
      *(*comstrp)++ = ch;                        /* save character  */
      if (ch == '[' && ! inquote) {              /* comment; find its end */
        if ((ch = saveTreeCom(comstrp)) == EOF)  break;
        *(*comstrp)++ = ch;                      /* add ] */
        }
      else if (ch == '\'') inquote = ! inquote;  /* start or end of quote */
      }

    return  ch;
  } /* saveTreeCom */


boolean processTreeCom (FILE *fp, tree *tr)
  { /* processTreeCom */
    int   text_started, functor_read, com_open;

    /*  Accept prefatory "phylip_tree(" or "pseudoNewick("  */

    functor_read = text_started = 0;
    (void) fscanf(fp, " p%nhylip_tree(%n", & text_started, & functor_read);
    if (text_started && ! functor_read) {
      (void) fscanf(fp, "seudoNewick(%n", & functor_read);
      if (! functor_read) {
        printf("Start of tree 'p...' not understood.\n");
        return FALSE;
        }
      }

    com_open = 0;
    (void) fscanf(fp, " [%n", & com_open);

    if (com_open) {                                  /* comment; read it */
      char  com[1024], *com_end;

      com_end = com;
      if (treeFinishCom(fp, & com_end) == EOF) {     /* omits enclosing []s */
        printf("Missing end of tree comment\n");
        return FALSE;
        }

      (void) readKeyValue(com, likelihood_key, "%lg",
                               (void *) &(tr->likelihood));
      (void) readKeyValue(com, opt_level_key,  "%d",
                               (void *) &(tr->opt_level));
      (void) readKeyValue(com, smoothed_key,   "%d",
                               (void *) &(tr->smoothed));

      if (functor_read) (void) fscanf(fp, " ,");   /* remove trailing comma */
      }

    return (functor_read > 0);
  } /* processTreeCom */


nodeptr uprootTree (tree *tr, nodeptr p)
  { /* uprootTree */
    nodeptr  q, r, s, start;
    int      n;

    if (p->tip || p->back) {
      printf("ERROR: Unable to uproot tree.\n");
      printf("       Inappropriate node marked for removal.\n");
      return (nodeptr) NULL;
      }

    n = --(tr->nextnode);               /* last internal node added */
    if (n != tr->mxtips + tr->ntips - 1) {
      printf("ERROR: Unable to uproot tree.  Inconsistent\n");
      printf("       number of tips and nodes for rooted tree.\n");
      return (nodeptr) NULL;
      }

    q = p->next->back;                  /* remove p from tree */
    r = p->next->next->back;
    hookup(q, r, defaultz);

    start = (r->tip || (! q->tip)) ? r : r->next->next->back;

    if (tr->ntips > 2 && p->number != n) {
      q = tr->nodep[n];            /* transfer last node's conections to p */
      r = q->next;
      s = q->next->next;
      hookup(p,             q->back, q->z);   /* move connections to p */
      hookup(p->next,       r->back, r->z);
      hookup(p->next->next, s->back, s->z);
      if (start->number == q->number) start = start->back->back;
      q->back = r->back = s->back = (nodeptr) NULL;
      }
    else {
      p->back = p->next->back = p->next->next->back = (nodeptr) NULL;
      }

    tr->rooted = FALSE;
    return  start;
  } /* uprootTree */


static char buffer[] = "foobar";



boolean treeReadLen (FILE *fp, tree *tr)
  { /* treeReadLen */
    nodeptr  p;
    int      i, ch;
    boolean  is_fact;

    for (i = 1; i <= tr->mxtips; i++) tr->nodep[i]->back = (node *) NULL;
    tr->start       = tr->nodep[tr->mxtips];
    tr->ntips       = 0;
    tr->nextnode    = tr->mxtips + 1;
    tr->opt_level   = 0;
   
    tr->smoothed    = FALSE;
    tr->rooted      = FALSE;

    is_fact = processTreeCom(fp, tr);

    p = tr->nodep[(tr->nextnode)++];
    if (! treeNeedCh(fp, '(', "at start of"))       return FALSE;
    if (! addElementLen(fp, tr, p))                 return FALSE;
    if (! treeNeedCh(fp, ',', "in"))                return FALSE;
    if (! addElementLen(fp, tr, p->next))           return FALSE;
    if (! tr->rooted) {
      if ((ch = treeGetCh(fp)) == ',') {        /*  An unrooted format */
        if (! addElementLen(fp, tr, p->next->next)) return FALSE;
        }
      else {                                    /*  A rooted format */
        tr->rooted = TRUE;
        if (ch != EOF)  (void) ungetc(ch, fp);
        }
      }
    else {
      p->next->next->back = (nodeptr) NULL;
      }
    if (! treeNeedCh(fp, ')', "in"))                return FALSE;
    (void) treeFlushLabel(fp);
    if (! treeFlushLen(fp))                         return FALSE;
    if (is_fact) {
      if (! treeNeedCh(fp, ')', "at end of"))       return FALSE;
      if (! treeNeedCh(fp, '.', "at end of"))       return FALSE;
      }
    else {
      if (! treeNeedCh(fp, ';', "at end of"))       return FALSE;
      }

    if (tr->rooted) {
      p->next->next->back = (nodeptr) NULL;
      tr->start = uprootTree(tr, p->next->next);
      if (! tr->start)                              return FALSE;
      }
    else {
      tr->start = p->next->next->back;  /* This is start used by treeString */
      }

    return  (initrav(tr, tr->start) && initrav(tr, tr->start->back));
  } /* treeReadLen */










/*=======================================================================*/
/*                        Read a tree from a string                      */
/*=======================================================================*/

int str_treeFinishCom (char **treestrp, char **strp)
                      /* treestrp -- tree string pointer */
                      /* strp -- comment string pointer */
  { /* str_treeFinishCom */
    int  ch;
    while ( (ch=*(*treestrp)++)!='\0' && ch!=']' ) {
      if (strp != NULL) *(*strp)++ = ch;    /* save character  */
      if (ch == '[') {                      /* nested comment; find its end */
        if ((ch=str_treeFinishCom(treestrp,strp)) == '\0')  break;
        if (strp != NULL) *(*strp)++ = ch;  /* save closing ]  */
        }
      }
    if (strp != NULL) **strp = '\0';        /* terminate string  */
    return  ch;
  } /* str_treeFinishCom */


int str_treeGetCh (char **treestrp)
    /* get next nonblank, noncomment character */
  { /* str_treeGetCh */
    int  ch;
    while ((ch=*(*treestrp)++) != '\0') {
      if (whitechar(ch)) ;
      else if (ch == '[') {                  /* comment; find its end */
        if ((ch = str_treeFinishCom(treestrp,(char**)NULL)) == '\0')  break;
        }
      else  break;
      }

    return  ch;
  } /* str_treeGetCh */




boolean  str_treeGetLabel (char **treestrp, char *lblPtr, int maxlen)
  { /* str_treeGetLabel */
    int      ch;
    boolean  done, quoted, lblfound;


    if (--maxlen < 0)
      lblPtr = (char*)NULL;  /* reserves space for '\0' */
    else if(lblPtr == NULL)
      maxlen = 0;

    ch = *(*treestrp)++;
    done = treeLabelEnd(ch);

    lblfound = !done;
    quoted = (ch == '\'');
    if (quoted && ! done) {
      ch = *(*treestrp)++;
      done = (ch == '\0');
    }

    while(!done) {
      if (quoted) 
	{
	  if (ch == '\'') 
	    {
	      ch = *(*treestrp)++;
	      if (ch != '\'') break;
	    }
	}
      else 
	if (treeLabelEnd(ch))
	  break;
      /*else 
	  if (ch == '_')
	    ch = ' ';*/
      if (--maxlen >= 0) *lblPtr++ = ch;
      ch = *(*treestrp)++;
      if (ch == '\0') break;
    }

    (*treestrp)--;

    if (lblPtr != NULL) *lblPtr = '\0';

    return lblfound;
  } /* str_treeGetLabel */


boolean  str_treeFlushLabel (char **treestrp)
  { /* str_treeFlushLabel */
    return  str_treeGetLabel(treestrp, (char*)NULL, (int)0);
  } /* str_treeFlushLabel */


int  str_treeFindTipName (char **treestrp, tree *tr)         /*DKB-orig*/
/*int  str_treeFindTipName (char **treestrp, tree *tr, int ch)*/   /*DKB-change*/
  { /* str_treeFindTipName */
    nodeptr  q;
    char    *nameptr, str[nmlngth+2];
    int      i, n;

    if (tr->prelabeled) {
      if (str_treeGetLabel(treestrp, str, nmlngth+2)) {
        n = treeFindTipByLabel(str, tr);
      }
      else
        n = 0;
      }

    else if (tr->ntips < tr->mxtips) {
      n = tr->ntips + 1;
      /* NAME-REMOVE*/
      /*nameptr = tr->nodep[n]->name;*/
      nameptr = tr->nameList[n];
      /* NAME-REMOVE*/
      if (! str_treeGetLabel(treestrp, nameptr, nmlngth+1)) n = 0;
      }

    else {
      n = 0;
      }

    return  n;
  } /* str_treeFindTipName */


boolean str_treeProcessLength (char **treestrp, double *dptr)
  { /* str_treeProcessLength */
    int     used;

    if(!str_treeGetCh(treestrp))  return FALSE;    /*  Skip comments */
    (*treestrp)--;

    if (sscanf(*treestrp, "%lf%n", dptr, &used) != 1) {
      printf("ERROR: str_treeProcessLength: Problem reading branch length\n");
      printf("%40s\n", *treestrp);
      *dptr = 0.0;
      return FALSE;
      }
    else {
      *treestrp += used;
      }

    return  TRUE;
  } /* str_treeProcessLength */


boolean  str_treeFlushLen (char **treestrp)
  { /* str_treeFlushLen */
    int  ch;
    double  x;

    if ((ch = str_treeGetCh(treestrp)) == ':')
    /*return str_treeProcessLength(treestrp, (double*)NULL);*/   /*DKB-orig*/
      return str_treeProcessLength(treestrp, &x);              /*DKB-change*/
    else {
      (*treestrp)--;
      return TRUE;
      }
  } /* str_treeFlushLen */


boolean  str_treeNeedCh (char **treestrp, int c1, char *where)
  { /* str_treeNeedCh */
    int  c2, i;

    if ((c2 = str_treeGetCh(treestrp)) == c1)  return TRUE;

    printf("ERROR: Missing '%c' %s tree; ", c1, where);
    if (c2 == '\0') 
      printf("end-of-string");
    else {
      putchar('"');
      for (i = 24; i-- && (c2 != '\0'); c2 = *(*treestrp)++)  putchar(c2);
      putchar('"');
      }

    printf(" found instead\n");
    return FALSE;
  } /* str_treeNeedCh */


boolean str_processTreeCom(tree *tr, char **treestrp)
  { /* str_processTreeCom */
    char  *com, *com_end;
    int  text_started, functor_read, com_open;

    com = *treestrp;

    /* Comment must begin with either "phylip_tree" or "pseudoNewick".
     * If it is neither, return FALSE. */
    functor_read = text_started = 0;
    sscanf(com, " p%nhylip_tree(%n", &text_started, &functor_read);
    if (functor_read) {
      com += functor_read;
    }
    else if (text_started) {
      com += text_started;
      sscanf(com, "seudoNewick(%n", &functor_read);
      if (! functor_read) {
        printf("Start of tree 'p...' not understood.\n");
        return  FALSE;
      }
      else {
        com += functor_read;
      }
    }

    /* Find opening bracket of comment */
    com_open = 0;
    sscanf(com, " [%n", &com_open);
    com += com_open;

    /* Read comment. */
    if (com_open) {
      if (!(com_end = strchr(com, ']'))) {
        printf("Missing end of tree comment.\n");
        return  FALSE;
      }
      *com_end = 0;
      (void)readKeyValue(com,likelihood_key,"%lg",(void*)&(tr->likelihood));
      (void)readKeyValue(com,opt_level_key, "%d", (void*)&(tr->opt_level));
      (void)readKeyValue(com,smoothed_key,  "%d", (void*)&(tr->smoothed));
      *com_end = ']';
      com_end++;

      /* Remove trailing comma, and return addr of next char in treestp */
      if (functor_read) {
        text_started = 0;
        sscanf(com_end, " ,%n", & text_started);
        com_end += text_started;
      }
      *treestrp = com_end;
    }
    return (functor_read > 0);
  } /* str_processTreeCom */

boolean str_processTreeComMerge(int *ntaxa, char **treestrp)
  { /* str_processTreeCom */
    char  *com, *com_end;
    int  text_started, functor_read, com_open;

    com = *treestrp;

    /* Comment must begin with either "phylip_tree" or "pseudoNewick".
     * If it is neither, return FALSE. */
    functor_read = text_started = 0;
    sscanf(com, " p%nhylip_tree(%n", &text_started, &functor_read);
    if (functor_read) {
      com += functor_read;
    }
    else if (text_started) {
      com += text_started;
      sscanf(com, "seudoNewick(%n", &functor_read);
      if (! functor_read) {
        printf("Start of tree 'p...' not understood.\n");
        return  FALSE;
      }
      else {
        com += functor_read;
      }
    }

    /* Find opening bracket of comment */
    com_open = 0;
    sscanf(com, " [%n", &com_open);
    com += com_open;

    /* Read comment. */
    if (com_open) {
      if (!(com_end = strchr(com, ']'))) {
        printf("Missing end of tree comment.\n");
        return  FALSE;
      }
      *com_end = 0;
      /*(void)readKeyValue(com,likelihood_key,"%lg",(void*)&(tr->likelihood));*/
      (void)readKeyValue(com,ntaxa_key, "%d", (void *)ntaxa);   
      /*      (void)readKeyValue(com,opt_level_key, "%d", (void*)&(tr->opt_level));
	      (void)readKeyValue(com,smoothed_key,  "%d", (void*)&(tr->smoothed));*/
      *com_end = ']';
      com_end++;

      /* Remove trailing comma, and return addr of next char in treestp */
      if (functor_read) {
        text_started = 0;
        sscanf(com_end, " ,%n", & text_started);
        com_end += text_started;
      }
      *treestrp = com_end;
    }
    return (functor_read > 0);
  } /* str_processTreeCom */


boolean  str_addElementLen (char **treestrp, tree *tr, nodeptr p)
  { /* str_addElementLen */
    double   z, branch;
    nodeptr  q;
    int      n, ch;

    if ((ch = str_treeGetCh(treestrp)) == '(') { /*  A new internal node */
      n = (tr->nextnode)++;
      if (n > 2*(tr->mxtips) - 2) {
        if (tr->rooted || n > 2*(tr->mxtips) - 1) {
          printf("ERROR: too many internal nodes.  Is tree rooted?\n");
          printf("Deepest splitting should be a trifurcation.\n");
          return  FALSE;
        }
        else {
          tr->rooted = TRUE;
        }
      }
      q = tr->nodep[n];
      if (! str_addElementLen(treestrp, tr, q->next))          return FALSE;
      if (! str_treeNeedCh(treestrp, ',', "in"))               return FALSE;
      if (! str_addElementLen(treestrp, tr, q->next->next))    return FALSE;
      if (! str_treeNeedCh(treestrp, ')', "in"))               return FALSE;
    /*if (! str_treeFlushLabel(treestrp))                      return FALSE;*//*DKB-orig*/
    }

    else {                           /*  A new tip */
    /*n = str_treeFindTipName(treestrp, tr, ch);*/ /*DKB-orig*/
      if(ch=='\'') (*treestrp)--;                  /*DKB-change*/
      n = str_treeFindTipName(treestrp, tr);       /*DKB-change*/
      if (n <= 0) return FALSE;
      q = tr->nodep[n];
      if (tr->start->number > n)  tr->start = q;
      (tr->ntips)++;
    }

    /*  Master and Slave always use lengths */

    if (! str_treeNeedCh(treestrp, ':', "in"))                 return FALSE;
    if (! str_treeProcessLength(treestrp, &branch))            return FALSE;
    if(adef->model == M_GTR || adef->model == M_GTRCAT || adef->model == M_GTRGAMMA)
      z = exp(-branch / tr->rdta->fracchangeav); 
    else
      z = exp(-branch / tr->rdta->fracchange);
      if (z > zmax)  z = zmax;
    hookup(p, q, z);

    return  TRUE;
  } /* str_addElementLen */




boolean str_treeReadLen (char *treestr, tree *tr)
    /* read string with representation of tree */
  { /* str_treeReadLen */
    nodeptr  p;
    int  i;
    boolean  is_fact, found;

    for(i=1; i<=(tr->mxtips); i++) tr->nodep[i]->back = (node*)NULL;
    tr->start       = tr->nodep[tr->mxtips];
    tr->ntips       = 0;
    tr->nextnode    = tr->mxtips + 1;
    tr->opt_level   = 0;
    tr->smoothed    = 1;/*(myprogtype==DNAML_MASTER); DANGER !!!!!!!!!*/
    tr->rooted      = FALSE;

    is_fact = str_processTreeCom(tr,&treestr);
    p = tr->nodep[(tr->nextnode)++];

    if(!str_treeNeedCh(&treestr, '(', "at start of"))       return FALSE;
    if(!str_addElementLen(&treestr, tr, p))                 return FALSE;
    if(!str_treeNeedCh(&treestr, ',', "in"))                return FALSE;
    if(!str_addElementLen(&treestr, tr, p->next))           return FALSE;
    if(!tr->rooted) {
      if(str_treeGetCh(&treestr) == ',') {        /*  An unrooted format */
        if(!str_addElementLen(&treestr,tr,p->next->next)) return FALSE;
      }
      else {                                       /*  A rooted format */
        p->next->next->back = (nodeptr) NULL;
        tr->rooted = TRUE;
        treestr--;
      }
    }
    if(!str_treeNeedCh(&treestr, ')', "in"))                 return FALSE;
  /*if(!str_treeFlushLabel(&treestr))                        return FALSE;*//*DKB-orig*/
    if(!str_treeFlushLen(&treestr))                          return FALSE;
    if(is_fact) {
      if(!str_treeNeedCh(& treestr, ')', "at end of"))       return FALSE;
      if(!str_treeNeedCh(& treestr, '.', "at end of"))       return FALSE;
    }
    else {
      if(!str_treeNeedCh(& treestr, ';', "at end of"))       return FALSE;
    }

    if(tr->rooted)  if (! uprootTree(tr, p->next->next))     return FALSE;
    tr->start = p->next->next->back;  /* This is start used by treeString */

    return  (initrav(tr,tr->start) && initrav(tr,tr->start->back));
  } /* str_treeReadLen */








boolean treeEvaluate (tree *tr, double smoothFactor)       /* Evaluate a user tree */
  { /* treeEvaluate */
    
   
    if (! smoothTree(tr, (int)((double)smoothings * smoothFactor))) 
      {
	return FALSE;      
      }
   
    if (evaluate(tr, tr->start) == badEval)  
      {
	return FALSE;
      }
     

    return TRUE;
  } /* treeEvaluate */



boolean evalFast (tree *tr)
  {
       
    if (! smoothTree(tr, 1)) 
      {
	return FALSE;      
      }
    if (evaluate(tr, tr->start) == badEval)  
      {
	return FALSE;
      }
     
    return TRUE;
  } /* treeEvaluate */

















double str_readTreeLikelihood (char *treestr)
  { /* str_readTreeLikelihood */
    double lk1;
    char    *com, *com_end;
    boolean  readKeyValue();

    if ((com = strchr(treestr, '[')) /*&& (com < strchr(treestr, '('))*/
                                     && (com_end = strchr(com, ']'))) 
      {
      com++;
      *com_end = 0;
      if (readKeyValue(com, likelihood_key, "%lg", (void *) &(lk1))) {
        *com_end = ']';
        return lk1;
        }
      }

    printf( "ERROR reading likelihood in receiveTree\n");
    printf("%s \n", treestr);
    return  badEval;
  } /* str_readTreeLikelihood */



















int randomInt(int n)
{
  return rand() %n;
}





void makePermutation(int *perm, int n)
{    
  int  i, j, k;
  int sum1 = 0, sum2 = 0;

  
  srand((unsigned int) time(NULL));
  
  for (i = 1; i <= n; i++)
    {
      perm[i] = i;
      sum1 += i;
    }

  for (i = 1; i <= n; i++) 
    {
      k        = randomInt(n + 1 - i);
      j        = perm[i];
      perm[i]     = perm[i + k];
      perm[i + k] = j; 
    }

}


void initAdef(analdef *adef)
{
  adef->model = M_HKY85;
  adef->timeLimit = 0;
  adef->t0 = 3.0;
  adef->repeats = 0;
  adef->max_rearrange = 21;  
  adef->stepwidth = 5;
  adef->initial = 10;
  adef->restart = FALSE;
  adef->mode = TREE_INFERENCE;
  adef->categories = 50;
  adef->boot    =           0;
  adef->interleaved =    TRUE;  
  adef->interleaved = 1;
  adef->maxSubProblemSize = 100;
  adef->useWeightFile = FALSE;
  adef->protmodel = M_JTT;
  adef->checkpoints = 0;
  adef->startingTreeOnly = 0;
}


void get_args(int argc, char *argv[], boolean print_usage, analdef *adef, rawdata *rdta)
{ /* get_args */
  int        c;
  boolean    bad_opt=FALSE;
  int        i, time, temperature;
  char       buf[2048];
  char       model[2048] = "";
  char       modelChar;
  int nameSet = 0, 
    alignmentSet = 0, 
    catSet = 0,
    checkpointSet = 0,
    startingTreeOnly = 0;
  
  run_id[0] = 0;
  workdir[0] = 0;
  seq_file[0] = 0;
  tree_file[0] = 0;
  model[0] = 0;
  weightFileName[0] = 0;
  


  

  while(!bad_opt && ((c=getopt(argc,argv,"a:b:c:h:d:f:g:i:k:l:m:r:t:w:s:n:e:y:"))!=-1) ) {
    switch(c) 
      {
      case 'y':
	sscanf(optarg,"%d", &startingTreeOnly);
	adef->startingTreeOnly = startingTreeOnly;
	break;
      case 'h':
	sscanf(optarg,"%d", &checkpointSet);
	if(checkpointSet > 0)
	  adef->checkpoints = 1;
	else
	  adef->checkpoints = 0;
	break;
      case 'a':
	strcpy(weightFileName,optarg);
	adef->useWeightFile = TRUE;
        break;      
      case 'e':
	sscanf(optarg,"%d", &adef->maxSubProblemSize);	
	break;
      case 'b':
	sscanf(optarg,"%ld", &adef->boot);
	printf( "Bootstrap random number seed = %ld\n\n", adef->boot);
	break;
      case 'c':
	sscanf(optarg, "%d", &adef->categories);
	catSet = 1;
	break;	      
      case 'd':
	sscanf(optarg, "%d", &temperature);
	adef->t0 = (double)temperature;	
	break;
      case 'g':
	sscanf(optarg, "%d", &adef->repeats);
	break;
      case 'l':
	sscanf(optarg, "%d", &time);
	adef->timeLimit = (double)time;
	break;
      case 'f': 
	sscanf(optarg, "%c", &modelChar);
	switch(modelChar)
	  {
	  case 'f': adef->mode = FAST_MODE;break;
	  case 's': adef->mode = SIMULATED_ANNEALING; break;
	  case 'e': adef->mode = TREE_EVALUATION; break;
	  case 'c': adef->mode = TREE_INFERENCE; break;	
	    /*	  case 'r': adef->mode = TREE_RECURSIVE; break;  */
	  case 'h': adef->mode = HEAVY_MODE; break;
	    /*case 'p': adef->mode = PRE_SELECT_MODE; break;*/
#ifdef PARALLEL
	  case 'p': adef->mode = PARALLEL_MODE; break;
#endif
	  case 'x': adef->mode = SPR; break;
	  case 'b': adef->mode = BIG_MODE; break;      
	  case 'a': adef->mode =  BIG_FAST_MODE; break;
	  case 'd': adef->mode =  BIG_RAPID_MODE; break;
	  default: 
	    {
	      printf("Error select on of the following algorithms:\n");
	      printf("-f c : standard hill-climbing (default)\n");
	      printf("-f f : fast hill climbing\n");
	      printf("-f e : optimization of model parameters and branch lengths\n");
	      printf("-f s : simulated annealing algorithm\n");
	      printf("-f h : hierarchical optimization mode\n");
	      printf("-f d : very fast hill-climbing for huge data\n");
	      /*printf("-f r : divide-and-conquer algorithm\n");*/
	      /*printf("-f p : pre-select-mode\n");*/
	      exit(-1);
	    }
	  }
	break;      
      case 'i':
	sscanf(optarg, "%d", &adef->initial);
	break;     
      case 'n':
        strcpy(run_id,optarg);
	nameSet = 1;
        break;
      case 'w':
        strcpy(workdir,optarg);
        break;                 
      case 't':
	strcpy(tree_file, optarg);
	adef->restart = TRUE;
	break;
      case 'r':
	sscanf(optarg, "%d", &adef->stepwidth);	
	if(adef->stepwidth < 2) adef->stepwidth = 2;
	break;    
      case 'k':
	sscanf(optarg, "%d", &adef->max_rearrange);
	if(adef->max_rearrange < 5) adef->max_rearrange = 5;
	break;  
      case 's':
	strcpy(seq_file, optarg);
	alignmentSet = 1;
	break;
      case 'm':
	strcpy(model, optarg);	
	if(strcmp(model, "HKY85\0") == 0)
	  {
	    adef->model = M_HKY85;
	    goto l_1;
	  }
	if(strcmp(model, "GTR\0") == 0)
	  {
	    adef->model = M_GTR;
	    goto l_1;
	  }
	if(strcmp(model, "HKY85CAT\0") == 0)
	  {
	    adef->model = M_HKY85CAT;
	    goto l_1;
	  }
	if(strcmp(model, "HKY85GAMMA\0") == 0)
	  {
	    adef->model = M_HKY85GAMMA;
	    goto l_1;
	  }	
	if(strcmp(model, "GTRGAMMA\0") == 0)
	  {
	    adef->model = M_GTRGAMMA;
	    goto l_1;
	  }
	if(strcmp(model, "GTRCAT\0") == 0)
	  {
	    adef->model = M_GTRCAT;
	    goto l_1;
	  }
	if(strcmp(model, "JTT\0") == 0)
	  {
	    adef->protmodel = M_JTT;
	    adef->model = M_PROTEINS;
	    goto l_1;
	  }
	if(strcmp(model, "PMB\0") == 0)
	  {
	    adef->protmodel = M_PMB;
	    adef->model = M_PROTEINS;
	    goto l_1;
	  }
	if(strcmp(model, "PAM\0") == 0)
	  {
	    adef->protmodel = M_PAM;
	    adef->model = M_PROTEINS;
	    goto l_1;
	  }
	if(strcmp(model, "JTTCAT\0") == 0)
	  {
	    adef->protmodel = M_JTT;
	    adef->model = M_PROTEINSCAT;
	    goto l_1;
	  }
	if(strcmp(model, "PMBCAT\0") == 0)
	  {
	    adef->protmodel = M_PMB;
	    adef->model = M_PROTEINSCAT;
	    goto l_1;
	  }
	if(strcmp(model, "PAMCAT\0") == 0)
	  {
	    adef->protmodel = M_PAM;
	    adef->model = M_PROTEINSCAT;
	    goto l_1;
	  }


	printf("Model %s does not exist\n", model);
	printf("please use either HKY85, HKY85CAT, HKY85GAMMA, GTR, GTRCAT for DNA DATA\n");
	printf("       and either JTT, JTTCAT, PMB, PMBCAT, PAM, PAMCAT for PROTEIN DATA\n");
	exit(-1);
      l_1:
	break;      
      default:
        bad_opt = TRUE;
        break;
    }

    

  }
  
  


  if(adef->model == M_PROTEINS || adef->model == M_PROTEINSCAT)
    {
      if(!catSet)
	adef->categories = 20;

      if(adef->mode == SIMULATED_ANNEALING || adef->mode == TREE_RECURSIVE || adef->mode == FAST_MODE)
	{
	  printf("\n Error: RAxML currently implements only the standard hill-climbing algorithm -f c\n");
	  printf("  and user-tree evaluation -f e for PROTEIN SUBSTITUTION models\n");
	  exit(1);
	}	

    }

  if(adef->mode == TREE_EVALUATION && (!adef->restart))
    {
      printf("\n Error: please specify a treefile for the tree you want to evaluate with -t\n");
      exit(-1);
    }

  if(!nameSet)
    {
      printf("\n Error: please specify a name for this run with -n\n");
      exit(-1);
    }
    
  if(! alignmentSet)
    {
      printf("\n Error: please specify an alignment for this run with -s\n");
      exit(-1);
    }


#ifdef WIN32
  if(workdir[0]==0 || (workdir[1] != ':' && workdir[2] != '\\'))
    {
      getcwd(buf,sizeof(buf));
      if( buf[strlen(buf)-1] != '\\') strcat(buf,"\\");
      strcat(buf,workdir);
      if( buf[strlen(buf)-1] != '\\') strcat(buf,"\\");
      strcpy(workdir,buf);
    }
#else
  if(workdir[0]==0 || workdir[0] != '/') 
    {
      getcwd(buf,sizeof(buf));
      if( buf[strlen(buf)-1] != '/') strcat(buf,"/");
      strcat(buf,workdir);
      if( buf[strlen(buf)-1] != '/') strcat(buf,"/");
      strcpy(workdir,buf);
    }
#endif

  if(bad_opt) 
    {
      
      exit(-1);
    }
    return;
}










boolean str_treeReadLenSpecial (char *treestr, tree *tr)
    /* read string with representation of tree */
  { /* str_treeReadLen */
    nodeptr  p;
    int  i;
    boolean  is_fact, found;

    for (i = 1; i <= tr->mxtips; i++) tr->nodep[i]->back = (node *) NULL;
    tr->start       = tr->nodep[tr->mxtips];
    tr->ntips       = 0;
    tr->nextnode    = tr->mxtips + 1;
    tr->opt_level   = 0;
    tr->smoothed    = FALSE;
    tr->rooted      = FALSE;


    is_fact = str_processTreeCom(tr,&treestr);
    p = tr->nodep[(tr->nextnode)++];

    if(!str_treeNeedCh(&treestr, '(', "at start of"))       return FALSE;
    if(!str_addElementLen(&treestr, tr, p))                 return FALSE;
    if(!str_treeNeedCh(&treestr, ',', "in"))                return FALSE;
    if(!str_addElementLen(&treestr, tr, p->next))           return FALSE;
    if (! tr->rooted) 
      { 
	if(str_treeGetCh(&treestr) == ',')
	  {
	    if (! str_addElementLen(&treestr, tr, p->next->next)) return FALSE;
	  }
	else
	  {
	    tr->rooted = TRUE;
	    if(*treestr != '\n') treestr--;
	  }
      }
    else
      {
	p->next->next->back = (nodeptr) NULL;
      }
      
    if(!str_treeNeedCh(&treestr, ')', "in"))                 return FALSE;
    str_treeFlushLabel(&treestr);           
    if(!str_treeFlushLen(&treestr))                          return FALSE;
    if(is_fact) {
      if(!str_treeNeedCh(& treestr, ')', "at end of"))       return FALSE;
      if(!str_treeNeedCh(& treestr, '.', "at end of"))       return FALSE;
    }
    else 
      {
	if(!str_treeNeedCh(& treestr, ';', "at end of"))       return FALSE;
      }

    if(tr->rooted)
      {
	p->next->next->back = (nodeptr) NULL;
	tr->start =  uprootTree(tr, p->next->next);
	if(!tr->start) return FALSE;
	}
    else
      {
	tr->start = p->next->next->back;
      }

    return  (initrav(tr,tr->start) && initrav(tr,tr->start->back));
  } /* str_treeReadLen */



void optimizeAlpha(tree *tr)
{
 int k;
 
 double currentTT = tr->alpha;
 double startLikelihood = tr->likelihood;
 double maxLikelihoodMinus = tr->likelihood;
 double maxLikelihoodPlus = tr->likelihood;
 double maxTTPlus = currentTT;
 double maxTTMinus = currentTT;
 double spacing = 0.5/optimizeTTinvocations;
 double tree_likelihood = startLikelihood;
 optimizeAlphaInvocations++;
      
  k = 1;
      
  while(tree_likelihood >= maxLikelihoodMinus && (currentTT - spacing * k) > 0) 
    {      
      tr->alpha = currentTT - spacing * k;     
      makeGammaCats(tr);
      initrav(tr, tr->start);
      initrav(tr, tr->start->back);
      evaluate(tr, tr->start);
      
      /*printf("alpha %f lh %f\n", tr->alpha, tr->likelihood);*/

      tree_likelihood = tr->likelihood;
     
      if(tr->likelihood > maxLikelihoodMinus)
	{
	  maxLikelihoodMinus = tr->likelihood;
	  maxTTMinus = currentTT - spacing * k;
	}    

      k++;
    }
  
  k = 1;
  tree_likelihood = startLikelihood;
  while(tree_likelihood >= maxLikelihoodPlus) 
    {      
      tr->alpha = currentTT + spacing * k;     
      makeGammaCats(tr);
      initrav(tr, tr->start);
      initrav(tr, tr->start->back);
      evaluate(tr, tr->start);      
      
      /*      printf("alpha %f lh %f\n", tr->alpha, tr->likelihood);*/

      tree_likelihood = tr->likelihood;

      if(tr->likelihood > maxLikelihoodPlus)
	{
	  maxLikelihoodPlus = tr->likelihood;
	  maxTTPlus = currentTT + spacing * k;
	}	 
      k++;
    }
  
  if(maxLikelihoodPlus > startLikelihood || maxLikelihoodMinus > startLikelihood)
    {
      if(maxLikelihoodPlus > maxLikelihoodMinus)
	{
	  tr->alpha = maxTTPlus;
	}
      else
	{
	  tr->alpha = maxTTMinus;
	}     
      makeGammaCats(tr);
      initrav(tr, tr->start);
      initrav(tr, tr->start->back);
      evaluate(tr, tr->start);

      infoFile = fopen(infoFileName, "a");

      fprintf(infoFile,"Alpha shape parameter: %f\n", tr->alpha);
      fclose(infoFile);
    }
  else
    { 
      tr->alpha = currentTT;
      makeGammaCats(tr);
      initrav(tr, tr->start);
      initrav(tr, tr->start->back);      
      evaluate(tr, tr->start);
    }
  /*printf("FINAL alpha %f lh %f\n", tr->alpha, tr->likelihood);*/

  /*treeEvaluate(tr, 1);

  printf("EVAL alpha %f lh %f\n", tr->alpha, tr->likelihood);*/

}

void optimizeTT(tree *tr)
{
  int k;
 
  double currentTT = tr->t_value;
  double startLikelihood = tr->likelihood;
  double maxLikelihoodMinus = tr->likelihood;
  double maxLikelihoodPlus = tr->likelihood;
  double maxTTPlus = currentTT;
  double maxTTMinus = currentTT;
  double spacing = 0.5/optimizeTTinvocations;
  double tree_likelihood = startLikelihood;
  optimizeTTinvocations++;
      
  k = 1;
      
  while(tree_likelihood >= maxLikelihoodMinus && (currentTT - spacing * k) > 0) 
    {
      
      alterTT(tr, currentTT - spacing * k);     
      initrav(tr, tr->start);
      initrav(tr, tr->start->back);
      evaluate(tr, tr->start);
      
      tree_likelihood = tr->likelihood;
      if(tr->likelihood > maxLikelihoodMinus)
	{
	  maxLikelihoodMinus = tr->likelihood;
	  maxTTMinus = currentTT - spacing * k;
	}
      k++;
    }
  
  k = 1;
  tree_likelihood = startLikelihood;
  while(tree_likelihood >= maxLikelihoodPlus) 
    {      
      alterTT(tr, currentTT + spacing * k);     
      initrav(tr, tr->start);
      initrav(tr, tr->start->back);
      evaluate(tr, tr->start);      

      tree_likelihood = tr->likelihood;
      if(tr->likelihood > maxLikelihoodPlus)
	{
	  maxLikelihoodPlus = tr->likelihood;
	  maxTTPlus = currentTT + spacing * k;
	}	 
      k++;
    }
  
  if(maxLikelihoodPlus > startLikelihood || maxLikelihoodMinus > startLikelihood)
    {
      if(maxLikelihoodPlus > maxLikelihoodMinus)
	{
	  alterTT(tr, maxTTPlus);
	}
      else
	{
	  alterTT(tr, maxTTMinus);
	}     
      initrav(tr, tr->start);
      initrav(tr, tr->start->back);
      evaluate(tr, tr->start);
    }
  else
    { 
      alterTT(tr, currentTT);	
      initrav(tr, tr->start);
      initrav(tr, tr->start->back);      
      evaluate(tr, tr->start);
    }
}


void baseFrequenciesGTR(rawdata *rdta, cruncheddata *cdta)
{
  double  sum, suma, sumc, sumg, sumt, wj, fa, fc, fg, ft, freqa, freqc, freqg, freqt;
  int     i, j, k, code;
  yType  *yptr;

  freqa = 0.25;
  freqc = 0.25;
  freqg = 0.25;
  freqt = 0.25;
  for (k = 1; k <= 8; k++) 
    {
      suma = 0.0;
      sumc = 0.0;
      sumg = 0.0;
      sumt = 0.0;
      for (i = 0; i < rdta->numsp; i++) 
	{
	  yptr = rdta->y[i];
	  for (j = 0; j < cdta->endsite; j++) 
	    {
	      code = *yptr++;
	      fa = freqa * ( code       & 1);
	      fc = freqc * ((code >> 1) & 1);
	      fg = freqg * ((code >> 2) & 1);
	      ft = freqt * ((code >> 3) & 1);
	      wj = cdta->aliaswgt[j] / (fa + fc + fg + ft);
	      suma += wj * fa;
	      sumc += wj * fc;
	      sumg += wj * fg;
	      sumt += wj * ft;
	    }
        }
      sum = suma + sumc + sumg + sumt;
      freqa = suma / sum;
      freqc = sumc / sum;
      freqg = sumg / sum;
      freqt = sumt / sum;
    }

  rdta->freqa = freqa;
  rdta->freqc = freqc;
  rdta->freqg = freqg;
  rdta->freqt = freqt;

  
      
  return;
}

void initReversibleGTR(rawdata *rdta, cruncheddata *cdta)
{
  
  double r[4][4], a[4][4], *initialRates = rdta->initialRates, 
    f[4], e[4], d[4];
    int i, j, k, m;
    
    f[0] = rdta->freqa;
    f[1] = rdta->freqc;
    f[2] = rdta->freqg;
    f[3] = rdta->freqt;

    i = 0;
    for (j = 0; j < 2; j++)
      for (k = j+1; k< 4; k++)
	r[j][k] = initialRates[i++];
    r[2][3] = 1;
    for (j = 0; j < 4; j++) 
	{
	  r[j][j] = 0;
	  for (k = 0; k < j; k++)
	    r[j][k] = r[k][j];
	}
    
    
    rdta->fracchangeav = 0;
    for (j = 0; j<4; j++)
      for (k = 0; k<4; k++)
	rdta->fracchangeav += f[j] * r[j][k] * f[k];
   
    
    
    m = 0;
    
      for(i=0; i<4; i++) 
	a[i][i] = 0;
      
      for(i=0; i<4; i++) 
	{
	  for(j=i+1; j<4; j++) 
	    {
	      double factor = ((m>=5) ? 1 : initialRates[m++]);
	      a[i][j] = a[j][i] = factor * sqrt( f[i] * f[j]);
	      a[i][i] -= factor * f[j];
	      a[j][j] -= factor * f[i];
	    }
	}
      
 

      tred2((double *)a,4,4,d,e);       
      tqli(d, e, 4 , 4, (double *)a);
           
      for(i=0; i<4; i++) 
	{
	  for(j=0; j<4; j++) 
	    {
	      a[i][j] *= sqrt(f[j]);
	    }
	}    


     
      for (i=0; i<4; i++)
	{	  
	  if (d[i] > -1e-8) 
	    {
	      
	      if (i != 0) 
		{		    
		  double tmp = d[i], sum=0;
		  d[i] = d[0];
		  d[0] = tmp;
		  for (j=0; j < 4; j++) 
		    {
		      tmp = a[i][j];
		      a[i][j] = a[0][j];
		      sum += (a[0][j] = tmp);
		    }
		  for (j=0; j < 4; j++) 
		    a[0][j] /= sum;
		}
	      break;
	    }
	}
      for (i=0; i< 4; i++) 
	{
	  rdta->EIGN[0][i] = -d[i];
	  for (j=0; j<4; j++)
	    rdta->EIGV[0][i][j] = a[j][i];
	}
     
      rdta->EIGN[1][0] = rdta->EIGN[0][0];
      rdta->EIGN[1][1] = rdta->EIGN[0][1];
      rdta->EIGN[1][2] = rdta->EIGN[0][2];
      rdta->EIGN[1][3] = rdta->EIGN[0][3];

     
      rdta->matread = TRUE;
      

      for (i = 0; i<4; i++) 
      {
	rdta->invfreq[0][i] = 1 / rdta->EIGV[0][i][0];	
      }


      /*invfreq[4], EIGV[4][4], EIGN[4]*/

}








void alterRates(tree *tr, int k)
{
  int i, increasing;
  double granularity = 0.1/((double)optimizeRatesInvocations);
  double bestLikelihood = tr->likelihood, 
    maxLikelihoodMinus = tr->likelihood, maxLikelihoodPlus = tr->likelihood,
    treeLikelihood, originalRate, maxRateMinus, maxRatePlus;
 
  
  i = 1;
  originalRate = tr->rdta->initialRates[k];
  maxRateMinus = maxRatePlus = originalRate;
  treeLikelihood = bestLikelihood;


  while(treeLikelihood >= maxLikelihoodMinus && (originalRate - granularity * i) > 0)
    {
      tr->rdta->initialRates[k] = originalRate - granularity * i;
      initReversibleGTR(tr->rdta, tr->cdta);
      initrav(tr, tr->start);
      initrav(tr, tr->start->back);
      evaluate(tr, tr->start);      
      treeLikelihood = tr->likelihood;
      if(tr->likelihood > maxLikelihoodMinus)
	{
	  increasing = 1;
	  maxLikelihoodMinus = tr->likelihood;
	  maxRateMinus = originalRate - granularity * i;
	}      
      i++;
    }

  i = 1;
  treeLikelihood = bestLikelihood;
  tr->rdta->initialRates[k] = originalRate;
  while(treeLikelihood >= maxLikelihoodPlus)
    {
      tr->rdta->initialRates[k] = originalRate + i * granularity;
      initReversibleGTR(tr->rdta, tr->cdta);
      initrav(tr, tr->start);
      initrav(tr, tr->start->back);
      evaluate(tr, tr->start);      
      treeLikelihood = tr->likelihood;
      if(tr->likelihood > maxLikelihoodPlus)
	{
	  increasing = 1;
	  maxLikelihoodPlus = tr->likelihood;
	  maxRatePlus = originalRate + granularity * i;
	}      
      i++;
    }
  if(maxLikelihoodPlus > bestLikelihood || maxLikelihoodMinus > bestLikelihood)
    {
      if(maxLikelihoodPlus > maxLikelihoodMinus)
	{	  
	  tr->rdta->initialRates[k] = maxRatePlus;
	  initReversibleGTR(tr->rdta, tr->cdta);
	}
      else
	{
	 tr->rdta->initialRates[k] = maxRateMinus;
	 initReversibleGTR(tr->rdta, tr->cdta);	  
	}     
      
      initrav(tr, tr->start);
      initrav(tr, tr->start->back);
      evaluate(tr, tr->start);     
    }
  else
    {      
      tr->rdta->initialRates[k] = originalRate;
      initReversibleGTR(tr->rdta, tr->cdta);      
      initrav(tr, tr->start);
      initrav(tr, tr->start->back);
      evaluate(tr, tr->start);     
    }

  
}

void optimizeRates(tree *tr)
{
  int i;
  double initialLikelihood, improvedLikelihood;
 
  for(i = 0; i < 5; i++)
    {	      
      alterRates(tr, i);     
    }

  optimizeRatesInvocations++;
}



void printModelAndProgramInfo(tree *tr, analdef *adef, int argc, char *argv[])
{ 
  int i;
  
  infoFile = fopen(infoFileName, "w");

  if(adef->repeats == 0)
    adef->repeats = tr->mxtips;


  switch(adef->mode)
     {   
     case TREE_INFERENCE : 
       printf("RAxML\n\n"); 
       fprintf(infoFile, "RAxML\n\n");
       break;		
     case FAST_MODE : 
       printf("RAxML FAST MODE\n\n");  
       fprintf(infoFile, "RAxML FAST MODE\n\n");
       break;  
     case SIMULATED_ANNEALING : 
       printf("RAxML SIMULATED ANNEALING\n");
       printf("start temperature: %f repeats: %d\n\n",adef->t0, adef->repeats);
       fprintf(infoFile, "RAxML SIMULATED ANNEALING\n");
       fprintf(infoFile, "start temperature: %f repeats: %d\n\n",adef->t0, adef->repeats);
       break;    
     case TREE_EVALUATION : 
       printf("RAxML Model Optimization\n\n");
       fprintf(infoFile, "RAxML Model Optimization\n\n");
       break; 
     case  TREE_RECURSIVE:  
       printf("RAxML DIVIDE & CONQUER MODE\nMAXIMUM SUBPROBLEM SIZE: %d\n\n", adef->maxSubProblemSize);
       fprintf(infoFile, "RAxML DIVIDE & CONQUER MODE\nMAXIMUM SUBPROBLEM SIZE: %d\n\n", adef->maxSubProblemSize);       
       break; 
     case  HEAVY_MODE:
       printf("RAxML hierarchical model optimization mode\n\n"); 
       fprintf(infoFile, "RAxML  hierarchical model optimization mode\n\n");
       break;
     case  BIG_RAPID_MODE:
       printf("RAxML rapid hill-climbing mode for huge data\n\n"); 
       fprintf(infoFile, "RAxML   rapid hill-climbing mode for huge data\n\n");
       break;
     }
    
  switch(adef->model)
    {
    case M_HKY85:
       printf( "HKY85 model of nucleotide substitution\n");  
       printf( "Transition/Transversion parameter will be estimated\n"); 
       fprintf(infoFile, "HKY85 model of nucleotide substitution\n");  
       fprintf(infoFile, "Transition/Transversion parameter will be estimated\n");
       break;      
    case M_HKY85CAT:
      printf( "HKY85 model of nucleotide substitution\n");  
      printf( "Transition/Transversion parameter will be estimated\n");  
      printf( "ML estimate of %d per site rate categories\n", adef->categories);
      fprintf(infoFile, "HKY85 model of nucleotide substitution\n");  
      fprintf(infoFile, "Transition/Transversion parameter will be estimated\n");  
      fprintf(infoFile, "ML estimate of %d per site rate categories\n", adef->categories);
      break;
    case M_GTR:
       printf( "GTR model of nucleotide substitution\n"); 
       printf( "All model parameters will be estimated by RAxML\n");
       fprintf(infoFile, "GTR model of nucleotide substitution\n"); 
       fprintf(infoFile, "All model parameters will be estimated by RAxML\n");
       break;
    case M_GTRCAT:
       printf( "GTR model of nucleotide substitution\n"); 
       printf( "All model parameters will be estimated by RAxML\n");
       printf( "ML estimate of %d per site rate categories\n", adef->categories);
       fprintf(infoFile, "GTR model of nucleotide substitution\n"); 
       fprintf(infoFile,"All model parameters will be estimated by RAxML\n");
       fprintf(infoFile, "ML estimate of %d per site rate categories\n", adef->categories);
       break;
    case M_HKY85GAMMA:
      printf( "HKY85 model of nucleotide substitution\n");  
      printf( "Transition/Transversion parameter will be estimated\n");  
      printf( "GAMMA model of rate heteorgeneity, ML estimate of alpha-parameter\n");
      fprintf(infoFile, "HKY85 model of nucleotide substitution\n");  
      fprintf(infoFile, "Transition/Transversion parameter will be estimated\n");  
      fprintf(infoFile, "GAMMA model of rate heteorgeneity, ML estimate of alpha-parameter\n");
      break;
    case M_GTRGAMMA:
      printf( "GTR model of nucleotide substitution\n");  
      printf( "All model parameters will be estimated by RAxML\n");  
      printf( "GAMMA model of rate heteorgeneity, ML estimate of alpha-parameter\n");
      fprintf(infoFile, "GTR model of nucleotide substitution\n");  
      fprintf(infoFile, "All model parameters will be estimated by RAxML\n");  
      fprintf(infoFile, "GAMMA model of rate heterogeneity, ML estimate of alpha-parameter\n");
      break;

    }
  if(adef->model == M_PROTEINS || adef->model == M_PROTEINSCAT)
    {
      switch(adef->protmodel)
	{
	case M_JTT:
	  printf( "JTT model of amino acid substitution\n");
	  fprintf(infoFile, "JTT model of amino acid substitution\n");  
	  break;
	case M_PAM:
	  printf( "PAM model of amino acid substitution\n");  
	  fprintf(infoFile, "PAM model of amino acid substitution\n");
	  break;
	case M_PMB:
	  printf( "PMB model of amino acid substitution\n");  
	  fprintf(infoFile, "PMB model of amino acid substitution\n");  
	  break;
	}
      if(adef->model == M_PROTEINSCAT)
	{
	   printf( "ML estimate of %d per site rate categories\n", adef->categories);
	   fprintf(infoFile, "ML estimate of %d per site rate categories\n", adef->categories);
	}


      printf("Protein Base Frequencies:\n");
      for(i = 0; i < 20; i++)
	printf("%f ",   tr->rdta->freqaa[i]);
      printf("\n");  
      
      printf("\n\n");
      
      fprintf(infoFile, "Protein Base Frequencies:\n");
      for(i = 0; i < 20; i++)
	fprintf(infoFile, "%f ",   tr->rdta->freqaa[i]);
      fprintf(infoFile, "\n"); 
      
      
      fprintf(infoFile, "\n\n");
      
      fprintf(infoFile,"RAxML was called as follows:\n\n");
      for(i = 0; i < argc; i++)
	fprintf(infoFile,"%s ", argv[i]);
      fprintf(infoFile,"\n\n\n");
    }
  else
    {
      printf("Empirical Base Frequencies:\n");
      
      printf("pi(A): %f ",   tr->rdta->freqa);
      printf("pi(C): %f ",   tr->rdta->freqc);
      printf("pi(G): %f ",   tr->rdta->freqg);
      printf("pi(T): %f \n",   tr->rdta->freqt);  
      
      printf("\n\n");
      
      fprintf(infoFile, "Empirical Base Frequencies:\n");
      
      fprintf(infoFile, "pi(A): %f ",   tr->rdta->freqa);
      fprintf(infoFile, "pi(C): %f ",   tr->rdta->freqc);
      fprintf(infoFile, "pi(G): %f ",   tr->rdta->freqg);
      fprintf(infoFile, "pi(T): %f \n",   tr->rdta->freqt);  
      
      fprintf(infoFile, "\n\n");
      
      fprintf(infoFile,"RAxML was called as follows:\n\n");
      for(i = 0; i < argc; i++)
	fprintf(infoFile,"%s ", argv[i]);
      fprintf(infoFile,"\n\n\n");
    }

  fclose(infoFile);
}








void checkTime(analdef *adef, tree *tr)
{
  if(adef->timeLimit == 0) return;
  {
    double delta = gettime() - masterTime;
    if(delta > adef->timeLimit)
      {
	printf("time limit exceeded ... exiting with Lh %f at time %f\n", 
		tr->likelihood, delta);
	treeEvaluate(tr, 4);
	printResult(tr, 0);
	exit(1);
      }
  }
}


/* start of Divide and Conquer methods */


void treeOptimizeSubtree(tree *tr, int _mintrav, int _maxtrav, int initial, subNodes *sn)
{
  int i;
  int mintrav = _mintrav;
  int maxtrav = _maxtrav;
  int subsequent = 0;

  if (maxtrav > tr->ntips - 3)  
    maxtrav = tr->ntips - 3;
  
  tr->opt_level = 0;
  
  resetBestTree(bt);
  saveBestTree(bt, tr);

 

  /*fprintf(stderr," treeOptimizeSubtree() %d %d %d %d\n", mintrav, maxtrav, Multiple, Thorough);*/

  tr->startLH = tr->endLH = tr->likelihood;


  for(i = 0; i < sn->count; i++)
    {           
      rearrangeParsimony(tr, sn->nodeArray[i], mintrav, maxtrav);
           
      if(Multiple && (tr->endLH > tr->startLH))                 	
	{	  
	  recallBestTree(bt, 1, tr);
	  /*fprintf(stderr,"Tree improved %f\n", tr->likelihood);*/
	  tr->startLH = tr->endLH = tr->likelihood;
	  subsequent = 1;
	}	
    }
 
  
  Multiple = subsequent;
  return;     
}





void computeSubtree (tree *tr, analdef *adef, subNodes *sn) 
  { 
    int i, rearrangementsMax, rearrangementsMin, noImproveCount, impr;                
    double lh, previousLh, difference, epsilon;
    bestlist *bestT;


    bestT = (bestlist *) malloc(sizeof(bestlist));
    bestT->ninit = 0;
    initBestTree(bestT, 1, tr->mxtips, tr->cdta->endsite);
  
    difference = 10.0;
    epsilon = 0.001;
    
    Multiple = 0;    
        
    resetBestTree(bt);            
    lh = tr->likelihood;    

   
    saveBestTree(bestT, tr);  
   
    impr = 1;		  		  
    
    while(1)
      {		
	recallBestTree(bestT, 1, tr);
	treeEvaluate(tr, 2);
	lh = tr->likelihood;

     	 
	if(impr)
	{
	    rearrangementsMin = 1;
	    rearrangementsMax = 5;    
	}
	else
	{
	  
	  if(rearrangementsMax < 10)
	    {
	      rearrangementsMin = 6;
	      rearrangementsMax = 10;			
	    }
	  else
	    goto cleanup; 	   	    
	}

	

	treeOptimizeSubtree(tr, rearrangementsMin, rearrangementsMax, adef->initial, sn);
	
	impr = 0;			      	
	previousLh = lh;
	for(i = 1; i <= bt->nvalid; i++)
	  {	    		  	   
	    recallBestTree(bt, i, tr);
	    /*fprintf(stderr,"%d %f\n", i, tr->likelihood);*/
	    treeEvaluate(tr, 0.25);
	    /*fprintf(stderr,"%d %f\n\n", i, tr->likelihood);*/
	    difference = ((tr->likelihood > previousLh)? 
			  tr->likelihood - previousLh: 
			  previousLh - tr->likelihood); 	    
	    if(tr->likelihood > lh && difference > epsilon)
	      {
		impr = 1;	       
		lh = tr->likelihood;	       
		saveBestTree(bestT, tr);		
	      }	   	   
	  }	
      }
    
  cleanup: 
    freeBestTree(bestT);
  }


void computeSubtreeReconnect (tree *tr, analdef *adef, subNodes *sn, int areaSize, int algoStage) 
  { 
    int i, rearrangementsMax, rearrangementsMin, noImproveCount, impr;                
    double lh, previousLh, difference, epsilon;
    bestlist *bestT;


    bestT = (bestlist *) malloc(sizeof(bestlist));
    bestT->ninit = 0;
    initBestTree(bestT, 1, tr->mxtips, tr->cdta->endsite);
  
    difference = 10.0;
    epsilon = 0.001;
    
    Multiple = 1;
    
        
    resetBestTree(bt);            
    lh = tr->likelihood;    

 
    saveBestTree(bestT, tr);  
   
    impr = 1;		  		  
    rearrangementsMin = 1;
    rearrangementsMax = areaSize;

    while(1)
      {
	recallBestTree(bestT, 1, tr);
	treeEvaluate(tr, 2);
	lh = tr->likelihood;
	/*fprintf(stderr,"%f\n", lh);*/

     	 
	if(!impr)
	  {	    
	    goto cleanup; 	   
	  }
	
	treeOptimizeSubtree(tr, rearrangementsMin, rearrangementsMax, adef->initial, sn);
	
	impr = 0;			      	
	previousLh = lh;
		    		  	   
	if(algoStage == 1)
	  {
	    for(i = 1; i <= bt->nvalid; i++)
	      {	    		  	   
		recallBestTree(bt, i, tr);		
		treeEvaluate(tr, 0.25);
		/*fprintf(stderr,"%d %f\n\n", i, tr->likelihood);*/
		difference = ((tr->likelihood > previousLh)? 
			      tr->likelihood - previousLh: 
			      previousLh - tr->likelihood); 	    
		if(tr->likelihood > lh && difference > epsilon)
		  {
		    impr = 1;	       
		    lh = tr->likelihood;	       
		    saveBestTree(bestT, tr);		
		  }	   	   
	      }	
	  }
	else
	  {
	    recallBestTree(bt, 1, tr);
	    treeEvaluate(tr, 1);

	    difference = ((tr->likelihood > previousLh)? 
			  tr->likelihood - previousLh: 
			  previousLh - tr->likelihood); 

	    if(tr->likelihood > lh && difference > epsilon)
	      {
		impr = 1;	       
		lh = tr->likelihood;	       
		saveBestTree(bestT, tr);		
	      }	 

	  }


	


  	   
      }
    
  cleanup: 
    freeBestTree(bestT);
  }

void computeFastSubtree (tree *tr, analdef *adef, subNodes *sn) 
{ 
  int i, mintrav, maxtrav, which;
  double initialLH, epsilon, diff;
 

  epsilon = 0.01;
  Multiple = 1;
  mintrav = 1;
  maxtrav = 10;
  
  if (maxtrav > tr->ntips - 3)  maxtrav = tr->ntips - 3;
          
  tr->opt_level = 0; 
  resetBestTree(bt);
  saveBestTree(bt, tr);
  /*fprintf(stderr,"computeFastSubtree() %d %d %d %d\n", mintrav, maxtrav, Multiple, Thorough);*/
  do
    {   
      /*fprintf(stderr,"%f\n", tr->likelihood);*/
      initialLH = tr->likelihood;
      tr->startLH = tr->endLH = tr->likelihood;

      for(i = 0; i < sn->count; i++)
	{ 
	  
	  rearrangeParsimony(tr, sn->nodeArray[i], mintrav, maxtrav);
	  if(tr->endLH > tr->startLH)   
	    {
	      recallBestTree(bt, 1, tr);
	      tr->startLH = tr->endLH = tr->likelihood;		    
	    }
	 
	}
      recallBestTree(bt, 1, tr);      
      treeEvaluate(tr, 1);
      checkTime(adef, tr);
      saveBestTree(bt, tr);
      diff = (tr->likelihood > initialLH)? tr->likelihood - initialLH: -1.0;        
    }
  while(diff > epsilon);

  initialLH = unlikely;
   	    	
  recallBestTree(bt, 1, tr);			       
  treeEvaluate(tr, 2); 
}

void computeRapidSubtree (tree *tr, analdef *adef, subNodes *sn) 
{ 
  int i, mintrav, maxtrav, which;
  double initialLH, epsilon, diff;
 

  epsilon = 0.01;
  Multiple = 1;
  mintrav = 1;
  maxtrav = 5;
  
  if (maxtrav > tr->ntips - 3)  maxtrav = tr->ntips - 3;
          
  tr->opt_level = 0; 
  resetBestTree(bt);
  saveBestTree(bt, tr);

  fprintf(stderr,"computeRapidSubtree() %d %d %d %d\n", mintrav, maxtrav, Multiple, Thorough);
   
  fprintf(stderr,"%f\n", tr->likelihood);
  initialLH = tr->likelihood;
  tr->startLH = tr->endLH = tr->likelihood;

  for(i = 0; i < sn->count; i++)
    { 
	  
      rearrangeParsimony(tr, sn->nodeArray[i], mintrav, maxtrav);
      if(tr->endLH > tr->startLH)   
	{
	  recallBestTree(bt, 1, tr);
	  tr->startLH = tr->endLH = tr->likelihood;		    
	}	 
    }

  recallBestTree(bt, 1, tr);      
  treeEvaluate(tr, 2); 
  fprintf(stderr,"%f\n", tr->likelihood);
}



void getNodes(tree *tr, nodeptr p,subNodes *sn)
{
  nodeptr q;

  sn->nodeArray[sn->count] = p;
  sn->count++;

  if(p->tip)          
    return;

  q = p->next;
  while(q != p)
    {
      getNodes(tr, q->back, sn);
      q = q->next;
    }

  return;
}


int countTips(tree *tr, nodeptr p)
{
  nodeptr q;
  int acc = 0;

  if(p->tip)
    return 1;

  q = p->next;
  while(q != p)
    {
      acc += countTips(tr, q->back);
      q = q->next;
    }

  return acc;
}


int makeArea(nodeptr p, int max, subNodes *sn)
{
  nodeptr q;
  int acc = 0;

  if(max == 0)
    return 0;

  sn->nodeArray[sn->count] = p;
  sn->count++;

  if(p->tip)     
    return 1;  

  q = p->next;
  acc = 1;
  while(q != p)
    {	
      acc += makeArea(q->back, max - 1, sn);
      q = q->next;
    }

  return acc;
}


static int BTypeCompare(const void *p1, const void *p2)
{
 BType *rc1 = (BType *)p1;
 BType *rc2 = (BType *)p2;

 int i = rc1->difference;
 int j = rc2->difference;
  
  if (i > j)
    return (1);
  if (i < j)
    return (-1);
  return (0);
}



int bisectNode(tree *tr, subNodes *sn)
{
  int i, leftTips, rightTips, balance, difference, result, sum;
  double zMax;
  nodeptr p, q;
  
  balance = 2 * tr->mxtips;

  for (i = 0; i < sn->count; i++) 
    {
      p = sn->nodeArray[i];
     
      q = p->back;
     
      if((!p->tip) && (!q->tip))
	{
	  if(q->back != p)
	    {
	      fprintf(stderr,"Bad rear at %d\n", i);
	      exit(1);
	    }
	  
	  leftTips = countTips(tr, p);
	  rightTips = countTips(tr,q);     
	  difference = ABS(leftTips - rightTips);

	  if(balance > difference)
	    {
	      balance = difference; 
	      result = p->number;
	    }	
	}
      
    }
  
  return result;
}






double reconnect(tree *tr, nodeptr p, nodeptr q, nodeptr p1, nodeptr q1, subNodes *sn, int algoStage)
{
  nodeptr p2, q2;
  int areaSize;

  p2 = p1->back;
  if(p2->back != p1)
    {
      fprintf(stderr,"Bad rear\n");
      exit(1);
    }

  p->next->back = p1; 
  p1->back = p->next;

  p->next->z = p->next->back->z = defaultz;

  p->next->next->back = p2;
  p2->back = p->next->next;

  p->next->next->z = p->next->next->back->z = defaultz;
  
  q2 = q1->back;
  if(q2->back != q1)
    {
      fprintf(stderr,"Bad rear\n");
      exit(1);
    }
  
  q->next->back = q1; 
  q1->back = q->next;

  q->next->z = q->next->back->z = defaultz;

  q->next->next->back = q2;
  q2->back = q->next->next;

  q->next->next->z = q->next->next->back->z = defaultz;
  
  tr->start = p;  

  initrav(tr, p);  
  initrav(tr, q);
  evaluate(tr, p); 
   
  sn->count = 0;
 
  /*areaSize = makeArea(p, 3, sn) + makeArea(q, 3, sn);*/

  makeArea(p1, 4, sn);
  makeArea(p2, 4, sn);
  makeArea(q1, 4, sn);
  makeArea(q2, 4, sn);
  makeArea(p, 4, sn);
  makeArea(q, 4, sn);
  computeSubtreeReconnect(tr,adef, sn, 10, algoStage);
  /* 10 or areaSize*/
 
     
  return tr->likelihood;
}




void recursiveAnalysis(tree *tr, analdef *adef, subNodes *sn, int algoStage)
{
  nodeptr p, q;
  int leftTips, rightTips, subtreeSize, splitNumber;
  p = tr->start;
  q = p->back;

  leftTips = countTips(tr, p);
  rightTips = countTips(tr,q);

  subtreeSize = leftTips + rightTips;

  if(subtreeSize > adef->maxSubProblemSize)
    {
      nodeptr p1, p2, q1, q2;
      subNodes *sn2;
      bestlist *T1, *T2;
      int areaSize;

      T1 = (bestlist *) malloc(sizeof(bestlist));
      T2 = (bestlist *) malloc(sizeof(bestlist));
      T1->ninit = 0;
      T2->ninit = 0;
      initBestTree(T1, 1, tr->mxtips, tr->cdta->endsite);
      initBestTree(T2, 1, tr->mxtips, tr->cdta->endsite);
      sn2 = (subNodes *)malloc(sizeof(subNodes));

      sn2->count = 0;
      sn2->nodeArray = (node **)malloc(2 * tr->mxtips * sizeof(node *));

     
      splitNumber = bisectNode(tr, sn);

      p = tr->nodep[splitNumber];
     
      q = p->back; 
      
      if(p->tip || q->tip)
	{
	  fprintf(stderr,"recursiveAnalysis split at tip, doing subtree inference instead\n");
	  freeBestTree(T1);
	  freeBestTree(T2);
	  free(sn2->nodeArray);
	  free(sn2);  
	  goto subtreeInference;
	}

      if(q->back->number != p->number) 
	{
	  fprintf(stderr,"recursiveAnalysis bad rear\n");
	  exit(-1);
	}

      p1 = p->next->back;
      p2 = p->next->next->back;
      hookup(p1, p2, defaultz);
      initrav(tr, p1); 
      initrav(tr, p2);
      evaluate(tr, p1);
      tr->start = p1;
      evalFast(tr);    
      saveBestTree(T1, tr);
  
      q1 = q->next->back;
      q2 = q->next->next->back;
      hookup(q1, q2, defaultz);
      initrav(tr, q1);  
      initrav(tr, q2);
      evaluate(tr, q1); 
      tr->start = q1;
      evalFast(tr);      
      saveBestTree(T2, tr);

      recallBestTree(T1, 1, tr);
      sn2->count = 0;
      getNodes(tr, p1, sn2);
      getNodes(tr, p2, sn2);   
      recursiveAnalysis(tr, adef, sn2, algoStage);
      saveBestTree(T1, tr);
    
      recallBestTree(T2, 1, tr);
      sn2->count = 0;
      getNodes(tr, q1, sn2);
      getNodes(tr, q2, sn2);
      recursiveAnalysis(tr, adef, sn2, algoStage);
      saveBestTree(T2, tr);

      recallBestTreeRecursive(T1, 1, tr);
      recallBestTreeRecursive(T2, 1, tr);

      initrav(tr, p1);  
      initrav(tr, p1->back);
      evaluate(tr, p1); 
 
      initrav(tr, q1);  
      initrav(tr, q1->back);
      evaluate(tr, q1); 
  
      
      
      hookup(q, p, defaultz);  
      tr->start = p;
      
    
      sn2->count = 0;
      
     
      reconnect(tr, p, q, p1, q1, sn2, algoStage);


      freeBestTree(T1);
      freeBestTree(T2);
      free(sn2->nodeArray);
      free(sn2);   
    }
  else
    { 
    subtreeInference:
       computeSubtree(tr, adef, sn);
      /*computeFastSubtree(tr, adef, sn);*/

      /*switch(algoStage)
	{
	case 0: 	  
	  computeFastSubtree(tr, adef, sn);
	  break;
	case 1: 
	  computeSubtree(tr, adef, sn);
	  break;
	default: 
	  fprintf(stderr,"fatal error\n"); 
	  exit(1);
	  }*/

    }
}

void computeRecursive2 (tree *tr, analdef *adef) 
{
  int lc, i, algoStage;      
  subNodes *sn;
  double startLH, endLH;
 
  sn = (subNodes *)malloc(sizeof(subNodes)); 
  sn->nodeArray = (node **)malloc(2 * tr->mxtips * sizeof(node *));
 
  if(adef->model == M_PROTEINS || adef->model == M_PROTEINSCAT)
    Thorough = 1;
  else
    Thorough = 0;

  algoStage = 0;
  lc = 10;

  while(lc > 0)
    {
      optimizeModel(tr, adef, 1);	  
      treeEvaluate(tr, 2);    
      printLog(tr, adef);
      printResult(tr, 0);
      startLH = tr->likelihood;
           
      sn->count = 0;
      getNodes(tr, tr->start, sn);
      getNodes(tr, tr->start->back, sn);
      recursiveAnalysis(tr, adef, sn, algoStage);
      
      endLH = tr->likelihood;
     
	  
     
      if(lc < 8)
	algoStage = 1;
      fprintf(stderr,"Algo stage of next it %d\n", algoStage);
      Thorough = 1;
      lc--;
    }

}


void computeRecursive (tree *tr, analdef *adef) 
{
  int lc, i, algoStage;      
  subNodes *sn;
  double startLH, endLH;
 
  sn = (subNodes *)malloc(sizeof(subNodes)); 
  sn->nodeArray = (node **)malloc(2 * tr->mxtips * sizeof(node *));
 
  if(adef->model == M_PROTEINS || adef->model == M_PROTEINSCAT)
    Thorough = 1;
  else
    Thorough = 0;

  algoStage = 0;
  lc = 10;

 
  optimizeModel(tr, adef, 1);	  
  treeEvaluate(tr, 2);    
  printLog(tr, adef);
  printResult(tr, 0);
  startLH = tr->likelihood;           
  sn->count = 0;
  getNodes(tr, tr->start, sn);
  getNodes(tr, tr->start->back, sn);
  recursiveAnalysis(tr, adef, sn, algoStage);
     	            
  Thorough = 1;
  optimizeModel(tr, adef, 1);	  
  treeEvaluate(tr, 2);    
  printLog(tr, adef);
  printResult(tr, 0);
  startLH = tr->likelihood;           
  sn->count = 0;
  getNodes(tr, tr->start, sn);
  getNodes(tr, tr->start->back, sn);
  recursiveAnalysis(tr, adef, sn, algoStage);

  exit(1);
  /* TODO */

  compute(tr, adef);
  

}






/* end of Divide and Conquer methods */



void computePreSelect (tree *tr, analdef *adef) 
  { 
    int i, rearrangementsMax, rearrangementsMin, noImproveCount, impr;                
    double lh, previousLh, difference, epsilon;
    bestlist *bestT;

    bestT = (bestlist *) malloc(sizeof(bestlist));
    bestT->ninit = 0;
    initBestTree(bestT, 1, tr->mxtips, tr->cdta->endsite);
  
    tr->nodeScoresOne = (preScore *)malloc(sizeof(preScore) * 2 * tr->mxtips);
    tr->nodeCount = (int *)malloc(sizeof(int) * 2 * tr->mxtips);
    for(i = 0; i < 2 * tr->mxtips; i++)
      tr->nodeCount[i] = 0;

    difference = 10.0;
    epsilon = 0.001;
    
    Multiple = 1;
    
    Thorough = 0;  
        
    if(adef->model == M_PROTEINS || adef->model == M_PROTEINSCAT)
      {
	Thorough = 1;       
	epsilon = 0.1;
      }


    resetBestTree(bt);            
    lh = tr->likelihood;    

 
    saveBestTree(bestT, tr);  
   
    impr = 1;		  		  
 
    while(1)
      {	


	recallBestTree(bestT, 1, tr);		
	optimizeModel(tr, adef, 1);       
	treeEvaluate(tr, 2);		
	
	lh = tr->likelihood;
	saveBestTree(bestT, tr);
	printLog(tr, adef);	
       
	checkTime(adef, tr);
     	        
	if(impr)
	  {	 	   
	    noImproveCount = 0;
	    printResult(tr, 0);	   
	    rearrangementsMin = 1;
	    rearrangementsMax = adef->stepwidth;	    
	  }			  			
	else
	  {		       
	    if(Thorough == 0)
	      { 		
		Thorough = 1;
		rearrangementsMax = adef->stepwidth;
		rearrangementsMin = 1; 
	      }
	    else
	      { 
		rearrangementsMax += adef->stepwidth;
		rearrangementsMin += adef->stepwidth; 
	      }	    	      
	    if(rearrangementsMax > adef->max_rearrange)	     	     	 
	      goto cleanup; 	   
	  }

	

	treeOptimizePreSelect(tr, rearrangementsMin, rearrangementsMax, adef);
	
	impr = 0;			      	
	previousLh = lh;

	checkTime(adef, tr);
	
	for(i = 1; i <= bt->nvalid; i++)
	  {	    		  	   
	    recallBestTree(bt, i, tr);	    
	    treeEvaluate(tr, 0.25);	    
	    /*printf("%d %f\n", i, tr->likelihood);*/
	    difference = ((tr->likelihood > previousLh)? 
			  tr->likelihood - previousLh: 
			  previousLh - tr->likelihood); 	    
	    if(tr->likelihood > lh && difference > epsilon)
	      {
		impr = 1;	       
		lh = tr->likelihood;	       
		saveBestTree(bestT, tr);		
	      }	   	   
	  }
	

      }
    
  cleanup: 
    recallBestTree(bestT, 1, tr);
    printLog(tr, adef);
    printResult(tr, 0);	   
    freeBestTree(bestT);
  }






double distanceML(tree *tr, int ip, int jp)
{

#ifndef HPC
  likelivector *lp, *lq;
  double  *abi, *bci, *sumci, *abptr, *bcptr, *sumcptr;
  double   dlnLidlz, dlnLdlz, d2lnLdlz2, z, zprev, zstep, lz, xvlz,
    ki, suma, sumb, sumc, ab, bc, inv_Li, t1, t2,
    fx1a, fx1c, fx1g, fx1t, fx1r, fx1y, fx2r, fx2y;
  double  zz, zv;
  double   *wrptr, *wr2ptr;
  int      i, curvatOK;
  homType *mlvP, *mlvQ;
  nodeptr realQ, realP;
  register  int *sQ; 
  register  int *sP;
  long refQ, refP;  
  int maxiter = 10;
  double z0 = defaultz;
  nodeptr p, q;

 

  p = tr->nodep[ip];
  q = tr->nodep[jp];

 

  { 
    unsigned scratch_size;
    scratch_size = sizeof(double) * tr->cdta->endsite;
    abi   = (double *) malloc(scratch_size);
    bci   = (double *) malloc(scratch_size);
    sumci = (double *) malloc(scratch_size);      
  }      
       
  abptr   = abi;
  bcptr   = bci;
  sumcptr = sumci;
  realQ =  tr->nodep[q->number];
  realP =  tr->nodep[p->number];
  mlvP = realP->mlv;
  mlvQ = realQ->mlv;
  sQ = realQ->equalityVector;
  sP = realP->equalityVector;    
     
  for (i = 0; i < tr->cdta->endsite; i++) 
    {
      refQ = *sQ++;
      refP = *sP++;
      
      mlvP = &realP->mlv[refP];
      mlvQ = &realQ->mlv[refQ];
      fx1a = tr->rdta->freqa * mlvP->a;
      fx1g = tr->rdta->freqg * mlvP->g;
      fx1c = tr->rdta->freqc * mlvP->c;
      fx1t = tr->rdta->freqt * mlvP->t;
      suma = fx1a * mlvQ->a + fx1c * mlvQ->c + fx1g * mlvQ->g + fx1t * mlvQ->t;
      fx2r = tr->rdta->freqa * mlvQ->a + tr->rdta->freqg * mlvQ->g;
      fx2y = tr->rdta->freqc * mlvQ->c + tr->rdta->freqt * mlvQ->t;
      fx1r = fx1a + fx1g;
      fx1y = fx1c + fx1t;
      *sumcptr++ = sumc = (fx1r + fx1y) * (fx2r + fx2y);
      sumb       = fx1r * fx2r * tr->rdta->invfreqr + fx1y * fx2y * tr->rdta->invfreqy;
      *abptr++   = suma - sumb;
      *bcptr++   = sumb - sumc;		        	          
    }

   


    z = z0;
    do {
      zprev = z;
      zstep = (1.0 - zmax) * z + zmin;
      curvatOK = FALSE;

      do {
        if (z < zmin) z = zmin;
        else if (z > zmax) z = zmax;

        lz    = log(z);
        xvlz  = tr->rdta->xv * lz;         
        
	zz = exp(lz);
	zv = exp(xvlz);
          
	abptr   = abi;
	bcptr   = bci;
	sumcptr = sumci;        
	wrptr   = &(tr->cdta->wr[0]);
	wr2ptr  = &(tr->cdta->wr2[0]);
	dlnLdlz = 0.0;                 /*  = d(ln(likelihood))/d(lz) */
	d2lnLdlz2 = 0.0;               /*  = d2(ln(likelihood))/d(lz)2 */



        for (i = 0; i < tr->cdta->endsite; i++) {
         
          ab     = *abptr++ * zz;
          bc     = *bcptr++ * zv;
          sumc   = *sumcptr++;
          inv_Li = 1.0/(ab + bc + sumc);
          t1     = ab * inv_Li;
          t2     = tr->rdta->xv * bc * inv_Li;
          dlnLidlz   = t1 + t2;
          dlnLdlz   += *wrptr++  * dlnLidlz;
          d2lnLdlz2 += *wr2ptr++ * (t1 + tr->rdta->xv * t2 - dlnLidlz * dlnLidlz);
          }



        if ((d2lnLdlz2 >= 0.0) && (z < zmax))
	  {
	    zprev = z = 0.37 * z + 0.63;  /*  Bad curvature, shorten branch */
	   
	  }
        else
          curvatOK = TRUE;

        } while (! curvatOK);

      if (d2lnLdlz2 < 0.0) {
        z *= exp(-dlnLdlz / d2lnLdlz2);
        if (z < zmin) z = zmin;
        if (z > 0.25 * zprev + 0.75)    /*  Limit steps toward z = 1.0 */
          z = 0.25 * zprev + 0.75;
        }

      if (z > zmax) z = zmax;

      } while ((--maxiter > 0) && (ABS(z - zprev) > zstep));

    free(abi);
    free(bci);
    free(sumci);
    

    /*printf("D %d <-> %d : %f\n", ip, jp, z);*/
    
    return  z;
#endif
#ifdef HPC
    return 0.0;
#endif
}




void makeDistMat(tree *tr, analdef *adef) 
{
  int i, j, k, n;
  double start_time;

  n = tr->mxtips + 1;
  k = 0;

  start_time = gettime();

  for(i = 1; i < n; i++)    
    {
      for(j = 1; j < i; j++)      
	{
	  tr->dist[k] = distanceML(tr, i, j);      
	  k++;
	}
    }

  printf("Time: %f\n", gettime() - start_time);

  printf("Indices required: %d and %d bytes\n", k, k * 4);
}


/*double subtreeDistance(tr *tree, nodpetr p, nodeptr q)
{
  int np;

  np = markNodes(tr->nodes1, p);

  nq = markNodes(tr->nodes1, q);
  


  while((pp = p->next) != p)
    {
      
    }


  while((qq = q->next) != q)
    {

    }


    }*/

 void computeSPR (tree *tr, analdef *adef) 
{
  /*int n = tr->mxtips + 1;

  n *= n;
  n = n / 2;

  tr->dist = (double *)malloc(sizeof(double) * n);

  makeDistMat(tr, adef);

  subtreeDistance(

  exit(1);*/
  }


void compute (tree *tr, analdef *adef) 
  { 
    int i, rearrangementsMax, rearrangementsMin, noImproveCount, impr;                
    double lh, previousLh, difference, epsilon;
    bestlist *bestT;

    bestT = (bestlist *) malloc(sizeof(bestlist));
    bestT->ninit = 0;
    initBestTree(bestT, 1, tr->mxtips, tr->cdta->endsite);
  
    difference = 10.0;
    epsilon = 0.001;
    
    Multiple = 1;
    
    Thorough = 0;  
        
    if(adef->model == M_PROTEINS || adef->model == M_PROTEINSCAT)
      {
	Thorough = 1;       
	epsilon = 0.1;
      }


    resetBestTree(bt);            
    lh = tr->likelihood;    

 
    saveBestTree(bestT, tr);  
   
    impr = 1;		  		  
 
    while(1)
      {	


	recallBestTree(bestT, 1, tr);		
	optimizeModel(tr, adef, 1);       
	treeEvaluate(tr, 2);		
	
	lh = tr->likelihood;
	saveBestTree(bestT, tr);
	printLog(tr, adef);	
       
	checkTime(adef, tr);
     	        
	if(impr)
	  {	 	   
	    noImproveCount = 0;
	    printResult(tr, 0);	   
	    rearrangementsMin = 1;
	    rearrangementsMax = adef->stepwidth;	    
	  }			  			
	else
	  {		       
	    if(Thorough == 0)
	      { 		
		Thorough = 1;
		rearrangementsMax = adef->stepwidth;
		rearrangementsMin = 1; 
	      }
	    else
	      { 
		rearrangementsMax += adef->stepwidth;
		rearrangementsMin += adef->stepwidth; 
	      }	    	      
	    if(rearrangementsMax > adef->max_rearrange)	     	     	 
	      goto cleanup; 	   
	  }

	

	treeOptimize(tr, rearrangementsMin, rearrangementsMax, adef);
	
	impr = 0;			      	
	previousLh = lh;

	checkTime(adef, tr);
	
	for(i = 1; i <= bt->nvalid; i++)
	  {	    		  	   
	    recallBestTree(bt, i, tr);	    
	    treeEvaluate(tr, 0.25);	    

	    /*printf("%d %f nodemNUm %d\n", i, tr->likelihood,  bt->byScore[i]->number);
	      fprintf(logFile, "%d %f NodeNUM %d\n", i, tr->likelihood, bt->byScore[i]->number);*/

	    difference = ((tr->likelihood > previousLh)? 
			  tr->likelihood - previousLh: 
			  previousLh - tr->likelihood); 	    
	    if(tr->likelihood > lh && difference > epsilon)
	      {
		impr = 1;	       
		lh = tr->likelihood;	       
		saveBestTree(bestT, tr);		
	      }	   	   
	  }
	

      }
    
  cleanup: 
    recallBestTree(bestT, 1, tr);
    printLog(tr, adef);
    printResult(tr, 0);	   
    freeBestTree(bestT);
  }

/********************** COMPUTE BIG ***************************************************************************/






static int infoCompare(const void *p1, const void *p2)
{
 bestInfo *rc1 = (bestInfo *)p1;
 bestInfo *rc2 = (bestInfo *)p2;

 int i = rc1->node;
  int j = rc2->node;
  
  if (i > j)
    return (1);
  if (i < j)
    return (-1);
  return (0);
}


void sortInfoList()
{

  qsort(&(iList.list[0]), iList.n, sizeof(bestInfo), infoCompare);
}

void resetInfoList()
{
  int i;

  for(i = 0; i < iList.n; i++)
    {
      iList.list[i].node = 0;
      iList.list[i].likelihood = unlikely;
    }
}

void initInfoList(int n)
{
  int i;

  iList.n = n;
  iList.valid = 0;
  iList.list = (bestInfo *)malloc(sizeof(bestInfo) * n);

  for(i = 0; i < n; i++)
    {
      iList.list[i].node = 0;
      iList.list[i].likelihood = unlikely;
    }
}

void insertInfoList(int node, double likelihood)
{
  int i;
  int min = 0;
  double min_l =  iList.list[0].likelihood;

  for(i = 1; i < iList.n; i++)
    {
      if(iList.list[i].likelihood < min_l)
	{
	  min = i;
	  min_l = iList.list[i].likelihood;
	}
    }

  if(likelihood > min_l)
    {
      iList.list[min].likelihood = likelihood;
      iList.list[min].node = node;
    }
}

void printInfoList()
{
  int i;
  
  for(i = 0; i < iList.n; i++)
    {
      printf("L : %f Node : %d\n", iList.list[i].likelihood, iList.list[i].node);
    }
  
}


boolean smoothRegion (tree *tr, nodeptr p, int region)
  { /* smooth */
    nodeptr  q;
   
    if (! update(tr, p))               return FALSE; /*  Adjust branch */
    if(region > 0)
      {
	if (! p->tip) 
	  {                                 
	    q = p->next;
	    while (q != p) 
	      {
		if (! smoothRegion(tr, q->back, --region))   return FALSE;
		q = q->next;
	      }	
	    if (! newview(tr, p)) return FALSE;	
	  }
      }
    
    return TRUE;
  } /* smooth */

boolean regionalSmooth (tree *tr, nodeptr p, int maxtimes, int region)
  {
    nodeptr  q;

    if (p->tip) return FALSE;            /* Should be an error */

    while (--maxtimes >= 0) 
      {
	tr->smoothed = TRUE;
	q = p;
	do 
	  {
	    if (! smoothRegion(tr, q, region)) return FALSE;
	    q = q->next;
	  } 
	while (q != p);
	if (tr->smoothed)  break;
      }

    tr->smoothed = FALSE;             /* Only smooth locally */
    return TRUE;
  } /* localSmooth */



nodeptr  removeNodeBIG (tree *tr, nodeptr p)
{
  double   zqr;
  nodeptr  q, r;
        
  q = p->next->back;
  r = p->next->next->back;
  zqr = q->z * r->z;
       
  if ((zqr = makenewz(tr, q, r, zqr, iterations)) == badZ) return (node *) NULL;      

  tr->zqr = zqr;
    
  hookup(q, r, zqr);

  p->next->next->back = p->next->back = (node *) NULL;
     
  return  q;
}

nodeptr  removeNodeRestoreBIG (tree *tr, nodeptr p)
{
  nodeptr  q, r;
        
  q = p->next->back;
  r = p->next->next->back;  

  newview(tr, q);
  newview(tr, r);
  
  hookup(q, r, tr->currentZQR);

  p->next->next->back = p->next->back = (node *) NULL;
     
  return  q;
}


boolean insertBIG (tree *tr, nodeptr p, nodeptr q, boolean glob)
{
  nodeptr  r, s;
  
  r = q->back;
  s = p->back;

  if(Thorough)
    { 
      double  zqr, zqs, zrs, lzqr, lzqs, lzrs, lzsum, lzq, lzr, lzs, lzmax;      
            
      if ((zqr = makenewz(tr, q, r, q->z, iterations)) == badZ) return FALSE; 
      
      if ((zqs = makenewz(tr, q, s, defaultz, iterations)) == badZ) return FALSE;      
      
      if ((zrs = makenewz(tr, r, s, defaultz, iterations)) == badZ) return FALSE;
      
      lzqr = (zqr > zmin) ? log(zqr) : log(zmin); 
      lzqs = (zqs > zmin) ? log(zqs) : log(zmin);
      lzrs = (zrs > zmin) ? log(zrs) : log(zmin);
      lzsum = 0.5 * (lzqr + lzqs + lzrs);
      
      lzq = lzsum - lzrs;
      lzr = lzsum - lzqs;
      lzs = lzsum - lzqr;
      lzmax = log(zmax);
      
      if      (lzq > lzmax) {lzq = lzmax; lzr = lzqr; lzs = lzqs;} 
      else if (lzr > lzmax) {lzr = lzmax; lzq = lzqr; lzs = lzrs;}
      else if (lzs > lzmax) {lzs = lzmax; lzq = lzqs; lzr = lzrs;}          
            
      hookup(p->next,       q, exp(lzq));
      hookup(p->next->next, r, exp(lzr));
      hookup(p,             s, exp(lzs));      		  
    }
  else
    { 
      
      double  z;
           
      z = sqrt(q->z);      
      hookup(p->next,       q, z);
      hookup(p->next->next, r, z);
    }   
    
  if (! newview(tr, p)) return FALSE;    

  if(Thorough)
    {     
      localSmooth(tr, p, smoothings);

      /* regionalSmooth() for special ? */

      /* Did not show to be more efficient */
      tr->lzq = p->next->z;
      tr->lzr = p->next->next->z;
      tr->lzs = p->z;      
    }           
     
  return  TRUE;
}

boolean insertRestoreBIG (tree *tr, nodeptr p, nodeptr q, boolean glob)
{
  nodeptr  r, s;
  
  r = q->back;
  s = p->back;

  if(Thorough)
    {                        
      hookup(p->next,       q, tr->currentLZQ);
      hookup(p->next->next, r, tr->currentLZR);
      hookup(p,             s, tr->currentLZS);      		  
    }
  else
    { 
      
      double  z;
      
      z = sqrt(q->z);      
      hookup(p->next,       q, z);
      hookup(p->next->next, r, z);
    }   
    
  if (! newview(tr, p)) return FALSE;    
       
  return  TRUE;
}


void restoreTopologyOnly(tree *tr)
{
  nodeptr p = tr->removeNode;
  nodeptr q = tr->insertNode;
  double qz, pz, p1z, p2z;
  nodeptr p1, p2, r, s;
  double currentLH = tr->likelihood;

  p1 = p->next->back;
  p2 = p->next->next->back;

  p1z = p1->z;
  p2z = p2->z;
  
  hookup(p1, p2, tr->currentZQR);

  p->next->next->back = p->next->back = (node *) NULL;  

  /* remove */

  /* testInsert */

  qz = q->z;
  pz = p->z;

  /* testInsert */

  /* insert */

  r = q->back;
  s = p->back;
    
  if(Thorough)
    {                        
      hookup(p->next,       q, tr->currentLZQ);
      hookup(p->next->next, r, tr->currentLZR);
      hookup(p,             s, tr->currentLZS);      		  
    }
  else
    { 	
      double  z;	
      z = sqrt(q->z);      
      hookup(p->next,       q, z);
      hookup(p->next->next, r, z);
    }     

  tr->likelihood = tr->bestOfNode;
   
  saveBestTree(bt, tr);

  tr->likelihood = currentLH;

  hookup(q, r, qz);

  p->next->next->back = p->next->back = (nodeptr) NULL;

  if(Thorough)
    {      
      hookup(p, s, pz);      
    }  

   hookup(p->next,       p1, p1z); 
   hookup(p->next->next, p2, p2z);      
}


boolean testInsertBIG (tree *tr, nodeptr p, nodeptr q)
{
  double  qz, pz;
  nodeptr  r;
  
  r = q->back; 
  qz = q->z;
  pz = p->z;
  
  
  if (! insertBIG(tr, p, q, FALSE))       return FALSE;         
  
  if (evaluate(tr, p->next->next) == badEval) return FALSE;   
   
  if(tr->likelihood > tr->bestOfNode)
    {
      tr->bestOfNode = tr->likelihood;
      tr->insertNode = q;
      tr->removeNode = p;   
      tr->currentZQR = tr->zqr;           
      tr->currentLZR = tr->lzr;
      tr->currentLZQ = tr->lzq;
      tr->currentLZS = tr->lzs;
    }
  
  if(tr->likelihood > tr->endLH)
    {	           
      tr->insertNode = q;
      tr->removeNode = p;   
      tr->currentZQR = tr->zqr;      
      tr->endLH = tr->likelihood;           
      
      /*tr->currentLZR = tr->lzr;
	tr->currentLZQ = tr->lzq;
	tr->currentLZS = tr->lzs;*/
    }    
 
  hookup(q, r, qz);
  
  p->next->next->back = p->next->back = (nodeptr) NULL;

  if(Thorough)
    {
      nodeptr s = p->back;
      hookup(p, s, pz);      
    } 
  
  return TRUE;
}



 
void addTraverseBIG(tree *tr, nodeptr p, nodeptr q, int mintrav, int maxtrav)
{  
  if (--mintrav <= 0) 
    {              
      if (! testInsertBIG(tr, p, q))  return;        
    }
  
  if ((! q->tip) && (--maxtrav > 0)) 
    {    
      addTraverseBIG(tr, p, q->next->back, mintrav, maxtrav);
      addTraverseBIG(tr, p, q->next->next->back, mintrav, maxtrav);    
    }
} 


int  rearrangeBIGEXP(tree *tr, nodeptr p, int mintrav, int maxtrav)
   
  {
    double   p1z, p2z, q1z, q2z;
    nodeptr  p1, p2, q, q1, q2;
    int      mintrav2;
    int result = 0;

    double iniLH;
      
    if (maxtrav < 1 || mintrav > maxtrav)  return 0;
    q = p->back;

    if (! p->tip) 
      {     
	p1 = p->next->back;
	p2 = p->next->next->back;

	if (! p1->tip || ! p2->tip) 
	  {
	    p1z = p1->z;
	    p2z = p2->z;
	   
	    iniLH = tr->endLH;

	    if (! removeNodeBIG(tr, p)) return badRear;
	  
	    if (! p1->tip) 
	      {
		addTraverseBIG(tr, p, p1->next->back,
			       mintrav, maxtrav);         
		addTraverseBIG(tr, p, p1->next->next->back,
			       mintrav, maxtrav);          
	      }

	    if (! p2->tip) 
	      {
		addTraverseBIG(tr, p, p2->next->back,
			       mintrav, maxtrav);
		addTraverseBIG(tr, p, p2->next->next->back,
			       mintrav, maxtrav);          
	      }
	    
	    /*if(q->tip && tr->endLH > iniLH)
	      {
		hookup(p->next,       p1, p1z); 
		hookup(p->next->next, p2, p2z);
	      }
	      else*/
	      {
		hookup(p->next,       p1, p1z); 
		hookup(p->next->next, p2, p2z);	   	    	    
		initravDIST(tr, p, 1);	   
	      }
	  }
      }  

   
    
    /*if (! q->tip && maxtrav > 0) 
      {
	q1 = q->next->back;
	q2 = q->next->next->back;

	if (! q1->tip && (!q1->next->back->tip || !q1->next->next->back->tip) ||
	    ! q2->tip && (!q2->next->back->tip || !q2->next->next->back->tip)) 
	  {
	    q1z = q1->z;
	    q2z = q2->z;

	    if (! removeNodeBIG(tr, q)) return badRear;
	  
	    mintrav2 = mintrav > 2 ? mintrav : 2;

	    if (! q1->tip) 
	      {
		addTraverseBIG(tr, q, q1->next->back,
			       mintrav2 , maxtrav);
		addTraverseBIG(tr, q, q1->next->next->back,
			       mintrav2 , maxtrav);         
	      }

	    if (! q2->tip) 
	      {
		addTraverseBIG(tr, q, q2->next->back,
			       mintrav2 , maxtrav);
		addTraverseBIG(tr, q, q2->next->next->back,
			       mintrav2 , maxtrav);          
	      }	   
	   
	    hookup(q->next,       q1, q1z); 
	    hookup(q->next->next, q2, q2z);
	   
	    initravDIST(tr, q, 1); 	   
	  }
	  } */

    return  1;
  } 




int  rearrangeBIG(tree *tr, nodeptr p, int mintrav, int maxtrav)
   
  {
    double   p1z, p2z, q1z, q2z;
    nodeptr  p1, p2, q, q1, q2;
    int      mintrav2;
    int result = 0;

    double iniLH;
      
    if (maxtrav < 1 || mintrav > maxtrav)  return 0;
    q = p->back;

    if (! p->tip) 
      {     
	p1 = p->next->back;
	p2 = p->next->next->back;

	if (! p1->tip || ! p2->tip) 
	  {
	    p1z = p1->z;
	    p2z = p2->z;
	   
	    iniLH = tr->endLH;

	    if (! removeNodeBIG(tr, p)) return badRear;
	  
	    if (! p1->tip) 
	      {
		addTraverseBIG(tr, p, p1->next->back,
			       mintrav, maxtrav);         
		addTraverseBIG(tr, p, p1->next->next->back,
			       mintrav, maxtrav);          
	      }

	    if (! p2->tip) 
	      {
		addTraverseBIG(tr, p, p2->next->back,
			       mintrav, maxtrav);
		addTraverseBIG(tr, p, p2->next->next->back,
			       mintrav, maxtrav);          
	      }
	    
	   
	    hookup(p->next,       p1, p1z); 
	    hookup(p->next->next, p2, p2z);	   	    	    
	    initravDIST(tr, p, 1);	   	    
	  }
      }  

   
    
    if (! q->tip && maxtrav > 0) 
      {
	q1 = q->next->back;
	q2 = q->next->next->back;

	if (! q1->tip && (!q1->next->back->tip || !q1->next->next->back->tip) ||
	    ! q2->tip && (!q2->next->back->tip || !q2->next->next->back->tip)) 
	  {
	    q1z = q1->z;
	    q2z = q2->z;

	    if (! removeNodeBIG(tr, q)) return badRear;
	  
	    mintrav2 = mintrav > 2 ? mintrav : 2;

	    if (! q1->tip) 
	      {
		addTraverseBIG(tr, q, q1->next->back,
			       mintrav2 , maxtrav);
		addTraverseBIG(tr, q, q1->next->next->back,
			       mintrav2 , maxtrav);         
	      }

	    if (! q2->tip) 
	      {
		addTraverseBIG(tr, q, q2->next->back,
			       mintrav2 , maxtrav);
		addTraverseBIG(tr, q, q2->next->next->back,
			       mintrav2 , maxtrav);          
	      }	   
	   
	    hookup(q->next,       q1, q1z); 
	    hookup(q->next->next, q2, q2z);
	   
	    initravDIST(tr, q, 1); 	   
	  }
      } 

    return  1;
  } 


boolean testInsertRestoreBIG (tree *tr, nodeptr p, nodeptr q)
  { 
    nodeptr  r;

    r = q->back; 


    if(Thorough)
      {
        if (! insertBIG(tr, p, q, FALSE))       return FALSE;    
	if (evaluate(tr, p->next->next) == badEval) return FALSE;

	/*	
	  if (! insertRestoreBIG(tr, p, q, FALSE))       return FALSE;
    
	  {
	  nodeptr x, y;
	  
	  x = p->next->next;
	  y = p->back;
	  while ((! x->x) || (! y->x)) 
	  {
	  if (! (x->x)) if (! newview(tr, x)) return badEval;
	  if (! (y->x)) if (! newview(tr, y)) return badEval;
	  }
	  }
	  
	  tr->likelihood = tr->endLH;
	*/

      }
    else
      {
	if (! insertRestoreBIG(tr, p, q, FALSE))       return FALSE;
    
	{
	  nodeptr x, y;
	  
	  x = p->next->next;
	  y = p->back;
	  while ((! x->x) || (! y->x)) 
	    {
	      if (! (x->x)) if (! newview(tr, x)) return badEval;
	      if (! (y->x)) if (! newview(tr, y)) return badEval;
	    }
	}
	
	tr->likelihood = tr->endLH;
      }
     
    return TRUE;
  } /* testInsert */

void restoreTreeFast(tree *tr)
{
  removeNodeRestoreBIG(tr, tr->removeNode);  
  testInsertRestoreBIG(tr, tr->removeNode, tr->insertNode);
}

void treeOptimizeBIG2(tree *tr, int _mintrav, int _maxtrav, analdef *adef)
{
  int i;
  int mintrav = _mintrav;
  int maxtrav = _maxtrav;
  int subsequent = 0;
  int initial = adef->initial;
  double lh_res;
  double optTime;
  int oldThorough;
  int impCount = 0;
   
  optTime = gettime();
  
  if (maxtrav > tr->ntips - 3)  
    maxtrav = tr->ntips - 3;  
  
  resetInfoList();
 
  /*  printf("ADEF-INITIAL %d\n", adef->initial);*/

  /* NEW */
  resetBestTree(bt);
  saveBestTree(bt, tr);
  /* NEW */

  if(Multiple && !Thorough)
    {
      mintrav = 1;       
      if(maxtrav < initial)	
	maxtrav = initial;
    }

  /*printf("%d %d %d %d iterations %d\n", mintrav, maxtrav, Multiple, Thorough, iterations);
    printf("starting treeOptimize at %f\n", gettime() - masterTime);*/
  /*fprintf(logFile, "starting treeOptimize at %f\n", gettime() - masterTime);
    fflush(logFile);*/

  tr->startLH = tr->endLH = tr->likelihood;

  tr->specialOPT = 0; /* option to use areasmooth */

  for(i = 1; i <= tr->mxtips + tr->mxtips - 2; i++)
    {           
     
      tr->bestOfNode = unlikely;

      rearrangeBIG(tr, tr->nodep[i], mintrav, maxtrav);
      
      /* OLD insertInfoList(i, tr->bestOfNode);*/

      if(Multiple && (tr->endLH > tr->startLH))                 	
	{		 
	  restoreTreeFast(tr);	        	  	 	  	      
	  tr->startLH = tr->endLH = tr->likelihood;	  	 	  	  	  	  	  
	  subsequent = 1;
	  impCount++;
	}	    
	  
      /* smoothRegion (tr, tr->nodep[i], 3); TODO */	  	  	
    }

  /* OLD treeEvaluate(tr, 0.25); 

  oldThorough = Thorough;
  Thorough = 1;
    
  tr->startLH = tr->endLH = tr->likelihood;
    
  tr->specialOPT = 1;
    
  for(i = 0; i < iList.n; i++)
    {     
      rearrangeBIG(tr, tr->nodep[iList.list[i].node], 1, 10);
      
      if(tr->endLH > tr->startLH)                 	
	{		 	  
	  restoreTreeFast(tr);	        	  	 
	  tr->startLH = tr->endLH = tr->likelihood;	 
	}
      
    }    

  treeEvaluate(tr, 0.25); TODO fehlerquelle

  tr->specialOPT = 0;  option to use areasmooth 
  
  Thorough =  oldThorough;*/    

  Multiple = subsequent;

  return;     
}



void treeOptimizeBIG3(tree *tr, int _mintrav, int _maxtrav, analdef *adef)
{
  int i;
  int mintrav = _mintrav;
  int maxtrav = _maxtrav;
  int subsequent = 0;
  int initial = adef->initial;
  double lh_res;
  double optTime;
  int oldThorough;
  int impCount = 0;
   
  optTime = gettime();
  
  if (maxtrav > tr->ntips - 3)  
    maxtrav = tr->ntips - 3;  
  
  resetInfoList();  

  if(Multiple && !Thorough)
    {
      mintrav = 1;       
      if(maxtrav < initial)	
	maxtrav = initial;
    } 

  tr->startLH = tr->endLH = tr->likelihood;

  for(i = 1; i <= tr->mxtips + tr->mxtips - 2; i++)
    {           
     
      tr->bestOfNode = unlikely;

      rearrangeBIG(tr, tr->nodep[i], mintrav, maxtrav);
      
      insertInfoList(i, tr->bestOfNode);

      if(tr->endLH > tr->startLH)                 	
	{		 
	  restoreTreeFast(tr);	        	  	 	  	      
	  tr->startLH = tr->endLH = tr->likelihood;	  	 	  	  	  	  	  
	  printf("%f \n", tr->likelihood);
	  subsequent = 1;
	  impCount++;
	}	    	         	  	
    }


  oldThorough = Thorough;
  Thorough = 1;
    
 
    
  tr->specialOPT = 1;
  
  initrav(tr, tr->start);
  initrav(tr, tr->start->back);
  evaluate(tr, tr->start);
  printf("Lh now %f\n", tr->likelihood);
 
  resetBestTree(bt);
  saveBestTree(bt, tr);

  tr->startLH = tr->endLH = tr->likelihood;

  for(i = 0; i < iList.n; i++)
    {        
      tr->bestOfNode = unlikely;   

      rearrangeBIG(tr, tr->nodep[iList.list[i].node], 1, 10);
      
      if(tr->endLH > tr->startLH)                 	
	{		 	  
	  restoreTreeFast(tr);	        	  	 
	  tr->startLH = tr->endLH = tr->likelihood;	 
	  saveBestTree(bt, tr);
	}
      else
	{
	  if(tr->bestOfNode != unlikely)
	    {	     
	      restoreTopologyOnly(tr);
	    }	    
	}            
    }    

    
  Thorough =  oldThorough;  

  Multiple = subsequent;

  return;     
}





int treeOptimizeBIG(tree *tr, int _mintrav, int _maxtrav, analdef *adef)
{
  int i;
  int mintrav = _mintrav;
  int maxtrav = _maxtrav;
  int subsequent = 0;
  int initial = adef->initial;
  double lh_res;
  double optTime;
  int oldThorough;
  int impCount = 0;

  /* NEW */

  nodeptr *list = (nodeptr *)malloc(sizeof(nodeptr) * 3 * tr->mxtips);
  int n = 0;

  for(i = 1; i <= tr->mxtips + tr->mxtips - 2; i++)
    {    
      if(tr->nodep[i]->tip)
	list[n++] = tr->nodep[i]->back;
      else
	{
	  list[n++] = tr->nodep[i];
	  if(!tr->nodep[i]->back->tip)
	    list[n++] = tr->nodep[i]->back;
	}
    }
      /*NEW*/

  optTime = gettime();
  
  if (maxtrav > tr->ntips - 3)  
    maxtrav = tr->ntips - 3;  
  
 
  resetBestTree(bt);
  saveBestTree(bt, tr);

  if(Multiple && !Thorough)
    {
      mintrav = 1;       
      if(maxtrav < initial)	
	maxtrav = initial;
    }
 
  tr->startLH = tr->endLH = tr->likelihood;

  lh_res = tr->likelihood;

  /*for(i = 1; i <= tr->mxtips + tr->mxtips - 2; i++)*/
  for(i = 0; i < n; i++)
    {           
     
      tr->bestOfNode = unlikely;
     
      /*rearrangeBIG(tr, tr->nodep[i], mintrav, maxtrav);*/
      rearrangeBIG(tr, list[i], mintrav, maxtrav);
           
      if(Multiple && (tr->endLH > tr->startLH))
	{		 
	  restoreTreeFast(tr);	        	  	 	  	      
	  tr->startLH = tr->endLH = tr->likelihood;	  	 	  	  	  	  	  
	  subsequent = 1;
	  saveBestTree(bt, tr);	 
	}	    
      else
	{	 
	  if(tr->bestOfNode != unlikely)
	    {	     
	      restoreTopologyOnly(tr);
	    }	    
	}
	       	  	  	
    }

  /*printf("\n %d %d %d %d %f -> %f\n", mintrav, maxtrav, Thorough, subsequent, lh_res, tr->endLH);*/
   
  Multiple = subsequent;

  free(list);

  return subsequent; 
}



void computeBIG (tree *tr, analdef *adef) 
  { 
    int i, rearrangementsMax, rearrangementsMin, impr, subsequent;                
    double lh, previousLh, difference, epsilon;              
    bestlist *bestT;

    bestT = (bestlist *) malloc(sizeof(bestlist));
    bestT->ninit = 0;
    initBestTree(bestT, 1, tr->mxtips, tr->cdta->endsite);   
    
    /*initInfoList(20);*/
 
    difference = 10.0;

    epsilon = 0.001;
    
    Multiple = 1;
    
    Thorough = 0;  
        
    if(adef->model == M_PROTEINS || adef->model == M_PROTEINSCAT)
      {
	Thorough = 1;       
	epsilon = 0.1;
      }    
    
    lh = tr->likelihood;           
   
    impr = 1;		  		  
     
    while(1)
      {		
	recallBestTree(bestT, 1, tr);	

	optimizeModel(tr, adef, 1);       
	treeEvaluate(tr, 2);		
			
	lh = tr->likelihood;	
	saveBestTree(bestT, tr);	

	printLog(tr, adef);	       	
     	printf("NWC: %d\n", NWC);
	
	if(impr)
	  {	    
	    printResult(tr, 0);	   
	    rearrangementsMin = 1;
	    rearrangementsMax = adef->stepwidth;	    
	  }			  			
	else
	  {		       
	    if(Thorough == 0)
	      { 		
		Thorough = 1;
		rearrangementsMax = adef->stepwidth;
		rearrangementsMin = 1; 
	      }
	    else
	      { 
		rearrangementsMax += adef->stepwidth;
		rearrangementsMin += adef->stepwidth; 
	      }	    	      
	    if(rearrangementsMax > adef->max_rearrange)	     	     	 
	      goto cleanup; 	   
	  }

	/*subsequent =*/ 
	treeOptimizeBIG(tr, rearrangementsMin, rearrangementsMax, adef);
	
	/*if(subsequent == 0)
	  {	   
	    Multiple = 0;	    
	    }*/
			
	impr = 0;			      	
	previousLh = lh;
		
	for(i = 1; i <= bt->nvalid; i++)
	  {	    		  	   
	    recallBestTree(bt, i, tr);	    
	    treeEvaluate(tr, 0.25);	    
	  
	    difference = ((tr->likelihood > previousLh)? 
			  tr->likelihood - previousLh: 
			  previousLh - tr->likelihood); 	    
	    if(tr->likelihood > lh && difference > epsilon)
	      {
		impr = 1;	       
		lh = tr->likelihood;	       
		saveBestTree(bestT, tr);		
	      }	   	   
	  }	
      }
    
  cleanup: 
    printLog(tr, adef);
    printResult(tr, 0);	     
  }


/*****************************COMPUTE BIG FAST ***************************************************************/

int determineRearrangementSetting(tree *tr,  analdef *adef, bestlist *bestT)
{
  int i, mintrav, maxtrav, bestTrav, impr;
  double rTime, startLH, bestRatio, ratio;
  int MaxFast;

  MaxFast = 26;

  startLH = tr->likelihood;
    
  mintrav = 1;
  maxtrav = 5;

  bestTrav = maxtrav = 5;

  impr = 1;

  resetBestTree(bt);

  while(impr && maxtrav < MaxFast)
    {	
      recallBestTree(bestT, 1, tr);

      rTime = gettime();
      if (maxtrav > tr->ntips - 3)  
	maxtrav = tr->ntips - 3;    
 
      tr->startLH = tr->endLH = tr->likelihood;

      for(i = 1; i <= tr->mxtips + tr->mxtips - 2; i++)
	{                
	  tr->bestOfNode = unlikely;
	  rearrangeBIG(tr, tr->nodep[i], mintrav, maxtrav);                 
	  if(tr->endLH > tr->startLH)                 	
	    {		 	 	      
	      restoreTreeFast(tr);	        	  	 	  	      
	      tr->startLH = tr->endLH = tr->likelihood;	  	 	  	  	  	  	  	  	      
	    }	         	       	  	  	       
	}

      treeEvaluate(tr, 0.25);
      saveBestTree(bt, tr);                                    

      /*printf("%d %f\n", maxtrav, tr->likelihood);*/

      if(tr->likelihood > startLH)
	{	 
	  startLH = tr->likelihood; 	  
	  printLog(tr, adef);
	  bestTrav = maxtrav;
	  bestRatio = ratio;
	  impr = 1;
	}
      else
	{
	  impr = 0;
	}
      maxtrav += 5;
    }

  recallBestTree(bt, 1, tr);   

  return bestTrav;     
}


double treeOptimizeQuick(tree *tr, int mintrav, int maxtrav, analdef *adef)
{
  int i;
  double lh_res;
  int oldThorough;     
  
  if (maxtrav > tr->ntips - 3)  
    maxtrav = tr->ntips - 3;  
  
  resetInfoList();
  resetBestTree(bt);
 
  tr->startLH = tr->endLH = tr->likelihood;

  for(i = 1; i <= tr->mxtips + tr->mxtips - 2; i++)
    {           
     
      tr->bestOfNode = unlikely;

      rearrangeBIG(tr, tr->nodep[i], mintrav, maxtrav);
      
      insertInfoList(i, tr->bestOfNode);

      if(tr->endLH > tr->startLH)                 	
	{		 
	  restoreTreeFast(tr);	        	  	 	  	      
	  tr->startLH = tr->endLH = tr->likelihood;	  	 	  	  	  	  	  	  
	}	    	  
    }

  oldThorough = Thorough;
  Thorough = 1;  
 
  for(i = 0; i < iList.n; i++)
    {      
      tr->bestOfNode = unlikely;

      rearrangeBIG(tr, tr->nodep[iList.list[i].node], 1, 10);                 

      if(tr->endLH > tr->startLH)                 	
	{		 	  
	  restoreTreeFast(tr);	 
	  /*printf("IMP %d %f %f %d bets of this %f\n", i, tr->startLH, tr->endLH, tr->nodep[iList.list[i].node]->number,  tr->bestOfNode);*/
	  tr->startLH = tr->endLH = tr->likelihood;	 
	  saveBestTree(bt, tr);
	}
      else
	{ 
	  /*printf("TOPT %d %f %f %d bets of this %f\n", i, tr->startLH, tr->endLH, tr->nodep[iList.list[i].node]->number, tr->bestOfNode);*/
	  if(tr->bestOfNode != unlikely)
	    {	     
	      restoreTopologyOnly(tr);
	    }	
	}      
    }       

  Thorough =  oldThorough;
  
  return tr->startLH;     
}



void computeBIGFAST (tree *tr, analdef *adef) 
{ 
  int i, rearrangementsMax, rearrangementsMin, noImproveCount, impr, goodTrav, bestTrav, which;                
  double lh, previousLh, difference, epsilon, currentLH, bestLH;              
  bestlist *bestT;
  
  bestT = (bestlist *) malloc(sizeof(bestlist));
  bestT->ninit = 0;
  initBestTree(bestT, 1, tr->mxtips, tr->cdta->endsite);
    
  initInfoList(40);
 
  difference = 10.0;
  epsilon = 0.01;    
    
  Thorough = 0;  
        
  if(adef->model == M_PROTEINS || adef->model == M_PROTEINSCAT)
    {
      Thorough = 1;       
      epsilon = 0.1;
    }

  optimizeModel(tr, adef, 1);       
  treeEvaluate(tr, 2);
  printLog(tr, adef);   

  saveBestTree(bestT, tr);   	  		      
       
  bestTrav = determineRearrangementSetting(tr, adef, bestT);       
  printLog(tr, adef);  
  printResult(tr, 0);

  impr = 1;

  while(impr)
    {                            	      

      optimizeModel(tr, adef, 1);    
      treeEvaluate(tr, 2);

      currentLH = tr->likelihood;
      
      bestLH = treeOptimizeQuick(tr, 1, bestTrav, adef);
      
      treeEvaluate(tr, 0.25);

      if(bestLH > currentLH)
	{	  
	  printLog(tr, adef);
	  printResult(tr, 0);

	  difference = (bestLH - currentLH); 	    
	  if(difference > epsilon)	      
	    impr = 1;
	  else
	    impr = 0;
	}
      else
	impr = 0;
    }
  
  bestLH = unlikely;

  for(i = 1; i <= bt->nvalid; i++)
    {	    	    	
      recallBestTree(bt, i, tr);			    
      treeEvaluate(tr, 0.25);
      /*printf("%d %f\n", i, tr->likelihood);*/
      if(tr->likelihood > bestLH)
	{
	  which = i;
	  bestLH = tr->likelihood;
	}           
    }

  recallBestTree(bt, which, tr);  
  optimizeModel(tr, adef, 1);
  treeEvaluate(tr, 2);
  printLog(tr, adef);
  printResult(tr, 0);    
}


double treeOptimizeRapid(tree *tr, int mintrav, int maxtrav, analdef *adef)
{
  int i;  
  int oldThorough;     
  
  if (maxtrav > tr->ntips - 3)  
    maxtrav = tr->ntips - 3;  
  
  resetInfoList();
  resetBestTree(bt);
 
  tr->startLH = tr->endLH = tr->likelihood;

  for(i = 1; i <= tr->mxtips + tr->mxtips - 2; i++)
    {           
     
      tr->bestOfNode = unlikely;

      rearrangeBIG(tr, tr->nodep[i], mintrav, maxtrav);
      /*NEW*/
      if(Thorough)
	{
	  if(tr->endLH > tr->startLH)                 	
	    {		 	  
	      restoreTreeFast(tr);	 	 
	      tr->startLH = tr->endLH = tr->likelihood;	 
	      saveBestTree(bt, tr);
	    }
	  else
	    { 
	      
	      if(tr->bestOfNode != unlikely)
		{	     
		  restoreTopologyOnly(tr);
		}	
	    }
	  /* NEW */
	}
      else
	{
	  insertInfoList(i, tr->bestOfNode);
           
	  if(tr->endLH > tr->startLH)                 	
	    {		 
	      restoreTreeFast(tr);	        	  	 	  	      
	      tr->startLH = tr->endLH = tr->likelihood;	  	 	  	  	  	  	  	  
	    }	    	  
	}
    }

  /* NEW */
  if(!Thorough)
    {
      /* NEW */
      Thorough = 1;  
 
      for(i = 0; i < iList.n; i++)
	{      
	  tr->bestOfNode = unlikely;
	  
	  rearrangeBIG(tr, tr->nodep[iList.list[i].node], mintrav, maxtrav);                 
	  
	  if(tr->endLH > tr->startLH)                 	
	    {		 	  
	      restoreTreeFast(tr);	 	 
	      tr->startLH = tr->endLH = tr->likelihood;	 
	      saveBestTree(bt, tr);
	    }
	  else
	    { 
	      
	      if(tr->bestOfNode != unlikely)
		{	     
		  restoreTopologyOnly(tr);
		}	
	    }      
	}       
      
      Thorough = 0;
      /* NEW */
    }
  /* NEW */
  return tr->startLH;     
}


void computeBIGRAPID (tree *tr, analdef *adef) 
{ 
  int i, rearrangementsMax, rearrangementsMin, noImproveCount, impr, goodTrav, bestTrav, which;                
  double lh, previousLh, difference, epsilon, currentLH, bestLH;              
  bestlist *bestT;
  
  bestT = (bestlist *) malloc(sizeof(bestlist));
  bestT->ninit = 0;
  initBestTree(bestT, 1, tr->mxtips, tr->cdta->endsite);
    
  initInfoList(50);
 
  difference = 10.0;
  epsilon = 0.01;    
    
  Thorough = 0;  
        
  if(adef->model == M_PROTEINS || adef->model == M_PROTEINSCAT)
    {
      Thorough = 1;       
      epsilon = 0.1;
    }

  optimizeModel(tr, adef, 1);       
  treeEvaluate(tr, 2);
  printLog(tr, adef);   

  saveBestTree(bestT, tr);   	  		      
       
  bestTrav = determineRearrangementSetting(tr, adef, bestT);       
  saveBestTree(bestT, tr);

  impr = 1;

  while(impr)
    {     
      recallBestTree(bestT, 1, tr);
      optimizeModel(tr, adef, 1);    
      treeEvaluate(tr, 2);
      saveBestTree(bestT, tr);

      printLog(tr, adef);
      printResult(tr, 0);
      saveBestTree(bestT, tr);
      lh = previousLh = tr->likelihood;
     
      treeOptimizeRapid(tr, 1, bestTrav, adef);
      
      impr = 0;

      for(i = 1; i <= bt->nvalid; i++)
	{	    		  	   
	  recallBestTree(bt, i, tr);	    
	  treeEvaluate(tr, 0.25);	    	 	

	  difference = ((tr->likelihood > previousLh)? 
			tr->likelihood - previousLh: 
			previousLh - tr->likelihood); 	    
	  if(tr->likelihood > lh && difference > epsilon)
	    {
	      impr = 1;	       
	      lh = tr->likelihood;	       
	      saveBestTree(bestT, tr);		
	    }	   	   
	}	
    }
  
  printf("SWITCHING\n");

  /* NEW */
  Thorough = 1;
  /* NEW */


  impr = 1;
  
  while(1)
    {		
      recallBestTree(bestT, 1, tr);	

      optimizeModel(tr, adef, 1);       
      treeEvaluate(tr, 2);					
      previousLh = lh = tr->likelihood;	
      saveBestTree(bestT, tr);	

      printLog(tr, adef);      	
     	        
      if(impr)
	{	    
	  printResult(tr, 0);	   
	  rearrangementsMin = 1;
	  rearrangementsMax = adef->stepwidth;	    
	}			  			
      else
	{		       	   
	  rearrangementsMax += adef->stepwidth;
	  rearrangementsMin += adef->stepwidth; 	        	      
	  if(rearrangementsMax > adef->max_rearrange)	     	     	 
	    goto cleanup; 	   
	}
      
      treeOptimizeRapid(tr, rearrangementsMin, rearrangementsMax, adef);
	

      impr = 0;			      	

		
      for(i = 1; i <= bt->nvalid; i++)
	{	    		  	   
	  recallBestTree(bt, i, tr);	    
	  treeEvaluate(tr, 0.25);	    
	  
	  difference = ((tr->likelihood > previousLh)? 
			tr->likelihood - previousLh: 
			previousLh - tr->likelihood); 	    
	  if(tr->likelihood > lh && difference > epsilon)
	    {
	      impr = 1;	       
	      lh = tr->likelihood;	       
	      saveBestTree(bestT, tr);		
	    }	   	   
	}	
    }
    
  cleanup: 
    printLog(tr, adef);
    printResult(tr, 0);	         
}


void computeBIGRAPIDFAST (tree *tr, analdef *adef) 
{ 
  int i, mintrav, maxtrav, which;
  double initialLH, epsilon, diff;
  bestlist *bestT;
  
  bestT = (bestlist *) malloc(sizeof(bestlist));
  bestT->ninit = 0;
  initBestTree(bestT, 1, tr->mxtips, tr->cdta->endsite);
 
  epsilon = 0.01;
  
  Thorough = 0;

  if(adef->model == M_PROTEINS || adef->model == M_PROTEINSCAT)
    Thorough = 1;

  optimizeModel(tr, adef, 1);
  treeEvaluate(tr, 2);
  saveBestTree(bestT, tr); 

  Multiple = 1;
  mintrav = 1;
  maxtrav = determineRearrangementSetting(tr, adef, bestT);

  optimizeModel(tr, adef, 1);
  treeEvaluate(tr, 2);
  
  if (maxtrav > tr->ntips - 3)  maxtrav = tr->ntips - 3;
          
  tr->opt_level = 0;
  
  do
    {
      printLog(tr, adef);
      printResult(tr, 0);

      initialLH = tr->likelihood;
      tr->startLH = tr->endLH = tr->likelihood;

      for(i = 1; i < (2 * tr->mxtips - 1); i++)
	{ 
	  tr->bestOfNode = unlikely;

	  rearrangeBIG(tr, tr->nodep[i], mintrav, maxtrav);
     	  	  
	  if(tr->endLH > tr->startLH)                 	
	    {		
	      
	      restoreTreeFast(tr);	 	 
	      tr->startLH = tr->endLH = tr->likelihood;	 
	      saveBestTree(bt, tr);
	    }
	  else
	    { 	      
	      if(tr->bestOfNode != unlikely)
		{	     
		  restoreTopologyOnly(tr);
		}	
	    }	  
	}	  	      
      treeEvaluate(tr, 1);            
      diff = (tr->likelihood > initialLH)? tr->likelihood - initialLH: -1.0;            
    }
  while(diff > epsilon);

  initialLH = unlikely;

  for(i = 1; i <= bt->nvalid; i++)
    {	    	    	
      recallBestTree(bt, i, tr);			    
      smoothTree(tr, smoothings/4);
      evaluate(tr, tr->start);		           
      if(tr->likelihood > initialLH)
	{
	  which = i;
	  initialLH = tr->likelihood;
	}           
    }

  recallBestTree(bt, which, tr);
  treeEvaluate(tr, 1);
  checkTime(adef, tr);
  optimizeModel(tr, adef, 1);
  printLog(tr, adef);
  printResult(tr, 0);

}


void simulatedAnnealingBIG(tree *tr, analdef *adef)
{
  double lh1, lh2, best, p1, p2, deltaf, alpha, t0;
  int repeats;
  int node, mintrav, maxtrav, rn, c_imp, selectMove, counter, topologyMove;
  bestlist *consensusList, *bestT;
  int maxR, i, statt, stati;
  int *subtreeStats, *nodeList, nodeCount;
  
  bestT = (bestlist *) malloc(sizeof(bestlist));
  bestT->ninit = 0;
  initBestTree(bestT, 1, tr->mxtips, tr->cdta->endsite);

  srand((unsigned int) time(NULL));
 
  maxR = adef->max_rearrange; 
    
  Multiple = 0;

  /* TODO passt noch nicht fuer prots*/

 
  Thorough = 0;
   
  optimizeModel(tr, adef, 1);
  treeEvaluate(tr, 4);

  alpha = 0.9;
  best = unlikely;
  t0 = adef->t0;
  
  if(adef->repeats == 0)
    adef->repeats = tr->mxtips;
  else
    repeats = adef->repeats;
  
  c_imp = 0;
  counter = 0;
 
  while(1)
    {      
      lh1 = tr->likelihood;
      
      if(lh1 > best)
	{
	  best = lh1;
	  printLog(tr, adef);	 
	  printResult(tr, 0);
	}
      
      saveBestTree(bestT, tr);
     
      node = 1 + randomInt(2 * tr->mxtips - 2); 
      
      selectMove = randomInt(100);
      
     

      if(selectMove < 95)
	{
	  topologyMove = 1;
	  mintrav = 1;	  
	  maxtrav = 2 + randomInt(maxR);  	
  
	  tr->startLH = tr->endLH = unlikely;
	  rearrangeBIG(tr, tr->nodep[node], mintrav, maxtrav);
	  restoreTreeFast(tr);	
	  if(selectMove > 94)
	    evalFast(tr);
	  c_imp++;
	}
      else	
	{
	  topologyMove = 0;
	  selectMove = randomInt(100);
	  
	  if(selectMove < 95)
	    {	      
	      nodeptr p;
	      nodeptr q;
	      double z0, z;
	      
	      p = tr->nodep[node];
	      
	      q = p->back;
	      z0 = q->z;		
	      z = makenewz(tr, p, q, z0, newzpercycle); 	  
	      p->z = q->z = z;
	      
	      evaluate(tr, tr->start);	
	    }
	  else
	    {
	      optimizeModel(tr, adef, 1);
	      treeEvaluate(tr, 4);
	    }
	}
           
      lh2 = tr->likelihood;

      if(lh1 > lh2)
	{	  
	  deltaf = lh2 - lh1;
	  p1 = exp( deltaf / t0);
	  rn = randomInt(10000);
	  if(rn == 0) 
	    p2 = 0;
	  else 
	    p2 = ((double)rn)/10000.0;	  	  
	  
	  if(p1 < p2)	  
	    recallBestTree(bestT, 1, tr);	  	  
	}    
      
      if(c_imp >= repeats)
	{
	  t0 = t0 * alpha;	  
	  c_imp = 0;
	}
     
      counter++;     

      if((counter % (2 * tr->mxtips)) == 0) 	
	{	 
	  counter = 0;
	}          
    }          
}










/********************** COMPUTE BIG ***************************************************************************/










/********************** COMPUTE PARALLEL ***************************************************************************/

#ifdef PARALLEL

  
nodeptr  removeNodePARALLEL (tree *tr, nodeptr p)
{
  double   zqr;
  nodeptr  q, r;
        
  q = p->next->back;
  r = p->next->next->back;
  zqr = q->z * r->z;
       
  tr->removeNumberP = p->number;
  tr->removeNumberQ = q->number;
  tr->removeNumberR = r->number;
  

  if ((zqr = makenewz(tr, q, r, zqr, iterations)) == badZ) return (node *) NULL;      

  tr->zqr = zqr;
    
  hookup(q, r, zqr);

  p->next->next->back = p->next->back = (node *) NULL;
     
  return  q;
}



boolean insertPARALLEL (tree *tr, nodeptr p, nodeptr q, boolean glob)
{
  nodeptr  r, s;
  
  r = q->back;
  s = p->back;

  

  if(Thorough)
    { 
      double  zqr, zqs, zrs, lzqr, lzqs, lzrs, lzsum, lzq, lzr, lzs, lzmax;      
            
       

      if ((zqr = makenewz(tr, q, r, q->z, iterations)) == badZ) return FALSE; 
     
      if ((zqs = makenewz(tr, q, s, defaultz, iterations)) == badZ) return FALSE;      
     
      if ((zrs = makenewz(tr, r, s, defaultz, iterations)) == badZ) return FALSE;
     

      lzqr = (zqr > zmin) ? log(zqr) : log(zmin); 
      lzqs = (zqs > zmin) ? log(zqs) : log(zmin);
      lzrs = (zrs > zmin) ? log(zrs) : log(zmin);
      lzsum = 0.5 * (lzqr + lzqs + lzrs);
      
      lzq = lzsum - lzrs;
      lzr = lzsum - lzqs;
      lzs = lzsum - lzqr;
      lzmax = log(zmax);
      
      if      (lzq > lzmax) {lzq = lzmax; lzr = lzqr; lzs = lzqs;} 
      else if (lzr > lzmax) {lzr = lzmax; lzq = lzqr; lzs = lzrs;}
      else if (lzs > lzmax) {lzs = lzmax; lzq = lzqs; lzr = lzrs;}          
            
      hookup(p->next,       q, exp(lzq));
      hookup(p->next->next, r, exp(lzr));
      hookup(p,             s, exp(lzs));     

     
    }
  else
    { 
      
      double  z;
           
      z = sqrt(q->z);      
      hookup(p->next,       q, z);
      hookup(p->next->next, r, z);
    }   
    
  if (! newview(tr, p)) return FALSE;    

  if(Thorough)
    {     
     
      localSmooth(tr, p, smoothings);
      

      /* regionalSmooth() for special ? */

      /* Did not show to be more efficient
	tr->lzq = p->next->z;
	tr->lzr = p->next->next->z;
	tr->lzs = p->z;
      */
    }           
     
  return  TRUE;
}




boolean testInsertPARALLEL (tree *tr, nodeptr p, nodeptr q)
{
  double  qz, pz;
  nodeptr  r;
  
  r = q->back; 

  qz = q->z;
  pz = p->z;
  
  
  if (! insertPARALLEL(tr, p, q, FALSE))       return FALSE;         
  
  if (evaluate(tr, p->next->next) == badEval) return FALSE;   
   
  if(tr->likelihood > tr->bestOfNode)
    {
      tr->bestOfNode = tr->likelihood;
    }
  
  if(tr->likelihood > tr->endLH)
    {	           
      tr->insertNode = q;
      tr->removeNode = p;   
      tr->currentZQR = tr->zqr;      
      tr->endLH = tr->likelihood;           
    
      tr->removedNumberP = tr->removeNumberP;
      tr->removedNumberQ = tr->removeNumberQ;
      tr->removedNumberR = tr->removeNumberR;
      tr->insertNumberQ = q->number;
      tr->insertNumberR = r->number;
    }    
  
  hookup(q, r, qz);
  
  p->next->next->back = p->next->back = (nodeptr) NULL;

  if(Thorough)
    {
      nodeptr s = p->back;
      hookup(p, s, pz);      
    } 
  
  return TRUE;
}



 
void addTraversePARALLEL(tree *tr, nodeptr p, nodeptr q, int mintrav, int maxtrav)
{  
  if (--mintrav <= 0) 
    {              
      if (! testInsertPARALLEL(tr, p, q))  return;        
    }
  
  if ((! q->tip) && (--maxtrav > 0)) 
    {    
      addTraversePARALLEL(tr, p, q->next->back, mintrav, maxtrav);
      addTraversePARALLEL(tr, p, q->next->next->back, mintrav, maxtrav);    
    }
} 


int  rearrangePARALLEL(tree *tr, nodeptr p, int mintrav, int maxtrav)
    /* rearranges the tree, globally or locally */
  { /* rearrange */
    double   p1z, p2z, q1z, q2z;
    nodeptr  p1, p2, q, q1, q2;
    int      mintrav2;
    int result = 0;
   
    if (maxtrav < 1 || mintrav > maxtrav)  return 0;
    

    if (! p->tip) 
      {     
	p1 = p->next->back;
	p2 = p->next->next->back;

	if (! p1->tip || ! p2->tip) 
	  {
	    p1z = p1->z;
	    p2z = p2->z;
	    if (! removeNodePARALLEL(tr, p)) return badRear;
	  
	    if (! p1->tip) 
	      {
		addTraversePARALLEL(tr, p, p1->next->back,
			       mintrav, maxtrav);         
		addTraversePARALLEL(tr, p, p1->next->next->back,
			       mintrav, maxtrav);          
	      }

	    if (! p2->tip) 
	      {
		addTraversePARALLEL(tr, p, p2->next->back,
			       mintrav, maxtrav);
		addTraversePARALLEL(tr, p, p2->next->next->back,
			       mintrav, maxtrav);          
	      }

	    hookup(p->next,       p1, p1z);
	    hookup(p->next->next, p2, p2z);

	     initravDIST(tr, p, 1); 

	    /*if (! (initrav(tr, tr->start) && initrav(tr, tr->start->back))) return badRear;	  */
	  }
      }  

    q = p->back;
    
    if (! q->tip && maxtrav > 1) 
      {
	q1 = q->next->back;
	q2 = q->next->next->back;

	if (! q1->tip && (!q1->next->back->tip || !q1->next->next->back->tip) ||
	    ! q2->tip && (!q2->next->back->tip || !q2->next->next->back->tip)) 
	  {
	    q1z = q1->z;
	    q2z = q2->z;
	    if (! removeNodePARALLEL(tr, q)) return badRear;
	  
	    mintrav2 = mintrav > 2 ? mintrav : 2;

	    if (! q1->tip) 
	      {
		addTraversePARALLEL(tr, q, q1->next->back,
			       mintrav2 , maxtrav);
		addTraversePARALLEL(tr, q, q1->next->next->back,
			       mintrav2 , maxtrav);         
	      }

	    if (! q2->tip) 
	      {
		addTraversePARALLEL(tr, q, q2->next->back,
			       mintrav2 , maxtrav);
		addTraversePARALLEL(tr, q, q2->next->next->back,
			       mintrav2 , maxtrav);          
	      }

	    hookup(q->next,       q1, q1z); 
	    hookup(q->next->next, q2, q2z);

	    initravDIST(tr, q, 1); 

	    /*if (! (initrav(tr, tr->start) && initrav(tr, tr->start->back))) return badRear;*/
	  }
      } 

    return  1;
  } 


nodeptr verifyRemoveNode(tree *tr)
{
  nodeptr p = tr->nodep[tr->removedNumberP]; 
  nodeptr correctSector = (nodeptr) NULL;

  if(p->tip)
    {
      printf("ERROR in  verifyRemoveNode() is a TIP\n");
      exit(-1);
    } 

  if(p->back->number == tr->removedNumberQ && p->next->back->number == tr->removedNumberR)    
    correctSector = p->next->next;          
  else
    {
      if(p->next->back->number == tr->removedNumberQ && p->next->next->back->number == tr->removedNumberR)	
	correctSector = p;	
      else
	{
	  if(p->next->next->back->number == tr->removedNumberQ && p->back->number == tr->removedNumberR)	    
	    correctSector = p->next;	    
	}
    } 

  return correctSector;
}


nodeptr verifyInsertNode(tree *tr)
{
  nodeptr q = tr->nodep[tr->insertNumberQ];
  nodeptr r;
  nodeptr insertSector = (nodeptr) NULL;

  q = tr->nodep[tr->insertNumberQ];
  
  if(q->tip)
    {      
      if(q->back->number == tr->insertNumberR && q->back->back->number == tr->insertNumberQ);
      insertSector = q;    
    }
  else
    {
      if(q->back->number == tr->insertNumberR && q->back->back->number == tr->insertNumberQ)    
	insertSector = q;    
      else
	{
	  if(q->next->back->number == tr->insertNumberR && q->next->back->back->number == tr->insertNumberQ)
	    insertSector = q->next;
	  else
	    {
	      if(q->next->next->back->number == tr->insertNumberR && q->next->next->back->back->number == tr->insertNumberQ)
		insertSector = q->next->next;	      
	    }
	}     
    }
  return  insertSector;
}







int inSub(nodeptr p,  int insertSectorNumber)
{
  nodeptr q;
  int result = 0;

  if(p->tip)
    {
      if(p->number == insertSectorNumber) 
	return 1;
      else
	return 0;
    }
  else
    {
       if(p->number == insertSectorNumber) 
	 return 1;
       else
	 {
	   q = p->next;
	   while(q != p)
	     {
	       result = (result || inSub(q->back, insertSectorNumber));
	       q = q->next;
	     }
	     
	   return result;
	 }
    }
      
}
   
 
int inSubtree(tree *tr, nodeptr q, int insertSectorNumber)
{
  int r1, r2;
  nodeptr r = q->back;

  r1 = inSub(q,  insertSectorNumber);
  r2 = inSub(r,  insertSectorNumber);

  return (r1 || r2);
}

double bestLikelihood;
int restCount = 0;
int restSucCount = 0;

#define STEPS 99
#define LENGTH 1002


double startOptTime;
int impCnt;

int restoreTreeFastPARALLEL(tree *tr)
{
  nodeptr removeSector, insertSector; 
  nodeptr p1, p2, r, s;
  double p1z, p2z, zqr, qz, ps, z, zqs, zrs, lzqr, lzqs, lzrs, lzsum, lzq, lzr, lzs, lzmax;   
  int r1, r2;

  restCount++;

  removeSector = verifyRemoveNode(tr); 
 
  if(!removeSector) 
    return 0;
  
  insertSector = verifyInsertNode(tr);  

  if(!insertSector) 
    return 0;
  
  p1 = removeSector->next->back;
  p2 = removeSector->next->next->back;
 
  r1 = inSub(p1,  insertSector->number);
  r2 = inSub(p2,  insertSector->number);

  if((!removeSector->tip) && (! p1->tip || ! p2->tip) && ((r1 || r2) != 0))
    {      
      restSucCount++;

      p1z = p1->z;
      p2z = p2->z;   	      
      
      zqr = p1->z * p2->z;

      if ((zqr = makenewz(tr, p1, p2, zqr, iterations)) == badZ) 
	return 0;      

      hookup(p1, p2, zqr);		

      removeSector->next->next->back = removeSector->next->back = (node *) NULL;    
                   	
      r = insertSector->back;
      qz = insertSector->z;
          
      s = removeSector->back;

      if(Thorough)
	{ 	     	  
	  ps = s->z; 
	  
	  if ((zqr = makenewz(tr, insertSector, r, insertSector->z, iterations)) == badZ) return FALSE; 
	  
	  if ((zqs = makenewz(tr, insertSector, s, defaultz, iterations)) == badZ) return FALSE;      
	  
	  if ((zrs = makenewz(tr, r, s, defaultz, iterations)) == badZ) return FALSE;
      
	  lzqr = (zqr > zmin) ? log(zqr) : log(zmin); 
	  lzqs = (zqs > zmin) ? log(zqs) : log(zmin);
	  lzrs = (zrs > zmin) ? log(zrs) : log(zmin);
	  lzsum = 0.5 * (lzqr + lzqs + lzrs);
	  
	  lzq = lzsum - lzrs;
	  lzr = lzsum - lzqs;
	  lzs = lzsum - lzqr;
	  lzmax = log(zmax);
	  
	  if      (lzq > lzmax) {lzq = lzmax; lzr = lzqr; lzs = lzqs;} 
	  else if (lzr > lzmax) {lzr = lzmax; lzq = lzqr; lzs = lzrs;}
	  else if (lzs > lzmax) {lzs = lzmax; lzq = lzqs; lzr = lzrs;}
	  
      
	  hookup(removeSector->next,       insertSector, exp(lzq));
	  hookup(removeSector->next->next, r, exp(lzr));
	  hookup(removeSector,             s, exp(lzs));      		  
	}
      else
	{ 	  	  
	  z = sqrt(insertSector->z);
	  
	  hookup(removeSector->next,       insertSector, z);
	  hookup(removeSector->next->next, r, z);
	}
      
      if (! newview(tr, removeSector)) return 0;
      
      tr->opt_level = 0;
      
      if(Thorough)
	{
	  if (! localSmooth(tr, removeSector, smoothings)) return 0;	
	}                        
      
      evaluate(tr, removeSector->next->next);   
	
      if(tr->likelihood > bestLikelihood)
	{
	  bestLikelihood = tr->likelihood;	  

	  /*if(myId == 0)
	    printf("MASTER %f %f %d %d\n", receivedLH, tr->likelihood, restSucCount, restCount);
	  else
	  printf("WORKER %f %f %d %d\n", receivedLH, tr->likelihood, restSucCount, restCount);*/
	  impCnt++;
	  if(myId == 0 && (impCnt % 50 == 0))
	    {
	      printf("%f %f\n", gettime() - startOptTime, tr->likelihood);
	    }
	}
      else
	{	 
	  hookup(insertSector, r, qz);
	  removeSector->next->next->back =  removeSector->next->back = (nodeptr) NULL;	  
	  /*if(myId == 0)
	    printf("MASTER restoring\n");
	  else
	  printf("WORKER restoring\n", receivedLH, tr->likelihood);*/
	  hookup(removeSector->next,       p1, p1z);
	  hookup(removeSector->next->next, p2, p2z);

	  if(Thorough)
	    removeSector->z = removeSector->back->z = ps;
	 
	  initravDIST(tr, removeSector, 1); 

	  /*if (! (initrav(tr, tr->start) && initrav(tr, tr->start->back))) return 0;*/
	  evaluate(tr, tr->start);	  
	}
	            
    }
  else
    return 0;

  return 1;
}


void masterProcess(tree *tr, analdef *adef)
{
  double rSpec[LENGTH], *rSpecPtr;
  MPI_Status msgStatus;
  int msgFrom, i, improved, loopCounter;
  double accProbeTime = 0;
  double time;
  double accRecvTime = 0;
  double accRestoreTime = 0;

  bestLikelihood = tr->likelihood;

  impCnt = 0;

  startOptTime = gettime();

  loopCounter = 0;

  while(1)
    {
      time = gettime();

      MPI_Probe(MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, &msgStatus);

      time = gettime() - time;
      accProbeTime += time;
     
      switch(msgStatus.MPI_TAG)
	{
	case REARRANGEMENT_SPECIFICATION:
	  msgFrom = msgStatus.MPI_SOURCE;	     
	  
	  time = gettime();

	  MPI_Recv(rSpec, LENGTH, MPI_DOUBLE, msgFrom, REARRANGEMENT_SPECIFICATION, MPI_COMM_WORLD, &msgStatus);          	  

	  time = gettime() - time;
	  accRecvTime += time;
	  
	  improved = rSpec[0];


	  time = gettime();

	  for(i = 0; i < improved; i++)
	    {
	      rSpecPtr = &rSpec[1 + i * 10];
	    
	      Thorough = (int) rSpecPtr[0];
	      tr->removedNumberP = (int) rSpecPtr[1];
	      tr->removedNumberQ = (int) rSpecPtr[2];
	      tr->removedNumberR = (int) rSpecPtr[3];
	      tr->insertNumberQ = (int) rSpecPtr[4];
	      tr->insertNumberR = (int) rSpecPtr[5];
	      tr->currentZQR =  rSpecPtr[6];
	      tr->currentLZQ =  rSpecPtr[7];
	      tr->currentLZR =  rSpecPtr[8];
	      tr->currentLZS =  rSpecPtr[9];	 
	      
	      restoreTreeFastPARALLEL(tr);
	    }	  

	  time = gettime() - time;
	  accRestoreTime += time;
	  
	  loopCounter++;

	  if(loopCounter % 10 == 0)
	    printf("MASTER breakdwon: Prpbe %f Recv %f Restore %f\n", accProbeTime, accRecvTime ,accRestoreTime);
	}
    }	      
}


void workerProcess(tree *tr, analdef *adef)
{
  int mintrav = 1;
  int maxtrav = 10;
  double rSpec[LENGTH], rSpecIn[LENGTH], *rSpecPtr,  *rSpecInPtr;
  int node, i, msgFrom, receivedSomething, improved, upper, loopCounter;
  MPI_Request  msgRequest;
  MPI_Status msgStatus; 
  double time;
  double startTime;
  double currentBest;
  
  double accRecvTime = 0;
  double accOptTime = 0;
  double accSendTime = 0;
  double accRestTime = 0;
  double globalBest = unlikely;

  if (maxtrav > tr->ntips - 3)  
    maxtrav = tr->ntips - 3;  
     
  startTime = gettime();

  srand((unsigned int) gettimeSrand());

  tr->startLH = tr->endLH = tr->likelihood;
  
  Thorough = 0;
 
  bestLikelihood = tr->likelihood;

  loopCounter = 0;

  while(1)
    {	

      time = gettime();

      MPI_Iprobe(MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, &receivedSomething, &msgStatus);

      time = gettime() - time;
      accRecvTime += time;
      
      

      while(receivedSomething)
	{
	  switch(msgStatus.MPI_TAG)
	    {
	    case REARRANGEMENT_SPECIFICATION:
	      msgFrom = msgStatus.MPI_SOURCE;
	      
	      time = gettime();
	      
	      MPI_Recv(rSpecIn, LENGTH, MPI_DOUBLE, msgFrom, REARRANGEMENT_SPECIFICATION, MPI_COMM_WORLD, &msgStatus);          	  
	      
	      time = gettime() - time;
	      accRecvTime += time;

	      improved = rSpecIn[0];
	     

	      time = gettime();

	      for(i = 0; i < improved; i++)
		{
		  rSpecInPtr = &rSpecIn[2 + 10 * i];
		  
		  Thorough = (int) rSpecInPtr[0];
		  tr->removedNumberP = (int) rSpecInPtr[1];
		  tr->removedNumberQ = (int) rSpecInPtr[2];
		  tr->removedNumberR = (int) rSpecInPtr[3];
		  tr->insertNumberQ = (int) rSpecInPtr[4];
		  tr->insertNumberR = (int) rSpecInPtr[5];
		  tr->currentZQR =  rSpecInPtr[6];
		  tr->currentLZQ =  rSpecInPtr[7];
		  tr->currentLZR =  rSpecInPtr[8];
		  tr->currentLZS =  rSpecInPtr[9];	      
		  
		  restoreTreeFastPARALLEL(tr);	
		}
	      

	      if(rSpecIn[1] > globalBest)
		

	      if(myId == 0)
		{

		  currentBest = MAX(rSpecIn[1], bestLikelihood);

		  

		  if(rSpecIn[1] > currentBest)
		    {
		      globalBest = rSpecIn;
		      printf("%f %f\n", gettime() - startTime, globalBest);
		    }
		}
			

	      time = gettime() - time;
	      accRestTime += time;

	    }
	  
	  time = gettime();

	  MPI_Iprobe(MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, &receivedSomething, &msgStatus);
	  
	  time = gettime() - time;
	  accRecvTime += time;
	  
	}
      
 		         
      node = 1 + randomInt(2 * tr->mxtips - 2);

      /*      node = 1 + ((myId - 1) * 100) + (10 * loopCounter);*/
	
      improved = 0;

      upper = node + STEPS;

      if(upper > 2 * tr->mxtips - 2)
	upper =  2 * tr->mxtips - 2;

      time = gettime();

      for(i = node; i <= upper; i++)
        {
	 
	  rearrangePARALLEL(tr, tr->nodep[i], mintrav, maxtrav);
	  	 	  
	  if(tr->endLH > tr->startLH)                 	
	    {		 
	      restoreTreeFast(tr);
		
	      tr->startLH = tr->endLH = tr->likelihood;	  	  		 

	      rSpecPtr = &rSpec[2 + improved * 10];
	      
	      rSpecPtr[0] = (double) Thorough;
	      rSpecPtr[1] = (double) tr->removedNumberP;
	      rSpecPtr[2] = (double) tr->removedNumberQ;
	      rSpecPtr[3] = (double) tr->removedNumberR;
	      rSpecPtr[4] = (double) tr->insertNumberQ;
	      rSpecPtr[5] = (double) tr->insertNumberR;
	      rSpecPtr[6] = tr->currentZQR;
	      rSpecPtr[7] = tr->currentLZQ;
	      rSpecPtr[8] = tr->currentLZR;
	      rSpecPtr[9] = tr->currentLZS;	      
	      improved++;
	    }
	}

      time = gettime() - time;
      accOptTime += time;

      
      /*MPI_Send(rSpec, 16, MPI_DOUBLE, 0, REARRANGEMENT_SPECIFICATION, MPI_COMM_WORLD);*/

      /*MPI_Isend(rSpec, 16, MPI_DOUBLE, 0, REARRANGEMENT_SPECIFICATION, MPI_COMM_WORLD, &msgRequest);*/


      if(improved)
	{
	  rSpec[0] = improved;
	  rSpec[1] = tr->endLH;
	  
	  time = gettime();
	  for(i = 0; i < numOfWorkers; i++)
	    {
	      if(i != myId)
		MPI_Isend(rSpec, LENGTH, MPI_DOUBLE, i, REARRANGEMENT_SPECIFICATION, MPI_COMM_WORLD, &msgRequest);	  	     
	    }
	  
	  time = gettime() - time;
	  accSendTime += time;
	}
      
      loopCounter++;

      /*if(loopCounter % 10 == 0 && myId == 1)
	printf("WORKER %d breakdown at %f Recv %f Sned %f Opt %f Restore %f LH %f\n", myId, gettime() - startTime, accRecvTime, 
	accSendTime, accOptTime, accRestTime, bestLikelihood);*/

    }	       
}




void computePARALLEL (tree *tr, analdef *adef) 
  {       
    initInfoList(20);              
    	
    optimizeModel(tr, adef, 1);       
    treeEvaluate(tr, 2);		

    if(myId == 0)
      printf("Master\n");
    else
      printf("Worker %d\n", myId - 1);
		   
    if(myId == 0)
      masterProcess(tr, adef);
    else
      workerProcess(tr, adef);
  }


#endif










/************************************ Compute PARALLEL *********************************************/





void computeHeavy (tree *tr, analdef *adef) 
  { 
    int i, rearrangementsMax, rearrangementsMin, noImproveCount, impr;                
    double lh, previousLh, difference, epsilon;
    bestlist *bestT;

    bestT = (bestlist *) malloc(sizeof(bestlist));
    bestT->ninit = 0;
    initBestTree(bestT, 1, tr->mxtips, tr->cdta->endsite);
  
    difference = 10.0;
    epsilon = 0.5;
    
    Multiple = 0;
    rearrangementsMin = 1;
    rearrangementsMax = 5;
    Thorough = 1;  
        
    if(adef->model == M_PROTEINS || adef->model == M_PROTEINSCAT)
      {    
	epsilon = 0.1;
      }


    resetBestTree(bt);            
    lh = tr->likelihood;    
    saveBestTree(bestT, tr);  
   
    impr = 1;		  		  
 
    while(1)
      {	

	recallBestTree(bestT, 1, tr);		
	optimizeModel(tr, adef, 1);       
	treeEvaluate(tr, 2);		
	
	lh = tr->likelihood;
	saveBestTree(bestT, tr);
	printLog(tr, adef);	
	checkTime(adef, tr);
     	
	if(impr)
	  {	 	   
	    noImproveCount = 0;
	    printResult(tr, 0);	   	    
	  }			  			
	else	  
	  goto cleanup; 	   
	         
	treeOptimize(tr, rearrangementsMin, rearrangementsMax, adef);
	
	impr = 0;			      	
	previousLh = lh;

	checkTime(adef, tr);
      
	for(i = 1; i <= bt->nvalid; i++)
	  {	    		  	   
	    recallBestTree(bt, i, tr);	    
	    treeEvaluate(tr, 0.25);	    
	    difference = ((tr->likelihood > previousLh)? 
			  tr->likelihood - previousLh: 
			  previousLh - tr->likelihood); 	    
	    if(tr->likelihood > lh && difference > epsilon)
	      {
		impr = 1;	       
		lh = tr->likelihood;	       
		saveBestTree(bestT, tr);		
	      }	   	   
	  }
	
      }
    
  cleanup: 
    recallBestTree(bestT, 1, tr);
    printLog(tr, adef);
    printResult(tr, 0);	   
    freeBestTree(bestT);
  }

void computeFast (tree *tr, analdef *adef) 
{ 
  int i, mintrav, maxtrav, which;
  double initialLH, epsilon, diff;
 
  epsilon = 0.01;
  
  Thorough = 0;

  if(adef->model == M_PROTEINS || adef->model == M_PROTEINSCAT)
    Thorough = 1;

  Multiple = 1;
  mintrav = 1;
  maxtrav = 8;
  
  if (maxtrav > tr->ntips - 3)  maxtrav = tr->ntips - 3;
          
  tr->opt_level = 0;
  optimizeModel(tr, adef, 1);
  resetBestTree(bt);
  saveBestTree(bt, tr);

  do
    {
      printLog(tr, adef);
      printResult(tr, 0);

      initialLH = tr->likelihood;
      tr->startLH = tr->endLH = tr->likelihood;

      for(i = 1; i < (2 * tr->mxtips - 1); i++)
	{ 
	  
	  rearrangeParsimony(tr, tr->nodep[i], mintrav, maxtrav);
	  if(tr->endLH > tr->startLH)   
	    {
	      recallBestTree(bt, 1, tr);
	      tr->startLH = tr->endLH = tr->likelihood;		    
	    }
	 
	}
      recallBestTree(bt, 1, tr);      
      treeEvaluate(tr, 1);
      checkTime(adef, tr);
      saveBestTree(bt, tr);
      diff = (tr->likelihood > initialLH)? tr->likelihood - initialLH: -1.0;            
    }
  while(diff > epsilon);

  initialLH = unlikely;

  for(i = 1; i <= bt->nvalid; i++)
    {	    	    	
      recallBestTree(bt, i, tr);			    
      smoothTree(tr, smoothings/4);
      evaluate(tr, tr->start);		           
      if(tr->likelihood > initialLH)
	{
	  which = i;
	  initialLH = tr->likelihood;
	}           
    }

  recallBestTree(bt, which, tr);
  treeEvaluate(tr, 1);
  checkTime(adef, tr);
  optimizeModel(tr, adef, 1);
  printLog(tr, adef);
  printResult(tr, 0);
}


double calcPartialLikelihood(tree *tr, int i, double ki)
{     
  initravPartial(tr, tr->start, i, ki);
  initravPartial(tr, tr->start->back, i, ki);
  return evaluatePartial(tr, tr->start, i, ki);
}

boolean newviewSPECIAL (tree *tr, nodeptr p)        /*  Update likelihoods at node */
  { /* newview */
    double   zq, lzq, xvlzq, zr, lzr, xvlzr, tip_t;
    nodeptr  q, r;
    likelivector *lp, *lq, *lr;
    int  i;
   
    if (p->tip) {             /*  Make sure that data are at tip */      
      likelivector *l;
      int           code;
      yType        *yptr;
      
      if (p->x) 
	{	 
	  return TRUE;  
	}

      if (! getxtip(p)) return FALSE; /*  They are not, so get memory */
      l = p->x->lv;           /*  Pointer to first likelihood vector value */
      yptr = p->tip;          /*  Pointer to first nucleotide datum */
      for (i = 0; i < tr->cdta->endsite; i++) {
        code = *yptr++;
        l->a =  code       & 1;
        l->c = (code >> 1) & 1;
        l->g = (code >> 2) & 1;
        l->t = (code >> 3) & 1;
        l->exp = 0;

	
        l++;
        }
    
      return TRUE;
      }

/*  Internal node needs update */

    q = p->next->back;
    r = p->next->next->back;

    

    while ((! p->x) || (! q->x) || (! r->x)) {
      if (! q->x) if (! newviewSPECIAL(tr, q))  return FALSE;
      if (! r->x) if (! newviewSPECIAL(tr, r))  return FALSE;
      if (! p->x) if (! getxnode(p)) return FALSE;
      }

    lp = p->x->lv;

    lq = q->x->lv;
    zq = q->z;
    lzq = (zq > zmin) ? log(zq) : log(zmin);
    xvlzq = tr->rdta->xv * lzq;

    lr = r->x->lv;
    zr = r->z;
    lzr = (zr > zmin) ? log(zr) : log(zmin);
    xvlzr = tr->rdta->xv * lzr;

    { 
      double  fxqr, fxqy, fxqn, sumaq, sumgq, sumcq, sumtq,
              fxrr, fxry, fxrn, ki, tempi, tempj;
    
      double zzq, zvq, zzr, zvr;
      int cat;

     
    
      

    
     

        for (i = 0; i < tr->cdta->endsite; i++) {
          ki = tr->cdta->patratStored[i];
	    
	  
          zzq = exp(ki *   lzq);
          zvq = exp(ki * xvlzq);
	  
          fxqr = tr->rdta->freqa * lq->a + tr->rdta->freqg * lq->g;
          fxqy = tr->rdta->freqc * lq->c + tr->rdta->freqt * lq->t;
          fxqn = fxqr + fxqy;
          tempi = fxqr * tr->rdta->invfreqr;
          tempj = zvq * (tempi-fxqn) + fxqn;
          sumaq = zzq * (lq->a - tempi) + tempj;
          sumgq = zzq * (lq->g - tempi) + tempj;
          tempi = fxqy * tr->rdta->invfreqy;
          tempj = zvq * (tempi-fxqn) + fxqn;
          sumcq = zzq * (lq->c - tempi) + tempj;
          sumtq = zzq * (lq->t - tempi) + tempj;

          zzr = exp(ki *   lzr);
          zvr = exp(ki * xvlzr);
          fxrr = tr->rdta->freqa * lr->a + tr->rdta->freqg * lr->g;
          fxry = tr->rdta->freqc * lr->c + tr->rdta->freqt * lr->t;
          fxrn = fxrr + fxry;
          tempi = fxrr * tr->rdta->invfreqr;
          tempj = zvr * (tempi-fxrn) + fxrn;
          lp->a = sumaq * (zzr * (lr->a - tempi) + tempj);
          lp->g = sumgq * (zzr * (lr->g - tempi) + tempj);
          tempi = fxry * tr->rdta->invfreqy;
          tempj = zvr * (tempi-fxrn) + fxrn;
          lp->c = sumcq * (zzr * (lr->c - tempi) + tempj);
          lp->t = sumtq * (zzr * (lr->t - tempi) + tempj);
          lp->exp = lq->exp + lr->exp;
	 
	  

          if (lp->a < minlikelihood && lp->g < minlikelihood
           && lp->c < minlikelihood && lp->t < minlikelihood) {
            lp->a   *= twotothe256;
            lp->g   *= twotothe256;
            lp->c   *= twotothe256;
            lp->t   *= twotothe256;
            lp->exp += 1;
            }
          lp++;
          lq++;
          lr++;
          }


      return TRUE;
      }
  } /* newview */






boolean initravP (tree *tr, nodeptr p)
  { /* initrav */
    nodeptr  q;

    if (! p->tip) {      
      q = p->next;      
      do 
	{
	  if (! initravP(tr, q->back))  return FALSE;	
	  q = q->next;	
        } while (q != p);
            
      if (! newviewSPECIAL(tr, p)) return FALSE;	      
      }

    return TRUE;
  } /* initrav */




void partialEval(tree *tr, double *lhs, nodeptr p)
{ 
  double   sum, z, lz, xvlz,
    ki, fxpa, fxpc, fxpg, fxpt, fxpr, fxpy, fxqr, fxqy,
    suma, sumb, sumc, term;
  
  double   zz, zv;
  likelivector *lp, *lq;
  nodeptr       q;
  int            i, *wptr;

  q = p->back;
  while ((! p->x) || (! q->x)) {
    if (! (p->x)) if (! newviewSPECIAL(tr, p)) return;
    if (! (q->x)) if (! newviewSPECIAL(tr, q)) return;
    }

  lp = p->x->lv;
  lq = q->x->lv;
  
  z = p->z;
  if (z < zmin) z = zmin;
  lz = log(z);
  xvlz = tr->rdta->xv * lz;
  
 
  
  wptr = &(tr->cdta->aliaswgt[0]);
  
  
 
  
  
  for (i = 0; i < tr->cdta->endsite; i++) {
    ki = tr->cdta->patratStored[i];
    zz   = exp(ki *   lz);
    zv   = exp(ki * xvlz);
    fxpa = tr->rdta->freqa * lp->a;
    fxpg = tr->rdta->freqg * lp->g;
    fxpc = tr->rdta->freqc * lp->c;
    fxpt = tr->rdta->freqt * lp->t;
    suma = fxpa * lq->a + fxpc * lq->c + fxpg * lq->g + fxpt * lq->t;
    fxqr = tr->rdta->freqa * lq->a + tr->rdta->freqg * lq->g;
    fxqy = tr->rdta->freqc * lq->c + tr->rdta->freqt * lq->t;
    fxpr = fxpa + fxpg;
    fxpy = fxpc + fxpt;
    sumc = (fxpr + fxpy) * (fxqr + fxqy);
    sumb = fxpr * fxqr * tr->rdta->invfreqr + fxpy * fxqy * tr->rdta->invfreqy;
    suma -= sumb;
    sumb -= sumc;
    term = log(zz * suma + zv * sumb + sumc) + (lp->exp + lq->exp)*log(minlikelihood);
    
    lhs[i] = *wptr++ * term;
        
    lp++;
    lq++;
  }
}


void calcPartialIni(tree *tr, double *lhs)
{
   initravP(tr, tr->start);
   initravP(tr, tr->start->back);
   partialEval(tr, lhs, tr->start);
}

static int doublecompare2(const void *p1, const void *p2)
{
  double i = *((double *)p1);

  double j = *((double *)p2);
  
  if (i > j)
    return (1);
  if (i < j)
    return (-1);
  return (0);
}

static int catCompare(const void *p1, const void *p2)
{
 rateCategorize *rc1 = (rateCategorize *)p1;
 rateCategorize *rc2 = (rateCategorize *)p2;

  double i = rc1->accumulatedSiteLikelihood;
  double j = rc2->accumulatedSiteLikelihood;
  
  if (i > j)
    return (1);
  if (i < j)
    return (-1);
  return (0);
}



void categorize(tree *tr, rateCategorize *rc)
{
  int i, k, found;
  double temp, diff, min;

  for (i = 0; i < tr->cdta->endsite; i++) 
      {
	temp = tr->cdta->patrat[i];
	found = 0;
	for(k = 0; k < tr->NumberOfCategories; k++)
	  {
	    if(temp == rc[k].rate || (fabs(temp - rc[k].rate) < 0.001))
	      {
		found = 1;
		tr->cdta->rateCategory[i] = k;				
		break;
	      }
	  }
	if(!found)
	  {
	    min = fabs(temp - rc[0].rate);
	    tr->cdta->rateCategory[i] = 0;

	    for(k = 1; k < tr->NumberOfCategories; k++)
	    {
	      diff = fabs(temp - rc[k].rate);
	      if(diff < min)
		{
		  min = diff;
		  tr->cdta->rateCategory[i] = k;
		}
	    }
	  }
      }

  for(k = 0; k < tr->NumberOfCategories; k++)
    tr->cdta->patrat[k] = rc[k].rate; 

}




void optimizeRateCategories(tree *tr, int categorized, int _maxCategories)
{
  int i, k;
  double initialRate, initialLikelihood, v, leftRate, rightRate, leftLH, rightLH, temp, wtemp;   
  double lower_spacing, upper_spacing;
  int maxCategories = _maxCategories;
  double initialLH = tr->likelihood;
  double *oldRat = (double *)malloc(sizeof(double) * tr->cdta->endsite);
  double *ratStored = (double *)malloc(sizeof(double) * tr->cdta->endsite);
  double *oldwr = (double *)malloc(sizeof(double) * tr->cdta->endsite);
  double *oldwr2 = (double *)malloc(sizeof(double) * tr->cdta->endsite);
  double *lhs = (double *)malloc(sizeof(double) * tr->cdta->endsite);
  int *oldCategory = (int *)malloc(sizeof(int) * tr->cdta->endsite);  
  int oldNumber; 
  double epsilon = 0.00001;
  double optt = gettime();
  double help;
  double accT = 0;

  if(optimizeRateCategoryInvocations == 1)
    {
      lower_spacing = 0.5 / ((double)optimizeRateCategoryInvocations);
      upper_spacing = 1.0 / ((double)optimizeRateCategoryInvocations);
    }
  else
    {
      lower_spacing = 0.05 / ((double)optimizeRateCategoryInvocations);
      upper_spacing = 0.1 / ((double)optimizeRateCategoryInvocations);
    }

  if(lower_spacing < 0.001)
    lower_spacing = 0.001;

  if(upper_spacing < 0.001)
    upper_spacing = 0.001;

  optimizeRateCategoryInvocations++;
  
  oldNumber = tr->NumberOfCategories;
  for(i = 0; i < tr->cdta->endsite; i++)
    {    
      oldCategory[i] = tr->cdta->rateCategory[i];
      ratStored[i] = tr->cdta->patratStored[i];    
      oldRat[i] = tr->cdta->patrat[i];
      oldwr[i] =  tr->cdta->wr[i];
      oldwr2[i] =  tr->cdta->wr2[i];
    }

  /*  calcPartialIni(tr, lhs);*/

  for(i = 0; i < tr->cdta->endsite; i++)
    {    
      tr->cdta->patrat[i] = tr->cdta->patratStored[i];     
      initialRate = tr->cdta->patrat[i];
      
      initialLikelihood = calcPartialLikelihood(tr, i, initialRate);
      
      /*initialLikelihood = lhs[i];*/
      leftLH = rightLH = initialLikelihood;
      leftRate = rightRate = initialRate;
       
      k = 1;
      while((initialRate - k * lower_spacing > 0.0001) && 
	    ((v = calcPartialLikelihood(tr, i, initialRate - k * lower_spacing)) > leftLH) && 
	    (fabs(leftLH - v) > epsilon))
	{	  
	  leftLH = v;
	  leftRate = initialRate - k * lower_spacing;
	  k++;	  
	}      
   
      k = 1;
      while(((v = calcPartialLikelihood(tr, i, initialRate + k * upper_spacing)) > rightLH) &&
	    (fabs(rightLH - v) > epsilon))
	{
	  rightLH = v;
	  rightRate = initialRate + k * upper_spacing;	 
	  k++;
	}
      
     


      if(rightLH > initialLikelihood || leftLH > initialLikelihood)
	{
	  if(rightLH > leftLH)	    
	    {	     
	      tr->cdta->patrat[i] = rightRate;
	      lhs[i] = rightLH;
	    }
	  else
	    {	      
	      tr->cdta->patrat[i] = leftRate;
	      lhs[i] = leftLH;
	    }
	}
      else
	lhs[i] = initialLikelihood;

      tr->cdta->patratStored[i] = tr->cdta->patrat[i];     
      
    }
     
 
  

  {     
    rateCategorize *rc = (rateCategorize *)malloc(sizeof(rateCategorize) * tr->cdta->endsite);
    int where;
    int found = 0;
    for (i = 0; i < tr->cdta->endsite; i++)
      {
	rc[i].accumulatedSiteLikelihood = 0;
	rc[i].rate = 0;
      }
      
    where = 1;   
    rc[0].accumulatedSiteLikelihood = lhs[0];/*calcPartialLikelihood(tr, 0, tr->cdta->patrat[0]);*/
    rc[0].rate = tr->cdta->patrat[0];
    tr->cdta->rateCategory[0] = 0;
    
    for (i = 1; i < tr->cdta->endsite; i++) 
      {
	temp = tr->cdta->patrat[i];
	found = 0;
	for(k = 0; k < where; k++)
	  {
	    if(temp == rc[k].rate || (fabs(temp - rc[k].rate) < 0.001))
	      {
		found = 1;						
		rc[k].accumulatedSiteLikelihood += lhs[i];
		/*		  calcPartialLikelihood(tr, i, temp);*/
		break;
	      }
	  }
	if(!found)
	  {	    
	    rc[where].rate = temp;	    
	    rc[where].accumulatedSiteLikelihood += lhs[i];
	    /*	      calcPartialLikelihood(tr, i, temp);*/
	    where++;
	  }
	}

    qsort(rc, where, sizeof(rateCategorize), catCompare);

    if(where < maxCategories)
      {
	tr->NumberOfCategories = where;
	categorize(tr, rc);
      }
    else
      {
	tr->NumberOfCategories = maxCategories;	
	categorize(tr, rc);
      }
      
    free(rc);
  
    for (i = 0; i < tr->cdta->endsite; i++) 
      {
	/*printf("%d %f\n", i, tr->cdta->patrat[tr->cdta->rateCategory[i]]);*/
	temp = tr->cdta->patrat[tr->cdta->rateCategory[i]];

	tr->cdta->wr[i]  = wtemp = temp * tr->cdta->aliaswgt[i];
	tr->cdta->wr2[i] = temp * wtemp;
      }
      
    initrav(tr, tr->start);
    initrav(tr, tr->start->back);
    evaluate(tr, tr->start);
   

    if(tr->likelihood < initialLH)
      {
	/*printf("No improvement->restoring %f < %f\n", tr->likelihood, initialLH);*/
	tr->NumberOfCategories = oldNumber;
	for (i = 0; i < tr->cdta->endsite; i++)
	    {
	      tr->cdta->patratStored[i] = ratStored[i]; 
	      tr->cdta->rateCategory[i] = oldCategory[i];
	      tr->cdta->patrat[i] = oldRat[i];	    
	      tr->cdta->wr[i]  = oldwr[i];
	      tr->cdta->wr2[i] = oldwr2[i];
	    } 
	 	  
	initrav(tr, tr->start);
	initrav(tr, tr->start->back);
        evaluate(tr, tr->start);

      }
    }
  free(oldCategory);
  free(oldRat);
  free(ratStored);
  free(oldwr);
  free(oldwr2); 
  free(lhs);

  /*  printf("OPT-Time: %f CATS %d LH %f INITIME %f\n", gettime() - optt, tr->NumberOfCategories, tr->likelihood, accT); */
}
  


void columnAnalysis(tree *tr, int i)
{
  int j;
  int counters[16];
  int val;

  for(j = 0; j < 16; j++)
    counters[j] = 0;

  for(j = 1; j <= tr->mxtips; j++)
    {
      val = tr->nodep[j]->tip[i];     
      counters[val] = counters[val] + 1;
    }


  printf("%d ", counters[1]);
  printf("%d ", counters[2]);
  printf("%d ", counters[4]);
  printf("%d ", counters[8]);
  printf("%d ", counters[15]);
  printf("\n");
}







int optimizeModel (tree *tr, analdef *adef, int finalOptimization)
{
  double startLH = tr->likelihood;
  double initialLH;
  int oldInv;

  switch(adef->model)
    {
    case M_PROTEINSCAT:
      {
	oldInv = optimizeRateCategoryInvocations;
	optimizeRateCategories(tr, finalOptimization, adef->categories);
      }
      break;
    case M_GTR:
      {
	oldInv = optimizeRatesInvocations;
	do
	  {	    
	    initialLH = tr->likelihood;
	    optimizeRates(tr);       
	  }
	while(tr->likelihood > initialLH && optimizeRatesInvocations < oldInv + 10);   
      }
      break;
    case M_GTRCAT:
       {	 
	 oldInv = optimizeRatesInvocations;
	 do
	   {	    
	     initialLH = tr->likelihood;
	     optimizeRates(tr);       
	   }
	 while(tr->likelihood > initialLH && optimizeRatesInvocations < oldInv + 10);  	



	 oldInv = optimizeRateCategoryInvocations;
	 optimizeRateCategories(tr, finalOptimization, adef->categories);	
       }
      break;
    case M_HKY85:
      {
	oldInv = optimizeTTinvocations;
	
	do
	  {
	    initialLH = tr->likelihood;
	    optimizeTT(tr);
	  }
	while(tr->likelihood > initialLH && optimizeTTinvocations < oldInv + 10);      
      }
      break;
    case M_GTRGAMMA:
       {
	 oldInv = optimizeRatesInvocations;

	 do
	   {	    
	     initialLH = tr->likelihood;
	     optimizeRates(tr);       
	   }
	 while(tr->likelihood > initialLH && optimizeRatesInvocations < oldInv + 10);

	 oldInv = optimizeAlphaInvocations;

	 do
	   {  
	     initialLH = tr->likelihood;
	     optimizeAlpha(tr);
	   }
	 while(tr->likelihood > initialLH && optimizeAlphaInvocations < oldInv + 10);      
	 
       }
      break;         
    case M_HKY85GAMMA:
       {
	oldInv = optimizeTTinvocations;
	
	do
	  {
	    initialLH = tr->likelihood;
	    optimizeTT(tr);
	  }
	while(tr->likelihood > initialLH && optimizeTTinvocations < oldInv + 10);      


	oldInv = optimizeAlphaInvocations;

	do
	  {  
	    initialLH = tr->likelihood;
	    optimizeAlpha(tr);
	  }
	while(tr->likelihood > initialLH && optimizeAlphaInvocations < oldInv + 10);      
	
	}
      break;
    case M_HKY85CAT:
       {
	oldInv = optimizeTTinvocations;
	
	do
	  {
	    initialLH = tr->likelihood;
	    optimizeTT(tr);
	  }
	while(tr->likelihood > initialLH && optimizeTTinvocations < oldInv + 10); 

	oldInv = optimizeRateCategoryInvocations;
	optimizeRateCategories(tr, finalOptimization, adef->categories);
       }
    }

 

  if(optimizeRatesInvocations > 90)
    optimizeRatesInvocations = 90;
  if(optimizeTTinvocations > 90)
    optimizeTTinvocations = 90;
  if(optimizeRateCategoryInvocations > 90)
    optimizeRateCategoryInvocations = 90;
  if(optimizeAlphaInvocations > 90)
    optimizeAlphaInvocations = 90;

  if(startLH > tr->likelihood) return 0;
  else return 1;
}

void consensusTree(bestlist *consensusList, tree *tr)
{
  
  FILE *temporary;  
  char consensusFileName[1024], str[16], infileName[1024],
    consensusCommand[1024];
  int i;
  bestlist *bestT;

  strcpy(infileName,workdir);
  strcat(infileName,"RAxML_TreesForConsensus.");
  strcat(infileName,run_id);
  
  strcpy(consensusFileName,workdir);
  strcat(consensusFileName,"RAxML_consensus.");
  strcat(consensusFileName,run_id);
  strcat(consensusFileName,".");
  sprintf(str, "%d", consensusTreeCounter);
  strcat(consensusFileName, str);
  printf("Consensus Tree written to RAxML_consensus.%s.%s\n",
	  run_id, str);

  
  strcpy(consensusCommand,"./consense ");
  strcat(consensusCommand, infileName);
  strcat(consensusCommand," "); 
  strcat(consensusCommand, consensusFileName);

  temporary = fopen(infileName, "w");

  bestT = (bestlist *) malloc(sizeof(bestlist));
  bestT->ninit = 0;
  initBestTree(bestT, 1, tr->mxtips, tr->cdta->endsite);
  saveBestTree(bestT, tr);
  
  infoFile = fopen(infoFileName, "a");

  for(i = 1; i <= consensusList->nvalid; i++)
    {
      recallBestTree(consensusList, i, tr);
      if(i == 1)
	fprintf(infoFile, "CONSENSUS(%d): best LH %f ",consensusTreeCounter, tr->likelihood);
      if(i == consensusList->nvalid)
	fprintf(infoFile, " <-> worst LH %f out of %d trees\n",tr->likelihood, consensusList->nvalid);
      tr->start = tr->nodep[1];
      printTree(temporary, tr, 0);      
    }

  fclose(infoFile);

  fflush(temporary);
  fclose(temporary);

  system(consensusCommand);

  recallBestTree(bestT, 1, tr);
  freeBestTree(bestT);
  consensusTreeCounter++;
}





void simulatedAnnealing(tree *tr, analdef *adef)
{
  double lh1, lh2, best, p1, p2, deltaf, alpha, t0;
  int repeats;
  int node, mintrav, maxtrav, rn, c_imp, selectMove, counter, topologyMove;
  bestlist *consensusList, *bestT;
  int maxR, i, statt, stati;
  int *subtreeStats, *nodeList, nodeCount;
  

  srand((unsigned int) time(NULL));

  bestT = (bestlist *) malloc(sizeof(bestlist));
  bestT->ninit = 0;
  initBestTree(bestT, 1, tr->mxtips, tr->cdta->endsite);

  maxR = adef->max_rearrange; 

  subtreeStats = (int *)malloc( 2 * tr->mxtips * sizeof(int));
  nodeList = (int *)malloc( 10 * tr->mxtips * sizeof(int));

  for(i = 0; i < 2 * tr->mxtips; i++)
    subtreeStats[i] = 0;

  nodeCount = 0;
  for(i = 0; i < 2 * tr->mxtips - 2; i++)
    {
      nodeList[i] = i + 1;
      nodeCount++;
    }
  /*for(i = 0; i < tr->mxtips - 2; i++)
    {
      nodeList[nodeCount] = tr->mxtips + 1 + i;
      nodeCount++;
      }*/

  if(buildConsensus)
    {
      consensusList = (bestlist *) malloc(sizeof(bestlist));
      consensusList->ninit = 0;
      initBestTree(consensusList, 100, tr->mxtips, tr->cdta->endsite);
    }
  
  Multiple = 0;

  /* TODO passt noch nicht fuer prots*/

  if(adef->model == M_PROTEINS || adef->model == M_PROTEINSCAT)
    {
      Thorough = 1;
    }
  else
    Thorough = 0;
   
  optimizeModel(tr, adef, 1);
  treeEvaluate(tr, 4);

  alpha = 0.9;
  best = unlikely;
  t0 = adef->t0;
  
  if(adef->repeats == 0)
    adef->repeats = tr->mxtips;
  else
    repeats = adef->repeats;

  

  c_imp = 0;
  counter = 0;

 

  while(1)
    {      
      lh1 = tr->likelihood;
      
      if(lh1 > best)
	{
	  best = lh1;
	  printLog(tr, adef);	 
	  printResult(tr, 0);
	}
      checkTime(adef, tr);
      resetBestTree(bestT);
      saveBestTree(bestT, tr);
      resetBestTree(bt);
     
      node = 1 + randomInt(2 * tr->mxtips - 2);

      /*node = nodeList[randomInt(nodeCount)];*/
      
      selectMove = randomInt(100);
      
      /*printf("Node %d SelectMove %d\n", node, selectMove);*/

      if(selectMove < 95)
	{
	  topologyMove = 1;
	  mintrav = 1;	  
	  maxtrav = 2 + randomInt(maxR);  	  
	  rearrangeParsimony(tr, tr->nodep[node], mintrav, maxtrav);
	  recallBestTree(bt, 1, tr);	        	  	  	 
	  evalFast(tr);
	  c_imp++;
	}
      else	
	{
	  topologyMove = 0;
	  selectMove = randomInt(100);
	  
	  if(selectMove < 95)
	    {	      
	      nodeptr p;
	      nodeptr q;
	      double z0, z;
	      
	      p = tr->nodep[node];
	      
	      q = p->back;
	      z0 = q->z;		
	      z = makenewz(tr, p, q, z0, newzpercycle); 	  
	      p->z = q->z = z;
	      
	      evaluate(tr, tr->start);	
	    }
	  else
	    {
	      optimizeModel(tr, adef, 1);
	      treeEvaluate(tr, 4);
	    }
	}
      
     
      lh2 = tr->likelihood;

      if(lh1 > lh2)
	{	  
	  deltaf = lh2 - lh1;
	  p1 = exp( deltaf / t0);
	  rn = randomInt(10000);
	  if(rn == 0) 
	    p2 = 0;
	  else 
	    p2 = ((double)rn)/10000.0;	  	  

	  if(buildConsensus && topologyMove && p1 < p2)
	    saveBestTree(consensusList, tr);	
	  if(p1 < p2)	  
	    recallBestTree(bestT, 1, tr);	  
	  else
	    {
	      /*printf("BACKWARD %f -> %f \n ", lh1, lh2);	     	     */
	    }
	}    
      else
	{
	  if(topologyMove)
	    {
	      subtreeStats[node] =  subtreeStats[node] + 1;
	      if(buildConsensus)
		saveBestTree(consensusList, tr);
	    }
	  /* TODO introduce penalties for missed moves ?*/
	  /* increase node selection probability while computation advances ? */
	}

      if(c_imp >= repeats)
	{
	  t0 = t0 * alpha;	  
	  c_imp = 0;
	}
     
      counter++;     

      if((counter % (2 * tr->mxtips)) == 0) 	
	{
	  if(buildConsensus) 
	    consensusTree(consensusList, tr);
	  /*
	    statt = 0;
	    stati = 0;
	    for(i = 0; i < 2 * tr->mxtips; i++)
	    {
	    if(subtreeStats[i] > 0)
	    {
	    printf("%d %d\n", i, subtreeStats[i]);
	    
	    if(i <= tr->mxtips)
	    statt++;
	    else
	    stati++;
	    }
	    }
	    printf("move-nodes tips %d inner %d\n", statt, stati);*/
	  counter = 0;
	}          
    }          
}







void treeEvaluation(tree *tr, analdef *adef)
{
  int i;
  
  for(i = 0; i < 10; i++)
    {
      optimizeModel(tr, adef, 1);      
      if(i < 9)
	treeEvaluate(tr, 1);
      else
	treeEvaluate(tr, 4);           
    }
  /*printf("optimized LH: %f\n", tr->likelihood);*/
  printLog(tr, adef);

}



void makeprotfreqs(rawdata *rdta)
{
  /* calculate amino acid frequencies based on eigmat */
  int i, mineig;

  mineig = 0;
  for (i = 0; i < 20; i++)
    {
      if(fabs(rdta->eigmatPROT[i]) < fabs(rdta->eigmatPROT[mineig]))
	mineig = i;
    }
  


  memcpy(rdta->freqaa, rdta->probmatPROT[mineig], 20 * sizeof(double));
 
  /*rdta->freqaa[0] = 0.076748;
  rdta->freqaa[1] = 0.051691;
  rdta->freqaa[2] = 0.042645;
  rdta->freqaa[3] = 0.051544;
  rdta->freqaa[4] = 0.019803;
  rdta->freqaa[5] = 0.040752;
  rdta->freqaa[6] = 0.061830;
  rdta->freqaa[7] = 0.073152;
  rdta->freqaa[8] = 0.022944;
  rdta->freqaa[9] = 0.053761;
  rdta->freqaa[10] = 0.091904;
  rdta->freqaa[11] = 0.058676;
  rdta->freqaa[12] = 0.023826;
  rdta->freqaa[13] = 0.040126;
  rdta->freqaa[14] = 0.050901;
  rdta->freqaa[15] = 0.068765;
  rdta->freqaa[16] = 0.058565;
  rdta->freqaa[17] = 0.014261;
  rdta->freqaa[18] = 0.032102;
  rdta->freqaa[19] = 0.066005;*/


  for (i = 0; i < 20; i++)
    {
      rdta->freqaa[i] = fabs(rdta->freqaa[i]);
      rdta->invfreqaa[i] = 1.0 / rdta->freqaa[i];
    }

} /* makeprotfreqs */



void init_protmats(rawdata *rdta, analdef *adef)
{
  int i, j, k;

  rdta->probmatPROT = (double **) malloc (20 * sizeof(double *));
  for(i = 0; i < 20; i++)
    rdta->probmatPROT[i] = (double *) malloc (20 * sizeof(double));

  switch(adef->protmodel)
    {
    case M_JTT:
      rdta->eigmatPROT = jtteigmat;
      for(i = 0; i < 20; i++)
	for(j = 0; j < 20; j++)
	  rdta->probmatPROT[i][j] = jttprobmat[i][j];
      break;
    case M_PAM:
      rdta->eigmatPROT = pameigmat;
      for(i = 0; i < 20; i++)
	for(j = 0; j < 20; j++)
	  rdta->probmatPROT[i][j] = pamprobmat[i][j];
      break;
    case M_PMB:
      rdta->eigmatPROT = pmbeigmat;
      for(i = 0; i < 20; i++)
	for(j = 0; j < 20; j++)
	  rdta->probmatPROT[i][j] = pmbprobmat[i][j];
      break;
    }

  for(k = 0; k < 20; k++)
    for(i = 0; i < 20; i++)
      for(j = 0; j < 20; j++)
	rdta->crossMAT[i][j][k] = rdta->probmatPROT[k][i] * rdta->probmatPROT[k][j];
} 


double LnGamma (double alpha)
{
/* returns ln(gamma(alpha)) for alpha>0, accurate to 10 decimal places.  
   Stirling's formula is used for the central polynomial part of the procedure.
   Pike MC & Hill ID (1966) Algorithm 291: Logarithm of the gamma function.
   Communications of the Association for Computing Machinery, 9:684
*/
  double x=alpha, f=0, z, result;

   if (x<7) 
     {
       f=1;  
       z=x-1;
       while (++z<7)  f*=z;
       x=z;   
       f=-log(f);
     }

   z = 1/(x*x);
   
   result = f + (x-0.5)*log(x) - x + .918938533204673 
	  + (((-.000595238095238*z+.000793650793651)*z-.002777777777778)*z
	       +.083333333333333)/x;  

   return result;
}


double IncompleteGamma (double x, double alpha, double ln_gamma_alpha)
{
/* returns the incomplete gamma ratio I(x,alpha) where x is the upper 
	   limit of the integration and alpha is the shape parameter.
   returns (-1) if in error
   ln_gamma_alpha = ln(Gamma(alpha)), is almost redundant.
   (1) series expansion     if (alpha>x || x<=1)
   (2) continued fraction   otherwise
   RATNEST FORTRAN by
   Bhattacharjee GP (1970) The incomplete gamma integral.  Applied Statistics,
   19: 285-287 (AS32)
*/
   int i;
   double p=alpha, g=ln_gamma_alpha;
   double accurate=1e-8, overflow=1e30;
   double factor, gin=0, rn=0, a=0,b=0,an=0,dif=0, term=0, pn[6];

   if (x==0) return (0);
   if (x<0 || p<=0) return (-1);

   factor=exp(p*log(x)-x-g);   
   if (x>1 && x>=p) goto l30;
   /* (1) series expansion */
   gin=1;  term=1;  rn=p;
 l20:
   rn++;
   term*=x/rn;   gin+=term;

   if (term > accurate) goto l20;
   gin*=factor/p;
   goto l50;
 l30:
   /* (2) continued fraction */
   a=1-p;   b=a+x+1;  term=0;
   pn[0]=1;  pn[1]=x;  pn[2]=x+1;  pn[3]=x*b;
   gin=pn[2]/pn[3];
 l32:
   a++;  b+=2;  term++;   an=a*term;
   for (i=0; i<2; i++) pn[i+4]=b*pn[i+2]-an*pn[i];
   if (pn[5] == 0) goto l35;
   rn=pn[4]/pn[5];   dif=fabs(gin-rn);
   if (dif>accurate) goto l34;
   if (dif<=accurate*rn) goto l42;
 l34:
   gin=rn;
 l35:
   for (i=0; i<4; i++) pn[i]=pn[i+2];
   if (fabs(pn[4]) < overflow) goto l32;
   for (i=0; i<4; i++) pn[i]/=overflow;
   goto l32;
 l42:
   gin=1-factor*gin;

 l50:
   return (gin);
}

double PointNormal (double prob)
{
/* returns z so that Prob{x<z}=prob where x ~ N(0,1) and (1e-12)<prob<1-(1e-12)
   returns (-9999) if in error
   Odeh RE & Evans JO (1974) The percentage points of the normal distribution.
   Applied Statistics 22: 96-97 (AS70)

   Newer methods:
     Wichura MJ (1988) Algorithm AS 241: the percentage points of the
       normal distribution.  37: 477-484.
     Beasley JD & Springer SG  (1977).  Algorithm AS 111: the percentage 
       points of the normal distribution.  26: 118-121.

*/
   double a0=-.322232431088, a1=-1, a2=-.342242088547, a3=-.0204231210245;
   double a4=-.453642210148e-4, b0=.0993484626060, b1=.588581570495;
   double b2=.531103462366, b3=.103537752850, b4=.0038560700634;
   double y, z=0, p=prob, p1;

   p1 = (p<0.5 ? p : 1-p);
   if (p1<1e-20) return (-9999);

   y = sqrt (log(1/(p1*p1)));   
   z = y + ((((y*a4+a3)*y+a2)*y+a1)*y+a0) / ((((y*b4+b3)*y+b2)*y+b1)*y+b0);
   return (p<0.5 ? -z : z);
}


double PointChi2 (double prob, double v)
{
/* returns z so that Prob{x<z}=prob where x is Chi2 distributed with df=v
   returns -1 if in error.   0.000002<prob<0.999998
   RATNEST FORTRAN by
       Best DJ & Roberts DE (1975) The percentage points of the 
       Chi2 distribution.  Applied Statistics 24: 385-388.  (AS91)
   Converted into C by Ziheng Yang, Oct. 1993.
*/
   double e=.5e-6, aa=.6931471805, p=prob, g;
   double xx, c, ch, a=0,q=0,p1=0,p2=0,t=0,x=0,b=0,s1,s2,s3,s4,s5,s6;

   if (p<.000002 || p>.999998 || v<=0) return (-1);

   g = LnGamma(v/2);

   xx=v/2;   c=xx-1;
   if (v >= -1.24*log(p)) goto l1;

   ch=pow((p*xx*exp(g+xx*aa)), 1/xx);
   if (ch-e<0) return (ch);
   goto l4;
l1:
   if (v>.32) goto l3;
   ch=0.4;   a=log(1-p);
l2:
   q=ch;  p1=1+ch*(4.67+ch);  p2=ch*(6.73+ch*(6.66+ch));
   t=-0.5+(4.67+2*ch)/p1 - (6.73+ch*(13.32+3*ch))/p2;
   ch-=(1-exp(a+g+.5*ch+c*aa)*p2/p1)/t;
   if (fabs(q/ch-1)-.01 <= 0) goto l4;
   else                       goto l2;
  
l3: 
   x=PointNormal (p);
   p1=0.222222/v;   ch=v*pow((x*sqrt(p1)+1-p1), 3.0);
   if (ch>2.2*v+6)  ch=-2*(log(1-p)-c*log(.5*ch)+g);
l4:
   q=ch;   p1=.5*ch;
   if ((t=IncompleteGamma (p1, xx, g))<0) {
      printf ("\nerr IncompleteGamma");
      return (-1);
   }
   p2=p-t;
   t=p2*exp(xx*aa+g+p1-c*log(ch));   
   b=t/ch;  a=0.5*t-b*c;

   s1=(210+a*(140+a*(105+a*(84+a*(70+60*a))))) / 420;
   s2=(420+a*(735+a*(966+a*(1141+1278*a))))/2520;
   s3=(210+a*(462+a*(707+932*a)))/2520;
   s4=(252+a*(672+1182*a)+c*(294+a*(889+1740*a)))/5040;
   s5=(84+264*a+c*(175+606*a))/2520;
   s6=(120+c*(346+127*c))/5040;
   ch+=t*(1+0.5*t*s1-b*c*(s1-b*(s2-b*(s3-b*(s4-b*(s5-b*s6))))));
   if (fabs(q/ch-1) > e) goto l4;

   return (ch);
}

#define PointGamma(prob,alpha,beta)  PointChi2(prob,2.0*(alpha))/(2.0*(beta))




void makeGammaCats(tree *tr)
{
  int i, K;
  double gap05, t, factor, lnga1, alfa, beta;

  K = tr->gammaCATS;
  alfa = beta = tr->alpha;
  gap05 = 1.0 / (2.0 * K);
  factor = alfa / beta * K;
  
  if(K==1) 
     {
       printf("ERROR in makeGammaCats\n");
       exit(-1);
     }
  
   lnga1=LnGamma(alfa+1);

   for (i=0; i<K-1; i++)
     tr->gammaProbs[i]=PointGamma((i+1.0)/K, alfa, beta);

   for (i=0; i<K-1; i++)
     tr->gammaProbs[i]=IncompleteGamma(tr->gammaProbs[i]*beta, alfa+1, lnga1);

   tr->gammaRates[0] = tr->gammaProbs[0]*factor;

   tr->gammaRates[K-1] = (1-tr->gammaProbs[K-2])*factor;

   for (i=1; i<K-1; i++)  
     tr->gammaRates[i] = (tr->gammaProbs[i]-tr->gammaProbs[i-1])*factor;

   for (i=0; i<K; i++) tr->gammaProbs[i]=1.0/K;

   /*for(i = 0; i < K; i++)
     printf("%d rate %f prob %f\n", i,  tr->gammaRates[i], tr->gammaProbs[i]);*/

   
   return;
  
}

void initModel(tree *tr, rawdata *rdta, cruncheddata *cdta, analdef *adef)
{
  rdta->ttratio =         2.0;
  tr->t_value = 2.0;
  switch(adef->model)
    {
#ifndef HPC
    case M_PROTEINS:
      newview = newviewPROT;
      makenewz = makenewzPROT;
      evaluate = evaluatePROT;
      init_protmats(rdta, adef);
      makeprotfreqs(rdta);
      break;
    case M_PROTEINSCAT:
      newview = newviewPROTCAT;
      makenewz = makenewzPROTCAT;
      evaluate = evaluatePROTCAT;
      evaluatePartial = evaluatePartialPROTCAT;
      newviewPartial =  newviewPartialPROTCAT;
      initravPartial = initravPartialPROTCAT;   
      init_protmats(rdta, adef);
      makeprotfreqs(rdta);
      break;
    case M_HKY85:
      newview = newviewHKY85;
      makenewz = makenewzHKY85;
      evaluate = evaluateHKY85;
      empiricalfreqs(adef, rdta, cdta);
      break;
    case M_HKY85GAMMA:
      newview = newviewHKY85GAMMA;
      makenewz = makenewzHKY85GAMMA;
      evaluate = evaluateHKY85GAMMA;
      empiricalfreqs(adef, rdta, cdta);
      tr->alpha = 1.0;
      tr->gammaCATS = 4;
      tr->gammaRates = (double *)malloc(sizeof(double) * tr->gammaCATS);
      tr->gammaProbs = (double *)malloc(sizeof(double) * tr->gammaCATS);
      makeGammaCats(tr);
      break;  
#endif 
    case M_HKY85CAT:
      newview = newviewHKY85CAT;
      makenewz = makenewzHKY85CAT;
      evaluate = evaluateHKY85CAT;
      evaluatePartial = evaluatePartialHKY85CAT;
      newviewPartial =  newviewPartialHKY85CAT;
      initravPartial = initravPartialHKY85CAT;     
      empiricalfreqs(adef, rdta, cdta);
      break;
#ifndef HPC
    case M_GTR:
      newview = newviewGTR;
      makenewz = makenewzGTR;
      evaluate = evaluateGTR;      
      baseFrequenciesGTR(rdta, cdta);       
      rdta->initialRates[0] = 0.5;
      rdta->initialRates[1] =  0.5;
      rdta->initialRates[2] = 0.5;
      rdta->initialRates[3] = 0.5;
      rdta->initialRates[4] = 0.5;      
      initReversibleGTR(rdta, cdta);
      break;
    case M_GTRGAMMA:
      newview = newviewGTRGAMMA;
      makenewz = makenewzGTRGAMMA;
      evaluate = evaluateGTRGAMMA;      
      baseFrequenciesGTR(rdta, cdta);       
      rdta->initialRates[0] = 0.5;
      rdta->initialRates[1] =  0.5;
      rdta->initialRates[2] = 0.5;
      rdta->initialRates[3] = 0.5;
      rdta->initialRates[4] = 0.5;      
      initReversibleGTR(rdta, cdta);
      tr->alpha = 1.0;
      tr->gammaCATS = 4;
      tr->gammaRates = (double *)malloc(sizeof(double) * tr->gammaCATS);
      tr->gammaProbs = (double *)malloc(sizeof(double) * tr->gammaCATS);
      makeGammaCats(tr);
      break;
#endif
    case M_GTRCAT:
      newview = newviewGTRCAT;
      makenewz = makenewzGTRCAT;
      evaluate = evaluateGTRCAT;
      evaluatePartial = evaluatePartialGTRCAT;
      newviewPartial =  newviewPartialGTRCAT;
      initravPartial = initravPartialGTRCAT;
      baseFrequenciesGTR(rdta, cdta);       
      rdta->initialRates[0] = 0.5;
      rdta->initialRates[1] =  0.5;
      rdta->initialRates[2] = 0.5;
      rdta->initialRates[3] = 0.5;
      rdta->initialRates[4] = 0.5;      
      initReversibleGTR(rdta, cdta);
      break;
    }
}



void initModelPack(tree * tr, analdef * adef)
{
  switch (adef->model)
    {
    case M_GTR:
      tr->modelParameters = 8;
      tr->modelData = (double *) malloc(sizeof(double) * tr->modelParameters);
      break;
    case M_GTRCAT:
      tr->modelParameters = 9 + 5 * tr->cdta->endsite;
      tr->modelData = (double *) malloc(sizeof(double) * tr->modelParameters);
      break;
    case M_HKY85:
      tr->modelParameters = 4;
      tr->modelData = (double *) malloc(sizeof(double) * tr->modelParameters);
      break;
    case M_HKY85CAT:
      tr->modelParameters = 5 + 5 * tr->cdta->endsite;
      tr->modelData = (double *) malloc(sizeof(double) * tr->modelParameters);
      break;
    case M_PROTEINSCAT:
      tr->modelParameters = 3 + 5 * tr->cdta->endsite;
      tr->modelData = (double *) malloc(sizeof(double) * tr->modelParameters);
      break;
    case M_PROTEINS:
      tr->modelParameters = 3;      
      tr->modelData = (double *) malloc(sizeof(double) * tr->modelParameters);
      break;
  }
}

void packCategories(tree * tr, double *m)
{
  int i;
  double *m_ptr = m;

  for (i = 0; i < tr->cdta->endsite; i++)
    *m_ptr++ = tr->cdta->rateCategory[i];

  for (i = 0; i < tr->cdta->endsite; i++)
    *m_ptr++ = tr->cdta->patratStored[i];

  for (i = 0; i < tr->cdta->endsite; i++)
    *m_ptr++ = tr->cdta->patrat[i];

  for (i = 0; i < tr->cdta->endsite; i++)
    *m_ptr++ = tr->cdta->wr[i];

  for (i = 0; i < tr->cdta->endsite; i++)
    *m_ptr++ = tr->cdta->wr2[i];

}

void unpackCategories(tree * tr, double *m)
{
  int i;
  double *m_ptr = m;

  for (i = 0; i < tr->cdta->endsite; i++)
    tr->cdta->rateCategory[i] = *m_ptr++;

  for (i = 0; i < tr->cdta->endsite; i++)
    tr->cdta->patratStored[i] = *m_ptr++;

  for (i = 0; i < tr->cdta->endsite; i++)
    tr->cdta->patrat[i] = *m_ptr++;

  for (i = 0; i < tr->cdta->endsite; i++)
    tr->cdta->wr[i] = *m_ptr++;

  for (i = 0; i < tr->cdta->endsite; i++)
    tr->cdta->wr2[i] = *m_ptr++;

}

void packModel(tree * tr, analdef * adef)
{
  int i;
  printf("pack model LH %f\n", tr->likelihood);

  tr->modelData[0] = (double) optimizeTTinvocations;
  tr->modelData[1] = (double) optimizeRatesInvocations;
  tr->modelData[2] = (double) optimizeRateCategoryInvocations;

  switch (adef->model)
  {
    case M_GTR:
      for (i = 3; i < 8; i++)
        tr->modelData[i] = tr->rdta->initialRates[i - 3];
      break;
    case M_GTRCAT:
      for (i = 3; i < 8; i++)
        tr->modelData[i] = tr->rdta->initialRates[i - 3];
      tr->modelData[8] = (double) tr->NumberOfCategories;
      packCategories(tr, &tr->modelData[9]);
      break;
    case M_HKY85:
      tr->modelData[3] = tr->t_value;
      break;
    case M_HKY85CAT:
      tr->modelData[3] = tr->t_value;
      tr->modelData[4] = (double) tr->NumberOfCategories;
      packCategories(tr, &tr->modelData[5]);
      break;
  case M_PROTEINSCAT:
    packCategories(tr, &tr->modelData[3]);
    break;
  }
}

void unpackModel(tree * tr, analdef * adef)
{
  int i;

  optimizeTTinvocations = (int) tr->modelData[0];
  optimizeRatesInvocations = (int) tr->modelData[1];
  optimizeRateCategoryInvocations = (int) tr->modelData[2];

  switch (adef->model)
    {
    case M_GTR:
      for (i = 3; i < 8; i++)
        tr->rdta->initialRates[i - 3] = tr->modelData[i];
      initReversibleGTR(tr->rdta, tr->cdta);
      break;
    case M_GTRCAT:
      for (i = 3; i < 8; i++)
        tr->rdta->initialRates[i - 3] = tr->modelData[i];
      tr->NumberOfCategories = (int) tr->modelData[8];
      initReversibleGTR(tr->rdta, tr->cdta);
      unpackCategories(tr, &tr->modelData[9]);
      break;
    case M_HKY85:
      tr->t_value = tr->modelData[3];
      alterTT(tr, tr->t_value);
      break;
    case M_HKY85CAT:
      tr->t_value = tr->modelData[3];
      tr->NumberOfCategories = (int) tr->modelData[4];
      unpackCategories(tr, &tr->modelData[5]);
      alterTT(tr, tr->t_value);
      break;
    case M_PROTEINSCAT:
      unpackCategories(tr, &tr->modelData[3]);
      break;
  }
  initrav(tr, tr->start);
  initrav(tr, tr->start->back);
  evaluate(tr, tr->start);
  printf("unpack model LH %f\n", tr->likelihood);
}


int filexists(char *filename)
{
  FILE *fp;
  int res;
  fp = fopen(filename,"r");
  


  if(fp) 
    {
      res = 1;
      fclose(fp);
    }
  else 
    res = 0;
       
  return res;
} 



double pie[20];
double eigvecs[20][20];
double eig[20];

void givens(double (*a)[20], long i, long j, long n, double ctheta,
                        double stheta, boolean left)
{ /* Givens transform at i,j for 1..n with angle theta */
  long k;
  double d;

  for (k = 0; k < n; k++) {
    if (left) {
      d = ctheta * a[i - 1][k] + stheta * a[j - 1][k];
      a[j - 1][k] = ctheta * a[j - 1][k] - stheta * a[i - 1][k];
      a[i - 1][k] = d;
    } else {
      d = ctheta * a[k][i - 1] + stheta * a[k][j - 1];
      a[k][j - 1] = ctheta * a[k][j - 1] - stheta * a[k][i - 1];
      a[k][i - 1] = d;
    }
  }
}  /* givens */


void coeffs(double x, double y, double *c, double *s, double accuracy)
{ /* compute cosine and sine of theta */
  double root;

  root = sqrt(x * x + y * y);
  if (root < accuracy) {
    *c = 1.0;
    *s = 0.0;
  } else {
    *c = x / root;
    *s = y / root;
  }
}  /* coeffs */


void tridiag(double (*a)[20], long n, double accuracy)
{ /* Givens tridiagonalization */
  long i, j;
  double s, c;

  for (i = 2; i < n; i++) {
    for (j = i + 1; j <= n; j++) {
      coeffs(a[i - 2][i - 1], a[i - 2][j - 1], &c, &s, accuracy);
      givens(a, i, j, n, c, s, TRUE);
      givens(a, i, j, n, c, s, FALSE);
      givens(eigvecs, i, j, n, c, s, TRUE);
    }
  }
}  /* tridiag */


void shiftqr(double (*a)[20], long n, double accuracy)
{ /* QR eigenvalue-finder */
  long i, j;
  double approx, s, c, d, TEMP, TEMP1;

  for (i = n; i >= 2; i--) {
    do {
      TEMP = a[i - 2][i - 2] - a[i - 1][i - 1];
      TEMP1 = a[i - 1][i - 2];
      d = sqrt(TEMP * TEMP + TEMP1 * TEMP1);
      approx = a[i - 2][i - 2] + a[i - 1][i - 1];
      if (a[i - 1][i - 1] < a[i - 2][i - 2])
        approx = (approx - d) / 2.0;
      else
        approx = (approx + d) / 2.0;
      for (j = 0; j < i; j++)
        a[j][j] -= approx;
      for (j = 1; j < i; j++) {
        coeffs(a[j - 1][j - 1], a[j][j - 1], &c, &s, accuracy);
        givens(a, j, j + 1, i, c, s, TRUE);
        givens(a, j, j + 1, i, c, s, FALSE);
        givens(eigvecs, j, j + 1, n, c, s, TRUE);
      }
      for (j = 0; j < i; j++)
        a[j][j] += approx;
    } while (fabs(a[i - 1][i - 2]) > accuracy);
  }
}  /* shiftqr */



void qreigen(double (*prob)[20], long n)
{ /* QR eigenvector/eigenvalue method for symmetric matrix */
  double accuracy;
  long i, j;

  accuracy = 1.0e-6;
  for (i = 0; i < n; i++) 
    {
      for (j = 0; j < n; j++)
	eigvecs[i][j] = 0.0;
      eigvecs[i][i] = 1.0;
    }
  tridiag(prob, n, accuracy);
  shiftqr(prob, n, accuracy);

  for (i = 0; i < n; i++)
    {
      /*printf("KAKA %d %f\n", i, prob[i][i]);*/
      eig[i] = prob[i][i];
    }

  /*for (i = 0; i <= 19; i++) 
    {
      for (j = 0; j <= 19; j++)
	prob[i][j] = sqrt(pie[j]) * eigvecs[i][j];
	}*/
}


void experiment()
{
  int i, j;
  double prob[20][20];
  double pie[20];
  double sum, scaler;

  prob[ 0][ 0] =   0; prob[ 0][ 1] =  58; prob[ 0][ 2] =  54; prob[ 0][ 3] =  81; prob[ 0][ 4] =  56; 
  prob[ 0][ 5] =  57; prob[ 0][ 6] = 105; prob[ 0][ 7] = 179; prob[ 0][ 8] =  27; prob[ 0][ 9] =  36; 
  prob[ 0][10] =  30; prob[ 0][11] =  35; prob[ 0][12] =  54; prob[ 0][13] =  15; prob[ 0][14] = 194; 
  prob[ 0][15] = 378; prob[ 0][16] = 475; prob[ 0][17] =   9; prob[ 0][18] =  11; prob[ 0][19] = 298; 
  prob[ 1][ 0] =  58; prob[ 1][ 1] =   0; prob[ 1][ 2] =  45; prob[ 1][ 3] =  16; prob[ 1][ 4] = 113; 
  prob[ 1][ 5] = 310; prob[ 1][ 6] =  29; prob[ 1][ 7] = 137; prob[ 1][ 8] = 328; prob[ 1][ 9] =  22; 
  prob[ 1][10] =  38; prob[ 1][11] = 646; prob[ 1][12] =  44; prob[ 1][13] =   5; prob[ 1][14] =  74; 
  prob[ 1][15] = 101; prob[ 1][16] =  64; prob[ 1][17] = 126; prob[ 1][18] =  20; prob[ 1][19] =  17; 
  prob[ 2][ 0] =  54; prob[ 2][ 1] =  45; prob[ 2][ 2] =   0; prob[ 2][ 3] = 528; prob[ 2][ 4] =  34; 
  prob[ 2][ 5] =  86; prob[ 2][ 6] =  58; prob[ 2][ 7] =  81; prob[ 2][ 8] = 391; prob[ 2][ 9] =  47; 
  prob[ 2][10] =  12; prob[ 2][11] = 263; prob[ 2][12] =  30; prob[ 2][13] =  10; prob[ 2][14] =  15; 
  prob[ 2][15] = 503; prob[ 2][16] = 232; prob[ 2][17] =   8; prob[ 2][18] =  70; prob[ 2][19] =  16; 
  prob[ 3][ 0] =  81; prob[ 3][ 1] =  16; prob[ 3][ 2] = 528; prob[ 3][ 3] =   0; prob[ 3][ 4] =  10; 
  prob[ 3][ 5] =  49; prob[ 3][ 6] = 767; prob[ 3][ 7] = 130; prob[ 3][ 8] = 112; prob[ 3][ 9] =  11; 
  prob[ 3][10] =   7; prob[ 3][11] =  26; prob[ 3][12] =  15; prob[ 3][13] =   4; prob[ 3][14] =  15; 
  prob[ 3][15] =  59; prob[ 3][16] =  38; prob[ 3][17] =   4; prob[ 3][18] =  46; prob[ 3][19] =  31; 
  prob[ 4][ 0] =  56; prob[ 4][ 1] = 113; prob[ 4][ 2] =  34; prob[ 4][ 3] =  10; prob[ 4][ 4] =   0; 
  prob[ 4][ 5] =   9; prob[ 4][ 6] =   5; prob[ 4][ 7] =  59; prob[ 4][ 8] =  69; prob[ 4][ 9] =  17; 
  prob[ 4][10] =  23; prob[ 4][11] =   7; prob[ 4][12] =  31; prob[ 4][13] =  78; prob[ 4][14] =  14; 
  prob[ 4][15] = 223; prob[ 4][16] =  42; prob[ 4][17] = 115; prob[ 4][18] = 209; prob[ 4][19] =  62; 
  prob[ 5][ 0] =  57; prob[ 5][ 1] = 310; prob[ 5][ 2] =  86; prob[ 5][ 3] =  49; prob[ 5][ 4] =   9; 
  prob[ 5][ 5] =   0; prob[ 5][ 6] = 323; prob[ 5][ 7] =  26; prob[ 5][ 8] = 597; prob[ 5][ 9] =   9; 
  prob[ 5][10] =  72; prob[ 5][11] = 292; prob[ 5][12] =  43; prob[ 5][13] =   4; prob[ 5][14] = 164; 
  prob[ 5][15] =  53; prob[ 5][16] =  51; prob[ 5][17] =  18; prob[ 5][18] =  24; prob[ 5][19] =  20; 
  prob[ 6][ 0] = 105; prob[ 6][ 1] =  29; prob[ 6][ 2] =  58; prob[ 6][ 3] = 767; prob[ 6][ 4] =   5; 
  prob[ 6][ 5] = 323; prob[ 6][ 6] =   0; prob[ 6][ 7] = 119; prob[ 6][ 8] =  26; prob[ 6][ 9] =  12; 
  prob[ 6][10] =   9; prob[ 6][11] = 181; prob[ 6][12] =  18; prob[ 6][13] =   5; prob[ 6][14] =  18; 
  prob[ 6][15] =  30; prob[ 6][16] =  32; prob[ 6][17] =  10; prob[ 6][18] =   7; prob[ 6][19] =  45; 
  prob[ 7][ 0] = 179; prob[ 7][ 1] = 137; prob[ 7][ 2] =  81; prob[ 7][ 3] = 130; prob[ 7][ 4] =  59; 
  prob[ 7][ 5] =  26; prob[ 7][ 6] = 119; prob[ 7][ 7] =   0; prob[ 7][ 8] =  23; prob[ 7][ 9] =   6; 
  prob[ 7][10] =   6; prob[ 7][11] =  27; prob[ 7][12] =  14; prob[ 7][13] =   5; prob[ 7][14] =  24; 
  prob[ 7][15] = 201; prob[ 7][16] =  33; prob[ 7][17] =  55; prob[ 7][18] =   8; prob[ 7][19] =  47; 
  prob[ 8][ 0] =  27; prob[ 8][ 1] = 328; prob[ 8][ 2] = 391; prob[ 8][ 3] = 112; prob[ 8][ 4] =  69; 
  prob[ 8][ 5] = 597; prob[ 8][ 6] =  26; prob[ 8][ 7] =  23; prob[ 8][ 8] =   0; prob[ 8][ 9] =  16; 
  prob[ 8][10] =  56; prob[ 8][11] =  45; prob[ 8][12] =  33; prob[ 8][13] =  40; prob[ 8][14] = 115; 
  prob[ 8][15] =  73; prob[ 8][16] =  46; prob[ 8][17] =   8; prob[ 8][18] = 573; prob[ 8][19] =  11; 
  prob[ 9][ 0] =  36; prob[ 9][ 1] =  22; prob[ 9][ 2] =  47; prob[ 9][ 3] =  11; prob[ 9][ 4] =  17; 
  prob[ 9][ 5] =   9; prob[ 9][ 6] =  12; prob[ 9][ 7] =   6; prob[ 9][ 8] =  16; prob[ 9][ 9] =   0; 
  prob[ 9][10] = 229; prob[ 9][11] =  21; prob[ 9][12] = 479; prob[ 9][13] =  89; prob[ 9][14] =  10; 
  prob[ 9][15] =  40; prob[ 9][16] = 245; prob[ 9][17] =   9; prob[ 9][18] =  32; prob[ 9][19] = 961; 
  prob[10][ 0] =  30; prob[10][ 1] =  38; prob[10][ 2] =  12; prob[10][ 3] =   7; prob[10][ 4] =  23; 
  prob[10][ 5] =  72; prob[10][ 6] =   9; prob[10][ 7] =   6; prob[10][ 8] =  56; prob[10][ 9] = 229; 
  prob[10][10] =   0; prob[10][11] =  14; prob[10][12] = 388; prob[10][13] = 248; prob[10][14] = 102; 
  prob[10][15] =  59; prob[10][16] =  25; prob[10][17] =  52; prob[10][18] =  24; prob[10][19] = 180; 
  prob[11][ 0] =  35; prob[11][ 1] = 646; prob[11][ 2] = 263; prob[11][ 3] =  26; prob[11][ 4] =   7; 
  prob[11][ 5] = 292; prob[11][ 6] = 181; prob[11][ 7] =  27; prob[11][ 8] =  45; prob[11][ 9] =  21; 
  prob[11][10] =  14; prob[11][11] =   0; prob[11][12] =  65; prob[11][13] =   4; prob[11][14] =  21; 
  prob[11][15] =  47; prob[11][16] = 103; prob[11][17] =  10; prob[11][18] =   8; prob[11][19] =  14; 
  prob[12][ 0] =  54; prob[12][ 1] =  44; prob[12][ 2] =  30; prob[12][ 3] =  15; prob[12][ 4] =  31; 
  prob[12][ 5] =  43; prob[12][ 6] =  18; prob[12][ 7] =  14; prob[12][ 8] =  33; prob[12][ 9] = 479; 
  prob[12][10] = 388; prob[12][11] =  65; prob[12][12] =   0; prob[12][13] =  43; prob[12][14] =  16; 
  prob[12][15] =  29; prob[12][16] = 226; prob[12][17] =  24; prob[12][18] =  18; prob[12][19] = 323; 
  prob[13][ 0] =  15; prob[13][ 1] =   5; prob[13][ 2] =  10; prob[13][ 3] =   4; prob[13][ 4] =  78; 
  prob[13][ 5] =   4; prob[13][ 6] =   5; prob[13][ 7] =   5; prob[13][ 8] =  40; prob[13][ 9] =  89; 
  prob[13][10] = 248; prob[13][11] =   4; prob[13][12] =  43; prob[13][13] =   0; prob[13][14] =  17; 
  prob[13][15] =  92; prob[13][16] =  12; prob[13][17] =  53; prob[13][18] = 536; prob[13][19] =  62; 
  prob[14][ 0] = 194; prob[14][ 1] =  74; prob[14][ 2] =  15; prob[14][ 3] =  15; prob[14][ 4] =  14; 
  prob[14][ 5] = 164; prob[14][ 6] =  18; prob[14][ 7] =  24; prob[14][ 8] = 115; prob[14][ 9] =  10; 
  prob[14][10] = 102; prob[14][11] =  21; prob[14][12] =  16; prob[14][13] =  17; prob[14][14] =   0; 
  prob[14][15] = 285; prob[14][16] = 118; prob[14][17] =   6; prob[14][18] =  10; prob[14][19] =  23; 
  prob[15][ 0] = 378; prob[15][ 1] = 101; prob[15][ 2] = 503; prob[15][ 3] =  59; prob[15][ 4] = 223; 
  prob[15][ 5] =  53; prob[15][ 6] =  30; prob[15][ 7] = 201; prob[15][ 8] =  73; prob[15][ 9] =  40; 
  prob[15][10] =  59; prob[15][11] =  47; prob[15][12] =  29; prob[15][13] =  92; prob[15][14] = 285; 
  prob[15][15] =   0; prob[15][16] = 477; prob[15][17] =  35; prob[15][18] =  63; prob[15][19] =  38; 
  prob[16][ 0] = 475; prob[16][ 1] =  64; prob[16][ 2] = 232; prob[16][ 3] =  38; prob[16][ 4] =  42; 
  prob[16][ 5] =  51; prob[16][ 6] =  32; prob[16][ 7] =  33; prob[16][ 8] =  46; prob[16][ 9] = 245; 
  prob[16][10] =  25; prob[16][11] = 103; prob[16][12] = 226; prob[16][13] =  12; prob[16][14] = 118; 
  prob[16][15] = 477; prob[16][16] =   0; prob[16][17] =  12; prob[16][18] =  21; prob[16][19] = 112; 
  prob[17][ 0] =   9; prob[17][ 1] = 126; prob[17][ 2] =   8; prob[17][ 3] =   4; prob[17][ 4] = 115; 
  prob[17][ 5] =  18; prob[17][ 6] =  10; prob[17][ 7] =  55; prob[17][ 8] =   8; prob[17][ 9] =   9; 
  prob[17][10] =  52; prob[17][11] =  10; prob[17][12] =  24; prob[17][13] =  53; prob[17][14] =   6; 
  prob[17][15] =  35; prob[17][16] =  12; prob[17][17] =   0; prob[17][18] =  71; prob[17][19] =  25; 
  prob[18][ 0] =  11; prob[18][ 1] =  20; prob[18][ 2] =  70; prob[18][ 3] =  46; prob[18][ 4] = 209; 
  prob[18][ 5] =  24; prob[18][ 6] =   7; prob[18][ 7] =   8; prob[18][ 8] = 573; prob[18][ 9] =  32; 
  prob[18][10] =  24; prob[18][11] =   8; prob[18][12] =  18; prob[18][13] = 536; prob[18][14] =  10; 
  prob[18][15] =  63; prob[18][16] =  21; prob[18][17] =  71; prob[18][18] =   0; prob[18][19] =  16; 
  prob[19][ 0] = 298; prob[19][ 1] =  17; prob[19][ 2] =  16; prob[19][ 3] =  31; prob[19][ 4] =  62; 
  prob[19][ 5] =  20; prob[19][ 6] =  45; prob[19][ 7] =  47; prob[19][ 8] =  11; prob[19][ 9] = 961; 
  prob[19][10] = 180; prob[19][11] =  14; prob[19][12] = 323; prob[19][13] =  62; prob[19][14] =  23; 
  prob[19][15] =  38; prob[19][16] = 112; prob[19][17] =  25; prob[19][18] =  16; prob[19][19] =   0; 

  pie[ 0] = 0.076748;
  pie[ 1] = 0.051691;
  pie[ 2] = 0.042645;
  pie[ 3] = 0.051544;
  pie[ 4] = 0.019803;
  pie[ 5] = 0.040752;
  pie[ 6] = 0.061830;
  pie[ 7] = 0.073152;
  pie[ 8] = 0.022944;
  pie[ 9] = 0.053761;
  pie[10] = 0.091904;
  pie[11] = 0.058676;
  pie[12] = 0.023826;
  pie[13] = 0.040126;
  pie[14] = 0.050901;
  pie[15] = 0.068765;
  pie[16] = 0.058565;
  pie[17] = 0.014261;
  pie[18] = 0.032102;
  pie[19] = 0.066004; /*changed  to 0.066004 from 0.066005 such that Sum pie[i] == 1 */
	

  /* SYMMETRY CHECK */

  for(i = 0; i < 20; i++)
    for(j = 0; j < 20; j++)
      {
	if(prob[i][j] != prob[j][i])
	  {
	    printf("NOT SYMM\n");
	    exit(1);
	  }
      }

  /* verify THAT base-freqs sum to 1 */

  sum = 0;

  for (i=0; i<20; i++)
    sum += pie[i];    
  
 
  printf("Freq-Sum %1.20f\n", sum);           
    
  /* scale probs correctly, needed? */

  for(i = 0; i < 20; i++)
    for(j = 0; j < 20; j++)      
      prob[i][j] =  prob[i][j]/100;
   for(i = 0; i < 20; i++)
    for(j = 0; j < 20; j++)      
      prob[i][j] =  prob[i][j] * pie[j];
 
  /* scale by mean rate */

  scaler = 0.0;
  
  for (i=0; i<20; i++)
    {
      for (j=i+1; j<20; j++)
	{
	  scaler += pie[i] * prob[i][j];
	  scaler += pie[j] * prob[j][i];
	}
    }

  scaler = 1.0 / scaler;

  for (i=0; i<20; i++)
    for (j=0; j<20; j++)
      if(i != j)
	prob[i][j] *= scaler;
  
  sum = 0;
  for(i = 0; i < 20; i++)
    for(j = 0; j < 20; j++)
      if(i != j) 
	sum += pie[i] * prob[i][j];

  printf("SUM ONE is %f\n", sum);

  for(i = 0; i < 20; i++)
     {
       sum = 0;
      
       for(j = 0; j < 20; j++)      
	 sum += pie[j] * prob[i][j];
	   
       prob[i][i] = -sum / pie[i];
       printf("SUM %f\n", (prob[i][i] * pie[i]) + sum);
     }
     
   
  qreigen(prob, 20);


  for(j = 0; j < 20; j++)
    {
      printf("E%d: %f\n", j, eig[j]);
    }

  printf("\n");
  exit(1);
}





main (int argc, char *argv[]) 
  {   
    rawdata      *rdta;
    cruncheddata  *cdta;
    tree         *tr;   
    int *perm;    
    double t;
    char *parsimonyTree;

    NWC = 0;

    masterTime = gettime();

#ifdef PARALLEL
    MPI_Init(&argc, &argv);
    MPI_Comm_rank(MPI_COMM_WORLD, &myId);    
    MPI_Comm_size(MPI_COMM_WORLD, &numOfWorkers);
#endif   
       

    /*experiment();*/

    adef = (analdef *) malloc(sizeof(analdef));
    rdta = (rawdata *)malloc(sizeof(rawdata));
    cdta = (cruncheddata *)malloc(sizeof(cruncheddata));     
    tr = (tree *)malloc(sizeof(tree));

    initAdef(adef);
    get_args(argc,argv,(boolean)1, adef, rdta);              
   	    	         
    INFILE = fopen(seq_file, "r");
    if (!INFILE)
      {
	printf( "Could not open sequence file: %s\n", seq_file);
	return 1;
      }

     
    getinput(adef, rdta, cdta, tr);   
         
    fclose(INFILE);
   
    if (! makeweights(adef, rdta, cdta))                             return 1; 

   
    if (! makevalues(rdta, cdta))                                    return 1;
      
    initModel(tr, rdta, cdta, adef); /* consumes 4 secs during init */
            
    if (! linkdata2tree(rdta, cdta, tr, adef))                             return 1;  
   
    if (! linkxarray(3, 3, cdta->endsite, & freextip, & usedxtip, adef))   return 1;
        
    if (! setupnodex(tr, adef))                                            return 1; 
        
    if(adef->mode == PARALLEL_MODE)
      initModelPack(tr, adef);

    /*printf("After setup %d\n", myId);*/

#ifndef OPEN_MP
#ifndef HPC
   switch(adef->model)
     {
     case  M_HKY85:    
       if( ! setupEqualityVectorHKY85(tr)) return 1;
       break;
     case M_GTR:
        if( ! setupEqualityVectorGTR(tr)) return 1; 
	break;
     }   
#endif
#endif

   
    treeStringLength = (tr->mxtips * (nmlngth+32)) + 256 + (tr->mxtips * 2);   




#ifndef HPC
    perm = (int *)malloc((2 * tr->mxtips + 1) * sizeof(int));
#endif


    /* OLD if(!(adef->mode == BIG_MODE || adef->mode == PARALLEL_MODE))
    {
      bt = (bestlist *) malloc(sizeof(bestlist));
      bt->ninit = 0;
      initBestTree(bt, 20, tr->mxtips, tr->cdta->endsite);
      }*/

    /* NEW */
    bt = (bestlist *) malloc(sizeof(bestlist));
    bt->ninit = 0;
    initBestTree(bt, 20, tr->mxtips, tr->cdta->endsite);
    /* NEW */

    
    strcpy(permFileName, workdir);    
    strcpy(resultFileName, workdir);
    strcpy(logFileName, workdir);
    strcpy(checkpointFileName, workdir);
    strcpy(infoFileName, workdir);
    
    strcat(permFileName, "RAxML_parsimonyTree.");
    strcat(resultFileName, "RAxML_result.");
    strcat(logFileName, "RAxML_log.");
    strcat(checkpointFileName, "RAxML_checkpoint.");
    strcat(infoFileName, "RAxML_info.");

    strcat(infoFileName, run_id);
    strcat(permFileName, run_id);
    strcat(resultFileName, run_id);
    strcat(logFileName, run_id);
    
    strcat(checkpointFileName, run_id);
    strcat(checkpointFileName,".");

    
    /*printf("After filenames %d\n", myId);*/

#ifndef PARALLEL
   
	/*printf("in printInfo %d\n", myId);*/
	if(filexists(infoFileName))
	  {
	    printf("RAxML output files with the run ID <%s> already exist \n", run_id);
	    printf("in directory %s ...... exiting\n", workdir);
	    exit(-1);
	  }

	logFile = fopen(logFileName, "w");
        
	printModelAndProgramInfo(tr, adef, argc, argv);
      
#endif	
     
    /*printf("After printinfo %d\n", myId);*/
   

    if (adef->restart) 
      {	 	
	INFILE = fopen(tree_file, "r");	
	if (!INFILE)
	  {
	    printf( "Could not open input tree: %s\n", tree_file);
	    return FALSE;
	  }       
	if (! treeReadLen(INFILE, tr))          return FALSE;	/* consumes 12 secs during setup */
	
	treeEvaluate(tr, 4); 	/* consumes */		

	fclose(INFILE);
      }
#ifndef HPC
    else
      {       
	if(adef->model == M_PROTEINS || adef->model ==  M_PROTEINSCAT)	  
	  {
	    char protParsCommand[1024];
	    
	    
	    strcpy(protParsCommand,"./protpars ");
	    strcat(protParsCommand, seq_file);
	    strcat(protParsCommand," "); 
	    strcat(protParsCommand, permFileName);	    

	    system(protParsCommand);

	    INFILE = fopen(permFileName, "r");	
	    if (!INFILE)
	      {
		printf( "Could not open protein parsimony tree file: %s\n", permFileName);
		return FALSE;
	      }
	    if (! treeReadLen(INFILE, tr))          return FALSE;	
	  
	    treeEvaluate(tr, 4);       		/* consumes 580 seconds */    
	    fclose(INFILE);
	  }
	else
	  {
	    makePermutation(perm, tr->mxtips);	    
	    RAxML_parsimony(perm, treeStringLength, &parsimonyTree);	    
	    str_treeReadLenSpecial(parsimonyTree ,tr);
	    treeEvaluate(tr, 4);	
	    free(parsimonyTree);	    	       
	    permutationFile = fopen(permFileName,"w");
	    printTree(permutationFile, tr, 0);
	    fclose(permutationFile);
	    RAxML_parsimony_cleanup();
	  }
      }   
#endif   

    if(adef->startingTreeOnly)
      exit(1);        

    switch(adef->mode)
      {      
      case HEAVY_MODE: computeHeavy(tr, adef);
      case TREE_EVALUATION : treeEvaluation(tr, adef);break; 
      case TREE_INFERENCE : compute(tr, adef);break;
      case FAST_MODE : computeFast(tr, adef); break;
      case SIMULATED_ANNEALING : simulatedAnnealing(tr, adef);break;
      case TREE_RECURSIVE: computeRecursive(tr, adef);break;	
	/*case PRE_SELECT_MODE: computePreSelect(tr, adef); break;*/
#ifdef PARALLEL
      case PARALLEL_MODE: computePARALLEL(tr, adef); break;
#endif
      case SPR: computeSPR(tr, adef); break;
      case BIG_MODE: computeBIG(tr, adef); break;
      case BIG_FAST_MODE: computeBIGRAPIDFAST(tr, adef); break;
      case BIG_RAPID_MODE: computeBIGRAPID(tr, adef); break;	
      }
    
    fclose(logFile);
    
    printf("\n\n");

    if(adef->checkpoints)
      {	
	if(checkPointCounter > 1)
	  printf("intermediate checkpoints written to: %s0-%d\n", checkpointFileName, (checkPointCounter - 1));
	else
	  printf("checkpoint written to: %s0\n", checkpointFileName); 
      }
    
    if((!adef->restart) && (!adef->mode == TREE_EVALUATION))
      printf("Parsimony starting tree written to: %s\n", permFileName);   	          
    if(!adef->mode == TREE_EVALUATION)
      printf("Final tree written to: %s\n", resultFileName);  
    printf("Execution LogFile written to: %s\n", logFileName);   
    printf("Execution information file written to: %s\n", infoFileName);
  }
